// Making a class suitable as the key for an associative array

import std.c.stdlib; // for rand

class MyKey {

  int a,b;

  this(int aa, int bb) {
    a = aa;
    b = bb;
  }

  // Need next three functions for assoc arrays to work:
  hash_t toHash() {
    return a+b;
  }

  int opEquals(Object o) {
    // See if o can be a MyKey.
    MyKey mk = cast(MyKey) o;
    // Equal if it worked, and its a and b are the same as ours.
    return mk && a == mk.a && b == mk.b;
  }

  // Return -1, 0 or 1 for ordering
  int opCmp(Object o) {
    MyKey mk = cast(MyKey) o;
    if (!mk) return -1; // we're less than any non-MyKey
    if (a == mk.a)
      return b-mk.b; // if tie on a, decide using b
    // Else decide on a
    return a-mk.a;
  }

}

// Now use it
void main() {
  int[MyKey] mka;
  for (int i=0; i<10; i++) {
    int a = rand()%100;
    int b = rand()%100;
    mka[new MyKey(a,b)] = 100*a+b;
  }
  mka.rehash; // for efficiency, now we've filled it
  foreach (MyKey mk ; mka.keys.sort) {
    printf("mk(%d,%d) = %d\n",mk.a,mk.b,mka[mk]);
  }
}

