module getopts;

import std.conv;
//import std.stdio;

class Options {

private:

  bool[char] hasValue;  // true for switches, false for flags
  bool[char] isSet;     // whether each flag is set
  char[][char] valueOf; // value of each switch

public:

  this(char[] spec) {
    uint i=0;
    while (i<spec.length) {
      uint i1 = i+1;
      bool colon = (i1 < spec.length && spec[i1] == ':');
      hasValue[spec[i]] = colon;
      i += colon ? 2 : 1;
    }
  }

  // Return first non-switch arg position.
  uint parse(char[][] argv) {
    uint i=1;
    while (i < argv.length) {
      if (argv[i][0] != '-' || argv[i] == "-") break;
      // All but the last character are flags. If this is the final arg,
      // last one has to be a flag too.
      bool isFinal = (i == argv.length-1);
      int jLim = argv[i].length;
      if (!isFinal) jLim --;
      for (int j=1; j<jLim; j++) {
	char c = argv[i][j];
	if (!c in hasValue) {
	  throw new Exception("unexpected flag " ~ [c]);
	} else if (hasValue[c]) {
	  throw new Exception("missing value for switch " ~ [c]);
	} else {
	  isSet[c] = true;
	}
      }
      if (!isFinal) {
	char c = argv[i][$-1];
	if (!c in hasValue) {
	  throw new Exception("unexpected flag " ~ [c]);
	} else if (hasValue[c]) {
	  valueOf[c] = argv[i+1];
	  i++;
	} else {
	  isSet[c] = true;
	}
      }
      i++;
    }
    return i; // true args start at i
  }

  bool contains(char c) {
      if (hasValue[c])
	return (c in valueOf) != null;
      else
	return (c in isSet) != null;
  }

  bool flag(char c) {
    if (!c in hasValue) {
      throw new Exception("bad flag " ~ [c]);
    } else if (hasValue[c]) {
      throw new Exception("switch not flag " ~ [c]);
    } else {
      return (c in isSet) != null;
    }
  }

  char[] getString(char c) {
    if ((c in hasValue) == null) {
      throw new Exception("bad switch " ~ [c]);
    } else if (!hasValue[c]) {
      throw new Exception("flag not switch " ~ [c]);
    } else if ((c in valueOf) == null) {
      return "";
    } else {
      return valueOf[c];
    }
  }

  int getInt(char c) {
    return toInt(getString(c));
  }

  double getDouble(char c) {
    return toDouble(getString(c));
  }

  char getChar(char c) {
    char[] str = getString(c);
    if (str.length == 1) {
      return str[0];
    } else {
      throw new Exception("value is string not char for " ~ [c]);
    }
  }

}
