import std.stdio;
import std.stream;
import std.string;
import std.conv;

import tally;

void main(char[][] argv) {
  // Run meansd on every (proper) argument.
  foreach (name ; argv[1..$])
    meansd(name);
}

void meansd(char[] name) {
  // We want an array of tallies for every occurrence pattern.
  Tally[][char[]] tallies;
  foreach (char[] line ; new BufferedFile(name)) {
    // Words in this line of the file:
    char[][] words = split(line);
    // Values of numerical fields in this line:
    double[] vals;
    // Occurrence pattern: a string of '0' and '1', with 
    // '1' meaning "is a number".
    char[] pat;
    // Get the values and pattern ...
    uint len = fillValuesAndPattern(words,vals,pat);
    // .. and record the values under the pattern.
    augmentForPattern(pat,vals,tallies);
  }
  // Now report the mean, SD etc for each pattern.
  foreach (char[] pat; tallies.keys.sort) {
    report(pat,tallies[pat]);
  }
}

uint fillValuesAndPattern(char[][] words, out double[] vals, out char[] pat) {
  uint len = words.length;
  vals.length = pat.length = len;
  foreach (int i, char[] word ; words) {
    // If we can convert word to a double, put the result in vals,
    // and set pat[i] to '1', else set it to '0'.
    try {
      vals[i] = toDouble(word);
      pat[i] = '1';
    } catch (ConvError e) {
      pat[i] = '0';
    }
  }
  return len;
}

void augmentForPattern(char[] pat, double[] vals,
		       inout Tally[][char[]] tallies) {
  int len = pat.length;
  if ((pat in tallies) == null) {
    // First time we've seen this pattern: set the length of
    // the tally vector, and create a Tally in each '1' position.
    tallies[pat].length = len;
    for (int i=0; i<len; i++) {
      if (pat[i] == '1') {
	tallies[pat][i] = new Tally(vals[i]);
      }
    }
  } else {
    // Not first time. Add to each tally.
    for (int i=0; i<len; i++) {
      if (pat[i] == '1') {
	tallies[pat][i].add(vals[i]);
      }
    }
  }
}

void report(char[] pat, Tally[] tallyArray) {
  int len = pat.length; 
  writefln("%s Idx   N    Mean          SD          Min          Max",pat);
  foreach (int i, Tally t ; tallyArray) {
    writef("%s:%03d",pat,i);
    if (pat[i] == '1') {
      writef(" %s",t);
    }
    writefln("");
  }
  writefln("");
}
