import getopts;

import std.cstream;
import std.stream;
import std.string;
import std.stdio;

enum TokenType {CHAR, WORD, LINE};

void main(char[][] argv) {
  auto opts = new Options("lcnpr");
  uint iMin = opts.parse(argv);
  uint[char[]] count;
  TokenType tt = 
    opts.flag('l') ? TokenType.LINE : 
    opts.flag('c') ? TokenType.CHAR : 
    TokenType.WORD;
  if (iMin == argv.length) {
    countTokens("",tt,count);
  } else {
    foreach (name ; argv[iMin..$]) {
      countTokens(name,tt,count);
    }
  }
  report(opts,count);
}

void countTokens(char[] name, TokenType tt, inout uint[char[]] count) {
  // Would like to do this with "? :" but types make it fail...
  Stream ins;
  if (name == "-" || name == "") {
    ins = din;
  } else {
    ins = new BufferedFile(name);
  }
  foreach (char[] line ; ins) {
    if (tt == TokenType.LINE) {
      count[line.dup] ++;
    } else if (tt == TokenType.WORD) {
      foreach(char[] word ; split(line)) {
	count[word.dup] ++;
      }
    } else {
      // Weirdness: there appears to be a private iswhite in std.stream,
      // and compiler complains at unqualified iswhite because of that!
      foreach (char c ; line)
	if (!std.string.iswhite(c)) count[[c]] ++;
    }
  }
}

void report(Options opts, uint[char[]] count) {

  bool isRev = opts.flag('r');
  bool isProp = opts.flag('p');

  count.rehash;

  int sum;
  if (isProp) {
    foreach (uint c ; count.values)
      sum += c;
  }
  double fac = 1.0/sum;

  if (isRev) {
    // sort alphabetically by token
    if (isProp) {
      foreach (char[] word ; count.keys.sort) {
	writefln("%s %10.8f",word,count[word]*fac);
      }
    } else {
      foreach (char[] word ; count.keys.sort) {
	writefln("%s %d",word,count[word]);
      }
    }
  } else {

    class counted_word {
      char[] word;
      this(char[] w) {
	word = w;
      }
      int opCmp(Object other) {
	counted_word ocw = cast(counted_word) other;
	if (ocw == null) {
	  return -1;
	} else {
	  return count[ocw.word] - count[word];
	}
      }
    }

    char[][] words = count.keys;
    counted_word[] cwa;
    cwa.length = words.length;
    foreach (int i, char[] word ; words) {
      cwa[i] = new counted_word(word);
    }
    if (isProp) {
      foreach (counted_word cw ; cwa.sort) {
	writefln("%10.8f %s",count[cw.word]*fac,cw.word);
      }
    } else {
      foreach (counted_word cw ; cwa.sort) {
	writefln("%d %s",count[cw.word],cw.word);
      }
    }
  }

}
