import Data.List
import Data.Array -- for range

abc = "abc"

-- succ already does this:
-- nextChar :: Char -> Char
-- nextChar =  toEnum . succ . fromEnum

bcd :: String
bcd = map succ abc

aNbNc = intersperse 'N' abc

-- "intercalate" as defined in the documentation does
-- not exist. These are equivalent:
intercalate1 a b = concat $ intersperse a b
intercalate2 = curry $ concat . uncurry intersperse 

-- ["ac","bd","e"], i.e. ragged is OK
tran = transpose ["ab","cde"]

nums lo hi = if lo > hi then [] else lo:(nums (succ lo) hi)
n1to4 = nums 1 4

-- These are the same:
n10a = foldl (+) 0 n1to4
n10b = foldr (+) 0 n1to4
-- But these are not:
-- 0-1-2-3-4, left to right:
nm10 = foldl (-) 0 n1to4
-- Right to left, with starting value at the END:
-- (1-(2-(3-(4-0))) = 1-2+3-4+0 = -2
nm2  = foldr (-) 0 n1to4

-- Folds don't have to have args of same type:
n294a = foldl (\x y -> x + fromEnum y) 0 "abc"
n294b = foldr (\x y -> fromEnum x + y) 0 "abc"

-- But the '1' variants do:
n10c = foldl1 (+) n1to4
n10d = foldr1 (+) n1to4
-- 1-2-3-4 = 1-9 = -8
nm8a = foldl1 (-) n1to4
-- 1-(2-(3-4)) = 1-2+3-4 = -2
nm2b = foldr1 (+) n1to4

-- ["bcd","bcd"]
bcd1 =       map (map succ) [abc,abc]
-- "bcdbcd"
bcd2 = concatMap (map succ) [abc,abc]

-- True:
cond1 = and (map (>= 'a') abc)
-- False:
cond2a = and (map (>= 'b') abc)
cond2b = all (>= 'b') abc
-- True:
cond3a = or (map (>= 'b') abc)
cond3b = any (>= 'b') abc

-- sum == foldr (+) 0
-- product == foldr (*) 1
-- maximum = foldr1 max
-- minimum = foldr1 min

-- scanl: progressive results (on prefixes)
pascal1 = scanl (+) 0 (nums 1 10)
pascal2 = scanl1 (+) (nums 1 10)
-- scanr: on suffixes
list1 = scanr (+) 0 (nums 1 10)
list2 = scanr1 (+) (nums 1 10)

-- map and foldl together:
-- (55,[-1,-2,-3,-4,-5,-6,-7,-8,-9,-10])
pairs1a = mapAccumL (\a x -> (a+x,-x)) 0 (nums 1 10)
pairs1b = mapAccumR (\a x -> (a+x,-x)) 0 (nums 1 10)

-- iterate: infinite application
motorway = take 4 $ iterate (+2) 2
-- repeat == iterate id
corrs = take 14 $ repeat 2
-- finite to infinite list
march = take 6 $ cycle [1,2]

-- unfoldr: build a list, using Nothing to terminate
succ10 x = if x <= 10 then Just $ (x, succ x) else Nothing
n1to10 = unfoldr succ10 1

-- splitAt = take and drop in a tuple
pair1a = splitAt 5 n1to10
-- span = takeWhile and dropWhile in a tuple
pair1b = span (<=5) n1to10
-- break: negates the predicate
pair1c = break (>6) n1to10

-- stripPrefix: documented but not defined ...
stripPrefix [] x = Just x
stripPrefix (h1:t1) (h2:t2) | h1 == h2 = stripPrefix t1 t2
stripPrefix _ _ = Nothing

just6to10 = stripPrefix (nums 1 5) (nums 1 10)

-- ["M","i","ss","i","ss","i","pp","i"]
gmis1 = group "Mississippi" 

gsort a = group $ sort a
gmis2 = gsort "Mississippi" 

inits1 = inits "Mississippi" 
tails1 = tails "Mississippi" 

-- tails == unfoldr tailer ...
tailer [] = Nothing
tailer (h:t) = Just (h:t,t)
tails2 = unfoldr tailer "Mississippi" 

-- inits lst = "" : reverse (unfoldr header lst)
header [] = Nothing
header lst = Just (lst,init lst)

near x y = y >= x-1 && y <= x+1

-- Deletes 4, because that's the first value satisfying the predicate.
no4 = deleteBy near 5 (range (1,10))
-- Deletes 4,5 and 6:
no456 = filter (not . near 5) (range (1,10))