module GapDistribution where

import qualified Data.ByteString.Char8 as BS
import qualified Data.Char as CH
import qualified Data.List as DL
import qualified Data.Map as DM
import qualified Data.Array.IArray as DA

data VType = UNuc | LNuc | UGap | LGap | HGap
  deriving (Eq, Enum, Read, Show, Bounded, Ord, DA.Ix)

vTypeOfChar 'A' = UNuc
vTypeOfChar 'C' = UNuc
vTypeOfChar 'G' = UNuc
vTypeOfChar 'T' = UNuc
vTypeOfChar 'N' = UNuc
vTypeOfChar 'a' = LNuc
vTypeOfChar 'c' = LNuc
vTypeOfChar 'g' = LNuc
vTypeOfChar 't' = LNuc
vTypeOfChar 'n' = LNuc
vTypeOfChar '-' = UGap
vTypeOfChar d | d >= '0' && d <= '9' = UGap
vTypeOfChar '_' = LGap
vTypeOfChar '=' = HGap
vTypeOfChar c = error ("vTypeOfChar: bad value character: "++[c])

type QValue = Int

qValueOfChar :: Char -> QValue
qValueOfChar ' ' = 94
qValueOfChar qc =
  let qv = fromEnum qc - 33 in
  if qv >= 0 && qv < 94
  then qv
  else error ("qValueOfChar: bad quality character: "++[qc])

mapp f (a:(b:t)) = f a b : mapp f t
mapp f _ = []

pairListOfList :: [t] -> [(t, t)]
pairListOfList = mapp (curry id)
  
nonSpacePairsOfAlignmentLine line =
  filter (\ t -> not (fst t == ' ')) (pairListOfList line)

vTypesAndQValuesOfLine line =
  map vTypeAndQValueOfPair $ nonSpacePairsOfAlignmentLine line

vTypeAndQValueOfPair (vc,qc) =
  (vTypeOfChar vc, qValueOfChar qc)

{-

This works, but we don't want to use _immutable_ arrays!
Better to use maps.

type QualDistTable = DA.Array VType (DA.Array QValue Int)

zeroArray min max =
  DA.listArray (min,max) $ replicate (max-min+1) 0

emptyQualDistTable :: DA.Array VType (DA.Array QValue Int)
emptyQualDistTable =
  let vtmin = minBound
      vtmax = maxBound
      sz = (fromEnum vtmax) - (fromEnum vtmin) + 1
      qmax = qValueOfChar ' '
      zeros = zeroArray 0 qmax
  in DA.listArray (vtmin, vtmax)
                  (replicate sz zeros)

-}

type QualDistTable = DM.Map VType (DM.Map QValue Int)

zeroQualDistTable :: QualDistTable
zeroQualDistTable = DM.empty       

qualDistTableOfLines lines =
    foldr augmentQualDistTable zeroQualDistTable lines 

augmentQualDistTable line qdt =
    foldr augmentQualDistTable1 qdt $ vTypesAndQValuesOfLine line

augmentQualDistTable1 (vt,qv) qdt =
    case DM.lookup vt qdt of
      Just map -> DM.insert vt (initialQualMap qv) qdt
      Nothing -> DM.insert vt (initialQualMap qv) qdt
    ... think ... use insertWith[Key]?
