module Getopt where

import qualified System.Console.GetOpt as GO
import qualified System.Environment as SE

getoptsTest =
  do trip   <- SE.withArgs ["-a","aa","-b"] $ getopts "a:b"
     let ans = ([('a',"aa"),('b',"")],[],[])
     reportTest "getopts" trip ans

reportTest key val ans =
  if val == ans
   then putStrLn $ key ++ " succeeded"
   else putStrLn $ key ++ " failed"

-- Returns switch stuff as assoc list. Access with lookup:
-- lookup 'a' stuff :: Just String
getopts :: String -> IO ([(Char,String)], [String], [String])
getopts optstr =
  do args <- SE.getArgs
     return $ getoptsFor optstr args

getoptsFor :: String -> [String] -> ([(Char, String)], [String], [String])
getoptsFor optstr args =
  let od = mapChunks processOption optstr
  in GO.getOpt GO.RequireOrder od args

processOption (c:':':rest) =
  let reqArg = GO.ReqArg (\v -> (c,v)) []
  in (GO.Option [c] [] reqArg [],rest)
processOption (c:rest) =
  let noArg = GO.NoArg (c,"")
  in (GO.Option [c] [] noArg [],rest)

-- Version of "map" whose function can consume any number
-- of items from the list (though 0 would cause an infinite loop)

mapChunks :: ([t] -> (a, [t])) -> [t] -> [a]
mapChunks func [] = []
mapChunks func list =
  let (stuff,rest) = func list
  -- Note: concatChunks would have ++ for : below:
  in stuff : mapChunks func rest
