module Main where

import qualified System.IO as IO
import qualified System.Environment as SE
import qualified System.Console.GetOpt as GO

main = interline reverse

interline = interactA . byline

byline f = unlines . (map f) . lines

argContents = do args <- SE.getArgs
                 if args == []
                  then getContents
                  else do conts <- mapM IO.readFile args
                          return $ concat conts

interactA f = do conts <- argContents
                 putStr (f conts)

-- Arguments to GO.Option are:
--   * list of single chars (after single hyphen) to specify this option
--   * list of keywords (after double hyphen) to specify this option
--   * how to create resulting value: NoArg constValue, or ReqArg func foo
--     where "func" transforms option value, "foo"
--   * explanation
-- Usages:
--   "-a" or "--alpha" or "-A" gives "aval" in results
--   "-b x" or "--beta x" gives "val-x"

myOpts = [GO.Option "aA" ["alpha"] (GO.NoArg "aval") "first option",
          GO.Option "b" ["beta"]  (GO.ReqArg ("val-"++) "ID") "second option"]

-- Try:
--    getMyOpts ["-a","-b","x"]
--    getMyOpts ["-a","--beta","x"]
--    getMyOpts ["-A","--beta=x"]
--    getMyOpts ["--alpha"]
getMyOpts = GO.getOpt GO.RequireOrder myOpts

-- Try: putStr myUsageInfo
putMyUsageInfo = putStr $ GO.usageInfo "Usage: mycmd [-opts] files, with:" myOpts


              