module Main where

import qualified Data.Map as M
import System (getArgs)
import System.Console.GetOpt
import Char
import Data.List
import Data.Ord
import Control.Monad
import Text.Printf

-- http://www.haskell.org/haskellwiki/Tutorials/Programming_Haskell/Introduction

-- Compile with e.g.
--   ghc -o vocab vocab.hs

-- Impure bit

main = do args <- getArgs
          let (ty,isRev,isProp,files) = getVocabOpts args 
          lines <- linesInFiles files
          let voc = getVocab ty isRev lines
          let fac = (if isProp then 1.0 / sumFunc snd voc else 1.0) :: Float
          mapM_ (printPair isRev fac) voc
          return ()

linesInFiles files = (liftM concat) (mapM linesInFile files)

linesInFile f = (liftM lines) (readFile f)

-- c is forced to have same type as f, i.e. Float, so we print with %.0f.

printPair False 1 (w,c) = printf "%.0f %s\n" c w
printPair True  1 (w,c) = printf "%s %.0f\n" w c
printPair False f (w,c) = printf "%8.6f %s\n" (c*f) w
printPair True  f (w,c) = printf "%s %8.6f\n" w (c*f)

-- Pure bit

sumFunc f lst = sumFunc1 f 0 lst

sumFunc1 f v [] = v
sumFunc1 f v (h:t) = sumFunc1 f (f h + v) t

optSpec = [Option "c" [] (NoArg 'c') "characters",
           Option "l" [] (NoArg 'l') "lines",
           Option "w" [] (NoArg 'w') "words",
           Option "r" [] (NoArg 'r') "reverse",
           Option "p" [] (NoArg 'p') "proportion"]

getVocabOpts args =
  let (opts,files,_) = getOpt RequireOrder optSpec args in
    (itemType opts, elem 'r' opts, elem 'p' opts, files)

itemType (ch:t) | elem ch "clw" = ch
itemType (_:t) = itemType t
itemType _      = 'w'

getVocab ty isRev lines = sortBy (vocabSorter isRev) (M.toList (createVocab ty lines))

vocabSorter False = compareByRev snd
vocabSorter True = compareBy fst

createVocab ty = foldl (addToVocab (splitter ty)) M.empty

splitter 'w' = words
splitter 'l' = listMe
splitter 'c' = mapIf isVisible listMe

isVisible c = isPrint c && not (isSpace c)

mapIf cond func [] = []
mapIf cond func (h:t) =
  let rest = mapIf cond func t in
    if cond h
    then (func h):rest
    else rest

listMe x = [x] 

compareByRev f x y = compareBy f y x

compareBy f x y = compare (f x) (f y)

addToVocab spl voc line =
   foldl addWordToVocab voc (spl line)

addWordToVocab voc word =
  M.insert word (getCount word voc + 1) voc

getCount = M.findWithDefault 0
