module Perl2HS where

import qualified Control.Monad as CM
import qualified Data.ByteString.Char8 as BS
import qualified Data.Char as CH
import qualified Data.List as DL
import qualified Data.Map as DM
import qualified Directory
import qualified IO
import qualified Network.Socket as NS
import qualified System.Console.GetOpt as GO
import qualified System.Environment as SE
import qualified System.IO as SI
import qualified System.Posix.Directory as PD
import qualified System.Posix.Files as PF
import qualified System.Posix.Types as PT
import qualified System.Posix.Time
import qualified System.Process as SP
import qualified System.Random as SR
import qualified System.Time as ST
import qualified Text.Printf as TP
import qualified Text.Regex as TR
import qualified Text.Regex.Posix as RP

-- file switches: -d, -s etc

mA = mStat PF.accessTime
md = mStat PF.isDirectory
me = PF.fileExist
mf = mStat PF.isRegularFile
ml = mStat PF.isSymbolicLink
ms f = do
  ex <- me f
  if ex
   then mStat PF.fileSize f
   else return 0

mStat op file = (CM.liftM op) (statRecord file)

-- abs - absolute value function
-- abs :: (Num a) => a -> a

-- accept - accept an incoming socket connect
-- Note: or could use Network.accept	 :: Socket -> IO (Handle, HostName, PortNumber)
accept :: NS.Socket -> IO (NS.Socket, NS.SockAddr)
accept = NS.accept

-- alarm - schedule a SIGALRM
-- TODO

-- atan2 - arctangent of Y/X in the range -PI to PI
-- atan2 :: (RealFloat a) => a -> a-> a

-- bind - binds an address to a socket
-- Network.Socket.bindSocket	:: Socket -> SockAddr -> IO ()
bind :: NS.Socket -> NS.SockAddr -> IO ()
bind = NS.bindSocket

-- binmode - prepare binary files for I/O
-- Note: can also use openBinaryFile fileName ioMode
binmode :: IO.Handle -> IO ()
binmode h = SI.hSetBinaryMode h True

-- bless - create an object 
-- TODO

-- caller - get context of the current subroutine call
-- TODO

-- chdir - change your current working directory
-- Directory.setCurrentDirectory	:: FilePath -> IO ()
chdir :: FilePath -> IO ()
chdir = Directory.setCurrentDirectory

-- chmod - changes the permissions on a list of files
-- TODO: we should implement a translator from rwx format;
-- for now, just use modes in System-Posix-Files.html#1
chmod :: FilePath -> PT.FileMode -> IO ()
chmod = PF.setFileMode

-- chomp - remove a trailing record separator from a string
chomp :: String -> String
chomp s = if last s == '\n' then init s else s

chompBS :: BS.ByteString -> BS.ByteString
chompBS s = if BS.last s == '\n' then BS.init s else s

-- chop - remove the last character from a string
-- BS.init :: ByteString -> ByteString
chop :: String -> String
chop = init

chopBS :: BS.ByteString -> BS.ByteString
chopBS = BS.init

-- chown - change the owership on a list of files
chown :: FilePath -> PT.UserID -> PT.GroupID -> IO ()
chown = PF.setOwnerAndGroup

-- chr - get character this number represents
chr :: Char -> Int
chr = fromEnum

-- chroot - make directory new root for path lookups
-- TODO - Not in Haskell?

-- close - close file (or pipe or socket) handle
close :: SI.Handle -> IO ()
close = SI.hClose

-- closedir - close directory handle
closedir :: PD.DirStream -> IO ()
closedir = PD.closeDirStream

-- connect - connect to a remote socket
-- Network.Socket.connect :: Socket -> SockAddr -> IO ()
-- Network.connectTo	  :: HostName -> PortID -> IO Handle
connect :: NS.Socket -> NS.SockAddr -> IO ()
connect = NS.connect

-- continue - optional trailing block in a while or foreach
-- TODO

-- cos - cosine function
-- cos :: (Floating a) => a -> a

-- crypt - one-way passwd-style encryption
-- TODO

-- dbmclose - breaks binding on a tied dbm file
-- dbmopen - create binding on a tied dbm file
-- TODO

-- defined - test whether a value, variable, or function is defined
-- TODO

-- delete - deletes a value from a hash
delete :: Ord k => k -> DM.Map k a -> DM.Map k a
delete = DM.delete

-- die - raise an exception or bail out
die :: [Char] -> a
die = error

-- do - turn a BLOCK into a TERM
-- TODO

-- dump - create an immediate core dump
-- TODO

-- each - retrieve the next key/value pair from a hash
each :: DM.Map k a -> [(k,a)]
each = DM.toList

-- endgrent - be done using group file
-- endhostent - be done using hosts file
-- endnetent - be done using networks file
-- endprotoent - be done using protocols file
-- endpwent - be done using passwd file
-- endservent - be done using services file
-- TODO

-- eof - test a filehandle for its end
eof :: SI.Handle -> IO Bool
eof = SI.hIsEOF

-- eval - catch exceptions or compile and run code
-- TODO

-- exec - abandon this program to run another
-- TODO

-- exists - test whether a hash key is present
exists :: Ord k => k -> DM.Map k a -> Bool
exists = DM.member

-- exit - terminate this program
-- TODO

-- exp - raise I to a power
-- exp :: (Floating a) => a -> a

-- fcntl - file control system call
-- TODO

-- fileno - return file descriptor from filehandle
-- TODO

-- flock - lock an entire file with an advisory lock
-- TODO

-- fork - create a new process just like this one
-- TODO

-- format - declare a picture format with use by the write() function
-- TODO

-- formline - internal function used for formats
-- TODO

-- getc - get the next character from the filehandle
getc :: SI.Handle -> IO Char
getc = SI.hGetChar

-- $ENV{...}
getenv :: String -> IO String
getenv = SE.getEnv

-- %ENV
env :: IO (DM.Map String String)
env = (CM.liftM DM.fromList) SE.getEnvironment

-- @ARGV
argv :: IO [String]
argv = SE.getArgs

-- $0
dollar0 :: IO String
dollar0 = SE.getProgName

-- getgrent - get next group record
-- getgrgid - get group record given group user ID
-- getgrnam - get group record given group name
-- gethostbyaddr - get host record given its address
-- gethostbyname - get host record given name
-- gethostent - get next hosts record
-- getlogin - return who logged in at this tty
-- getnetbyaddr - get network record given its address
-- getnetbyname - get networks record given name
-- getnetent - get next networks record
-- getpeername - find the other end of a socket connection
-- getpgrp - get process group
-- getppid - get parent process ID
-- getpriority - get current nice value
-- getprotobyname - get protocol record given name
-- getprotobynumber - get protocol record numeric protocol
-- getprotoent - get next protocols record
-- getpwent - get next passwd record
-- getpwnam - get passwd record given user login name
-- getpwuid - get passwd record given user ID
-- getservbyname - get services record given its name
-- getservbyport - get services record given numeric port
-- getservent - get next services record
-- getsockname - retrieve the sockaddr for a given socket
-- getsockopt - get socket options on a given socket
-- TODO

-- Note small "o" here...
-- Given a Perl Getopt-style switch spec and an argument list
-- (list of strings), return a triple consisting of
--    - map from char string giving switch values; flags have "" value
--    - non-switch args
--    - error messages
getopt :: [Char] -> [String] -> (DM.Map Char String, [String], [String])
getopt spec args = 
    let optDescrList = convertOptionSpec spec
        (pairs,rest,errs) = GO.getOpt GO.RequireOrder optDescrList args
    in (DM.fromList pairs,rest,errs)

-- convert Perl Getopt-style spec into Haskell spec

convertOptionSpec [] = []
convertOptionSpec (c:':':spec) =
    let opt = GO.Option [c] [[c]] (GO.ReqArg (\x -> (c,x)) [c]) [c]
    in opt : convertOptionSpec spec
convertOptionSpecS (c:spec) =
    let opt = GO.Option [c] [[c]] (GO.NoArg (c,"")) [c]
    in opt : convertOptionSpec spec

cwd :: IO String
-- cwd = backtick1 "pwd"
cwd = Directory.getCurrentDirectory

backtick1 = (CM.liftM head) . backtick

backtick cmd = do (_,fh,_,_) <- SP.runInteractiveCommand cmd
                  (CM.liftM lines) $ IO.hGetContents fh

-- glob - expand filenames using wildcards. We ls -d and "lines"
-- to deal correctly with names containing spaces.
glob :: String -> IO [String]
glob = backtick . ("ls -d "++)

-- gmtime - convert UNIX time into record or string using Greenwich time
-- TODO

-- goto - create spaghetti code
-- TODO

-- grep - locate elements in a list test true against a given criterion
grep :: (a -> Bool) -> [a] -> [a]
grep = Prelude.filter

-- hex - convert a string to a hexadecimal number
-- TODO

-- import - patch a module's namespace into your own
-- Same in Haskell!

-- index - find a substring within a string
-- Note: substring is 2nd arg in perl but 1st in Haskell;
-- and this returns a Maybe, rather than -1 for failyre

index :: (Eq a, Num t) => [a] -> [a] -> Maybe t
index str sub = indexInd 0 str sub

indexInd i str sub
  | DL.isPrefixOf sub str = Just i
  | str == []             = Nothing
  | otherwise             = indexInd (i+1) (tail str) sub

indexBS :: BS.ByteString -> BS.ByteString -> Maybe Int
indexBS = flip BS.findSubstring

-- int - get the integer portion of a number
-- Note: truncate goes towards 0, round to nearest, ceiling down, floor up
-- Prelude.truncate :: Integral b => a -> b
-- Prelude.round :: Integral b => a -> b
-- Prelude.ceiling :: Integral b => a -> b
-- Prelude.floor :: Integral b => a -> b
int :: (Integral b, RealFrac a) => a -> b
int x = truncate x

intercalate = concat ## DL.intersperse

(##) f g x y = f (g x y) 

-- ioctl - system-dependent device control system call
-- TO DO

-- join - join a list into a string using a separator
-- Note: infix string is 1st arg in perl, 2nd arg in Haskell
joinBS :: BS.ByteString -> [BS.ByteString] -> BS.ByteString
joinBS = BS.intercalate

-- keys - retrieve list of indices from a hash
-- Data.Map.keys :: Map k a -> [k]
keys = DM.keys

-- kill - send a signal to a process or process group
-- Note: have to have got the ProcessHandle from runInteractiveProcess
kill :: SP.ProcessHandle -> IO ()
kill = SP.terminateProcess

-- last - exit a block prematurely
-- TO DO

-- lc - return lower-case version of a string

lc :: String -> String
lc   = map CH.toLower

lcBS :: BS.ByteString -> BS.ByteString
lcBS = BS.map CH.toLower 

-- lcfirst - return a string with just the next letter in lower case
lcfirst :: String -> String
lcfirst (h:t) = (CH.toLower h) : t

lcfirstBS :: BS.ByteString -> BS.ByteString
lcfirstBS = recaseFirstBS CH.toLower

recaseFirstBS f bs =
  if BS.null bs
  then bs
  else BS.cons (f $ BS.head bs) (BS.tail bs)

-- length - return the number of bytes in a string
-- BS.length :: BS.ByteString -> Int
-- List.length :: [a] -> Int
-- length = Prelude.length

lengthBS :: BS.ByteString -> Int
lengthBS = BS.length

-- link - create a hard link in the filesytem
link :: FilePath -> FilePath -> IO ()
link = PF.createLink

-- listen - register your socket as a server
listen :: NS.Socket -> Int -> IO ()
listen = NS.listen

-- local - create a temporary value for a global variable (dynamic scoping)
-- TO DO

-- localtime - convert UNIX time (integer seconds) into record or string using local time
localtimeString :: Integer -> IO String
localtimeString t = 
   do cal <- ST.toCalendarTime (ST.TOD t 0)
      return (ST.calendarTimeToString cal)

localtimeRecord :: Integer -> IO ST.CalendarTime
localtimeRecord t = ST.toCalendarTime (ST.TOD t 0)

localtimeList :: Integer -> IO [Int]
localtimeList t =
   do cal <- ST.toCalendarTime (ST.TOD t 0)
      return [ST.ctSec cal,
              ST.ctMin cal,
              ST.ctHour cal,
              fromEnum $ ST.ctDay cal,
              fromEnum $ ST.ctMonth cal,
              ST.ctYear cal,
              fromEnum $ ST.ctWDay cal,
              ST.ctYDay cal,
              fromEnum $ ST.ctIsDST cal]
   
-- lock - get a thread lock on a variable, subroutine, or method

-- log - retrieve the natural logarithm for a number
-- log :: (Floating a) => a -> a

-- lstat - stat a symbolic link
lstatRecord :: FilePath -> IO PF.FileStatus
lstatRecord = PF.getSymbolicLinkStatus

lstatList file =
  do s <- lstatRecord file
     return (statToList s)

-- m - match a string with a regular expression pattern. 
-- The regex is the FIRST arg. Return Nothing if fails;
-- if succeeds, return Just [v1,v2,...] where the v's are
-- the substrings matching (...) terms.
-- But see =~ below...
m :: String -> String -> Maybe [String]
m = TR.matchRegex . TR.mkRegex

-- =~: note that this can have multiple result types:
--   Bool                   -- success or not
--   String                 -- first substring that matches
--   (String,String,String) -- pre-context, matching string, post-context
--   (String,String,String,[String]) 
--                          -- adds subgroups of match
--   (Int,Int)              -- match offset and length, first match only
--   [(Int,Int)]            -- ditto, all matches
-- And everything works with ByteStrings too, e.g.:
--   (BS.pack "abcbd") =~ (BS.pack "b") :: BS.ByteString

(=~) str re = (RP.=~) str re

-- Returns [] if match fails, else list of substrings of str
-- matched to (...) patterns in re

-- map - apply a change to a list to get back a new list with the changes
-- Same as Prelude version.

-- mkdir - create a directory. Note need to supply mode!
mkdir :: FilePath -> PT.FileMode -> IO ()
mkdir = PD.createDirectory 

-- Default mode of rwxr-xr-x
mkdir1 :: FilePath -> IO ()
mkdir1 d = mkdir d (0o755 :: PT.FileMode)

-- msgctl - SysV IPC message control operations
-- msgget - get SysV IPC message queue
-- msgrcv - receive a SysV IPC message from a message queue
-- msgsnd - send a SysV IPC message to a message queue
-- TO DO

-- my - declare and assign a local variable (lexical scoping)
-- TO DO

-- next - iterate a block prematurely
-- TO DO

-- no - unimport some module symbols or semantics at compile time
-- TO DO

-- oct - convert a string to an octal number
-- TO DO

-- open - open a file, pipe, or descriptor
open :: String -> IO SI.Handle
open ('>':'>':name) = IO.openFile name IO.AppendMode
open ('>':name) = IO.openFile name IO.WriteMode
open ('<':name) = IO.openFile name IO.ReadMode
open ('|':cmd) = 
    do (gh,_,_,_) <- SP.runInteractiveCommand cmd
       return gh
open cmd = if last cmd == '|'
           then do (_,fh,_,_) <- SP.runInteractiveCommand (init cmd)
                   return fh
           else IO.openFile cmd IO.ReadMode

-- opendir - open a directory
opendir :: FilePath -> IO PD.DirStream
opendir = PD.openDirStream

-- ord - find a character's numeric representation
ord :: Char -> Int
ord = fromEnum

-- our - declare and assign a package variable (lexical scoping)
-- pack - convert a list into a binary representation
-- package - declare a separate global namespace
-- NO

-- pipe - open a pair of connected filehandles
-- TO DO

-- pop - remove the last element from an array and return it
-- NOTE: we return both
popList l = (init l, last l)

-- pos - find or set the offset for the last/next m//g search
-- TO DO

-- print - output a list to a filehandle
-- Best approximations ...
printStr :: SI.Handle -> String -> IO ()
printStr = IO.hPutStr

print :: (Show a) => SI.Handle -> a -> IO ()
print = IO.hPrint

-- printf - output a formatted list to a filehandle.
-- sprintf would be the same, but we can't seem to specify
-- a string result type!
printf :: (TP.HPrintfType r) => SI.Handle -> String -> r
printf = TP.hPrintf

sprintf s = TP.printf s

-- prototype - get the prototype (if any) of a subroutine
-- NO

-- push - append one or more elements to an array
pushItem :: (([a] -> [a] -> [a]) -> [t] -> t1) -> t -> t1
pushItem a b = a (++) [b]

pushList :: [a] -> [a] -> [a]
pushList = (++)

-- q - singly quote a string
-- qq - doubly quote a string
-- qr - Compile pattern
-- quotemeta - quote regular expression magic characters
-- qw - quote a list of words
-- qx - backquote quote a string
-- NO

-- rand - retrieve the next pseudorandom number
randStart :: IO (Double, SR.StdGen)
randStart = do g <- SR.getStdGen
               return $ rand g

rand :: SR.RandomGen g => g -> (Double, g)
rand = SR.random

-- srand - seed the random number generator, with
-- given arg or with time if none
-- TO DO

-- Possible use: Control.Monad.liftM (take 10) randList
randList :: IO [Double]
randList = do g <- SR.getStdGen
              return (SR.randoms g)

-- read - fixed-length buffered input from a filehandle
-- TO DO

-- readdir - get a directory from a directory handle
readdir :: PD.DirStream -> IO FilePath
readdir = PD.readDirStream

-- readdirAll - get list of basenames in directory from its name
readdirAll :: FilePath -> IO [FilePath]
readdirAll d = do dh <- opendir d
                  ans <- readdirAll1 dh
                  return $ sort ans

readdirAll1 dh =
   do dir <- readdir dh
      if dir == ""
        then return []
        else do rest <- readdirAll1 dh
                return (dir:rest)

-- readline - fetch a record from a file
-- ??

-- readlink - determine where a symbolic link is pointing
readlink :: FilePath -> IO FilePath
readlink = PF.readSymbolicLink

-- readpipe - execute a system command and collect standard output
-- ??

-- recv - receive a message over a Socket
-- TO DO

-- redo - start this loop iteration over again
-- NO

-- ref - find out the type of thing being referenced
-- NO

-- rename - change a filename
rename = PF.rename

-- require - load in external functions from a library at runtime
-- TO DO

-- reset - clear all variables of a given name
-- NO

-- return - get out of a function early
-- NO

-- reverse - flip a string or a list
-- use Prelude one

-- rewinddir - reset directory handle
rewinddir = PD.rewindDirStream

-- rindex - right-to-left substring search
rindex :: (Eq a, Num t) => [a] -> [a] -> Maybe t
rindex str sub = rindexInd 0 str sub

rindexInd _ [] _ = Nothing
rindexInd i str sub =
  let res = rindexInd (i+1) (tail str) sub
  in case res of
       Just _  -> res
       Nothing -> if DL.isPrefixOf sub str
                  then Just i
                  else Nothing

-- This is not very efficient, probably...
rindexBS :: BS.ByteString -> BS.ByteString -> Maybe Int
rindexBS str sub = 
   let inds = BS.findSubstrings sub str
    in case inds of
         [] -> Nothing
         _  -> Just (last inds)

-- rmdir - remove a directory
-- Not recursive...but removeDirectoryRecursive follows symlinks!
rmdir :: FilePath -> IO ()
rmdir = PD.removeDirectory

-- s - replace a pattern with a string
-- Note: remember to double any backslashes in final arg as well as value ...
-- Args are: searchPattern stringToMatch replacement, e.g.
-- s "c" "cat" "f" == "fat"
s :: String -> String -> String -> String
s = TR.subRegex . TR.mkRegex

-- scalar - force a scalar context
-- NO

-- seek - reposition file pointer for random-access I/O
-- TO DO

-- seekdir - reposition directory pointer
-- TO DO

-- select - reset default output or do I/O multiplexing
-- TO DO

-- semctl - SysV semaphore control operations
-- semget - get set of SysV semaphores
-- semop - SysV semaphore operations

-- send - send a message over a socket
-- TO DO

-- Sequences: to substitute for "for" loops
seq1 lo = lo:seq1 (succ lo)

seq2 lo hi = seq3 lo hi succ

seq3 lo hi f | lo > hi = []
seq3 lo hi f = lo:seq3 (f lo) hi f

-- Reverse (descending) sequences
rseq1 lo = lo:rseq1 (pred lo)

rseq2 lo hi = rseq3 lo hi pred

rseq3 lo hi f | lo < hi = []
rseq3 lo hi f = lo:rseq3 (f lo) hi f

-- setgrent - prepare group file for use
-- sethostent - prepare hosts file for use
-- setnetent - prepare networks file for use
-- setpgrp - set the process group of a process
-- setpriority - set a process's nice value
-- setprotoent - prepare protocols file for use
-- setpwent - prepare passwd file for use
-- setservent - prepare services file for use
-- setsockopt - set some socket options

-- shift - remove the first element of an array, and return it
shift l = (head l, tail l)

-- shmctl - SysV shared memory operations
-- shmget - get SysV shared memory segment identifier
-- shmread - read SysV shared memory
-- shmwrite - write SysV shared memory
-- shutdown - close down just half of a socket connection

-- sin - return the sine of a number
-- same as Prelude

-- sleep - block for some number of seconds
sleep n = 
  do ph <- SP.runCommand $ "sleep "++(show n)
     SP.waitForProcess ph

-- socket - create a socket
-- socketpair - create a pair of sockets
-- TO DO

-- sort - sort a list of values
sort :: (Ord a) => [a] -> [a]
sort = DL.sort

sortBy :: (a -> a -> Ordering) -> [a] -> [a]
sortBy = DL.sortBy

-- splice - add or remove elements anywhere in an array
-- TO DO

-- split - split up a string using a regexp delimiter
split = TR.splitRegex . TR.mkRegex

-- sqrt - square root function
-- same as prelude

-- stat - get a file's status information

--statRecord :: FilePath -> IO PF.FileStatus
-- Usage:
--    sr <- statRecord "foo.txt"
--    PF.accessTime sr
-- or
--    (CM.liftM PF.accessTime) (statRecord "foo.txt")
statRecord = PF.getFileStatus

--statList :: FilePath -> IO [Int]
statList file =
  do s <- statRecord file
     return (statToList s)

getFileSize file =
  do s <- statRecord file
     return $ toInteger (PF.fileSize s) 

{-
Safe version:
fromString s = 
  let [(v,"")] = reads s
   in v
-}

-- Allows ambiguities and/or trailing chars:
fromString s = (fst . head .reads) s

toString i = shows i ""

statToList :: PF.FileStatus -> [Int]
statToList s = 
    map (\f -> f s)
        [fromEnum . PF.deviceID, fromEnum . PF.fileID, fromEnum . PF.fileMode, 
         fromEnum . PF.linkCount, fromEnum . PF.fileOwner, fromEnum . PF.fileGroup, 
         fromEnum . PF.specialDeviceID, fromEnum . PF.fileSize, fromEnum . PF.accessTime, 
         fromEnum . PF.modificationTime, fromEnum . PF.statusChangeTime]

-- study - optimize input data for repeated searches
-- sub - declare a subroutine, possibly anonymously
-- NO

-- substr - get [or alter] a portion of a string
substr :: [a] -> Int -> Int -> [a]
substr str idx len = DL.take len (DL.drop idx str)

substrBS :: BS.ByteString -> Int -> Int -> BS.ByteString
substrBS bs idx len = BS.take len (BS.drop idx bs)

-- symlink - create a symbolic link to a file
symlink :: FilePath -> FilePath -> IO ()
symlink = PF.createSymbolicLink

-- syscall - execute an arbitrary system call
-- sysopen - open a file, pipe, or descriptor
-- sysread - fixed-length unbuffered input from a filehandle
-- sysseek - position I/O pointer on handle used with sysread and syswrite

-- system - run a separate program

-- syswrite - fixed-length unbuffered output to a filehandle

-- tell - get current seekpointer on a filehandle
-- telldir - get current seekpointer on a directory handle

-- tie - bind a variable to an object class
-- tied - get a reference to the object underlying a tied variable

-- time - return number of seconds since 1970
time :: IO Integer
time = do ST.TOD secs _ <- ST.getClockTime
          return secs

-- times - return elapsed time for self and child processes
-- TO DO

touch = PF.touchFile 

-- tr - transliterate a string after expanding first two
-- args on hyphens. Any excess chars in first are deleted;
-- any excess in second are ignored.
-- Examples: tr "a-z" "A-Z" "hello" == "HELLO"
--           tr "a-z" "A-E" "hello" == "E"
tr :: [Char] -> [Char] -> [Char] -> [Char]
tr from to string =
   let from' = expandRange from
       to' = expandRange to
   in (tr2 from' to' string)

expandRange [] = []
expandRange (lo:'-':hi:rest) =
  [lo .. hi] ++ expandRange rest
expandRange (h:t) =
  h : expandRange t

tr2 [] _ s = s
tr2 (fh:ft) (th:tt) s =
  tr2 ft tt (tr1 fh th s)
tr2 from [] s = trD from s

tr1 _ _ [] = []
tr1 from to (h:t) =
  (if h == from then to else h) : (tr1 from to t)

trD [] s = s
trD (fh:ft) s =
  trD ft (trD1 fh s)

trD1 _ [] = []
trD1 c (h:t) = if h == c then trD1 c t else h : (trD1 c t)

-- truncate - shorten a file

-- uc - return upper-case version of a string

uc :: String -> String
uc   = map CH.toUpper

ucBS :: BS.ByteString -> BS.ByteString
ucBS = BS.map CH.toUpper 

-- ucfirst - return a string with just the next letter in upper case
ucfirst :: String -> String
ucfirst (h:t) = (CH.toUpper h) : t

ucfirstBS :: BS.ByteString -> BS.ByteString
ucfirstBS = recaseFirstBS CH.toUpper

-- umask - set file creation mode mask

-- undef - remove a variable or function definition

-- unlink - remove one link to a file
unlink f = PF.removeLink f

-- unpack - convert binary structure into normal perl variables

-- unshift - prepend more elements to the beginning of a list

-- untie - break a tie binding to a variable

-- use - load in a module at compile time

-- utime - set a file's last access and modify times
utime f atime mtime = PF.setFileTimes atime mtime

-- values - return a list of the values in a hash

-- vec - test or set particular bits in a string

-- wait - wait for any child process to die

-- waitpid - wait for a particular child process to die

-- wantarray - get void vs scalar vs list context of current subroutine call

-- warn - print debugging info

-- write - print a picture record

-- -X - a file test (-r, -x, etc)

-- y - transliterate a string (see tr)
