import Data.Array.IArray
import System (getArgs)

main = do [s1,s2] <- getArgs
          putStrLn (show $ align s1 s2)
          return ()

prop_align_1 =  
  align "" "" 
  == 
  Path 0 []

prop_align_2 =
  align "ACGTACGTACGT" "ACTAGGTCACGT"
  ==
  Path 3 [Same "AC",Ins1 "G",Same "TA",Diff "C" "G",Same "GT",Ins2 "C",Same "ACGT"]

data Path = Path Int [Match]
  deriving (Eq,Ord,Show)

data Match = Same [Char] | Diff [Char] [Char] | Ins1 [Char] | Ins2 [Char]
  deriving (Eq,Ord,Show)

align :: [Char] -> [Char] -> Path
align s1 s2 =
  let len1 = length s1
      len2 = length s2
      a1 = listArray (0,len1) s1 :: Array Int Char
      a2 = listArray (0,len2) s2 :: Array Int Char
  in align1 a1 a2 len1 len2

-- align1 :: (IArray a Char, IArray a1 Char) =>
-- a Int Char -> a1 Int Char -> Int -> Int -> Path
align1 a1 a2 len1 len2 =
  let aa = array ((0,0),(len1,len2)) 
                 [((i,j),findAAElt i j aa a1 a2) | i <- [0..len1], j <- [0..len2]] :: Array (Int,Int) AAElt
      c = cost $ aa!(len1,len2)
      p = compressMatches (reverse $ followBestPath len1 len2 aa a1 a2)
  in Path c p

{- Alignment array elements are of type AAElt with fields for
   cost, dx and dy (d stands for "decrement"; boolean)
-}

data AAElt = AAElt Int Bool Bool
  deriving (Eq,Ord,Show)

cost (AAElt c _ _) = c
dx (AAElt _ d _) = d
dy (AAElt _ _ d) = d

findAAElt 0 0 aa _ _   = AAElt 0 False False
findAAElt i 0 aa _ _   = AAElt i True False
findAAElt 0 j aa _ _   = AAElt j False True
findAAElt i j aa a1 a2 =
  let i1 = i-1
      j1 = j-1 
      c12 = cost (aa!(i1,j1))
      same = (a1!i1 == a2!j1)
  in if same
     then AAElt c12 True True
     else 
     let ci = cost (aa!(i1,j))
         cj = cost (aa!(i,j1)) 
         cm = c12
         minC = foldl1 min [ci,cj,cm]
     in AAElt (minC+1) (min ci cm == minC) (min cj cm == minC)

min3 a b c =
  if a < b
  then if a < c then a else c
  else if b < c then b else c

deltaDiff x y | x == y = 0
deltaDiff _ _ = 1

followBestPath 0 0 _ _ _ = []
followBestPath i j aa a1 a2 = 
  let e = aa!(i,j)
      (c1,ii) = followDim (dx e) a1 i
      (c2,jj) = followDim (dy e) a2 j
  in createMatch c1 c2 : followBestPath ii jj aa a1 a2

followDim False _ i = ('-',i)
followDim True  a i = (a!(i-1),i-1)

createMatch c '-' = Ins1 [c]
createMatch '-' c = Ins2 [c]
createMatch c1 c2 = if c1 == c2 then Same [c1] else Diff [c1] [c2]

compressMatches (h1 : h2 : t) =
  case mayCompress h1 h2 of
    Just h  -> compressMatches (h:t)
    Nothing -> h1 : compressMatches (h2:t)
compressMatches l = l

mayCompress (Same s1) (Same s2) = Just (Same (s1++s2))
mayCompress (Ins1 s1) (Ins1 s2) = Just (Ins1 (s1++s2))
mayCompress (Ins2 s1) (Ins2 s2) = Just (Ins2 (s1++s2))
mayCompress (Diff s1a s1b)
            (Diff s2a s2b) = Just (Diff (s1a++s2a) (s1b++s2b))
mayCompress _ _                 = Nothing
