-- Arg: e.g. $G1K/v1

import System (getArgs)
import Perl2HS
import Utility
import qualified Data.Map as DM

type WTup = (Int, Double, Double)

main = do
  [dir]   <- getArgs
  files   <- glob (dir++"/*/NA*/libSC*/lane*/aln.map.err")
  conts   <- mapM readFile files
  let sizeMat = map getInsertSizeTriples conts
  let tuples  = DM.toList $ collateInsertSizes files sizeMat
  mapM_ printTuple tuples

printTuple (samp,(n,mu,sd)) = sprintf "%s %10d %7.2f %6.2f\n" samp n mu sd

getInsertSizeTriples =  justList . map insertSizeTriple . map words . lines
  
insertSizeTriple :: [String] -> Maybe WTup
insertSizeTriple ["[cal_insert_size]",count,_,_,_,_,_,mean,_,sdev] = do
  n  <- mayRead count
  mu <- mayRead mean
  sd <- mayRead sdev
  return (n,mu,sd)
insertSizeTriple _ = Nothing

collateInsertSizes files sizeMat = 
  let fileToName = (!!2) . reverse . split "/"
      samps = map fileToName files
   in collateInsertSizes1 DM.empty samps sizeMat
   
collateInsertSizes1 mp samps mat = 
  let step   = uncurry (DM.insertWith (++))
      mp     = foldr step DM.empty $ zip samps mat
      zero   = (0,0.0,0.0)
      recalc = foldr combineWeightedDists zero
   in DM.map recalc mp

combineWeightedDists :: WTup -> WTup -> WTup
combineWeightedDists (n1,mu1,sd1) (n2,mu2,sd2) = (n,mu,sd)
  where n    = n1+n2
        nx   = fromIntegral n
        p1   = (fromIntegral n1)/nx
        p2   = (fromIntegral n2)/nx
        mu = p1*mu1 + p2*mu2
        sumsq a b = a*a+b*b
        sd   = sqrt $ p1*sumsq sd1 (mu1-mu) + p2*sumsq sd2 (mu2-mu)


  



