import System.Time
import Perl2HS
import Utility

{-
Format of an event line is:

month day time place type details

with month and day as numbers, and the others as strings. For a (weekly) repeating
item, have multiple details fields
-}

-- Need this to be able to infer weekdays.
thisYear = 2008

data Event = Event {evDay :: Int, evItems :: [String]}
  deriving (Eq,Ord)

instance Show Event where
  show e = "|-\n| " ++ intercalate " || " (evItems e)

main = do
  home  <- getenv("HOME")
  cont  <- readFile(home++"/events.txt")
  now   <- getClockTime
  let lns = filter (not . isCommentLine) (lines cont)
  let eventList = concat $ map (createEvents . map normalize . split ",") lns
  writeFile (home++"/eventsw.txt") (unlines (map show (sort (discardBefore now eventList))))

isCommentLine "" = True
isCommentLine ('#':_) = True
isCommentLine _ = False

normalize = intercalate " " . words

discardBefore now lst =
  let today = ctYDay $ toUTCTime now
   in filter (\e -> evDay e >= today) lst

createEvents :: [String] -> [Event]
createEvents wds =
  case createEvents0 wds of
     Just e -> e
     Nothing  -> [Event 0 wds]

createEvents0 [date,time,place,what] =
  createEvents0 [date,time,place,what,""]
createEvents0 (date:time:place:what:dets) =
  case words date of
   [smon,sday] -> createEvents1 smon sday time place what dets
   _ -> Nothing
createEvents0 _ = Nothing

createEvents1 smon sday time place what dets = do
  mon <- mayRead smon
  day <- mayRead sday
  return $ createEvents2 mon day time place what (filter (not . (== "0")) dets)

createEvents2 _ _ _ _ _ [] = []
createEvents2 mon day time place what (det:detList) =
  let h = createEvent   mon  day    time place what det 
      t = createEvents2 mon (day+7) time place what detList
   in h:t

createEvent mon day time place what det =
  -- "Monday" is a space filler here ...
  let ct1 = CalendarTime thisYear (toEnum (mon-1)) day 0 0 0 0 Monday 0 "UTC" 0 False 
      ct  = (toUTCTime . toClockTime) ct1
   in Event (ctYDay ct)
        [(take 3 . show . ctWDay) ct,
         (take 3 . show . ctMonth) ct ++ " " ++ (sprintf "%2d" . fromEnum . ctDay) ct,
         time,place,what,det]
