module Main where

import System.Cmd as SC
import System.Process as SP
import System.Posix.Files as SF
import System.Posix.Process as SPP
import System.IO as SI

lsMr = do (ih,oh,eh,ph) <- SP.runInteractiveCommand "ls -o"
          top <- SI.hGetLine oh
          rest <- SI.hGetContents oh
          putStrLn top
          putStr $ unlines $ reverse $ lines rest

lsMat = do (ih,oh,eh,ph) <- SP.runInteractiveCommand "ls -o"
           top <- SI.hGetLine oh
           -- List of lists, one per file.
           mat <- hMatrix oh
           return $ rle $ map length mat

hMatrix h = do lns <- (hGetContents h)
               return $ map words $ lines lns

-- Run-length encoding: [item] -> [(count,item)]

rle [] = []
rle (h:t) = rle1 t h 1

rle1 [] v n = [(n,v)]
rle1 (h:t) v n | v == h    = rle1 t v (n+1)
               | otherwise = (n,v):(rle1 t h 1)

-- Inverse of rle

unrle = concatMap (uncurry unrle1)

unrle1 1 v = [v]
unrle1 n v = v:(unrle1 (n-1) v)

