package SGRP::AlignmentCoords;

use Exporter;
@ISA = qw(Exporter);

@EXPORT = qw(alignmentCoords alignmentCoords1);

use SGRP::Variables;

use Utility;
use File::Basename;

use Carp;

use strict;

sub alignmentCoords {
    my ($species,$filtMeth,$strainSet,@parts) = @_;
    unless (@parts) {
	confess("Usage: $0 alco species filtMeth strainSet part1 part2 ...");
    }
    my ($iter,$dir) = getCurrentIteration($species,$filtMeth,$strainSet,'p?');
    my $prev = previousDirectory($dir);
    foreach my $part (@parts) {
	$part = "p$part" if ($part =~ /^[0-9]$/);
	foreach my $chrDir (glob("$dir/$part/*")) {
	    alignmentCoords1("$chrDir/layout.gz","$chrDir/alicoords.gz");
	}
    }
}

sub alignmentCoords1 {
    my ($inFile,$outFile) = @_;

    return unless (isComplete($inFile,1));
    if (stamp($outFile)) {
	alignmentCoords2($inFile,$outFile);
	unstamp($outFile);
    }

    #my $dangFile = "$dir/dangles.txt";
    #if (isComplete($outFile,1) && stamp($dangFile)) {
	#findDangles($outFile,$dangFile);
	#unstamp($dangFile);
    #}

}

# Alignment coords line for each read is of form
# read chr pairingDirection iter rlo1=clo1:rhi1=chi1 ...

sub alignmentCoords2 {
    my ($inFile,$outFile) = @_;
    my $chr = basename(dirname($inFile));
    report("reading $inFile");
    my $fh = openToRead($inFile);
    my $off = 1; # offset in alignment
    my $ln = 0;  # line number in file
    # $current[$k] (for cols 2k and 2k+1) is
    #  [readName,iter,clo,rlo,dirn,rhi] where dirn is 1 or -1, rlo is "r for clo",
    # and rhi will have dirn added to it as soon as we see a nucleotide.
    # A tuple inside %alis is of the form
    #           [iter,clo,rlo,chi,rhi] (was: clo,chi,iter)
    my @current;
    push @current,['REF',0,1,1,1,0];
    my @suspended; # inside run of '='
    my %alis;
    my %mustBeFilled;
    while (<$fh>) {
	chomp;
	$ln ++;
	#last if ($ln == 10000);
	# We ignore REF lines, and also (in extra iteration) lines starting
	# with space.
	next if (/^>REF/ || /^ /);
	if (/^>(\S+):(\S+):(\S+) (\d+) (\S+)/) {
	    my ($name,$rlo,$rhi,$k,$iter) = ($1,$2,$3,$4+1,$5); # ADD ONE to the column
	    if ($current[$k]) {
		my ($name2,$iter2,$start2,$rstart2,$dirn2,$rend2) = @{$current[$k]};
		if ($suspended[$k]) {
		    $suspended[$k] = undef;
		} else {
		    push @{$alis{$name2}},[$iter2,$start2,$rstart2,$off-1,$rend2] if ($off > $start2);
		}
	    }
	    my $dirn = $rhi>=$rlo ? 1 : -1;
	    $current[$k] = [$name,$iter,$off,$rlo,$dirn,$rlo-$dirn];
	    $mustBeFilled{$k} = $_;
	} else {
	    foreach my $k (keys %mustBeFilled) {
		delete $mustBeFilled{$k};
	    }
	    my $kMax = length($_)/2-1;
	    foreach my $k (0 .. $kMax) {
		my $col = $k*2;
		my $v = substr($_,$col,1);
		# Allow for extended sequence being indicated in reference:
		$v = '=' if ($col == 0 && $v =~ /[a-z]/);
		# We don't count "-" as a reason to suspend ...
		if (($v eq ' ' && $k != 0) || !defined $v || $v eq '' || $v eq '=') {
		    mayRecordAlignment($current[$k],$suspended[$k],$off,\%alis);
		    if ($v eq '=') {
			unless ($suspended[$k]) {
			    unless ($current[$k]) {
				print STDERR "WARNING: hard gap at $inFile line $ln, offset $off, slot $k without current\n";
			    }
			    $suspended[$k] = 1;
			}
		    } else {
			$current[$k] = $suspended[$k] = undef;
		    }
		} else {
		    # A very few hard-to-fix cases of this still persist ... 14-9-07
		    #unless ($current[$k]) {
			#croakToFile($outFile,"Filled $k at line $ln\nHeader: none\nLine:   $_\n");
		    #}
		    if ($suspended[$k]) {
			if ($current[$k]) {
			    $current[$k][2] = $off;
			    $current[$k][3] = $current[$k][5]+$current[$k][4];
			}
			$suspended[$k] = undef;
		    }
		    $current[$k][5] += $current[$k][4] if ($current[$k] && $v =~ /[A-Za-z]/);
		}
	    }
	    foreach my $k ($kMax+1 .. $#current) {
		mayRecordAlignment($current[$k],$suspended[$k],$off,\%alis);
		$suspended[$k] = $current[$k] = undef;
	    }
	    while (@current && !defined $current[-1]) {
		$suspended[$#current] = undef;
		pop @current; # for efficiency
	    }
	    $off++;
	}
    }
    # Flush
    foreach my $k (0 .. $#current) {
	mayRecordAlignment($current[$k],$suspended[$k],$off,\%alis);
    }
    report();
    my @names = sort keys %alis;
    my $maxNL;
    foreach my $name (@names) {
	$maxNL = max($maxNL,length($name));
    }
    report("writing $outFile");
    my $gh = openToPipe("sort +4 -n | gzip -c > $outFile");
    foreach my $name (@names) {
	my $jname = $name . (' ' x ($maxNL-length($name)));
	printf($gh "%s %s %2d %8s",
	       $jname,
	       $chr,
	       pairingDirection($name,\%alis),
	       $alis{$name}[0][0]);
	foreach my $pTup (compactTuples($alis{$name})) {
	    my $expr = sprintf("%d %d %d %d",@$pTup);
	    printf($gh "   %27s",$expr);
	}
	print $gh "\n";
    }
    $gh->close();
    report();
}

sub mayRecordAlignment {
    my ($curr,$susp,$off,$pAlis) = @_;
    if ($curr && !$susp) {
	my ($name,$iter,$start,$rstart,$dirn,$rend) = @$curr;
	if ($off > $start) {
	    # start-6 allows up to a 5-bp deletion to be glossed over
	    if (defined $pAlis->{$name} && $pAlis->{$name}[-1][3] >= $start-6) {
		$pAlis->{$name}[-1][3] = $off-1; # extend existing one
		$pAlis->{$name}[-1][4] = $rend;
	    } else {
		push @{$pAlis->{$name}},[$iter,$start,$rstart,$off-1,$rend];
	    }
	}
    }
}

sub pairingDirection {
    my ($name,$pAlis) = @_;
    my $mate = $name;
    if ($mate =~ /1k$/) { # which it should!
	substr($mate,-3,1) =~ tr/pq/qp/;
    }
    return pairingDirection1($pAlis->{$name},$pAlis->{$mate});
}

sub pairingDirection1 {
    my ($pTups1,$pTups2) = @_;
    if (!$pTups2) {
	return 0; # no mate
    } elsif ($pTups1->[0][1] > $pTups2->[-1][3]) {
	return -1;
    } elsif ($pTups1->[-1][3] < $pTups2->[0][1]) {
	return 1;
    } else {
	return 0; # both present, but overlapping
    }
}

sub compactTuples {
    my ($pTups) = @_;
    # An element of $ans is an alignment tuple without the
    # initial $iter.
    my @ans;
    foreach my $pTup (@$pTups) {
	if (@ans && $pTup->[1] <= $ans[-1][2]+1) {
	    $ans[-1][2] = $pTup->[3];
	    $ans[-1][3] = $pTup->[4];
	} else {
	    push @ans,[@{$pTup}[1..4]];
	}
    }
    return @ans;
}

sub croakToFile {
    my ($g,$msg) = @_;
    $g =~ s/\.gz$//;
    my $gh = openToWrite("$g.err");
    print $gh $msg;
    $gh->close();
    croak("\n$msg");
}

sub findDangles {
    my ($f,$g) = @_;
    my $fh = openToRead($f);
    my (@segs,%isStart,%isEnd);
    while (<$fh>) {
	my @a = split;
	for (my $i=4; $i<$#a; $i+=2) {
	    my $lo = $a[$i];
	    my $hi = $a[$i+1];
	    push @segs,[$lo,$hi];
	    $isStart{$lo} = $isEnd{$hi} = 1;
	}
    }
    
    # Delete starts not preceded by an end
    foreach my $lo (keys %isStart) {
	unless ($isEnd{$lo-1}) {
	    delete $isStart{$lo};
	}
    }
    
    my $gh = openToWrite($g);
    my @starts = sort {$a <=> $b} keys %isStart;
    foreach my $pSeg (sort {$a->[0] <=> $b->[0]} @segs) {
	my ($lo,$hi) = @$pSeg;
	while (@starts && $starts[0] <= $hi) {
	    if ($starts[0] <= $lo) {
		print $gh $starts[0] . "\n";
	    }
	    shift @starts;
	}
    }
}


1;
