package SGRP::FindBranchLengths;

use Exporter;
@ISA = qw(Exporter);

@EXPORT = qw(findBranchLengths);

use SGRP::Variables;

use Utility;
use File::Basename;
use Carp;

use strict;

sub findBranchLengths {
    my ($key) = @_;
    # This is stand-alone code for easy exporting to Mark
    my $caFile = "${key}_ca";
    report("creating $caFile");
    `compressARG.pl $key.mgr $caFile`;
    report();
    return findBranchLengths1("${key}_ca","${key}_bl");
}

sub findBranchLengths1 {
    my ($caFile,$blFile) = @_;
    my $arch=`arch`;
    chomp $arch;
    my $bin = "$HOME/src/lengths/$arch/findBranchLengths";
    unless (-x $bin) {
	die("Missing or empty: $bin");
    }
    my $dir = dirname($caFile);
    # Output file $blFile has _bl where $caFile has _ca
    my $cmd = "$bin $dir/mgr_in $caFile $blFile $blFile.log 2>$blFile.err";
    # Try three times ...
    report("creating $blFile");
    foreach my $k (0 .. 2) {
	if ($k > 0) {
	    print " retrying...";
	}
	`$cmd`;
	if (-s "$blFile.err") {
	    if ($k < 2) {
		print "ERROR: see $blFile.err\n";
	    } else {
		die "ERROR: see $blFile.err";
	    }
	} elsif (checkArgsFileOK($blFile)) {
	    unlink("$blFile.err");
	    last;
	}
    }
    report();
    return $blFile;
}

sub checkArgsFileOK {
    my ($f) = @_;
    my $mf = sprintf("%s/mgr_in",dirname($f));
    my $mh = openToRead($mf);
    chomp ($_ = <$mh>);
    my (undef,$nm) = split;
    my @prevEnd;
    my $fh = openToRead($f);
    my $line = 0;
    while (<$fh>) {
	chomp;
	$line ++;
	my ($lo,$idx,$hi) = split;
	unless (defined $prevEnd[$idx]) {
	    $prevEnd[$idx] = -1;
	}
	unless ($lo == $prevEnd[$idx]+1) {
	    print "$f: ERROR: line $line: $lo $idx $hi, but prevEnd is $prevEnd[$idx]\n";
	    return 0; # not OK
	}
	$prevEnd[$idx] = $hi;
    }
    foreach my $i (0 .. $#prevEnd) {
	unless ($prevEnd[$i] == $nm-1) {
	    print "$f: ERROR: marker count is $nm, but ARG $i ends at $prevEnd[$i]\n";
	    return 0; # not OK
	}
    }
    return 1; # OK
}


1;
