package SGRP::GenerateHypotheses;

# An alternative to Filter.pm, using Prolog code.

use Exporter;
@ISA = qw(Exporter);

@EXPORT = qw(generateHypotheses);

use SGRP::Variables;
use SGRP::SGRP;

use Utility;
use File::Basename;

use Carp;

use strict;

sub generateHypotheses {
    my ($species,$setName) = @_;
    unless (defined $setName) {
	confess("Usage: $0 hyps species strainSetName");
    }
    my $state = "$ROOT/savedstates/$species";
    unless (-x $state) {
	confess("Missing or not executable: $state");
    }
    my $tgtDir1 = "$ROOT/hyps/$setName";
    my $tgtDir = "$tgtDir1/$species";
    my $tgtAli = "$tgtDir/ali_v4.txt";
    my $pStrains = SGRP::SGRP::getStrains($setName);
    unless (-s $tgtAli) {
	generateHypotheses1($species,$pStrains,$state,$tgtDir1,$tgtDir);
    }
    distributeAlignments($tgtAli,$species);
}

sub generateHypotheses1 {
    my ($species,$pStrains,$state,$tgtDir1,$tgtDir) = @_;
    copyAll($pStrains,"$ROOT/read","wm_lower.pl",$tgtDir1);
    copyAll($pStrains,"$ROOT/read","$species/meets_v1.pl",$tgtDir);
    copyAll($pStrains,"$ROOT/read","$species/ali_v3.pl",$tgtDir);
    report("generating hypotheses in Prolog");
    my $cmd = 'combine_meets,generate_hypotheses,hyp_bonuses_to_file,filter_hypotheses,';
    $cmd .=   'hypotheses_to_text,prune_alignments_using_hyps,create_alilines_file';
    SGRP::SGRP::runProlog($state,$tgtDir,$cmd,'hyps');
    report();
}

sub copyAll {
    my ($pStrains,$pre,$suf,$tgtDir) = @_;
    my $tgt = sprintf("%s/%s",$tgtDir,basename($suf));
    return if (-s $tgt);
    report("creating $tgt");
    my $gh = openToWrite($tgt);
    foreach my $str (sort @$pStrains) {
	my $fh = openToRead("$pre/$str/$suf");
	while (<$fh>) {
	    print $gh $_;
	}
    }
    $gh->close();
    report();
}

sub distributeAlignments {
    my ($src,$species) = @_;
    my $fh = openToRead($src);
    my ($g,$gh,$ghStr);
    while (<$fh>) {
	my (undef,undef,$str) = split;
	$str =~ s/-.*//;
	unless ($str eq $ghStr) {
	    if ($gh) {
		$gh->close();
		report();
	    }
	    $ghStr = $str;
	    $g = "$ROOT/read/$str/$species/methp/alilines.txt";
	    if (-s $g) {
		$gh = undef;
		report("already exists: $g",1);
	    } else {
		report("creating $g");
		$gh = openToWrite($g);
	    }
	}
	print $gh $_ if ($gh);
    }
    if ($gh) {
	$gh->close();
	report();
    }
}

1;
