package OffsetMap;

use Exporter;

@ISA = qw(Exporter);

use Utility;

use Carp;

use strict;

# The files should be *.map files, and partPat should be '?' or a digit.

# $self->{chr}[0] is list of "from" offsets
# $self->{chr}[1] is list of "to" offsets
# We don't need to store lengths, as they're differences of "from" offsets.

sub new {
    my ($class,$partPat,@files) = @_;
    my %h;
    my %max;
    foreach my $file (@files) {
	if (defined $file && -s $file) {
	    #report("reading offset map file $file");
	    my $fh = openToRead($file);
	    my $len;
	    while (<$fh>) {
		chomp;
		#my @a = split;
		my ($seq,$lo,$refLo,$len) = split;
		#if (@a == 4 || $seq !~ /^\d+$/) {
		#    ($seq,$lo,$refLo,$len) = @a;
		#} else {
		#    # for sorted12.map ...
		#    $seq = '';
		#    ($lo,$refLo,$len) = @a;
		#}
		if ($partPat ne '?') {
		    my (undef,$chr) = split(/\./,$seq);
		    # We use == so that chrm is part 0, correctly
		    next unless (substr($chr,-1) == $partPat);
		}
		push @{$h{$seq}[0]},$lo;
		push @{$h{$seq}[1]},$refLo;
		$max{$seq} = max($max{$seq},$lo+$len-1);
	    }
	    #report();
	}
    }
    if (%h) {
	$h{'MAX'} = \%max;
	my $self = bless \%h,$class;
	return $self;
    } else {
	return undef;
    }
}

sub alignedSegments {
    my ($self,$chr,$ignoreSz,$aLo,$aHi) = @_;
    my $pA = $self->{$chr};
    my ($kLo) = defined $aLo ? findOffset1($pA,$aLo,2) : 0;
    my ($kHi) = defined $aHi ? findOffset1($pA,$aHi,1) : $#{$pA->[0]};
    my @ans;
    # $pA->[0] is list of ref offsets
    # $pA->[1] is list of ali offsets
    foreach my $k ($kLo .. $kHi) {
	# rkLo should be ref coord corresponding to akLo ...
	my $rkLo = $pA->[0][$k];
	my $akLo = $pA->[1][$k];
	# ... but increase rkLo and akLo if aLo > akLo.
	if ($aLo > $akLo) {
	    $rkLo += $aLo-$akLo;
	    $akLo = $aLo;
	}

	my $akHi;
	if ($k == $#{$pA->[0]}) {
	    $akHi = $aHi;
	} else {
	    my $rkHi = $pA->[0][$k+1]-1;
	    $akHi = $akLo+$rkHi-$rkLo;
	}

	if (@ans && $akLo <= $ans[-1][1]+$ignoreSz+1) {
	    $ans[-1][1] = $akHi; # bridge small gap
	} else {
	    push @ans,[$akLo,$akHi];
	}
    }
    if (@ans) {
	unless (defined $ans[-1][1]) {
	    $ans[-1][1] = $self->{'MAX'}{$chr};
	}
	foreach my $k (0,1) {
	    if (defined $aLo && $ans[0][$k] < $aLo) {
		$ans[0][$k] = $aLo;
	    }
	    if (defined $aHi && $ans[-1][$k] > $aHi) {
		$ans[-1][$k] = $aHi;
	    }
	}
    }
    return \@ans;
}

sub findOffset1 {
    my ($pA,$off,$i) = @_;
    my $i0 = min($i,1);
    my $i1 = 1-$i0;
    my ($kLo,$vLo,$kHi,$vHi);
    # Deal with edge cases, because of sparsity in telomeres
    my $ans1;
    if ($off >= $pA->[$i0][-1]) {
	$kLo = $#{$pA->[0]};
    } elsif ($off >= $pA->[$i0][-2]) {
	$kLo = $#{$pA->[0]}-1;
    } elsif ($off < $pA->[$i0][1]) {
	$kLo = 0;
    } else {
	# Iterate:
	($kLo,$vLo) = (1,$pA->[$i0][1]);
	($kHi,$vHi) = ($#{$pA->[$i0]}-1,$pA->[$i0][-2]);
	while ($kHi-$kLo > 1) {
	    my $k = max($kLo+1,min($kHi-1,$kLo + int(($kHi-$kLo)*($off-$vLo)/($vHi-$vLo))));
	    my $v = $pA->[$i0][$k];
	    if ($off < $v) {
		$kHi = $k;
		$vHi = $v;
	    } else {
		$kLo = $k;
		last if ($off == $v);
		$vLo = $v;
	    }
	}
    }
    my $ans = $off+$pA->[$i1][$kLo]-$pA->[$i0][$kLo];
    if ($i > 0) {
	if ($kLo < $#{$pA->[$i0]} && $ans >= $pA->[$i1][$kLo+1]) {
	    my $gapR = $pA->[$i1][$kLo+1];
	    if ($i == 1 || $i == 3) {
		$ans = $gapR-1; # move down to just before gap
	    } else {
		$ans = $gapR; # move to just after gap
		$kLo ++;
	    }
	    if ($i >= 3) {
		$ans = -$ans; # signal "missing" with negation
	    }
	} elsif ($kLo == 0 && $ans < 1) {
	    if ($i == 1 || $i == 3) {
		$ans = 0; # move down to just before gap
	    } else {
		$ans = 1; # move to just after gap
		$kLo ++;
	    }
	}
    }
    return ($kLo,$ans);
}

sub alignmentRegions {
    my ($self,$gapTh,$flankSz) = @_;
    my %aregs;
    foreach my $seq (keys %$self) {
	unless ($seq eq 'MAX') {
	    $aregs{$seq} = alignmentRegions1($self->{$seq},$gapTh,$flankSz);
	}
    }
    return \%aregs;
}

sub alignmentRegions1 {
    my ($pA,$gapTh,$flankSz) = @_;
    my ($pFrom,$pTo) = @$pA;
    # Each member of @regs is initially [fromLo,fromHi]
    my @regs;
    # Start
    if ($pTo->[0] > $pFrom->[0]) {
	push @regs,[max(1,$pFrom->[0]-$flankSz),
		    $pFrom->[0]+$flankSz];
    }
    for (my $i=1; $i<=$#$pFrom; $i++) {
	my $len = $pFrom->[$i] - $pFrom->[$i-1];
	my $gapSz = $pTo->[$i] - $pTo->[$i-1] - $len;
	if ($gapSz >= $gapTh) {
	    if (@regs && $pFrom->[$i]-$flankSz <= $regs[-1][1]) {
		# Extend old region:
		$regs[-1][1] = $pFrom->[$i]+$flankSz;
	    } else {
		# Create new region:
		push @regs,[max(1,$pFrom->[$i]-$flankSz),
			    $pFrom->[$i]+$flankSz];
	    }
	}
    }
    # Extend each region with toLo,toHi
    foreach my $pR (@regs) {
	push @$pR,findOffset($pA,$pR->[0],0),findOffset($pA,$pR->[1],0);
    }
    return \@regs;
}

# $i = 0 -> forward translation
# $i > 0 is back translation, with different values
#   giving different results when we fall in a gap:
#    $i = 1 -> go to left of gap
#    $i = 2 -> go to right of gap
#    $i = 3 -> go to left of gap and negate
#    $i = 4 -> go to right of gap and negate

sub findOffset {
    my ($pA,$off,$i) = @_;
    my (undef,$off1) = findOffset1($pA,$off,$i);
    return $off1;
}

# Translate offset $off in alignment coords to position
# in strain coords. For $type, see below (default is 1).

sub backTranslation {
    my ($self,$seq,$off,$type) = @_;
    my $pA = $self->{$seq} || return undef;
    return findOffset($pA,$off,$type || 1);
}

# Returns just the cigar string

sub cigarString {
    my ($self,$seq,$lo,$hi) = @_;
    my $pA = $self->{$seq} || return 'M' . ($hi-$lo+1);
    my ($pos,@cs0) = findCigarString($pA,$lo,$hi);
    my $seq1 = reduceSequenceName($seq);
    my @ans;
    foreach my $op (@cs0) {
	my $type = substr($op,0,1);
	# We should never get I's, because dependent sequences are
	# always less complete
	#if ($type eq 'I') {
	#    push @ans,$op;
	if ($type eq 'M') {
	    my $len = substr($op,1);
	    push @ans,cigarString($self,$seq1,$pos,$pos+$len-1);
	    $pos += $len;
	} elsif ($type eq 'D') {
	    my $len = substr($op,1);
	    my $pA1 = $self->{$seq1};
	    if ($pA1) {
		my $d = findOffset($pA1,$pos+$len,0)-findOffset($pA1,$pos,0);
		push @ans,"D$d";
	    } else {
		push @ans,$op;
	    }
	    $pos += $len;
	}
    }
    return @ans;
}

# Returns start point in target AND cigar string
sub findCigarString {
    my ($pA,$lo,$hi) = @_;
    my ($k,$start) = findOffset1($pA,$lo,0);
    my $len=$hi-$lo+1;
    # We want to consume $len nucleotides of top sequence
    my @ans = ($start);
    my $delta = $lo - $pA->[0][$k];
    while ($len > 0) {
	if ($k == $#{$pA->[0]}) {
	    return (@ans,"M$len"); # end of the offset map
	} else {
	    my $dpa = $pA->[0][$k+1]-$pA->[0][$k];
	    my $segLen = $dpa - $delta;
	    if ($segLen > $len) {
		return (@ans,"M$len");
	    } else {
		my $dSeg = "D" . ($pA->[1][$k+1]-$pA->[1][$k] - $dpa);
		if ($segLen == $len) {
		    return (@ans,"M$len",$dSeg);
		} else {
		    push @ans,"M$segLen",$dSeg;
		    $k ++;
		    $len -= $segLen;
		    $delta = 0;
		}
	    }
	}
    }
}

# -------------------------------------------------------------

sub reduceSequenceName {
    my ($name) = @_;
    my @comps = split(/\./,$name);
    if (@comps == 3) {
	return "$comps[0].$comps[1]";
    } elsif (@comps == 2) {
	return $comps[1];
    } else {
	confess("cannot reduce $name");
    }
}

# Return [d1,d2], equal to off1-off2 in sequences
# seq1 and seq2 respectively; undef if cannot be found.

sub differences {
    my ($self,$seq1,$off1,$seq2,$off2) = @_;
    if ($seq1 eq $seq2) {
	# Speed: this will often be the case ...
	my $d = $off1-$off2;
	return [$d,$d];
    } else {
	my $off1in2 = translation($self,$seq1,$off1,$seq2);
	return undef unless (defined $off1in2);
	my $off2in1 = translation($self,$seq2,$off2,$seq1);
	return undef unless (defined $off2in1);
	return [$off1-$off2in1,$off1in2-$off2];
    }
}

# Return off1 translated from sequence 1 to sequence 2,
# or undef if impossible. If no seq2, go to underlying.

sub translation {
    my ($self,$seq1,$off1,$seq2) = @_;
    my $pA1 = $self->{$seq1} || return undef;
    my $offB = findOffset($pA1,$off1,0);
    if ($seq2) {
	my $pA2 = $self->{$seq2} || return undef;
	return findOffset($pA2,$offB,1);
    } else {
	return $offB;
    }
}

sub failsToSpanBreak {
    my ($self,$seq,$lo,$hi,$slop) = @_;
    my $pA = $self->{$seq};
    if ($pA) {
	foreach my $pos (@{$pA->[0]}) {
	    return 0 if ($pos > $lo+$slop && $pos <= $hi-$slop);
	}
	return 1;
    } else {
	return 0;
    }
}

sub getChromosomeNames {
    my ($self) = @_;
    my %isName;
    foreach my $key (keys %$self) {
	unless ($key eq 'MAX') {
	    my (undef,$name) = split(/\./,$key);
	    $isName{$name} = 1;
	}
    }
    return sort keys %isName;
}

# Return translation of $off to underlying chromosome.

sub resolution {
    my ($self,$seq,$off) = @_;
    my $pA = $self->{$seq} || return undef;
    return findOffset($pA,$off,0);
}

1;
