package SGRP::RunFregeneExperiment;

use Exporter;
@ISA = qw(Exporter);

@EXPORT = qw(runFregeneExperiment);

use SGRP::Variables;
use SGRP::BsubSGRP;
use Bsub;

use Utility;
use File::Basename;

use Carp;

use strict;

# This would be qw(09 91) for split version, for which we also need
# to enable $gHeld in fregeneQualityStrings.pl.
my @PAIRS = ('');

my $MAX_PART = 0; # do parts 0 to 0 only

sub runFregeneExperiment {
    if (@_ == 5) {
	runFregeneExperimentABI(@_);
    } elsif (@_ == 3) {
	runFregeneExperimentSolexa(@_);
    } else {
	confess("Usage: $0 rfre (species filtMeth strainSet | solexaFastqFile) numIndivs coverage");
    }
}

sub runFregeneExperimentABI {
    my ($species,$filtMeth,$strainSet,$numIndivs,$coverage) = @_;
    my $sim = "$ROOT/rin10k_simII.xml.gz";
    confess("$sim: empty or missing") unless (-s $sim);
    # Fix subdir 000 as most of the qualities.gz data will be there.
    my $srcDir = "$ROOT/match/$strainSet/$species/$filtMeth/000";
    #my ($iter,$srcDir) = getCurrentIteration($species,$filtMeth,$strainSet,'p?');
    confess("$srcDir: no such directory") unless (-d $srcDir);
    my $lKey = sprintf("freg%d_%d",$numIndivs,$coverage);
    my $uKey = sprintf("FREG%d_%d",$numIndivs,$coverage);
    my $tgtDir = "$ROOT/match/$uKey/freg";
    my $fDir = "$srcDir/$lKey";
    mkdir_p($fDir);
    my $tgt1 = sprintf("%s/freg%d_%d/fregeneQualities%s.gz",$srcDir,$numIndivs,$coverage,$PAIRS[0]);
    if (-s $tgt1) {
	report("$tgt1: already exists",1);
    } else {
	submitAndAwaitFregeneQualityStrings($numIndivs,$coverage,$species,
					    $filtMeth,$strainSet,$fDir,$lKey);
    }
    submitAndAwaitFregeneCreateLayout($fDir,$sim,$fDir,$tgtDir);
    #submitAndAwaitFregeneMainProcessing($uKey,$tgtDir);
}

sub submitAndAwaitFregeneQualityStrings {
    my ($numIndivs,$coverage,$species,$filtMeth,$strainSet,$fDir,$lKey) = @_;
    my $rVal = $MAX_PART+1;
    my $cmd = "fregeneQualityStrings.pl -r $rVal -n $numIndivs -c $coverage $species $filtMeth $strainSet";
    my $id = bsub("$fDir/fqs",$cmd,undef,undef,$lKey);
    report("submitted fqs job $id, and waiting");
    waitUntilDone($id);
    report();
}

sub submitAndAwaitFregeneCreateLayout {
    my ($fDir,$sim,$srcDir,$tgtDir) = @_;
    my $sim = "$ROOT/rin10k_simII.xml.gz";
    my @ids;
    foreach my $n (0 .. $MAX_PART) {
	my $lo = 1+$n*1000000;
	my $hi = (1+$n)*1000000;
	foreach my $treat (qw(errs noerrs)) {
	    my $eSw = ($treat eq 'errs') ? '-e' : '';
	    foreach my $pair (@PAIRS) {
		my $layout = "$tgtDir/$treat$pair/000/p$n/chr0$n/layout.gz";
		if (-s $layout) {
		    print "Already exists: $layout\n";
		} else {
		    my $cmd = "fregeneCreateLayout.pl -l $lo -h $hi $eSw $srcDir/fregeneQualities$pair.gz $sim $layout";
		    my $key = "fcl_${treat}${pair}_$n";
		    my $id = bsub("$fDir/$key",$cmd,undef,undef,$key);
		    push @ids,$id;
		    print "Submitted $id to create $layout\n";
		}
	    }
	}
    }
    #report("submitted fcl jobs, and waiting");
    #waitUntilDone(@ids);
    #report();
}

# $fqFile could be: 
# /lustre/work1/sanger/lh3/lh3/work/yeast/run/cere-SK1/raw/SK1.fastq.gz
sub runFregeneExperimentSolexa {
    my ($fqFile,$numIndivs,$coverage) = @_;
    my $sim = "$ROOT/rin10k_simII.xml.gz";
    confess("$sim: empty or missing") unless (-s $sim);
    my $lKey = sprintf("freg%d_%d",$numIndivs,$coverage);
    my $uKey = sprintf("FREG%d_%dS",$numIndivs,$coverage);
    my $tgtDir = "$ROOT/match/$uKey/freg";
    my $fDir = $tgtDir;
    mkdir_p($fDir);
    my $base = basename($fqFile,'.fastq');
    my $tgtDir1 = sprintf("%s/match/FQS/%s/freg%d_%d",$ROOT,$base,$numIndivs,$coverage);
    my $tgt1 = sprintf("%s/fregeneQualities%s.gz",$tgtDir1,$PAIRS[0]);
    if (-s $tgt1) {
	report("$tgt1: already exists",1);
    } else {
	submitAndAwaitFregeneQualityStrings($numIndivs,$coverage,$fqFile,
					    undef,undef,$fDir,$lKey);
    }
    submitAndAwaitFregeneCreateLayout($fDir,$sim,$tgtDir1,$tgtDir);
    #submitAndAwaitFregeneMainProcessing($uKey,$tgtDir);
}

sub submitAndAwaitFregeneMainProcessing {
    my ($uKey,$dir) = @_;
    my @ids;

    foreach my $op qw(proc smat smim) {
	my @pairs = @PAIRS;
	pop @pairs if (@pairs && $op ne 'proc'); # get rid of 91 if any
	foreach my $treat (qw(errs noerrs)) {
	    foreach my $pair (@pairs) {
		push @ids,bsubSGRP(undef,1,$op,3,'freg',"methc$pair",$uKey,(0 .. 9));
	    }
	}
	report("submitted $op jobs, waiting");
	waitUntilDone(@ids);
	report();
    }

    # This won't do, as we now have to compare errs and noerrs...
    #report("running runCompareImputedAcross");
    #`cd $dir; runCompareImputedAcross bl_`;
    #report();
}

1;
