package SGRP::SortLayoutFiles;

use Exporter;
@ISA = qw(Exporter);

@EXPORT = qw(sortLayoutFiles realignBuffer setDNABitsAndSymbols);

use SGRP::Variables;

use Utility;
use File::Basename;

use Carp;

use strict;

my @DNA_SYM;
my %DNA_BITS;

sub sortLayoutFiles {
    my ($species,$filtMeth,$strainSet,@parts) = @_;
    unless (@parts) {
	confess("Usage: $0 slof species filtMeth strainSet part1 part2 ...");
    }
    my ($iter,$dir) = getCurrentIteration($species,$filtMeth,$strainSet,'p?');
    foreach my $part (@parts) {
	$part = "p$part" if ($part =~ /^[0-9]$/);
	if ($IS_FINAL_ITERATION) {
	    copyClbsToCurrent($iter,dirname($dir),$part);
	}
	foreach my $chrDir (glob("$dir/$part/*")) {
	    sortLayoutFiles1($chrDir);
	}
	combineSequencedLocationCounts($dir);
    }
}

sub copyClbsToCurrent {
    my ($iter,$dir,$part) = @_;
    my @bases;
    foreach my $sub (glob("$dir/000/$part/*")) {
	push @bases,basename($sub) if (-d $sub);
    }
    foreach my $base (@bases) {
	my $tgt = sprintf("%s/%s/%s",iterationDir($dir,$iter),$part,$base);
	next if (-d $tgt);
	for (my $i=$iter-1; $i>=0; $i--) {
	    my $src = sprintf("%s/%s/%s",iterationDir($dir,$i),$part,$base);
	    if (-d $src) {
		mkdir_p($tgt);
		# We need layout.gz as well as sorted.gz so impx can create layout_y.gz
		report("Copying $src/clbs.gz, layout.gz and dangles.txt to $tgt");
		`cp -p $src/clbs.gz $src/layout.gz $src/dangles.txt $tgt`;
		report();
		last;
	    }
	}
    }
}

sub iterationDir {
    my ($dir,$iter) = @_;
    my $i3 = sprintf("%3d",$iter);
    $i3 =~ tr/ /0/;
    return "$dir/$i3";
}

sub sortLayoutFiles1 {
    my ($dir) = @_;
    my $src = "$dir/clbs.gz";
    return unless (isComplete($src,1));
    my $tgt1 = "$dir/sorted1.gz";
    my $tgt = "$dir/sorted.gz";
    if (stamp($tgt)) {
	if (stamp($tgt1)) {
	    sortLayoutFiles2($src,$tgt1);
	    unstamp($tgt1);
	}
	my $tgt2 = mayRealignGappedRegions($tgt1,"$dir/sorted2.gz","$dir/sorted12.map");
	#liftOverAuxRegions($dir);
	report("collecting gap runs");
	my $pGapRuns = collectGapRuns($tgt2);
	report();
	applyGapIndices($tgt2,$pGapRuns,$tgt);
	unlink($tgt2) unless ($IS_FINAL_ITERATION);
	unstamp($tgt);
	#report("created $tgt",1);
    }
    sequencedLocationCount($dir);
}

# sub liftOverAuxRegions {
#     my ($dir) = @_;
#     my $chrHere = basename($dir);
#     my $prevAux = previousAuxFile($dir);
#     my $pOM = new OffsetMap('?',"$dir/sorted12.map");
#     my $fh = openToRead($prevAux);
#     my $gh = openToWrite("$dir/extended.map");
#     while (<$fh>) {
# 	chomp;
# 	my ($name,$from,$to,$len) = split;
# 	my (undef,$chr,$extn) = split(/\./,$name);
# 	if ($chr eq $chrHere && defined $extn) {
# 	    my $tFrom = $pOM->translation($from);
# 	    my $tM
#     
# }
# 
# sub previousAuxFile {
#     my ($dir) = @_;
#     my @comps = split(/\//,$dir);
#     my $n3 = $comps[-3];
#     if ($n3 == 0) {
# 	my $sp = $comps[-4];
# 	foreach my $base (qw(extended augmented)) {
# 	    my $f = "$ROOT/ref/$sp/$base.map";
# 	    return $f if (-s $f);
# 	}
# 	die("Cannot find $ROOT/ref/$sp/extended.map or augmented.map");
#     } else {
# 	$n3 = sprintf("%3d",$n3-1);
# 	$n3 =~ tr/ /0/;
# 	$comps[-3] = $n3;
# 	my $f = join("/",@comps) . "/extended.map";
# 	return $f if (-s $f);
# 	die("Cannot find $f");
#     }
# }

sub sortLayoutFiles2 {
    my ($src,$tgt) = @_;
    my @strain;
    my $fh = openToRead($src);
    while (<$fh>) {
	if (/^>(\S+) (\d+)$/) {
	    unless ($1 eq 'REF') {
		$strain[$2] = $1;
	    }
	}
    }
    my @sorted = sort @strain;
    my %posn;
    foreach my $k (0 .. $#sorted) {
	$posn{$sorted[$k]} = $k;
    }
    my $ns = scalar(@sorted);
    my @map;
    foreach my $k (0 .. $#strain) {
	$map[$k] = $posn{$strain[$k]};
    }
    report("creating $tgt with $ns strains");
    my $gh = openToWrite($tgt);
    print $gh ">@sorted\n";
    # Reopen source:
    $fh = openToRead($src);
    while (<$fh>) {
	next if (/^>/);
	chomp;
	# $#sorted+2 because @sorted doesn't include ref,
	# and $#sorted = scalar(@sorted)-1.
	my $tgtLine = '  ' x ($#sorted+2);
	if (/^(..)(.*)/) {
	    my ($ref,$rest) = ($1,$2);
	    substr($tgtLine,0,2) = $ref;
	    my $kLim = length($rest)/2;
	    for (my $k=0; $k<$kLim; $k++) {
		# "2+" because first two positions are for ref
		substr($tgtLine,2+$map[$k]*2,2) = substr($rest,$k*2,2);
	    }
	} else {
	    substr($tgtLine,0,length($_)) = $_;
	}
	print $gh "$tgtLine\n";
	my $rv = substr($_,0,1);
    }
    $gh->close();
    report();
}

sub mayRealignGappedRegions {
    my ($src,$tgt,$map) = @_;
    return $src unless ($IS_FINAL_ITERATION);
    report("creating $tgt");
    realignGappedRegions($src,$tgt,$map);
    report();
    return $tgt;
}

sub realignGappedRegions {
    my ($src,$tgt,$map) = @_;
    setDNABitsAndSymbols();
    my $fh = openToRead($src);
    my $gh = openToWrite($tgt);
    my $mh = openToWrite($map);
    my @buf;
    my $hdr = <$fh>;
    print $gh $hdr;
    my @lns = (0,0);
    print $mh "@lns\n";
    while (<$fh>) {
	chomp;
	if (isPolyLine($_) && !(@buf && differentStrainsSequenced($buf[-1],$_))) {
	    push @buf,$_;
	} else {
	    if (@buf) {
		realignBuffer(\@buf,$gh,\@lns,$mh);
		@buf = ();
	    }
	    print $gh "$_\n";
	    $lns[0] ++;
	    $lns[1] ++;
	}
    }
    realignBuffer(\@buf,$gh,\@lns,$mh) if (@buf);
}

# Copied from alicat.pl:

# Bit positions are:
#  A 1
#  C 2
#  G 4
#  T 8
#  - 16

sub setDNABitsAndSymbols {
    %DNA_BITS = qw(A  1 C  2 G  4 T  8
                   M  3 R  5 W  9
                   S  6 Y 10 K 12
                   B 14 D 13 H 11 V  7
                   N 15 - 16 = 16
                   a 17 c 18 g 20 t 24
                   m 19 r 21 w 25
                   s 22 y 26 k 28
                   b 30 d 29 h 27 v 23
                   n 31);
    @DNA_SYM = ();
    while (my ($k,$v) = each %DNA_BITS) {
        # Make sure it's been "used as a number" so
        # bitwise "|" works numerically!
        $DNA_BITS{$k} += 0;
        $DNA_SYM[$v] = $k;
    }
}

sub isPolyLine {
    my ($line) = @_;
    my $r;
    my $len = length($line);
    my $inc = 2; # $VALS_ONLY ? 1 : 2;
    for (my $i=0; $i<$len; $i+=$inc) {
	my $v = substr($line,$i,1);
	# Note: hard gap does NOT qualify us as polymorphic,
	# else we get huge regions that it's slow and unsafe
	# to realign.
	if ($v ne ' ' && $v ne '=') { 
	    $v =~ tr/a-z/A-Z/;
	    $v = '-' if ($v !~ /[A-Z]/);
	    if (defined $r) {
		return 1 unless ($v eq $r);
	    } else {
		$r = $v;
	    }
	}
    }
    return 0;
}

sub differentStrainsSequenced {
    my ($line1,$line2) = @_;
    my $len1 = length($line1);
    my $len2 = length($line2);
    return 1 unless ($len1 == $len2);
    for (my $i=2; $i<$len1; $i+= 2) {
	return 1 unless ((substr($line1,$i,1) eq ' ') eq (substr($line2,$i,1) eq ' '));
    }
    return 0;
}

sub realignBuffer {
    my ($pBuf,$gh,$pLns,$mh) = @_;
    my $d = $pLns->[1]-$pLns->[0] if ($pLns);
    my ($refIn,$refOut) = realignPolymorphicRegion($pBuf,$gh);
    my $lenIn = length($refIn);
    my $lenOut = length($refOut);
    my ($i,$j) = (0,0);
    while ($i < $lenIn || $j < $lenOut) {
	my $nucIn =  ($i < $lenIn  && substr($refIn, $i,1) =~ /[A-Za-z]/);
	my $nucOut = ($j < $lenOut && substr($refOut,$j,1) =~ /[A-Za-z]/);
	my ($di,$dj) = (0,0);
	if ($nucIn && !$nucOut) {
	    $dj = 1;
	} elsif ($nucOut && !$nucIn) {
	    $di = 1;
	} else {
	    $di = 1 if ($i < $lenIn);
	    $dj = 1 if ($j < $lenOut);
	}
	if ($di) {
	    $i++;
	    $pLns->[0] ++ if ($pLns);
	}
	if ($dj) {
	    $j++;
	    $pLns->[1] ++ if ($pLns);
	}
	if ($mh && ($di != $dj) && $pLns) {
	    print $mh "@$pLns\n";
	}	    
    }
}

sub realignPolymorphicRegion {
    my ($pBuf,$gh) = @_;
    my @origV; # $origV[$k][$i] is value of strain k in line i, if sequenced
    my @origQ; # $origQ[$k][$i] is quality of strain k in line i, if sequenced
    my %seq;     # $seq{$k} is sequence of strain k (nucleotides only)
    my $origRef;
    my $maxLen;
    my %sumScore;
    foreach my $i (0 .. $#$pBuf) {
	my $line = $pBuf->[$i];
	$origRef .= substr($line,0,1);
	my $len = length($line);
	$maxLen = max($len,$maxLen);
        # For each sequenced strain...
	for (my $j=0; $j<$len; $j+=2) {
	    my $val = substr($line,$j,1);
	    if ($val ne ' ') {
		my $k = $j/2;
		$origV[$k][$i] = $val;
		my $qc = substr($line,$j+1,1);
		$origQ[$k][$i] = $qc;
		$val =~ tr/acgtn/ACGTN/;
		if ($val =~ /[ACGTN]/) {
		    $seq{$k} .= $val; # normalized
		    $sumScore{$k} += ($qc eq ' ' ? 127 : ord($qc)) - 33;
		}
	    }
	}
    }

    my %seqScore; # $seqScore{$seq} is sum of scores of all strains with that seq
    foreach my $k (keys %seq) {
	$seqScore{$seq{$k}} += $sumScore{$k};
    }
    # Order distinct sequences with highest-scoring first
    my ($cons,@rest) = sort {$seqScore{$b} <=> $seqScore{$a}} (keys %seqScore);
    my %ali = ($cons,$cons); # mapping from each distinct sequence to gapped version
    #print "  $cons -> $cons\n" if ($DEBUG);
    foreach my $sequ (@rest) {
	#print "  $sequ -> " if ($DEBUG);
	$cons = addToAlignment($sequ,\%ali,$cons);
    }
    my $nOut = printRealignment(\%ali,\%seq,\@origV,\@origQ,$maxLen,$gh);
    # $nOut is number of output lines; $origRef is original ref sequence;
    # $ali{$seq{0}} is how ref seq aligned if it was non-empty, or
    # the appropriate number of gap symbols.
    return ($origRef,$ali{$seq{0}} || ('-' x $nOut));
}    
	
sub addToAlignment {
    my ($seq,$pAli,$cons) = @_;
    return $cons if (defined $pAli->{$seq}); # should not happen
    my @cigar = align($seq,$cons);
    # Apply the cigar to everything currently in $pAli,
    # _if_ cigar has any 'i' in it (else no change)
    my $ci = containsInsertions(\@cigar);
    if ($ci) {
	foreach my $seq1 (keys %$pAli) {
	    $pAli->{$seq1} = applyCigar('i',\@cigar,$pAli->{$seq1});
	}
    }
    # Apply the cigar to $seq
    $pAli->{$seq} = applyCigar('d',\@cigar,$seq);
    $cons = merge($pAli->{$seq},applyCigar('i',\@cigar,$cons));
    #print "$pAli->{$seq} -> $cons\n" if ($DEBUG);
    return $cons;
}
    
sub align {
    my ($seq,$cons) = @_;
    my $ls = length($seq);
    my $lc = length($cons);
    my @score; # no. of nuc matches back to (0,0) from here
    my @op; # to go backwards
    $score[0][0] = 0;
    my $indelPenalty = 0.6;
    my $indelPenaltyExtra = 0.2;
    foreach my $j (1 .. $lc) {
	$score[0][$j] = -$indelPenalty*$j;
	$op[0][$j] = 'd';
    }
    foreach my $i (1 .. $ls) {
	$score[$i][0] = -$indelPenalty*$i;
	$op[$i][0] = 'i';
	my $vi = substr($seq,$i-1,1);
	foreach my $j (1 .. $lc) {
	    my $ms = matchScore($vi,substr($cons,$j-1,1));
	    my $mScore = $score[$i-1][$j-1] + $ms;
	    my $dScore = $score[$i][$j-1]-$indelPenalty;
	    # Cheating affine-gap adjustment...
	    $dScore -= $indelPenaltyExtra unless ($op[$i][$j-1] eq 'd');
	    my $iScore = $score[$i-1][$j]-$indelPenalty;
	    $iScore -= $indelPenaltyExtra unless ($op[$i-1][$j] eq 'i');
	    if ($mScore > $dScore && $mScore > $iScore) {
		$op[$i][$j] = 'm';
		$score[$i][$j] = $mScore;
	    } elsif ($dScore >= $iScore) {
		$op[$i][$j] = 'd';
		$score[$i][$j] = $dScore;
	    } else {
		$op[$i][$j] = 'i';
		$score[$i][$j] = $iScore;
	    }
	}
    }
    my @ops;
    my $i = $ls;
    my $j = $lc;
    while ($i > 0 || $j > 0) {
	my $opij = $op[$i][$j];
	push @ops,$opij;
	$i -- unless ($opij eq 'd');
	$j -- unless ($opij eq 'i');
    }
    my @cigar;
    while (@ops) {
	my $opn = $ops[-1];
	my $len = 0;
	while (@ops && $ops[-1] eq $opn) {
	    $len ++;
	    pop @ops;
	}
	push @cigar,[$opn,$len];
    }
    return @cigar;    
}

sub matchScore {
    my ($vi,$vj) = @_;
    if ($vi eq $vj) {
	return 1.1; # bonus for no mutations
    } else {
	my $v = consensusValue($vi,$vj);
	return ($v eq $vj) ? 1 : 0;
    }
}

sub consensusValue { # simplified from alicat.pl
    my ($v1,$v2) = @_;
    if ($v1 eq '-') {
	return $v2;
    } elsif ($v2 eq '-') {
	return $v1;
    } else {
	return $DNA_SYM[$DNA_BITS{$v1} | $DNA_BITS{$v2}];
    }
}

sub containsInsertions {
    my ($pCigar) = @_;
    foreach my $pA (@$pCigar) {
	return 1 if ($pA->[0] eq 'i');
    }
    return 0;
}

sub applyCigar {
    my ($insOp,$pCigar,$seq) = @_;
    my $ali;
    my $off = 0;
    foreach my $pA (@$pCigar) {
	my ($op,$len) = @$pA;
	if ($op eq $insOp) {
	    $ali .= '-' x $len;
	} else {
	    $ali .= substr($seq,$off,$len);
	    $off += $len;
	}
    }
    return $ali;
}

sub merge {
    my ($s1,$s2) = @_;
    my $len = length($s1);
    my $s;
    foreach my $i (0 .. $len-1) {
	$s .= consensusValue(substr($s1,$i,1),substr($s2,$i,1));
    }
    return $s;
}

sub printRealignment {
    my ($pAli,$pS,$pOrig,$pQ,$maxLen,$gh) = @_;
    my (@val,@qual); # TRANSPOSED for printing
    foreach my $k (keys %$pS) {
	my $seq = $pS->{$k};
	my @ali = split("",$pAli->{$seq});
	my $ii = 0; # in orig
	my ($vGap,$qGap); # calculate when needed
	foreach my $i (0 .. $#ali) {
	    my $v = $ali[$i];
	    if ($v eq '-') {
		if ($pOrig->[$k][$ii] =~ /[-_=0-9]/) {
		    # use it
		    $val[$i][$k] = $pOrig->[$k][$ii];
		    $qual[$i][$k] = $pQ->[$k][$ii];
		    $ii++;
		} else {
		    unless (defined $vGap) {
			($vGap,$qGap) = decideVQGap($pOrig,$pQ,$ii,$k);
		    }
		    $val[$i][$k] = $vGap;
		    $qual[$i][$k] = $qGap;
		}
	    } else {
		while ($pOrig->[$k][$ii] !~ /[A-Za-z]/) {
		    $ii++; # discard
		}
		$val[$i][$k] = $pOrig->[$k][$ii];
		$qual[$i][$k] = $pQ->[$k][$ii];
		$ii++;
	    }
	}
    }
    foreach my $k (0 .. $#$pOrig) {
	next unless ($pOrig->[$k] && !$pS->{$k}); # gaps only...
	my $iMax = min($#val,$#{$pOrig->[$k]});
	foreach my $i (0 .. $iMax) {
	    $val[$i][$k] = $pOrig->[$k][$i];
	    $qual[$i][$k] = $pQ->[$k][$i];
	}
    }
    my $kMax = $#$pOrig;
    foreach my $i (0 .. $#val) {
	my $pVI = $val[$i];
	my $pQI = $qual[$i];
	foreach my $k (0 .. $#$pVI) {
	    my $v = $pVI->[$k];
	    my $q = $pQI->[$k];
	    if ($v =~ /[^ ]/) {
		print $gh "$v$q";
	    } elsif ($k == 0) {
		print $gh '- '; # can't have space in reference!
	    } else {
		print $gh '  ';
	    }
	}
	my $extra = $maxLen - 2*scalar(@$pVI);
	print $gh ' ' x $extra if ($extra);
	print $gh "\n";
    }
    return scalar(@val);
}

sub decideVQGap {
    my ($pV,$pQ,$ii,$kk) = @_;
    # Same strain:
    foreach my $i (0 .. $#{$pV->[$kk]}) {
	my $vi = $pV->[$kk][$i];
	if ($vi eq '-' || $vi > 0) {
	    return ($vi,$pQ->[$kk][$i] || ' ');
	}
    }
    # Same position:
    foreach my $k (1 .. $#$pV) {
	my $vi = $pV->[$k][$ii];
	if ($vi eq '-' || $vi > 0) {
	    return ($vi,$kk == 0 ? ' ' : $pQ->[$k][$ii] || ' ');
	}
    }
    # Anywhere:
    foreach my $k (1 .. $#$pV) {
	foreach my $i (0 .. $#{$pV->[$k]}) {
	    my $vi = $pV->[$k][$i];
	    if ($vi eq '-' || $vi > 0) {
		my $qi = ' ';
		if ($kk > 0) {
		    $qi = $pQ->[$k][$i];
		    $qi = ':' unless (defined $qi && $qi ne ' ');
		}
		return ($vi,$qi);
	    }
	}
    }
    # Arbitrary:
    return ('-',':');
}

sub collectGapRuns {
    my ($f) = @_;
    my $fh = openToRead($f);
    <$fh>; # header line
    # Note: these 'off' values are in the FILE, not the chromosome,
    # i.e. they increment for reference gaps too.
    my $off = 0;
    my @gapStart;
    my @runs;
    while (<$fh>) {
	my $kMax = length($_)/2-1;
	for (my $k=0; $k <= $kMax; $k++) {
	    my $c = substr($_,2*$k,1);
	    # These "gaps" are only soft ones, as only they will get
	    # distinguished for imputation purposes.
	    if ($c eq '-') {
		unless (defined $gapStart[$k]) {
		    $gapStart[$k] = $off;
		}
	    } elsif (defined $gapStart[$k]) {
		push @runs,[$gapStart[$k],$off-1,$k];
		$gapStart[$k] = undef;
	    }
	}
	$off ++;
    }
    for (my $k=0; $k <= $#gapStart; $k++) {
	if (defined $gapStart[$k]) {
	    push @runs,[$gapStart[$k],$off-1,$k];
	}
    }
    @runs = sort {$a->[0] <=> $b->[0] || $a->[1] <=> $b->[1] || $a->[2] <=> $b->[2]} @runs;
    decideGapRunIndices(\@runs);
    return \@runs;
}

sub decideGapRunIndices {
    my ($pRuns) = @_;
    my @indexRange;
  RUN:
    foreach my $i (0 .. $#$pRuns) {
	my ($lo,$hi,$k) = @{$pRuns->[$i]};
	foreach my $idx (1 .. 5) {
	    if (defined $indexRange[$idx]) {
		if ($indexRange[$idx][0] == $lo &&
		    $indexRange[$idx][1] == $hi) {
		    $pRuns->[$i][3] = $idx;
		    next RUN;
		} elsif ($indexRange[$idx][1] < $lo) {
		    $indexRange[$idx] = undef;
		}
	    }
	}
	# We need to create a new one ...
	foreach my $idx (1 .. 5) {
	    unless (defined $indexRange[$idx]) {
		$indexRange[$idx] = [$lo,$hi];
		$pRuns->[$i][3] = $idx;
		next RUN;
	    }
	}
	# Overflow! Send to stdout to avoid contaminating .berr file.
	print "WARNING: re-using gap index 5 at $lo $hi $k\n";
	$pRuns->[$i][3] = 5;
    }
}

sub applyGapIndices {
    my ($src,$pGapRuns,$tgt) = @_;
    report("creating $tgt");
    my $fh = openToRead($src);
    my $gh = openToWrite($tgt);
    my $off = 0;
    my $j = 0; # offset in $pGapRuns
    my $hdr = <$fh>;
    print $gh $hdr;
    while (<$fh>) {
	my $kMax = length($_)/2-1;
	for (my $k=0; $k<=$kMax; $k++) {
	    my $p = $k*2;
	    my $sp1 = substr($_,$p,1);
	    if ($sp1 eq '-') { # and not '='; these are soft gaps only
		substr($_,$p,1) = findGapIndex($pGapRuns,$j,$off,$k);
	    }
	}
	print $gh $_;
	$off ++;
	while ($j < $#$pGapRuns && $pGapRuns->[$j][1] < $off) {
	    $j++;
	}
    }
    $gh->close();
    report();
}

sub findGapIndex {
    my ($pGapRuns,$jMin,$off,$k) = @_;
    for (my $j=$jMin; $j <= $#$pGapRuns; $j++) {
	my ($lo,$hi,$kk,$idx) = @{$pGapRuns->[$j]};
	last if ($lo > $off);
	if ($kk == $k && $hi >= $off) {
	    return $idx;
	}
    }
    confess("No gap index found for $jMin $off $k");
}

sub sequencedLocationCount {
    my ($d) = @_;
    my $f = "$d/sorted.gz";
    my $g = "$d/sequenced.txt";
    return unless (-s $f && ! -f "$f.touch" && stamp($g));
    report("$g: creating");
    my $nseq = sequencedLocationCount1($f);
    my $gh = openToWrite($g);
    print $gh "$nseq\n";
    $gh->close();
    unstamp($g);
    report();
}

sub sequencedLocationCount1 {
    my ($f) = @_;
    my $fh = openToRead($f);
    <$fh>; # header
    my $nseq = 0;
    while (<$fh>) {
	unless (/^>/) {
	    my $len = length($_);
	    for (my $i=2; $i<$len; $i+=2) {
		$nseq ++ if (substr($_,$i,1) =~ /[A-Za-z]/);
	    }
	}
    }
    return $nseq;
}

sub combineSequencedLocationCounts {
    my ($dir) = @_;
    my $tgt = "$dir/sequenced.txt";
    return if (-s $tgt);
    my $nseq = 0;
    my $dir0 = $dir;
    $dir0 =~ s/...$/000/; # iteration zero ...
    my $pos = length($dir0)-3;
    foreach my $d0 (glob("$dir0/p?/*")) {
	next unless (-d $d0);
	my $pat = $d0;
	substr($pat,$pos,3) = '???';
	my @dirs = reverse(glob($pat));
	my $d;
	foreach my $dd (@dirs) {
	    if (substr($dd,0,$pos+3) le $dir) {
		$d = $dd;
		last;
	    }
	}
	my $f = "$d/sequenced.txt";
	return unless (-s $f && ! -f "$f.touch");
	my $fh = openToRead($f);
	my $line = <$fh>;
	chomp $line;
	$nseq += $line;
    }
    return if (-s $tgt);
    report("creating $tgt");
    my $gh = openToWrite($tgt);
    print $gh "$nseq\n";
    $gh->close();
    report();
}

sub updateDeleted {
    my ($pRA,$pPosn,$pDeleted) = @_;
    foreach my $pA (@$pRA) {
	my ($type,$sense,undef,undef,@strains) = @$pA;
	if ($type eq 'del') {
	    my $val = ($sense eq 'ON');
	    foreach my $s (@strains) {
		if ($val) {
		    $pDeleted->{$pPosn->{$s}} = 1;
		} else {
		    delete $pDeleted->{$pPosn->{$s}};
		}
	    }
	}
    }
}


1;
