package SGRP::SplitAlignments;

use Exporter;
@ISA = qw(Exporter);

@EXPORT = qw(splitAlignments);

use SGRP::Variables;

use Utility;
use File::Basename;

use Carp;

use strict;

sub splitAlignments {
    my ($species,$srcMeth,$loDig,$nDigs,$tgtMeth,@strains) = @_;
    unless (@strains && $loDig =~ /^[0-9]$/ && $nDigs > 0 && $nDigs <= 10) {
	confess("Usage: $0 sali species srcMeth loDigit nDigs tgtMeth strain1 strain2 ...");
    }
    foreach my $str (@strains) {
	splitAlignments1($species,$srcMeth,$loDig,$nDigs,$tgtMeth,$str);
    }
}

sub splitAlignments1 {
    my ($species,$srcMeth,$loDig,$nDigs,$tgtMeth,$str) = @_;
    my $srcDir = "$ROOT/read/$str/$species/$srcMeth";
    my $tgtDir = "$ROOT/read/$str/$species/$tgtMeth";
    if (! -d $srcDir) {
	report("$srcDir: missing",1);
    } elsif (-d $tgtDir) {
	report("$tgtDir: already exists",1);
    } else {
	splitAlignments2($srcDir,$loDig,$nDigs,$tgtDir);
    }
}

sub splitAlignments2 {
    my ($srcDir,$loDig,$nDigs,$tgtDir) = @_;
    my %wanted;
    for (my $k=0; $k<$nDigs; $k++) {
	$wanted{($loDig+$k)%10} = 1;
    }
    splitAlilinesFile("$srcDir/alilines.txt",\%wanted,"$tgtDir/alilines.txt");
    splitAlilinesFile("$srcDir/alilines_status.txt",\%wanted,"$tgtDir/alilines_status.txt");
    splitClusterFile("$srcDir/clusters.txt",\%wanted,"$tgtDir/clusters.txt");
    splitAlignmentsFile("$srcDir/ssahaSNP.sorted.gz",\%wanted,"$tgtDir/ssahaSNP.sorted.gz");
}

sub splitAlilinesFile {
    my ($f,$pWanted,$g) = @_;
    report("$g: creating");
    my $fh = openToRead($f);
    my $gh = openToWrite($g);
    while (<$fh>) {
	my (undef,undef,$id) = split;
	print $gh $_ if ($pWanted->{readNameDigit($id)});
    }
    $fh->close();
    $gh->close();
    report();
}

# Takes 16*24*plateNumber + 24*[a-p] + final digit pair,
# and returns tens part.

sub readNameDigit {
    my ($id) = @_;
    # Strip off anything before '-' or after '.'
    $id =~ s/.*-//;
    $id =~ s/\..*//;
    my ($plate,$a,$n) = ($id =~ /^(\d+)([a-z])(\d+)$/);
    my $sum = $n + 24*(ord($a)-ord('a') + 16*$plate);
    return $sum % 10;
}

sub splitClusterFile {
    my ($f,$pWanted,$g) = @_;
    report("$g: creating");
    my $fh = openToRead($f);
    my $gh = openToWrite($g);
    my $active = 0;
    while (<$fh>) {
	if (/^(\S+) /) {
	    $active = $pWanted->{readNameDigit($1)};
	}
	print $gh $_ if ($active);
    }
    $fh->close();
    $gh->close();
    report();
}

sub splitAlignmentsFile {
    my ($f,$pWanted,$g) = @_;
    report("$g: creating");
    my $fh = openToRead($f);
    my $gh = openToWrite($g);
    my $active = 0;
    while (<$fh>) {
	if (/^ALIGNMENT/) {
	    my (undef,undef,$id) = split;
	    $active = $pWanted->{readNameDigit($id)};
	}
	print $gh $_ if ($active);
    }
    $fh->close();
    $gh->close();
    report();
}

1;
