package SGRP::Variables;

use Exporter;
@ISA = qw(Exporter);

@EXPORT = qw(setVariables getCurrentIteration previousDirectory
	     $HOME $LUS $ROOT $REF $ALIASES $ARG_COUNT $REPEAT_DB
	     $JAVA_IMPUTER $MJM_JAVA $SSAHA2_MEMORY $FIXED_ITERATION
	     @STATIC_IMPUTATION_SUFS $IS_FINAL_ITERATION $RELEASE_DIR);

use Utility;

use File::Basename;
use Carp;
use strict;

use vars qw($HOME $LUS $ROOT $REF $ALIASES $ARG_COUNT $REPEAT_DB $JAVA_IMPUTER 
	    $MJM_JAVA $SSAHA2_MEMORY $FIXED_ITERATION @STATIC_IMPUTATION_SUFS
            $IS_FINAL_ITERATION $RELEASE_DIR);

# Dumb, genomic, chromosome, five-hundreds, two-hundreds, hundreds, fifties, twenties, tens:
@STATIC_IMPUTATION_SUFS = qw(d h); # qw(d g c v w h f e t);

$REPEAT_DB = 'comrep'; # for PruneAlignments

$JAVA_IMPUTER = 0;

#$MJM_JAVA = '/lustre/work1/sanger/mjm/jre/bin/java';

$MJM_JAVA = '/nfs/team71/phd/dmc/software/jdk1.5.0_05/jre/bin/java';

# Megabytes to allocate to ssaha2:
$SSAHA2_MEMORY = 6000;


# If building Gbrowse data from a release dir, set $RELEASE_DIR here;
# if not defined, build from $SGR (scratch dir)

$RELEASE_DIR = '/lustre/work1/sanger/dmc/sgroot/release3';

sub setVariables {

    my ($rootDir,$fixedIter) = @_;

    $FIXED_ITERATION = $fixedIter; # undef by default
    setDirectory(\$HOME,$ENV{'HOME'});
    setDirectory(\$LUS,-d $ENV{'LUS'} ? $ENV{'LUS'} : $HOME);
    setDirectory(\$ROOT,$rootDir || $ENV{SGR} || "$LUS/sgroot");
    setDirectory(\$REF,"$ROOT/ref");
    setDirectory(\$ALIASES,"$ROOT/aliases");

}

sub setDirectory {
    my ($pVar,$dir) = @_;
    $dir = absoluteReadLink($dir) if (-l $dir);
    confess("$dir: no such directory") unless (-d $dir);
    $$pVar = $dir;
    return $dir;
}    

# Return NNN and the highest-index $ROOT/str/species/meth/NNN
# directory that exists (NNN should be >= 1) and has the specified
# subdir if any. If there's no specified subdir and nothing is
# found, we create 000.

sub getCurrentIteration {
    my ($species,$filtMeth,$strSet,$subdirNeeded) = @_;
    my $dir = "$ROOT/match/$strSet/$species/$filtMeth";
    if (-d "$dir/p0") {
	return (0,$dir); # old style
    }
    my $pat = iterationSubdirPattern();
    my @subs = glob("$dir/$pat");
    if (@subs) {
	for (my $k=$#subs; $k>=0; $k--) {
	    my $sub = $subs[$k];
	    if (defined $subdirNeeded) {
		my @lst = glob("$sub/$subdirNeeded");
		next unless (@lst && -d $lst[0]);
	    }
	    if (-f "$sub/isFinalIteration") {
		$IS_FINAL_ITERATION = 'FINAL';
		$ARG_COUNT = 25;
	    } elsif (-f "$sub/isExtraIteration") {
		$IS_FINAL_ITERATION = 'EXTRA';
		# and Margarita should not run!
	    } else {
		$IS_FINAL_ITERATION = undef;
		# For Margarita; save time on non-final iterations.
		$ARG_COUNT = 3;
	    }
	    return (0+substr($sub,-3), $sub);
	}
    } 
    if (defined $subdirNeeded) {
	return (0);
    } else {
	my $sub = "$dir/000";
	mkdir_p($sub);
	return (0,$sub);
    }
}

sub iterationSubdirPattern {
    if (!defined $FIXED_ITERATION) {
	return '[0-9][0-9][0-9]';
    } elsif ($FIXED_ITERATION =~ /^\d+$/) {	
	my $pat = sprintf("%3d",$FIXED_ITERATION);
	$pat =~ tr/ /0/;
	return $pat;
    } else {
	return $FIXED_ITERATION;
    }
}

sub previousDirectory {
    my ($dir) = @_;
    my @comps = split(/\//,$dir);
    if ($comps[-1] > 0) {
	($comps[-1] = sprintf("%3d",$comps[-1]-1)) =~ tr/ /0/;
	my $prev = join("/",@comps);
	return $prev if (-d $prev);
    }
    return undef;
}


1;
