#!/usr/bin/perl

# Args: 
#  dir to look for nrco.fa files in
#  dir to look for sorted.gz files in
#  dir to write augmented sorted.gz files to.

use Utility;
use File::Basename;
use strict;

my ($nrDir,$srcDir,$tgtDir) = @ARGV;

my $lnr = length($nrDir)+1;
foreach my $nrco (glob("$nrDir/p?/*/nrco.fa")) {
    my $infix = substr(dirname($nrco),$lnr);
    my $src = "$srcDir/$infix/sorted.gz";
    my $tgt = "$tgtDir/$infix/sorted.gz";
    if (-s $nrco && -s $src && ! -f $tgt) {
	report("$tgt: creating");
	addNonRefConsensus($nrco,$src,$tgt);
	report();
    }
}

sub addNonRefConsensus {
    my ($nrco,$src,$tgt) = @_;
    my $nh = openToRead($nrco);
    <$nh>; # title line
    my $nrseq;
    while (<$nh>) {
	chomp;
	$nrseq .= $_;
    }
    my $fh = openToRead($src);
    my $gh = openToWrite($tgt);
    my $hdr = <$fh>;
    chomp $hdr;
    print $gh "$hdr ANCESTOR\n";
    my @strains = split(" ",substr($hdr,1));
    my $ns = scalar(@strains);
    my $len = 2*($ns+1); # expected length of input lines
    my $off=0;
    while (<$fh>) {
	if (/^[A-Z]/) {
	    chomp;
	    my $v = isPolymorphic($_); # return winner if poly, else undef
	    if ($v) {
		$v = substr($nrseq,$off,1);
	    } else {
		$v = substr($_,0,1); # monomorphic so ref is fine
	    }
	    print $gh $_;
	    my $dl = $len - length($_);
	    print $gh ' ' x $dl if ($dl>0);
	    print $gh "$v \n";
	    $off ++;
	} else {
	    print $gh $_;
	}
    }
}

sub isPolymorphic {
    my ($line) = @_;
    my $iMax = length($line)-2;
    my %sum;
    for (my $i=0; $i<=$iMax; $i+=2) {
	my $v = substr($_,$i,1);
	$v =~ tr/a-z/A-Z/;
	unless ($v eq ' ') {
	    my $qc = substr($_,$i+1,1);
	    $qc = '~' if ($qc eq ' ');
	    my $diff = ord($qc)-33;
	    $sum{$v} += $diff if ($diff >= 40);
	}
    }
    my @vals = sort {$sum{$b} <=> $sum{$a}} keys %sum;
    if (@vals > 1) {
	return $vals[0];
    } else {
	return undef;
    }
}
