#!/usr/bin/env perl

# Given a dir, print alignment of imputed_m, sorted and layout to aligned.gz

use Utility;
use strict;

foreach my $d (@ARGV) {
    alignISL($d);
}

sub alignISL {
    my ($d) = @_;

    my ($ih,$sh,$lh,$ah);
    
    if (-s "$d/imputed_y.gz") {
	$ih = openToRead("$d/imputed_y.gz");
	$sh = openToRead("$d/sorted_y.gz");
	$lh = openToRead("$d/layout_y.gz");
        $ah = openToWrite("$d/aligned_y.gz");
    } else {
	$ih = openToRead("$d/imputed_m.gz");
	$sh = openToRead("$d/sorted.gz");
	$lh = openToRead("$d/layout.gz");
        $ah = openToWrite("$d/aligned.gz");
    }

    my $hdr = <$ih>;
    <$sh>;
    print $ah $hdr;
    chomp $hdr;
    my @strains = split(" ",substr($hdr,1));
    my %strIdx;
    foreach my $i (0 .. $#strains) {
	$strIdx{$strains[$i]} = charForIndex($i);
    }
    my $wid = 2*(scalar(@strains)+1);
    
    my @using;
    while (<$lh>) {
	chomp;
	if (/^>/) {
	    unless (/^>REF/) {
		my ($str,$pos) = (/^>(\S+)-\S+ (\d+) /);
		$using[$pos] = $strIdx{$str};
	    }
	    printf($ah "%s;%s\n",$_,layoutKey(scalar(@strains),length($_)+1,\@using));
	} else {
	    for (my $pos=0; $pos <= $#using; $pos++) {
		if (defined $using[$pos] && substr($_,2*($pos+1),1) !~ /[^ ]/) {
		    $using[$pos] = undef;
		}
	    }
	    my $iLine = <$ih>;
	    my $sLine = <$sh>;
	    chomp $iLine;
	    chomp $sLine;
	    $iLine .= ' ' x ($wid-length($iLine));
	    $sLine .= ' ' x ($wid-2-length($sLine));
	    print $ah "$iLine | $sLine | $_\n";
	}
    }
}

sub charForIndex {
    my ($i) = @_;
    if ($i < 10) {
	return $i;
    } elsif ($i < 36) {
	return chr(ord($i)+55); # A-Z
    } elsif ($i < 52) {
	return chr(ord($i)+61); # a-z
    } else {
	return $i; # as number
    }
}

sub layoutKey {
    my ($nstr,$lsofar,$pUsing) = @_;
    my $lst;
    my $taken = 4*$nstr+10;
    my $diff = $taken-$lsofar;
    my $iMin = 0;
    while ($diff < 0) {
	$iMin ++;
	$diff += 2;
    }
    $lst = ' ' x $diff if ($diff > 0);
    for (my $i=$iMin; $i<=$#$pUsing; $i++) {
	my $key = $pUsing->[$i];
	$key .= ' ' x (2-length($key));
	$lst .= $key;
    }
    return $lst;
}
