#!/usr/bin/env perl

use Utility;
use strict;

my $root = "$ENV{SGR}/read";

my $QCHUNK = 5;  # quality chunk size
my $OCHUNK = 50; # offset chunk size

foreach my $s (@ARGV) {
    alignmentByQualAndPos("$root/$s/renamed.fastq",
			  "$root/$s/cere/ssahaSNP.all.gz",
			  "$root/$s/cere/abqp.txt");
}

sub alignmentByQualAndPos {
    my ($fqFile,$aliFile,$outFile) = @_;

    my %quals;
    my $fh = openToRead($fqFile);
    while (1) {
	my $hdr = <$fh>;
	last unless ($hdr);
	chomp $hdr;
	my ($name) = ($hdr =~ /^.(\S+[pq]1k) /);
	die("Bad header line: $hdr") unless ($name);
	<$fh>;
	<$fh>;
	my $qLine = <$fh>;
	chomp $qLine;
	$quals{$name} = $qLine;
    }
    
    my @count;
    $fh = openToRead($aliFile);
    my ($name,$off,$d,$qseq,$oldName,@status);
    while (<$fh>) {
	chomp;
	if (/^ALIGNMENT/) {
	    my @a = split;
	    unless ($name eq $a[2]) {
		processRead($quals{$name},\@status,\@count);
		@status = ();
		$name = $a[2];
	    }
	} elsif (/^Query/) {
	    my $off2;
	    (undef,$off,$qseq,$off2) = split;
	    $d = $off>$off2 ? -1 : 1;
	    $off --; # to make it zero-offset
	} elsif (/^Sbjct/) {
	    my (undef,undef,$sseq) = split;
	    my $iMax = length($qseq)-1;
	    my $offOrig = $off;
	    for (my $i=0; $i<=$iMax; $i++) {
		my $qv = substr($qseq,$i,1);
		if ($qv ne '-') {
		    my $sv = substr($sseq,$i,1);
		    my $st = $sv eq $qv ? 2 : 1;
		    $status[$off] = max($status[$off],$st);
		    $off += $d;
		}
	    }
	    $off -= $d;
	}
    }
    processRead($quals{$name},\@status,\@count);

    my $gh = openToWrite($outFile);
    foreach my $q (0 .. $#count) {
	my $pC = $count[$q];
	if (defined @$pC) {
	    foreach my $i (0 .. $#$pC) {
		my $pCI = $pC->[$i];
		if (defined @$pCI) {
		    printf($gh "%2d %4d %d %d %d\n",
			   $q*$QCHUNK,$i*$OCHUNK,$pCI->[0],$pCI->[1],$pCI->[2]);
		}
	    }
	}
    }
    $gh->close();

}

sub processRead {
    my ($quals,$pStatus,$pCount) = @_;
    foreach my $i (0 .. length($quals)-1) {
	my $q = int((ord(substr($quals,$i,1))-33)/$QCHUNK);
	my $ii = int($i/$OCHUNK);
	$pCount->[$q][$ii][$pStatus->[$i]] ++;
    }
}
