#!/usr/bin/env perl

use Utility;
use Getopt::Std;
use strict;

my %OPT;
getopts('t:o:c:s:',\%OPT);

my $top = $OPT{t} || "$ENV{SGRW}/release3";
my $out = $OPT{o}; # || "$top/tables/assembledBasesTable.txt";
my $chrPat = $OPT{c} || '*';
my @spList = split(/,/,$OPT{s} || "cere,para");

my $gh = openToWrite($out);
autoflush $gh 1;

printf($gh "%13s %4s","strain","spec");
printf($gh " %9s %9s %6s %6s\n",
       "aliRef","aliAll","nPlaced","nAmbig");

foreach my $sp (@spList) {
    assembledBasesTable($top,$sp,$gh);
}

# $refCount{$str} = no. of raw bases (not gaps!) from $str aligned to ref nucs
# $allCount{$str} = total no. of bases from that $str, from specs

sub assembledBasesTable {
    my ($top,$sp,$gh) = @_;
    my (%refCount,%allCount);
    my (%placedRead,%ambigRead);
    foreach my $f (glob("$top/$sp/match/$chrPat/layout.gz")) {
	report("$f: reading") if ($out);
	my $fh = openToRead($f);
	my @strainAt;
	while (<$fh>) {
	    chomp;
	    if (/^>(\S+) (\d+) (\S+)/) {
		my ($spec,$pos,$iter) = ($1,$2,$3);
		my ($read,$from,$to) = split(/:/,$spec);
		my ($str,$ext) = split(/-/,$read);
		if ($iter =~ /^\d+$/) {
		    $strainAt[2*($pos+1)] = $str;
		    $to =~ s/\|.*//;
		    my ($lo,$hi) = sort {$a <=> $b} ($from,$to);
		    $allCount{$str} += $hi-$lo+1;
		    $placedRead{$str}{$read} = 1;
		} else {
		    $strainAt[2*($pos+1)] = undef;
		    $ambigRead{$str}{$read} = 1;
		}
	    } elsif (/^[ACGT]/) {
		my $iLim = length($_);
		for (my $i=2; $i<$iLim; $i+=2) {
		    my $v = substr($_,$i,1);
		    if ($v =~ /[ACGTacgt]/) {
			my $str = $strainAt[$i];
			if ($str) {
			    $refCount{$str} ++;
			}
		    }
		}
	    }
	}
	report() if ($out);
    }
    foreach my $str (sort keys %refCount) {
	printf($gh "%13s %4s %9d %9d %6d %6d\n",
	       $str,$sp,$refCount{$str},$allCount{$str},
	       scalar(keys %{$placedRead{$str}}),
	       scalar(keys %{$ambigRead{$str}}));
    }
}
