#!/usr/bin/perl

# Input: output of crossStrandData.pl

use Utility;
use Getopt::Std;
use strict;

# Type of data (on a given strand) is one of:
#   0: single nucleotide, NQS OK
#   1: single nucleotide, NQS failed
#   2: multiple equal nucleotides
#   3: disagreeing nucleotides (single or multi)
#   4-7: as 0-3 but gaps are involved

my %OPT;
getopts('f:',\%OPT);

my $FAC = $OPT{f};

my ($hNuc,$hGap,$nNuc,$nGap)=0;
my @count;
while (<>) {
    chomp;
    my (%data,$what);
    ($data{F},$data{C},$what) = split(/ :: /,$_);
    my (%val,%qual,%type);
    foreach my $side (qw(F C)) {
	($val{$side},$qual{$side},$type{$side}) = decideConsensus($data{$side});
    }
    my $clash = $val{F} ne $val{C};
    if ($clash) {
	# Probability that the error was in F, given the qualities
	# (really we should map these...)
	my $dq = $qual{F}-$qual{C};
	my $pf = 1/(1+exp(0.1*$dq*log(10)));
	$count[$qual{F}][$type{F}][1] += $pf;
	$count[$qual{C}][$type{C}][1] += 1-$pf;
    }
    $count[$qual{F}][$type{F}][0] ++;
    $count[$qual{C}][$type{C}][0] ++;
}
print " Qu";
foreach my $i (0 .. 7) {
    print "   Tot$i   Dis$i   Qu$i";
}
print "\n";
foreach my $q (0 .. $#count) {
    printf("%3d",$q);
    foreach my $i (0 .. 7) {
	my ($n,$b);
	if (defined $count[$q][$i]) {
	    ($n,$b) = @{$count[$q][$i]};
	}
	if ($n-$b > 0 && $b > 0) {
	    my $pe = ($b+0.5)/($n+1);
	    my $qq = int(0.5-10*log($pe)/log(10));
	    printf(" %6d %6.1f %5.2f",$n,$b,$qq);
	} else {
	    printf(" %6d %6.1f %5s",0,0,'NA');
	}
    }
    print "\n";
}

sub decideConsensus {
    my ($chars) = @_;
    my @items = split(/ , /,$chars);
    my %qual;
    my $hasGap;
    my $multi;
    my $seenAmp;
    foreach my $item (@items) {
	my ($v,@quals1) = split(" ",$item);
	my @quals;
	foreach my $q (@quals1) {
	    if ($q eq '&') {
		$seenAmp = 1;
	    } else {
		$multi = 1 if (@quals);
		push @quals,$q;
	    }
	}
	my $isGap = ($v =~ /[^A-Z]/);
	$hasGap = 1 if ($isGap);
	$qual{$v} = consensusQuality($isGap,@quals);
    }
    my @vals = sort {$qual{$b} <=> $qual{$a}} keys %qual;
    my $q = $qual{$vals[0]};
    $q -= $qual{$vals[1]} if (@vals > 1);
    #print ">> $q : $chars\n" if ($q >= 36 && $q <= 40);
    $q = 127 if ($q > 127); # keep it manageable
    my $type;
    if (@vals > 1) {
	$type = 3;
    } elsif ($multi) {
	$type = 2;
    } elsif ($seenAmp) {
	$type = 1;
    } else {
	$type = 0;
    }
    return ($vals[0],int(0.5+$q),$hasGap*4 + $type);
}

sub consensusQuality {
    my ($isGap,@quals) = @_;
    @quals = sort {$b <=> $a} @quals;
    my $sum = 0;
    my $fac = 1;
    foreach my $q (@quals) {
	$sum += $q*$fac;
	last unless ($FAC);
	$fac *= $FAC;
    }
    # See 0124 ...
    my $q = 55 * (1 - exp(-0.029 * $sum));
    return $q;
}
