#!/usr/bin/env perl

# Input: ssaha2 output for contigs vs ref genome (e.g Yong's contigs vs cere)

use Utility;
use strict;

my ($sc,%trans,@buf);
foreach my $scFile (qw(ref4600_shred_lon.supercontigs kell_notcov.supercontigs)) {
    unless (-s $scFile) {
	print STDERR "Warning: missing or empty: $scFile\n";
	next;
    }
    my $fh = openToRead($scFile);
    while (<$fh>) {
	chomp;
	my ($key,$val) = split;
	if ($key eq 'supercontig') {
	    my $n = scalar(@buf);
	    foreach my $i (0 .. $#buf) {
		$trans{$buf[$i]} = 
		    $n == 1 
		    ? sprintf("S%s=C%s",$sc,substr($buf[$i],-3))
		    : sprintf("S%s.%d/%d=C%s",$sc,$i+1,$n,substr($buf[$i],-3));
	    }
	    $sc = substr($val,-3);
	    @buf = ();
	} elsif ($key eq 'contig') {
	    push @buf,$val;
	}
    }
    my $n = scalar(@buf);
    foreach my $i (0 .. $#buf) {
	$trans{$buf[$i]} = sprintf("%s.%d.%d",$sc,$i+1,$n);
    }
}

my %alis;
my %contigLength;
while (<>) {
    chomp;
    if (/^ALIGNMENT /) {
	# Should happen for ssaha2 output ...
	my (undef,$score,$cont,$chr,$qLo,$qHi,$sLo,$sHi,$fc,@rest) = split;
	push @{$alis{$cont}{"$chr $fc"}},[$qLo,$qHi,$sLo,$sHi,$score];
	$contigLength{$cont} = $rest[-1];
    }
}

my $gh = openToPipe("sort +4 | pruneSyntenicPaths.pl");
foreach my $cont (sort keys %alis) {
    my $pH = $alis{$cont};
    my @bestCands;
    foreach my $chrFC (sort keys %$pH) {
	my ($score,@quads) = bestQuads($pH->{$chrFC},substr($chrFC,-1,1) eq 'C');
	push @bestCands,[$score,$chrFC,@quads];
    }
    my @sorted = sort {$b->[0] <=> $a->[0]} @bestCands;
    my $sf = $sorted[0][0] - $sorted[1][0];
    my ($score,$chrFC,@quads) = @{$sorted[0]};
    my ($chr,$fc) = split(" ",$chrFC);
    my $lenCov = 0;
    foreach my $i (0 .. $#quads) {
	$lenCov += $quads[$i][1]-$quads[$i][0]+1;
    }
    my $refLo = ($fc eq 'F') ? $quads[0][2] : $quads[-1][3];
    my $refHi = ($fc eq 'F') ? $quads[-1][3] : $quads[0][2];
    printf($gh "%14s %7d %7d %1s %5s %7d %7d %7d %7d %7d %7d %7d %2d :",
	   %trans ? ($trans{$cont} || die("No trans for $cont")) : $cont,
	   $quads[0][0],$quads[-1][1],
	   $fc,$chr,$refLo,$refHi,
	   $score,$sf,$lenCov,$quads[0][0]-1,$contigLength{$cont}-$quads[-1][1],
	   scalar(@quads));
    foreach my $i (0 .. $#quads) {
	foreach my $n (@{$quads[$i]}[0..3]) {
	    printf($gh " %d",$n);
	}
	print $gh ' :' if ($i<$#quads);
    }
    print $gh "\n";
}

sub bestQuads {
    my ($pQuadsIn,$isC) = @_;
    my @quadsIn = sort {$a->[0] <=> $b->[0]} @$pQuadsIn;
    my @cands;
    foreach my $i (0 .. $#quadsIn) {
	my @path = ($quadsIn[$i]);
	my $score = $path[0][4];
	foreach my $j ($i+1 .. $#quadsIn) {
	    my $pQ = $quadsIn[$j];
	    my $dq = $pQ->[0]-$path[-1][1]-1;
	    my $ds = $isC ? $path[-1][3]-$pQ->[2]-1 : $pQ->[2]-$path[-1][3]-1;
	    my $pen;
	    if ($dq > 0) {
		if ($ds > 0) {
		    $pen = max($dq*0.1,$ds*0.03);
		} else {
		    $pen = $dq-$ds;
		}
	    } elsif ($ds > 0) {
		$pen = $ds+$dq;
	    } else {
		$pen = max(-$dq,-$ds);
	    }
	    my $delta = $pQ->[4] - $pen;
	    if ($delta > 0) {
		push @path,$pQ;
		$score += $delta;
	    }
	}
	push @cands,[$score,@path];
    }
    my @sorted = sort {$b->[0] <=> $a->[0]} @cands;
    return @{$sorted[0]};
}
