#!/usr/bin/env perl

# Attempt to convert Blast output to something like Ssaha2 output.

use strict;

my $MIN_SCORE = 100;

my ($state,
    $query,$sbjct,$score,
    $qlo,$qhi,$slo,$shi,$qlen,$alen,@match,$percid,$fc);
while (<>) {
    chomp;
    if (/^Query=/) {
	(undef,$query) = split;
    } elsif (/^Sequences producing/) {
	$state = 1;
    } elsif (/^>/) {
	$sbjct = substr($_,1);
	$state = 2;
    } elsif (/^ Score = /) {
	if ($state == 2) {
	    my @a = split;
	    if ($a[2] >= $MIN_SCORE) {
		$state = 3;
		$score = $a[2];
	    }
	} elsif ($state == 4) {
	    $alen = $fc eq 'F' ? $qhi-$qlo+1 : $qlo-$qhi+1;
	    printf("ALIGNMENT %d %s %s %d %d %d %d %s %d %5.2f %d\n\n",
		   $score,$query,$sbjct,$qlo,$qhi,$slo,$shi,$fc,$alen,$percid,$qlen);
	    print join("\n",@match) . "\n";
	    $state = 2;
	}
    } elsif (/ letters; record/) {
	($qlen) = split;
	$qlen =~ s/[^0-9]//g;
    } elsif (/^ Identities = / && $state == 3) {
	my @a = split;
	if ($a[3] =~ /^\((\d+)%\),$/) {
	    $percid = $1;
	}
	$fc = $a[10] eq 'Plus' ? 'F' : 'C';
	($qlo,$qhi,$slo,$shi,@match) = undef;
	$state = 4;
    } elsif (/^Query: / && $state == 4) {
	my (undef,$q1,$qseq,$q2) = split;
	# qlo is "q location matching slo", ditto for hi, so reversed on a C match ...
	$qlo = $q1 unless (defined $qlo);
	$qhi = $q2;
	s/:/ /;
	push @match,$_;
	my $qLine = $_;
	<>; # comparison line, which we'll redo
	chomp (my $sLine = <>);
	$sLine =~ s/:/ /;
	my (undef,$s1,undef,$s2) = split(" ",$sLine);
	$slo = $s1 unless (defined $slo);
	$shi = $s2;
	<>; # blank line
	my $cLine = ' ' x length($qLine);
	for (my $i=0; $i<length($qLine); $i++) {
	    my $qc = substr($qLine,$i,1);
	    if ($qc eq '-') {
		substr($cLine,$i,1) = '-';
	    } elsif ($qc =~ /[ACGT]/) {
		my $sc = substr($sLine,$i,1);
		next if ($sc eq $qc);
		if ($sc eq '-') {
		    substr($cLine,$i,1) = '-';
		} elsif ($sc =~ /[ACGT]/) {
		    substr($cLine,$i,1) = (($qc =~ /[AG]/) eq ($sc =~ /[AG]/)) ? 'i' : 'v';
		}
	    }
	}
	$cLine =~ s/ +$//;
	push @match,$cLine,$sLine,'';
    }
}

	    
