#!/usr/bin/env perl

# Input: output of blastn or ssaha2

use strict;
my $idx = 0;
my ($qchr,$schr,$fc);
my ($qseq,$sseq,$qleft,$qright,$sleft,$sright);
while (<>) {
    #print;
    chomp;
    if (/^ALIGNMENT/) {
	printSegments($idx,$qchr,$schr,$fc,$qseq,$sseq,$qleft,$qright,$sleft,$sright);
	(undef,undef,$qchr,$schr,undef,undef,undef,undef,$fc) = split;
	$idx ++;
	($qseq,$sseq,$qleft,$qright,$sleft,$sright) = undef;
    } elsif (/^Query=/) {
	(undef,$qchr) = split;
    } elsif (/^>(.*)/) {
	$schr = $1;
    } elsif (/^  Minus Strand/) {
	$fc = 'C';
    } elsif (/^  Plus Strand/) {
	$fc = 'F';
    } elsif (/^ Score = /) {
	printSegments($idx,$qchr,$schr,$fc,$qseq,$sseq,$qleft,$qright,$sleft,$sright);
	$idx ++;
	($qseq,$sseq,$qleft,$qright,$sleft,$sright) = undef;
    } elsif (/^Query/) {
	my (undef,$q1,$qs,$q2) = split;
	$qseq .= $qs;
	$qleft = $q1 unless (defined $qleft);
	$qright = $q2;
    } elsif (/^Sbjct/) {
	my (undef,$s1,$ss,$s2) = split;
	$sseq .= $ss;
	$sleft = $s1 unless (defined $sleft);
	$sright = $s2;
    }
}
printSegments($idx,$qchr,$schr,$fc,$qseq,$sseq,$qleft,$qright,$sleft,$sright);

sub printSegments {
    my ($idx,$qchr,$schr,$fc,$qseq,$sseq,$qleft,$qright,$sleft,$sright) = @_;
    return unless ($qseq);
    my $dq = $qleft < $qright ? 1 : -1;
    my $fcq = $dq > 0 ? 'F' : 'C';
    my $ds = $sleft < $sright ? 1 : -1;
    my $fcs = $ds > 0 ? 'F' : 'C';
    my $iq1 = $qleft;
    my $is1 = $sleft;
    my $off1 = 0; # posn in qseq and sseq
    my $lim = length($qseq);
    while ($off1 < $lim) {
	my $off2 = $off1+1;
	while ($off2 < $lim && substr($qseq,$off2,1) ne '-' && substr($sseq,$off2,1) ne '-') {
	    $off2++;
	}
	my $segLen = $off2-$off1;
	printf("%d %s %d %s %d %s %d %s %d\n",$idx,
	       $qchr,$iq1,$fcq,$segLen,
	       $schr,$is1,$fcs,$segLen);
	$iq1 += $dq*$segLen;
	$is1 += $ds*$segLen;
	last if ($off2 >= $lim);
	# If we're not at the end, one of the sequences must have a gap.
	$off1 = $off2;
	$off2 = $off1+1;
	if (substr($qseq,$off1,1) eq '-') {
	    while ($off2 < $lim && substr($qseq,$off2,1) eq '-') {
		$off2++;
	    }
	    $segLen = $off2-$off1;
	    printf("%d %s %d %s %d %s %d %s %d\n",$idx,
		   $qchr,$iq1,$fcq,0,
		   $schr,$is1,$fcs,$segLen);
	    $is1 += $ds*$segLen;
	} else {
	    while ($off2 < $lim && substr($sseq,$off2,1) eq '-') {
		$off2++;
	    }
	    $segLen = $off2-$off1;
	    printf("%d %s %d %s %d %s %d %s %d\n",$idx,
		   $qchr,$iq1,$fcq,$segLen,
		   $schr,$is1,$fcs,0);
	    $iq1 += $dq*$segLen;
	}
	last if ($off2 >= $lim);
	$off1 = $off2;
    }
}
