#!/usr/bin/perl

use Utility;
use File::Basename;
use strict;

# Args: species filtMeth SET iter

if (@ARGV == 4) {
    checkArgsFiles(@ARGV);
} else {
    die("Usage: $0 species filtMeth SET NNN\n");
}

sub checkArgsFiles {
    my ($sp,$meth,$set,$iter) = @_;
    my $top = "$ENV{SGR}/match/$set/$sp/$meth/$iter";
    unless (-d $top) {
	print "$top: not a directory\n";
	return;
    }
    my $cmd = "sgrp.pl -bf marg $sp $meth $set";
    foreach my $d (glob("$top/p?/*")) {
	my $af = "$d/args_bl";
	unless (checkArgsFile($af)) {
	    print "rm $af\n" if (-f $af);
	    my $n = substr(basename(dirname($d)),1);
	    foreach my $md (glob("$d/m???")) {
		checkArgsFiles1($md,"$cmd $n");
	    }
	}
    }
}

sub checkArgsFiles1 {
    my ($d,$cmd) = @_;
    my $a = "$d/args_bl";
    return if checkArgsFile($a);
    if (-s $a) {
	print "rm $a\n";
    }
    foreach my $k (0 .. 24) {
	my $k3 = sprintf("%3d",$k);
	$k3 =~ tr/ /0/;
	my $f = "$d/arg${k3}_bl";
	next if checkArgsFile($f);
	my $tf = "$f.touch";
	if (-f $tf && fileAgeInHours($tf) < 4) {
	    print "echo $tf exists and is less than 4 hours old\n";
	    next;
	}
	my $mf = "$d/arg${k3}.mgr";
	my $sz = -s $mf;
	if ($sz <= 60) {
	    print "echo $mf: ";
	    if (defined $sz) {
		print "size is $sz\n";
	    } else {
		print "missing\n";
	    }
	}
	foreach my $ff (glob("$d/arg${k3}*")) {
	    print "rm $ff\n";
	}
	print "$cmd\n";
    }
}

sub fileAgeInHours {
    my ($f) = @_;
    my @a = stat($f);
    my $now = time();
    return ($now-$a[9])/3600;
}

# Return 1 if OK, 0 if not.
sub checkArgsFile {
    my ($f,$rep) = @_;
    my $mf = sprintf("%s/mgr_in",dirname($f));
    foreach my $ff ($f,$mf) {
	unless (-s $ff) {
	    print "echo $ff: missing or empty\n";
	    return 0;
	}
    }
    my $mh = openToRead($mf);
    chomp ($_ = <$mh>);
    my (undef,$nm) = split;
    my @prevEnd;
    my $fh = openToRead($f);
    my $line = 0;
    while (<$fh>) {
	chomp;
	$line ++;
	my ($lo,$idx,$hi) = split;
	unless (defined $prevEnd[$idx]) {
	    $prevEnd[$idx] = -1;
	}
	unless ($lo == $prevEnd[$idx]+1) {
	    print "echo $f: line $line: $lo $idx $hi, but prevEnd is $prevEnd[$idx]\n";
	    return 0;
	}
	$prevEnd[$idx] = $hi;
    }
    foreach my $i (0 .. $#prevEnd) {
	unless ($prevEnd[$i] == $nm-1) {
	    print "echo $f: marker count is $nm, but ARG $i ends at $prevEnd[$i]\n";
	    return 0;
	}
    }
    print "echo $f: OK\n" if ($rep);
    return 1;
}
