#!/usr/bin/env perl

use Utility;
use File::Basename;
use strict;

autoflush STDOUT 1;

my $DMC_QUALITY = 0;
my $HENG40 = 1;

printf("%s %s %2s %2s %2s %2s %2s %2s %3s %2s %8s %6s",
       'R','R','Ps','Ps','Q1','Q1','Q2','Q2','D','D','Bases','Errors');
printf(" %4s %4s %4s %4s %6s %6s %6s %6s %s\n",
       'Rows','PosR','MuQ1','MuQ2','MuD','RmsD','MuSim','RmsSim','Lane');

foreach my $f (glob("$ENV{G1K}/v1/*-*/NA?????/lib*/lane*/qualmap.txt")) {
    checkQualmap($f);
}

sub checkQualmap {
    my ($f) = @_;
    my $fh = openToRead($f);
    my $id = substr(dirname($f),length($ENV{G1K})+1);
    my ($pMin,$pMax,$q1Min,$q1Max,@mat);
    while (<$fh>) {
	chomp;
	my @a = split;
	push @mat,\@a;
	$pMin = min($pMin,$a[1]);
	$pMax = max($pMax,$a[1]);
	$q1Min = min($q1Min,$a[2]);
	$q1Max = max($q1Max,$a[2]);
    }
    my ($rMin,$rMax,$q2Min,$q2Max,$dMin,$dMax,$nSum,$mSum,$q1Sum,$q2Sum);
    my ($nRows,$nPosRows,$ssq,$ssqSim,$qSimSum);
    foreach my $pA (@mat) {
	my ($r,$p,$q1,$q2,$n,$m) = @$pA;
	$q2 = dmcQuality($n,$m) if ($DMC_QUALITY);
	my $pe1 = min(0.5,exp(-log(10)*0.1*$q1));
	$pMin = min($p,$pMin);
	$pMax = max($p,$pMax);
	next if ($p == $pMin || $p == $pMax);
	if ($m > $n) {
	    print STDERR "WARNING: n>m in $f: @$pA\n";
	    printf("%s %s %2s %2s %2s %2s %2s %2s %3s %2s %8s %6s",
		   '*','*','**','**','**','**','**','**','*','* ','*****','******');
	    printf(" %4s %4s %4s %4s %6s %6s %6s %6s %s\n",
		   '****','****','****','****','***','*****','***','*****',$id);
	    return;
	}
	$rMin = min($r,$rMin);
	$rMax = max($r,$rMax);
	$q2Min = min($q2,$q2Min);
	$q2Max = max($q2,$q2Max);
	$nRows ++;
	$nPosRows ++ if ($n > 0);
	if ($n>0) {

	    my $mSim = simulateCount($pe1,$n);
	    my $qSim;
	    if ($DMC_QUALITY) {
		$qSim = dmcQuality($n,$mSim);
	    } else {
		my $qm = $HENG40 ? 40 : $q1Max;
		my $peSim = ($mSim+0.1/($qm+1))/($n+0.1);
		$qSim = int(0.5-10*log($peSim)/log(10));
	    }
	    if ($HENG40) {
		my $pe2 = ($m+1)/($n+2);
		$q2 = int(0.5-10*log($pe2)/log(10));
	    }

	    $nSum += $n;
	    $mSum += $m;

	    my $d = $q2-$q1;
	    $ssq += $d*$d*$n;
	    $dMin = min($d,$dMin);
	    $dMax = max($d,$dMax);

	    my $dSim = $qSim-$q1;
	    $ssqSim += $dSim*$dSim*$n;

	    $q1Sum += $q1*$n;
	    $q2Sum += $q2*$n;
	    $qSimSum += $qSim*$n;
	}
    }
    if ($nSum == 0) {
	printf("%s %s %2s %2s %2s %2s %2s %2s %3s %2s %8s %6s",
	       '0','0','00','00','00','00','00','00','0','0 ','00000','000000');
	printf(" %4s %4s %4s %4s %6s %6s %6s %6s %s\n",
	       '0000','0000','0000','0000','000','00000','000','00000',$id);
    } else {
	my $rmse = sqrt($ssq/$nSum);
	my $q1Mean = $q1Sum/$nSum;
	my $q2Mean = $q2Sum/$nSum;
	my $qSimMean = $qSimSum/$nSum;
	my $rmseSim = sqrt($ssqSim/$nSum);
	printf("%d %d %2d %2d %2d %2d %2d %2d %3d %2d %8d %6d",
	       $rMin,$rMax,$pMin,$pMax,$q1Min,$q1Max,$q2Min,$q2Max,$dMin,$dMax,$nSum,$mSum);
	printf(" %4d %4d %4.1f %4.1f %6.2f %6.3f %6.2f %6.3f %s\n",
	       $nRows,$nPosRows,$q1Mean,$q2Mean,$q2Mean-$q1Mean,$rmse,$qSimMean-$q1Mean,$rmseSim,$id);
    }
}

sub simulateCount {
    my ($p,$n) = @_;
    my $c = 0;
    if ($n <= 20) {
	foreach my $i (1 .. $n) {
	    $c++ if (rand() < $p);
	}
    } else {
	my $mu = $p*$n;
	my $sd = sqrt($p*(1-$p)*$n);
	if (3*$sd < $mu) {
	    # normal approximation
	    my $r = -log(rand());
	    my $th = rand()*3.141592652*2;
	    my $z = $r*cos($th);
	    my $x = $mu+$z*$sd;
	    $c = int(0.5+$x);
	} else {
	    # Poisson approximation
	    my $mass = rand();
	    my $p = exp(-$mu);
	    while (1) {
		if ($c) {
		    $p *= $mu/$c;
		}
		$mass -= $p;
		last if ($mass < 0);
		$c ++;
	    }
	}
    }
    $c = max(0,min($c,$n));
    return $c;
}

sub dmcQuality {
    my ($n,$m) = @_;
    my $qMax = 40;
    return $qMax/2 if ($n == 0 || $m > $n);
    my $lch = 0;
    foreach my $i (0 .. $m-1) {
	$lch += log(($n-$i)/($m-$i));
    }
    my ($top,$bot);
    foreach my $q (0 .. $qMax) {
	my $p = min(0.5,exp(-log(10)*0.1*$q));
	my $lpm = $m*log($p)+($n-$m)*log(1-$p);
	my $lp = $lpm+$lch;
	my $post = exp($lp);
	$top += $post*$q;
	$bot += $post;
    }
    return $qMax/2 if ($bot == 0);
    my $q = int(0.5+$top/$bot);
    #print "$top $bot $q\n";
    return $q;
}
