#!/usr/bin/perl

use Utility;
use File::Basename;
use strict;

my ($QUICK,@files);
if (@ARGV) {
    $QUICK = 0;
    @files = @ARGV;
} else {
    $QUICK = 1;
    @files = glob("v1/*/*/*/*/recalibrated.fastq.gz");
}

chdir($ENV{G1K});

LOOP:
    foreach my $tgt (@files) {
    if (-s "$tgt.berr") {
	my $eh = new FileHandle("$tgt.berr");
	while (<$eh>) {
	    chomp;
	    unless (/^$/ || /^(Convert|Run|Break|rm:) /) {
                print "$tgt.berr:$_\n";
		print "rm ${tgt}*\n";
		next LOOP;
	    }
	}
    }
    unless (-f "$tgt.touch") {
	my $srcD = dirname($tgt);
	$srcD =~ s/v1/DATA/;
	my $src;
	foreach my $f (glob("$srcD/*")) {
	    if (-l $f) {
		$src = readlink($f);
		last;
	    }
	}
	if ($src) {
	    if ($QUICK) {
		my $sz1 = -s $src;
		my $sz2 = -s $tgt;
		printf("%8.6f %10d %10d %s\n",
		       $sz2/($sz1 || 1),$sz1,$sz2,$tgt);
	    } else {
		compareFastqFiles($src,$tgt);
	    }
	} else {
	    print "Cannot find source for $tgt\n";
	}
    }
}

sub compareFastqFiles {
    my ($f1,$f2) = @_;
    my $fh1 = openToRead($f1);
    my $fh2 = openToRead($f2);
    my $n=0;
    my @count;
    while (1) {
	my (@a,@b);
	foreach my $i (0 .. 3) {
	    my $line1 = <$fh1>;
	    my $line2 = <$fh2>;
	    if ($line1 && $line2) {
		push @a,$line1;
		push @b,$line2;
	    } else {
		print "Truncated: $f2\n" if ($line1);
		print "Extended: $f2\n" if ($line2);
		return;
	    }
	}
	foreach my $i (0 .. 2) {
	    $n++;
	    unless ($a[$i] eq $b[$i]) {
		print "Mismatch at line $n:\n";
		print "$f1:\n$a[$i]";
		print "$f2:\n$b[$i]";
		return;
	    }
	}
	$n++; # for a[3]
	# Only do this every 10th read:
	if ($n%40 == 0) {
	    chomp $b[3];
	    foreach my $v (split("",$b[3])) {
		$count[ord($v)-33] ++;
	    }
	}
    }
    my ($min,$s0,$s1,$s2);
    my $max = $#count;
    foreach my $i (0 .. $max) {
	$min = $i unless (defined $min);
	my $c = $count[$i];
	$s0 += $c;
	$s1 += $i*$c;
	$s2 += $i*$i*$c;
    }
    unless ($s0) {
	print "No qualities found in $f2\n";
	return;
    }
    my $mu = $s1/$s0;
    my $sd = sqrt($s2/$n-$mu);
    printf("%2d %2d %6.3f %6.3f %s\n",$min,$max,$mu,$sd,$f2);
}
