#!/usr/bin/env perl

use Utility;
use File::Basename;
use Cwd;
use strict;

# Run from NNN dir.

my @comps = split(/\//,cwd());
my $sp = $comps[-3];

my $refDir = "../../../../../ref/$sp";

my $pH = readFastaFile("$refDir/genome.fa");
foreach my $dir (glob("p?/*")) {
    my $seq = $pH->{basename($dir)} || die("No sequence found for $dir");
    checkRefSequences($dir,$seq);
}

sub checkRefSequences {
    my ($d,$seq) = @_;
    foreach my $f ("$d/sorted.gz",glob("$d/[si]*_[myz].gz")) {
	if (-f "$f.touch") {
	    print "$f: touch file still exists\n";
	} else {
	    checkRefSequence($f,$seq);
	}
    }
}

sub checkRefSequence {
    my ($f,$seq) = @_;
    my $fh = openToRead($f);
    <$fh>; # header
    my $off = 0;
    my $line = 1;
    my $errs = 0;
    while (<$fh>) {
	$line ++;
	if (/^[A-Z]/) {
	    my $r = substr($seq,$off,1);
	    $off++;
	    unless ($r eq substr($_,0,1)) {
		print "$f line $line offset $off: expected $r, found $_";
		$errs ++;
		if ($errs == 10) {
		    print "$f: giving up\n";
		    last;
		}
	    }
	}
    }
    if ($errs == 0) {
	print "$f: OK\n";
    } elsif ($errs < 10) {
	print "$f: $errs errors\n";
    }
}
