#!/usr/bin/env perl

use Utility;
use File::Basename;
use strict;

autoflush STDOUT 1;

chdir($ENV{G1K});
foreach my $rlFile (glob("DATA/*/NA*/lib*/lane*/*lengths")) {
    my $qmFile = dirname($rlFile) . "/qualmapBayesian.txt";
    $qmFile =~ s/DATA/v1/;
    my $d = dirname($qmFile);
    next unless (-d $d); # without comment
    unless (-s $qmFile) {
	print "$qmFile: missing or empty\n";
	next;
    }
    my $fh = openToRead($rlFile);
    chomp ($_ = <$fh>);
    my ($len1,$len2) = split;
    $fh = openToRead($qmFile);
    my @seen;
    while (<$fh>) {
	my ($r,$p) = split;
	$seen[$r][$p] = 1;
    }
    if (!@seen) {
	print "$qmFile: no data\n";
    } elsif ($len2 == 0) {
	# Single end
	if ($#seen > 0) {
	    print "$qmFile: paired but should be single\n";
	}
	unless ($#{$seen[0]} == $len1) {
	    print "$qmFile: single, but goes up to $#{$seen[0]} not $len1. ";
	    checkMap($d);
	}
    } elsif ($seen[0]) {
	print "$qmFile: single but should be paired\n";
    } else {
	foreach my $r (1,2) {
	    my $len = $r == 1 ? $len1 : $len2;
	    if ($seen[$r]) {
		unless ($#{$seen[$r]} == $len) {
		    print "$qmFile: read $r goes up to $#{$seen[$r]} not $len. ";
		    checkMap($d);
		    #last;
		}
	    } else {
		print "$qmFile: nothing for read $r. ";
		checkMap($d);
		#last;
	    }
	}
    }
}

sub checkMap {
    my ($d) = @_;
    my $sz = getSize("$d/sample/aln.map");
    print " $sz from";
    my @sz;
    foreach my $r (0..2) {
	printf(" %s",getSize("$d/sample/aln.$r.map"));
    }
    print "\n";
}

sub getSize {
    my ($f) = @_;
    return '-' unless (-e $f);
    return -s $f;
}

