#!/usr/bin/perl

# Args: srcDir tgtDir e.g. methc91 methc91c (c means "cleared")

use Utility;
use File::Basename;
use strict;

my ($srcD,$tgtD) = @ARGV;

my $pDiff = getDifficultRegions("$ENV{SGR}/ref/cere/ALL.gff");

my $len = length($srcD);
foreach my $f (glob("$srcD/p?/c*/sorted.gz")) {
    my $g = $tgtD . substr($f,$len);
    if (-s $g) {
	report("$g: already exists",1);
    } else {
	report("$g: creating");
	my $chr = basename(dirname($f));
	clearDifficultRegions($f,$pDiff->{$chr},$g);
	report();
    }
}

sub getDifficultRegions {
    my ($f) = @_;
    my $fh = openToRead($f);
    my %diff;
    while (<$fh>) {
	chomp;
	my ($chr,undef,$type,$lo,$hi) = split;
	$chr = 'chrm' if ($chr eq 'chrM');
	if ($type eq 'chromosome' && $chr ne 'chrm') {
	    push @{$diff{$chr}},[1,30000];
	    push @{$diff{$chr}},[$hi-29999,$hi];
	} elsif ($type eq 'repeat_region' || $type eq 'transposable_element') {
	    push @{$diff{$chr}},[$lo,$hi];
	}
    }
    foreach my $chr (keys %diff) {
	$diff{$chr} = [sort {$a->[0] <=> $b->[0] || $a->[1] <=> $b->[1]} @{$diff{$chr}}];
    }
    return \%diff;
}

sub clearDifficultRegions {
    my ($f,$pDiff,$g) = @_;
    my $fh = openToRead($f);
    my $gh = openToWrite($g);
    my $hdr = <$fh>;
    print $gh $hdr;
    my $off = 0;
    while (<$fh>) {
	$off ++ if (/^[A-Z]/);
	if (inDifficultRegion($off,$pDiff)) {
	    substr($_,2) =~ s/./ /g;
	}
	print $gh $_;
    }
}

sub inDifficultRegion {
    my ($off,$pDiff) = @_;
    return 0 unless ($pDiff);
    # Remove regions to the left of where we now are
    while (@$pDiff && $pDiff->[0][1] < $off) {
	shift @$pDiff;
    }
    foreach my $pA (@$pDiff) {
	return 0 if ($pA->[0] > $off);
	return 1 if ($pA->[1] >= $off);
    }
    return 0;
}

