#!/usr/bin/perl

use strict;

my %sequence;
my @ids;
<>; # header
while (<>) {
    chomp;
    if (/^\S/) {
	my ($id,$seq) = split;
	unless ($sequence{$id}) {
	    push @ids,$id;
	}
	$sequence{$id} .= $seq;
    }
}

# Numericalize gaps in all sequences ...

my %gapExtent;
foreach my $seq (values %sequence) {
    my $kMax = length($seq)-1;
    my $lo = 0;
  LOOP:
    while ($lo <= $kMax) {
	while (substr($seq,$lo,1) ne '-') {
	    $lo++;
	    last LOOP if ($lo > $kMax);
	}
	my $hi = $lo;
	while ($hi <= $kMax && substr($seq,$hi+1,1) eq '-') {
	    $hi ++;
	}
	$gapExtent{$lo}{$hi} = 1;
	$lo = $hi+1;
    }
}

# Assign codes 1 to 9 for gap extents
my @gaps;
foreach my $lo (keys %gapExtent) {
    foreach my $hi (keys %{$gapExtent{$lo}}) {
	push @gaps,[$lo,$hi];
    }
}
@gaps = sort {$a->[0] <=> $b->[0] || $a->[1] <=> $b->[1]} @gaps;
my @usedUntil;
my %gapCode;
foreach my $pA (@gaps) {
    my ($lo,$hi) = @$pA;
    foreach my $code (1 .. 9) {
	unless (defined $usedUntil[$code] && $usedUntil[$code] >= $lo) {
	    $gapCode{$lo}{$hi} = $code;
	    $usedUntil[$code] = $hi;
	    last;
	}
    }
}

# Apply to sequences
foreach my $seq (values %sequence) {
    #printf("1 Len %d\n",length($seq));
    my $lo = 0;
    my $kMax = length($seq)-1;
  LOOP:
    while ($lo <= $kMax) {
	while (substr($seq,$lo,1) ne '-') {
	    $lo++;
	    last LOOP if ($lo > $kMax);
	}
	my $hi = $lo;
	while ($hi <= $kMax && substr($seq,$hi+1,1) eq '-') {
	    $hi ++;
	}
	if (defined $gapCode{$lo}{$hi}) {
	    my $code = $gapCode{$lo}{$hi};
	    substr($seq,$lo,$hi-$lo+1) = $code x ($hi-$lo+1);
	} else {
	    die("No code for $lo $hi");
	}
    }
    #printf("2 Len %d\n",length($seq));
}

# We don't print the first ID, as that will have to stand for "ref" for
# consistency with yeast format ...

printf(">%s\n",join(" ",@ids[1 .. $#ids]));
my $kMax = length($sequence{$ids[0]})-1;
for (my $k=0; $k<=$kMax; $k++) {
    foreach my $id (@ids) {
	print substr($sequence{$id},$k,1) . ' ';
    }
    print "\n";
}

