#!/usr/bin/perl

use Utility;
use strict;

autoflush STDOUT 1;

my $QTH = 40;
my $TH = chr(33+$QTH);
my $PE = exp(log(0.1)*$QTH/10);

# Arg: layout.gz file

my $fh = openToRead($ARGV[0]);
my (@read,@strain,%nd);
while (<$fh>) {
    chomp;
    if (/^>REF/) {
	print "$_\n";
	my %active;
	for (my $i=0; $i<=$#read; $i+=2) {
	    $active{"$strain[$i] $read[$i]"} = 1 if ($read[$i]);
	}
	printUnlessActive(\%nd,\%active);
    } elsif (/^>(\S+) (\d+)/) {
	my ($sr,$p) = ($1,$2);
	$p *= 2; # so it's a column
	my ($s,$r) = split(/-/,$sr);
	$read[$p] = $r;
	$strain[$p] = $s;
    } else {
	s/^..//; # discard ref
	my $iMax = length($_)-4;
	my $jMax = $iMax+2;
	for (my $i=0;$i<=$iMax;$i+=2) {
	    my $si = $strain[$i];
	    my $vi = substr($_,$i,1);
	    if ($vi eq ' ') {
		$strain[$i] = $read[$i] = undef;
		next;
	    }
	    $vi =~ tr/a-z/A-Z/;
	    next unless ($vi =~ /[ACGT]/);
	    for (my $j=$i+2; $j<=$jMax; $j+=2) {
		my $sj = $strain[$j];
		next unless ($sj eq $si);
		my $vj = substr($_,$j,1);
		$vj =~ tr/a-z/A-Z/;
		next unless ($vj =~ /[ACGT]/);
		if (substr($_,$i+1,1) ge $TH && substr($_,$j+1,1) ge $TH) {
		    $nd{$strain[$i]}{$read[$i]}{$read[$j]}[0] ++ unless ($vi eq $vj);
		    $nd{$strain[$i]}{$read[$i]}{$read[$j]}[1] ++;
		}
	    }
	}
	if ($jMax < $#strain) {
	    for (my $j=$jMax+2; $j<=$#strain; $j+=2) {
		$strain[$j] = $read[$j] = undef;
	    }
	}
    }
}

printUnlessActive(\%nd,{});

sub printUnlessActive {
    my ($pND,$pActive) = @_;
    foreach my $s (sort keys %$pND) {
	my $pND1 = $nd{$s};
	foreach my $ri (sort keys %$pND1) {
	    unless ($pActive->{"$s $ri"}) {
		my $pND2 = $pND1->{$ri};
		foreach my $rj (sort keys %$pND2) {
		    unless ($pActive->{"$s $rj"}) {
			my ($n,$d) = @{$pND2->{$rj}};
			next unless ($n);
			my $mu = $PE*($d-1);
			if ($n-1 > $mu) {
			    my $z = ($n-1-$mu)/sqrt($mu);
			    printf("%s %s %s %d %d %5.3f\n",$s,$ri,$rj,$n,$d,$z);
			}
		    }
		}
		$pND1->{$ri} = undef;
	    }
	}
    }
}
