#!/usr/bin/env perl

use Utility;
use File::Basename;
use strict;

# Run in $SGR/match directory

# Fields in each line of FREG*/freg/fregErrStats.txt are
# nIndivs coverage method mafHi type count errorProportion

# Fields in each line of FREG*/freg/fregSNPStats.txt are

foreach my $d (glob("FREG?_?/freg"),glob("FREG??_?/freg"),glob("FREG???_?/freg")) {
    my ($n,$c,$s) = (dirname($d) =~ /^FREG(\d+)_(\d+)(S?)/);
    my $isSol = length($s);
    my (%num,%den,%snps);
    my $esFile = "$d/fregErrStats.txt";
    next unless (stamp($esFile));
    report("$esFile: creating");
    foreach my $f (glob("$d/cia*/cia.chr00.bout")) {
	my ($rhi) = (basename(dirname($f)) =~ /mac(.*)$/);
	$rhi =~ s/.*://;
	$rhi = 'mono' if ($rhi eq '1');
	my $fh = openFromPipe("grep -v '^Answer ' $f");
	my (@names,@rhiList);
	while (<$fh>) {
	    chomp;
	    if (/^Type /) {
		(undef,undef,undef,@names) = split;
		foreach my $i (0 .. $#names) {
		    $names[$i] =~ s/pe\((.)\)/\1/;
		}
	    } elsif (/^N.. /) { 
		my ($type,$count,undef,@peList) = split;
		if (scalar(@peList) != scalar(@names)) {
		    print STDERR "Mismatch: '@names' and '@peList' in '$_'\n";
		} else {
		    foreach my $i (0 .. $#names) {
			my $key1 = sprintf("%3d %2d %d %s %5.3f %s",
					   $n,$c,$isSol,$names[$i],$rhi,$type);
			my $key2 = $key1;
			my $key3 = $key1;
			substr($key2,-2,1) = '.';
			substr($key3,-1,1) = '.';
			foreach my $key123 ($key1,$key2,$key3) {
			    my $keya = $key123;
			    substr($keya,-9,5) = ' all ';
			    foreach my $key ($key123,$keya) {
				$num{$key} += $count*$peList[$i];
				$den{$key} += $count;
			    }
			}
		    }
		}
	    } elsif (/^Method /) {
		(undef,@rhiList) = split;
		$rhiList[0] = 'mono' if ($rhiList[0] == 0);
	    } elsif (/^   [a-z] / && ($rhi eq 'mono' || $rhi <= 0.5)) {
		my ($name,@counts) = split(" ",substr($_,3));
		# $rhi is the REAL minor allele frequency
		my $key1 = sprintf("%3d %2d %d %s %5s",$n,$c,$isSol,$name,$rhi);
		my @keyList = ($key1);
		if ($rhi ne 'mono') {
		    my $key2 = $key1;
		    substr($key2,-5) = ' poly';
		    push @keyList,$key2;
		}
		foreach my $i (0 .. $#rhiList) {
		    # $rhiList[$i] is the APPARENT MINOR allele frequency
		    # (compareImputedAcross.pl reduced values >0.5 to 0.5).
		    my @rhListA = $rhiList[$i];
		    push @rhListA,' poly' if ($rhiList[$i] > 0);
		    foreach my $rh (@rhListA) {
			foreach my $key (@keyList) {
			    $snps{sprintf("%s %5s",$key,$rh)} += $counts[$i];
			}
		    }
		}
	    }
	}
    }

    my $eh = openToWrite($esFile);
    foreach my $key (sort keys %num) {
	printf($eh "%s %8s %10.8f\n",$key,$den{$key},$num{$key}/($den{$key} || 1));
    }
    $eh->close();

    my $sh = openToWrite("$d/fregSNPStats.txt");
    foreach my $key (sort keys %snps) {
	printf($sh "%s %8d\n",$key,$snps{$key});
    }
    $sh->close();
    unstamp($esFile);
    report();
}
