#!/usr/bin/env perl

use Utility;
use SGRP::SequencedRegionSet;
use File::Basename;
use strict;

my %feats;
foreach my $f (glob("*.trna.txt")) {
    my $str = basename($f,".trna.txt");
    my $fh = openToRead($f);
    <$fh>;
    <$fh>;
    <$fh>;
    while (<$fh>) {
	chomp;
	my ($chr,undef,$lo,$hi,$aa,$anti,$ilo,$ihi,$score) = split;
	my $ori = '+';
	if ($lo > $hi) {
	    $ori = '-';
	    my $tmp = $lo;
	    $lo = $hi;
	    $hi = $lo;
	}
	push @{$feats{$chr}},[$lo,$hi,'tRNA',$aa,$ori, $str,$ilo,$ihi,$score,$anti];
    }
}
my %srs;
foreach my $f (glob("*.repeat_alilines_pruned.txt")) {
    my $str = basename($f,".repeat_alilines_pruned.txt");
    $srs{$str} = new SequencedRegionSet("$str/assembly/genome.seqr"); # ensure this is in place!
    my $fh = openToRead($f);
    while (<$fh>) {
	if (/LTR/) {
	    chomp;
	    my (undef,$score,$type,$chr,$rlo,$rhi,$clo,$chi,$ori,undef,$percid) = split;
	    ($clo,$chi) = sort {$a <=> $b} ($clo,$chi);
	    push @{$feats{$chr}},[$clo,$chi,'match',$type,$ori, $str,$rlo,$rhi,$score,$percid];
	}
    }
}    

my $fh = openToRead("genomeAligned.gff");
while (<$fh>) {
    chomp;
    my ($chr,undef,$type,$lo,$hi,undef,$ori,undef,$anno) = split(/\t/,$_);
    if ($type eq 'transposable_element' || $type eq 'tRNA' ||
	($type eq 'repeat_region' && $anno =~ /LTR/)) {
	$anno =~ s/;.*//;
	push @{$feats{$chr}},[$lo,$hi,'SGD',$type,$ori,$anno];
    }
}

foreach my $chr (sort keys %feats) {
    my @arr = sort {$a->[0] <=> $b->[0] || $a->[1] <=> $b->[1] || $a->[2] cmp $b->[2] || $a->[3] cmp $b->[3]} @{$feats{$chr}};
    foreach my $pRec (@arr) {
	my @gff = buildGFF($chr,$pRec);
	unless ($gff[1] eq 'REF') {
	    $gff[-1] .= sprintf(";PercSeq=%4.2f",percentSequenced($srs{$gff[1]},$gff[0],$gff[3],$gff[4]));
	}
	print join("\t",@gff) . "\n";
    }
}

sub buildGFF {
    my ($chr,$pR) = @_;
    if ($pR->[2] eq 'tRNA') {
	my $anno = sprintf("Type=%s;Anti=%s",$pR->[3],$pR->[9]);
	if ($pR->[6] > 0) {
	    $anno .= sprintf(";IntronLo=%d;IntronHi=%d",$pR->[6],$pR->[7]);
	}
	return ($chr,
		$pR->[5],'tRNA',
		$pR->[0],$pR->[1],
		$pR->[8],$pR->[4],'.',
		$anno);
    } elsif ($pR->[2] eq 'match') {
	my $type;
	if ($pR->[3] =~ /I#LTR/) {
	    $type = 'transposable_element';
	} else {
	    $type = 'repeat_region';
	}
	return ($chr,
	        $pR->[5],
		$type,
		$pR->[0],$pR->[1],
		$pR->[8],$pR->[4] eq 'F' ? '+' : '-','.',
		sprintf("Type=%s;From=%d;To=%d;PercID=%s",
			$pR->[3],$pR->[6],$pR->[7],$pR->[9]));
    } else {
	return ($chr,
		'REF',
		$pR->[3],
		$pR->[0],$pR->[1],
		$pR->[1]-$pR->[0]+1,$pR->[4],'.',
		$pR->[5]);
    }
}

sub percentSequenced {
    my ($pSRS,$chr,$lo,$hi) = @_;
    my $over = $pSRS->overlapsSequencedRegion($chr,$lo,$hi,0,1);
    my $ps = 100*$over/($hi-$lo+1);
    return $ps;
}

		
