#!/usr/bin/perl

use Utility;
use strict;

foreach my $d (@ARGV) {
    my $f1 = "$d/imputed_m.gz"; # M/F version
    my $f2 = "$d/imputed_g.gz"; # SD version
    my $g = "$d/imputed_u.gz"; # Union
    if (allFilesReady($f1,$f2) && stamp($g)) {
	report("$g: creating");
	combineAlignmentFiles($f1,$f2,$g);
	report();
	unstamp($g);
    }
}

sub combineAlignmentFiles {
    my ($f1,$f2,$g) = @_;

    my $fh1 = openToRead($f1);
    my $fh2 = openToRead($f2);
    my $gh = openToWrite($g);

    my $h1 = <$fh1>;
    my $h2 = <$fh2>;
    unless ($h1 eq $h2) {
	die("Headers differ:\n$h1$h2");
    }
    print $gh $h1;
    
    while (<$fh1>) {
	chomp (my $line1 = $_);
	chomp (my $line2 = <$fh2>);
	unless ($line2) {
	    die("Excess in $f1");
	}
	print $gh combineLines($line1,$line2) . "\n";
    }
    if (<$fh2>) {
	die("Excess in $f2");
    }
}

sub combineLines {
    my ($line1,$line2) = @_;
    my $line;
    my $len = length($line1);
    unless (length($line2) == $len) {
	die("Different line lengths:\n<$line1>\n<$line2>\n");
    }
    for (my $i=0; $i<$len; $i+=2) {
	my $v1 = substr($line1,$i,1);
	my $q1 = substr($line1,$i+1,1);
	my $v2 = substr($line2,$i,1);
	my $q2 = substr($line2,$i+1,1);
	(my $u1 = $v1) =~ tr/_a-z/-A-Z/;
	(my $u2 = $v2) =~ tr/_a-z/-A-Z/;
	my ($v,$q);
	if ($u1 eq $u2) {
	    $q = $q1 ge $q2 ? $q1 : $q2;
	    $v = ($u1 eq $v1 || $u2 eq $v2) ? $u1 : $v1;
	} elsif ($q1 ge $q2) {
	    $v = $v1;
	    $q = chr(33+ord($q1)-ord($q2));
	} else {
	    $v = $v2;
	    $q = chr(33+ord($q2)-ord($q1));
	}
	$line .= "$v$q";
    }
    return $line;
}

