#!/usr/bin/env perl

=head1 NAME

combineArgsRelatedness.pl

=head1 SYNOPSIS

    cd $SGR/match/CERE/cere/methp
    combineArgsRelatedness.pl

=head1 DESCRIPTION

Print the strain names, then a matrix (suitable for displaying with
Matlab) showing the proportion of times that each pair of strains
are neighbours in the ARG tree. However, snpMatrix probably gives
more useful information.

=head1 AUTHOR

David Carter, C<dmc@sanger.ac.uk> , August 2006

=head1 SEE ALSO

    snpMatrix.pl

=cut

use Utility;
use strict;

combineArgsRelatedness();

sub combineArgsRelatedness {
    my (%num,%den);
    foreach my $d1 (glob("p?")) {
	foreach my $d2 (glob("$d1/*")) {
	    my $sorted = "$d2/sorted.gz";
	    my $mgrin = "$d2/mgr_in";
	    my $rel = "$d2/args_rel.txt";
	    if (-s $sorted && -s $mgrin && -s $rel) {
		addArgsRelatedness($sorted,$mgrin,$rel,\%num,\%den);
	    }
	}
    }
    printArgsRelatedness(\%num,\%den);
}

sub addArgsRelatedness {
    my ($sorted,$mgrin,$rel,$pNum,$pDen) = @_;
    my $fh = openToRead($sorted);
    $_ = <$fh>;
    chomp;
    s/^>//;
    my @strains = ('REF',split);
    $fh = openToRead($mgrin);
    $_ = <$fh>;
    chomp;
    my (undef,$nm) = split;
    $fh = openToRead($rel);
    my $i=0;
    while (<$fh>) {
	chomp;
	my $si = $strains[$i];
	my @row = split;
	foreach my $j (0 .. $#row) {
	    my $sj = $strains[$j];
	    $pNum->{$si}{$sj} += $nm*$row[$j];
	    $pDen->{$si}{$sj} += $nm;
	}
	$i ++;
    }
}

sub printArgsRelatedness {
    my ($pNum,$pDen) = @_;
    my %rel;
    foreach my $si (keys %$pNum) {
	my $pNumI = $pNum->{$si};
	my $pDenI = $pDen->{$si};
	foreach my $sj (keys %$pNumI) {
	    $rel{$si}{$sj} = $pNumI->{$sj}/$pDenI->{$sj};
	}
    }
    my @strains = sort keys %rel;
    printf(">%s\n",join(" ",@strains[1 .. $#strains]));
    foreach my $i (0 .. $#strains) {
	my $pRelI = $rel{$strains[$i]};
	my @row;
	foreach my $j (0 .. $#strains) {
	    push @row,$pRelI->{$strains[$j]};
	}
	print "@row\n";
    }
}
