#!/usr/bin/env perl

# This script generates "cp -p" commands to copy (newly-created) recalibrated.fastq.gz
# files, resulting from "g1k.pl cal", to the repository. It doesn't actually do the
# copying; you should redirect its output to a file and then (after checking) execute
# that.

# Currently we assume the analysis type is "v1" (see "foreach" line below).

use Utility;
use File::Basename;
use strict;

chdir($ENV{G1K});

print "cd $ENV{G1K}\n";

foreach my $f (glob("v1/*/NA*/lib*/lane*/recalibrated.fastq.gz")) {
    if (0 && -s "$f.touch") {
	print "# $f still being made\n";
	next;
    }
    (my $dd = dirname($f)) =~ s/v1/DATA/;
    my ($pfx,$sfx);
  LOOP:
    foreach my $suf (qw(fastq fastq.gz fq.gz)) {
	foreach my $sl (glob("$dd/*.$suf")) {
	    if (-l $sl) {
		$pfx = readlink($sl);
		if (-s $pfx) {
		    $pfx = substr($pfx,0,length($pfx)-length($suf)-1);
		    $sfx = $suf;
		    last LOOP;
		} else {
		    $pfx = undef;
		}
	    }
	}
    }
    if ($pfx) {
	my $tgt = "$pfx.recal.$sfx";
	$tgt .= ".gz" unless ($tgt =~ /\.gz/);
	my $st = -s $tgt;
	unless (-s $f == $st && !newer($f,$tgt)) {
	    if ($st) {
		print "# Overwriting $tgt\n";
	    }
	    print "cp -p $f $tgt\n";
	}
    }
}

	
