#!/usr/bin/env perl

# Args: coverage file, then *.ali files

use Utility;
use File::Basename;
use strict;

my $TH = 0.5; # percent ident needed to count a match

my %cov;
my $fh = openToRead(shift(@ARGV));
while (<$fh>) {
    chomp;
    my ($str,$c) = split;
    $cov{$str} = $c;
}

my %count;
my @genes;
foreach my $f (@ARGV) {
    my $gene = basename($f,".ali");
    push @genes,$gene;
    my $fh = openToRead($f);
    my $n = 0;
    while (<$fh>) {
	chomp;
	if (/^>(\S+)-/) {
	    my @a = split(/,/,$_);
	    my ($num,$den) = split(/\//,$a[3]);
	    if ($den > 0 && $num/$den >= $TH) {
		$count{$1}{$gene} ++;
	    }
	} elsif (/^[A-Z]/) {
	    $n ++;
	    last if ($n == 10);
	}
    }
}

printf("%13s %10s %3s %4s %4s %4s %4s\n",
       "strain","gene","nal","low","mid","high","wref");
foreach my $str (sort keys %count) {
    my $pH = $count{$str};
    foreach my $g (@genes) {
	my $nAli = $pH->{$g};
	my $est1 = $nAli/$cov{$str};
	my $sigLo = 0.5*(-1.96+sqrt(1.96*1.96+4*$est1));
	my $sigHi = 0.5*( 1.96+sqrt(1.96*1.96+4*$est1));
	printf("%13s %10s %3d %4.1f %4.1f %4.1f",
	       $str,$g,$nAli,$sigLo*$sigLo,$est1,$sigHi*$sigHi);
	if ($count{S288c}{$g}) {
	    printf(" %4.1f\n",$est1*$cov{S288c}/$count{S288c}{$g});
	} else {
	    print "   NA\n";
	}
    }
}

	    
