#!/usr/bin/env perl

# For SNP and indel counts in main paper.
# We identify an insertion as any contiguous sequence of ref gaps,
# and a deletion as any contiguous sequence of ref non-gaps where
# there is a gap in at least one other position.

# Args: qmin, then sequenced.gz or imputed.gz files

use Utility;
use strict;

my $qc = shift @ARGV;
my $qMin = chr(ord('!')+$qc);

my ($tSNPs,$tIndels);
foreach my $f (@ARGV) {
    my ($nSNPs,$nIndels);
    my $fh = openToRead($f);
    my $state = 'n'; # n, i or d
    while (<$fh>) {
	next if (/^>/);
	if (/^[-_=0-9]/) {
	    # insertion here
	    unless ($state eq 'i') {
		$nIndels ++;
		$state = 'i';
	    }
	} else {
	    my ($hasDel,$hasPoly);
	    my $cMax = length($_)-1;
	    (my $r = substr($_,0,1)) =~ tr/a-z/A-Z/;
	    for (my $c=2; $c<=$cMax; $c+=2) {
		my $v = substr($_,$c,1);
		next if ($v eq ' ');
		my $qc = substr($_,$c+1,1);
		next if ($qc ne ' ' && $qc lt $qMin);
		if ($v =~ /[-_=0-9]/) {
		    $hasDel = 1;
		} else {
		    $v =~ tr/a-z/A-Z/;
		    $hasPoly = 1 unless ($v eq $r || $v eq 'N');
		}
	    }
	    if ($hasDel) {
		unless ($state eq 'd') {
		    $nIndels ++;
		    $state = 'd';
		}
	    }
	    $nSNPs ++ if ($hasPoly);
	}
    }
    printf("%s %6d %6d\n",$f,$nSNPs,$nIndels);
    $tSNPs += $nSNPs;
    $tIndels += $nIndels;
}
my $len = length($ARGV[0]);
printf("%${len}s %6d %6d\n",'TOTAL',$tSNPs,$tIndels);
