#!/usr/bin/env perl

use Utility;
use strict;

foreach my $d (glob("p?/*")) {
    createLayoutZ($d);
}

sub createLayoutZ {
    my ($d) = @_;
    my $layZ = "$d/layout_z.gz";
    my $sorY = "$d/sorted_y.gz";
    my $sorZ = "$d/sorted_z.gz";
    my $layY = "$d/layout_y.gz";
    foreach my $f ($sorY,$sorZ,$layY) {
	unless (-s $f) {
	    report("$f: missing or empty",1);
	    return;
	}
    }
    my $dlFile = "$d/delete_yz.txt";
    my @delLocs;
    if (stamp($dlFile)) {
	report("$d: finding locations to delete");
	@delLocs = locationsToDelete($sorY,$sorZ,$dlFile);
	printf("%d locations ... ",scalar(@delLocs));
	unstamp($dlFile);
	report();
    } else {
	@delLocs = locationsToDelete(undef,undef,$dlFile);
    }
    if (stamp($layZ)) {
	report("$layZ: creating");
	applyDeletionLocations($layY,\@delLocs,$layZ);
	unstamp($layZ);
	report();
    }
    my $danY = "$d/dangles_y.txt";
    my $danZ = "$d/dangles_z.txt";
    if (-s $danY && stamp($danZ)) {
	createDanglesZ($danY,\@delLocs,$danZ);
	unstamp($danZ);
    }
}

sub createDanglesZ {
    my ($f,$pLocs,$g) = @_;
    my $fh = openToRead($f);
    my $gh = openToWrite($g);
    my $i = 0;
    while (<$fh>) {
	chomp;
	my ($off,$lr) = split;
	while ($i <= $#$pLocs && $pLocs->[$i] <= $off) {
	    $i ++;
	}
	printf($gh "%d %s\n",$off-$i,$lr);
    }
}

sub locationsToDelete {
    my ($sorY,$sorZ,$dlFile) = @_;
    if (-s $dlFile) {
	my $fh = openToRead($dlFile);
	chomp (my @locs = <$fh>);
	return @locs;
    } else {
	my $gh = openToWrite($dlFile);
	my $yh = openToRead($sorY);
	my $zh = openToRead($sorZ);
	my $hdrY = <$yh>;
	my $hdrZ = <$zh>;
	my $pMap = createColumnMap($hdrY,$hdrZ);
	my $yLine;
	my $loc = 0;
	my @delLocs;
	while ($yLine = <$yh>) {
	    chomp $yLine;
	    my $zLine = <$zh>;
	    chomp $zLine;
	    $loc ++;
	    while (!closeEnoughMatch($yLine,$pMap,$zLine)) {
		push @delLocs,$loc;
		print $gh "$loc\n";
		if (@delLocs > 100) {
		    $gh->close();
		    unlink($dlFile);
		    die("Too many deletion locations: @delLocs");
		}
		chomp ($yLine = <$yh>);
		last unless ($yLine);
		$loc ++;
	    }
	}
	return @delLocs;
    }
}

sub createColumnMap {
    my ($hdr1,$hdr2) = @_;
    chomp $hdr1;
    chomp $hdr2;
    my @list1 = split(" ",substr($hdr1,1));
    my @list2 = split(" ",substr($hdr2,1));
    my %pos2;
    foreach my $i (0 .. $#list2) {
	$pos2{$list2[$i]} = $i;
    }
    my @cMap;
    my $change;
    foreach my $i (0 .. $#list1) {
	$cMap[$i] = $pos2{$list1[$i]};
    }
    return \@cMap;
}

sub closeEnoughMatch {
    my ($line1,$pMap,$line2) = @_;
    # Insist ref is same
    my $r1 = substr($line1,0,1);
    $r1 = '=' if ($r1 =~ /[a-z]/);
    my $r2 = substr($line2,0,1);
    return 0 unless ($r1 eq $r2);
    my ($nGood,$nBad) = (0,0);
    foreach my $i (0 .. $#$pMap) {
	my $v1 = substr($line1,2*($i+1),1);
	if (defined $v1 && $v1 ne '' && $v1 ne ' ') {
	    my $j = $pMap->[$i];
	    my $v2 = substr($line2,2*($j+1),1);
	    if ($v1 eq $v2) {
		$nGood ++;
	    } else {
		$nBad ++;
	    }
	}
	
    }
    return ($nBad <= 1 || $nGood > $nBad);
}

sub applyDeletionLocations {
    my ($layY,$pLocs,$layZ) = @_;
    my $yh = openToRead($layY);
    my $zh = openToWrite($layZ);
    my $loc = 0;
    my $i = 0;
    while (<$yh>) {
	if (/^>/) {
	    print $zh $_;
	} else {
	    $loc ++;
	    if ($i <= $#$pLocs && $pLocs->[$i] == $loc) {
		$i ++;
	    } else {
		print $zh $_;
	    }
	}
    }
}
