#!/usr/bin/env perl

# Run in (final) NNN dir.

use SGRP::Variables;
use File::Basename;
use Utility;
use strict;

setVariables();

my $ogDir = "$ROOT/match/PARS/cere/methc/017";

foreach my $d (glob("p?/*")) {
    my $chr = basename($d);
    createSortedWithOutgroup("$d/sorted_y.gz",
			     "$d/imputed_y.gz",
			     "$ogDir/$d/nrco.fq",
			     "$d/sorted_o.gz");
}

sub createSortedWithOutgroup {
    my ($fs,$fi,$fo,$gs) = @_;
    return if (-s $gs);
    return unless (-s $fs && -s $fi && -s $fo);
    return unless (stamp($gs));
    report("$gs: creating");
    my ($mSeq,$mQuals) = finalColumnSequence($fi);
    my $fh = openToRead($fs);
    my $oh = openToRead($fo);
    <$oh>; # header
    my $gh = openToWrite($gs);
    chomp (my $hdr = <$fh>);
    my @strains = split(" ",$hdr);
    my $expLen = 2*(scalar(@strains)+1);
    print $gh "$hdr MRCA OUT\n";
    my $off1 = 0; # from zero; line count
    while (<$fh>) {
	chomp;
	$_ .= ' ' x ($expLen-length($_));
	print $gh $_;
	print $gh substr($mSeq,$off1,1);
	print $gh substr($mQuals,$off1,1);
	$off1 ++;
	# Outgroup ...
	if (/^[A-Z]/) {
	    my $line = <$oh>;
	    # When everything else (of quality >= 40) has same value, give $line 
            # that value too, otherwise outgroup is too far away
	    my %seen;
	    for (my $i=0; $i<$expLen; $i+=2) {
		my $v = substr($_,$i,1);
		my $qc = substr($_,$i+1,1);
		unless ($v eq ' ' || ($qc ne ' ' && $qc lt 'I')) {
		    $v =~ tr/a-z/A-Z/;
		    $seen{$v} = 1;
		}
	    }
	    if (scalar(keys %seen) == 1) {
		substr($line,0,1) = substr($_,0,1);
	    }
	    print $gh $line;
	} else {
	    print $gh substr($_,0,1) . "!\n"; # make same as ref, but poor, when gap
	}
    }
    $gh->close();
    unstamp($gs);
    report();
}
    
sub finalColumnSequence {
    my ($f) = @_;
    my $fh = openToRead($f);
    <$fh>; # header
    my ($seq,$quals);
    while (<$fh>) {
	chomp;
	$seq .= substr($_,length($_)-2,1);
	$quals .= substr($_,length($_)-1,1);
    }
    return ($seq,$quals);
}

