#!/usr/bin/env perl

# Input: *.csv, from Richard's summary.xls on 080430

use strict;

$_ = <>; # header
my %stats;
while (<>) {
    chomp;
    s/"(\d+),(\d\d\d)"/\1\2/g;
    s/%//g;
    my (undef,undef,undef,$sample,$lib,$l_read1,$l_read2,$ins_mean,$ins_sd,
	$n_reads,$n_mapped,$n_paired,$err_rate,$dup_rate,$gc,$mbp) = split(",",$_);
    next unless ($mbp);
    my $gbp = $mbp/1000;
    if ($lib =~ /^lib/) {
	$lib = "SC-$sample-$lib";
    } elsif ($lib =~ /^Solexa-/) {
	$lib =~ s/Solexa/BGI/;
    } else {
	$lib = "BI-$lib";
    }
    if ($gbp < 4) {
	my ($org) = split(/-/,$lib);
	$lib = sprintf("%s-%s-*",$org,$sample);
    }
    if (defined $stats{$sample}{$lib}) {
	my @s = @{$stats{$sample}{$lib}};
	$stats{$sample}{$lib} = [$s[0]+$n_reads,$s[1]+$n_mapped,$s[2]+$n_paired,
				 $s[3]+$n_reads*$err_rate,$s[4]+$n_reads*$dup_rate,$s[5]+$gbp];
	
    } else {
	$stats{$sample}{$lib} = [$n_reads,$n_mapped,$n_paired,
				 $n_reads*$err_rate,$n_reads*$dup_rate,$gbp];
    }
}
printf("%10s %20s %6s %5s %5s %5s %5s\n",
       'Sample','Library','GBases','%Map','%Pair','%Err','%Dup');
foreach my $sample (sort keys %stats) {
    my $pH1 = $stats{$sample};
    foreach my $lib (sort {$pH1->{$b}[5] <=> $pH1->{$a}[5]} keys %$pH1) {
	my ($n_reads,$n_mapped,$n_paired,$err_count,$dup_count,$gbp) = @{$pH1->{$lib}};
	printf("%10s %20s %6.3f %5.1f %5.1f %5.2f %5.2f\n",
	       $sample,$lib,$gbp,
	       100*$n_mapped/$n_reads,
	       100*$n_paired/$n_reads,
	       $err_count/$n_reads,
	       $dup_count/$n_reads);
    }
}

	
