#!/usr/bin/perl

# Derive complete data for evaluating Fregene run.

# Args: e.g. 200 10 1000000 rin10k_simII.xml.gz $SGR/match/FREG_COMPLETE/200

use Utility;
use strict;

if (@ARGV != 5) {
    die("Usage: $0 nHaps nChrs chrSz rinFile tgtDir\n");
}

my ($nHaps,$nChrs,$chrSz,$f,$dir) = @ARGV;

my $nOffs = $nChrs*$chrSz;

my $fh = openToRead($f);
my $active;
my $ind = 0;
my %snp;
while (<$fh>) {
    chomp;
    if (/<DATA>/) {
	$active = 1;
	$_ = substr($_,6);
    }
    if ($active) {
	foreach my $pos (split) {
	    last if ($pos >= $nOffs);
	    push @{$snp{$pos}},$ind;
	}
	$ind ++;
	last if ($ind > $nHaps);
    }
}

my @posList = sort {$a <=> $b} keys %snp;
my @indList = sort {$a cmp $b} (0 .. $nHaps);
my @colMap;
foreach my $i (0 .. $#indList) {
    $colMap[$indList[$i]] = 2*$i;
}
shift @indList; # get rid of zero (ref) so we can use it as header

for (my $nc=0; $nc<$nChrs; $nc++) {
    my $chrBase = $chrSz*$nc+1;
    my $g = sprintf("%s/p%d/chr%2d/sorted.gz",$dir,substr($nc,-1),$nc);
    $g =~ tr/ /0/;
    report("creating $g");
    my $gh = openToWrite($g);
    print $gh ">@indList\n";
    my $kMax = $chrSz*($nc+1);
    for (my $k=$chrBase; $k<=$kMax; $k++) {
	my $line = 'A ' . ('A~' x $nHaps);
	if ($k == $posList[0]) {
	    shift @posList;
	    foreach my $ind (@{$snp{$k}}) {
		substr($line,$colMap[$ind],1) = 'C';
	    }
	    if (substr($line,0,1) eq 'C') {
		$line =~ tr/AC/CA/;
	    }
	}
	print $gh "$line\n";
    }
    report();
}
