#!/usr/bin/env perl

# Arg: cere or para

# cere.deltas, para.deltas: construct by hand, starting with sorted
# cere.strains and adding "dx dy" or "= dx dy" (to put in key below
# for close-packed strains) as required.

use Utility;
use strict;

my %FULL = (cere => 'cerevisiae',
	    cere_reduced1 => 'cerevisiae',
	    cere_clean => 'cerevisiae (clean)',
	    cere_syn => 'cerevisiae (synonymous sites)',
	    para => 'paradoxus',
	    euro => 'paradoxus (Europe)');

my %COLOUR_STRING =
    (cere => {clinical => '1 0 0',
	      wild => '0 0 1',
	      soil => '0.7 0.3 0',
	      unknown => '0 0 0',
	      fermentation => '0 0.5 0',
	      baker => '0 1 0',

	      Africa => '0.7 0.3 0',
	      Australia => '0.7 0.3 1',
	      EAsia => '1 0 0',
	      Europe => '0 0.5 0',
	      Hawaii => '1 0.5 0.5',
	      NAmerica => '0 0 1',
	      SAmerica => '0 1 1',
	      Unknown => '0 0 0'},

     para => {EAsia => '1 0 0',
	      Europe => '0 0 0',
	      Hawaii => '1 0.5 0.5',
	      NAmerica => '0 0 1',
	      SAmerica => '0 1 1'},


     euro => {londona => '0 0.7 0',
	      londoni => '0.5 0 0.5',
	      italy => '0 0 1',
	      denmark => '0 1 1',
	      russia => '1 0 0',
	      siberia => '1 0.7 0'});

$COLOUR_STRING{cere_clean} = $COLOUR_STRING{cere};
$COLOUR_STRING{cere_reduced1} = $COLOUR_STRING{cere};

foreach my $sp (@ARGV) {
    my $sp2 = $sp;
    $sp2 =~ s/_syn$//;
    if ($FULL{$sp}) {
	editPostscriptTree($sp,$COLOUR_STRING{$sp2});
    } else {
	die("Bad species name: $sp");
    }
}

sub editPostscriptTree {
    my ($sp,$pColourString) = @_;
    my $psIn = "$sp.ps";
    my $strFile = "$sp.strains";
    my $psOut = "${sp}_edited.ps";
    my $psOutN = "${sp}_edited_nonames.ps";
    my $pStrain = readStrains($strFile);
    my $pDelta = readDeltas("$sp.deltas");
    my $pColour = readDeltas("$sp.colours");
    my $fh = openToRead($psIn);
    my $gh = openToWrite($psOut);
    my $nh = openToWrite($psOutN);
    my $abbr = 'A';
    my @table;
    my $coloured;
    my $italic;
    my %usedColour;
    while (<$fh>) {
	chomp;
	my @a = split;
	my $noN = 0;
	# Contract cere diagram:
	if ($a[2] eq 'translate') {
	    $a[1] = 0;
	    print $gh "/italfont /Times-Italic findfont 12 scalefont def\n";
	    print $nh "/italfont /Times-Italic findfont 12 scalefont def\n";
	} elsif (@a == 16 && $a[-1] eq 'stroke') {
	    my $ylo = $sp eq 'cere' ? 120 : 150;
	    for (my $i=1; $i<=$#a; $i+=3) {
		if ($a[$i] == -10) {
		    $a[$i] = $ylo;
		}
	    }
	} elsif ($a[3] eq "($sp.tree") {
	    @a = (@a[0..2],'(S',$FULL{$sp},'strains');
        } elsif ($a[2] eq 'moveto' && $a[3] =~ /^\(([\.0-9]+)\)$/ && $a[4] eq 'show')  {
	    my $id = $1; # /1000; # divide down by 1000 ...
	    $a[3] = "($id)";
	    $a[0] += 5;
	    print $gh "@a\n";
	    print $nh "@a\n";
        } elsif ($a[2] eq 'moveto' && $a[3] =~ /^\((.)\)$/ && $a[4] eq 'show')  {
	    my $str = $pStrain->{$1} || die("No strain for key $1");
	    if (defined $pColour->{$str}) {
		my $pA = $pColour->{$str};
		my $box;
		if ($pA->[0]) {
		    $usedColour{$pA->[0]} = 1;
		    my $col = $pColourString->{$pA->[0]}
		    || die("Unknown colour $pA->[0]");
		    print $gh "$col setrgbcolor\n";
		    $coloured = $col;
		}
		if ($pA->[1] eq 'lab') {
		    print $gh "italfont setfont\n";
		    $italic = 1;
		    $usedColour{'lab'} = 1;
		}
	    }
	    my $pD = $pDelta->{$str};
	    unless ($pD && $pD->[0] eq '=') {
		$a[0] = max(-8,$a[0]-length($str)*4);
		$a[0] = min($a[0],510-length($str)*8);
	    }
	    if (defined @$pD) {
		if ($pD->[0] eq '=') {
		    push @table,[$abbr,$str,$coloured,$italic];
		    $str = $abbr;
		    $abbr = ($abbr eq 'Z' ? 'a' : chr(ord($abbr)+1));
		    if ($sp eq 'para' && $str =~ /[A-Q]/) {
			($a[0],$a[1]) = positionOnCircle($str);
		    } elsif ($#$pD >= 1) {
			$a[0] += $pD->[1];
			$a[1] += $pD->[2];
		    }
		} elsif ($pD->[0] eq '.') {
		    $str = ($str eq 'Z1_1' && $sp eq 'para') ? 'Europe' : '';
		    $a[0] += $pD->[1];
		    $a[1] += $pD->[2];
		} else {
		    $a[0] += $pD->[0];
		    $a[1] += $pD->[1];
		}
	    }
	    $a[3] = sprintf("(%s)",$str);
	    $noN = 1;
	} elsif ($a[0] eq 'showpage') {
	    printTable($sp,\@table,$pColourString,$gh);
	    if (defined $pColourString) {
		printColourKey($sp,$pColourString,\%usedColour,$gh);
	    }
	    if ($sp eq 'para') {
		addCerevisiaeToScale($gh);
		addCerevisiaeToScale($nh);
	    }
	}
	print $gh "@a\n";
	print $nh "@a\n" unless ($noN);
	if ($italic) {
	    print $gh "basefont setfont\n";
	    $italic = 0;
	}
	if ($coloured) {
	    print $gh "0 0 0 setrgbcolor\n";
	    $coloured = 0;
	}
    }
}
    
sub readStrains {
    my ($sFile) = @_;
    my %strain;
    my $sh = openToRead($sFile);
    my $key = 'A';
    while (<$sh>) {
	chomp;
	$strain{$key} = $_;
	if ($key eq 'Z') {
	    $key = 'a';
	} else {
	    $key = chr(ord($key)+1);
	}
    }
    return \%strain;
}

sub readDeltas {
    my ($f,$singles) = @_;
    my %delta;
    if (-s $f) {
	my $fh = openToRead($f);
	while (<$fh>) {
	    chomp;
	    my ($str,@a) = split;
	    if (@a) {
		if ($singles) {
		    $delta{$str} = $a[0];
		} else {
		    $delta{$str} = \@a;
		}
	    }
	}
    }
    return \%delta;
}

# Used for abbreviation table

sub printTable {
    my ($sp,$pTable,$pColourString,$gh) = @_;
    return unless (@$pTable);
    my $x = ($sp =~ /^cere/) ? 380 : 370; 
    my $y0 = $sp eq 'cere_clean' ? 350 : ($sp =~ /^cere/) ? 270 : 200;
    my $y = $y0;
    foreach my $pA (@$pTable) {
	my ($abbr,$name,$col,$ital) = @$pA;
	print $gh "$col setrgbcolor\n" if ($col);
	print $gh "italfont setfont\n" if ($ital);
	printf($gh "%d %d moveto (%s) show\n",$x,$y,$abbr);
	printf($gh "%d %d moveto (= %s) show\n",$x+12,$y,$name);
	print $gh "0 0 0 setrgbcolor\n" if ($col);
	print $gh "basefont setfont\n" if ($ital);
	$y -= 12;
    }
    drawBox($x-10,$y+5,$x+95,$y0+15,$gh);
}

sub drawBox {
    my ($x1,$y1,$x2,$y2,$gh) = @_;
    printf($gh "%d %d moveto %d %d lineto %d %d lineto %d %d lineto %d %d lineto stroke\n",
	   $x1,$y1,$x2,$y1,$x2,$y2,$x1,$y2,$x1,$y1);
}

sub positionOnCircle {
    my ($ch) = @_;
    my $idx = ord($ch)-ord('A');
    my ($cx,$cy) = (80,210);
    my ($rx,$ry) = (25,35);
    my $degs = 45 - 18*$idx;
    my $th = $degs*3.141592652/180;
    my $x = $cx + $rx*cos($th);
    my $y = $cy + $ry*sin($th);
    return ($x,$y);
}

sub printColourKey {
    my ($sp,$pColourString,$pUsed,$gh) = @_;
    my ($x,$y0);
    if ($sp eq 'cere') {
	($x,$y0) = (280,250);
    } elsif ($sp eq 'cere_clean') {
	($x,$y0) = (120,250);
    } elsif ($sp eq 'euro') {
	($x,$y0) = (320,250);
    } else {
	($x,$y0) = (50,500);
    }
    my $y = $y0;
    foreach my $col (sort keys %$pColourString) {
	if ($pUsed->{$col}) {
	    printf($gh "%s setrgbcolor\n",$pColourString->{$col});
	    if ($col eq 'londona') {
		$col = 'London/ABI';
	    } elsif ($col eq 'londoni') {
		$col = 'London/IAG';
	    } else {
		substr($col,0,1) =~ tr/a-z/A-Z/;
		if (substr($col,1,1) =~ /[A-Z]/) {
		    $col = sprintf("%s %s",
				   substr($col,0,1),
				   substr($col,1));
		}
	    }
	    printf($gh "%d %d moveto (%s) show\n",$x,$y,$col);
	    $y -= 12;
	}
    }
    if ($pUsed->{'lab'}) {
	print $gh "\nitalfont setfont\n0 0 0 setrgbcolor\n";
	printf($gh "%d %d moveto (Lab) show\nbasefont setfont\n",$x,$y);
	$y -= 12;
    }
    print $gh "0 0 0 setrgbcolor\n";
    drawBox($x-10,$y+5,$x+75,$y0+15,$gh);
   
}

sub addCerevisiaeToScale {
    my ($gh) = @_;
    my $scale = cereParaScale();
    my $fh = openToRead('cere.ps');
    print $gh "0.7 0.3 0 setrgbcolor\n";
    my $dx = 350;
    my $dy = 300;
    while (<$fh>) {
	chomp;
	my ($x1,$y1,$mt,$x2,$y2,$lt,$st,@rest) = split;
	last if ($x2 > 480 && $y2 >= 720);
	if ($mt eq 'moveto' && $lt eq 'lineto' && $st eq 'stroke' && !@rest) {
	    $x1 *= $scale;
	    $y1 *= $scale;
	    $x2 *= $scale;
	    $y2 *= $scale;
	    $x1 += $dx;
	    $y1 += $dy;
	    $x2 += $dx;
	    $y2 += $dy;
	    printf($gh "%d %d moveto %d %d lineto stroke\n",
		   $x1,$y1,$x2,$y2);
	}
    }
    printf($gh "%d %d moveto (S cerevisiae to same scale) show\n",$dx,$dy+50);
}

sub cereParaScale {
    return fileScale("cere.ps")/fileScale("para.ps");
}

sub fileScale {
    my ($f) = @_;
    my $fh = openFromPipe("tail -4 $f");
    chomp (my @lines = <$fh>);
    $lines[0] =~ s/.*\((.*)\).*/\1/;
    my @a = split(" ",$lines[1]);
    my @b = split(" ",$lines[2]);
    return $lines[0]/($b[0]-$a[0]);
}
