#!/usr/bin/env perl

# Run in NNN dir, after running
#    randomizeSortedACRegions.pl
# to get sorted_ac_regions.txt.rand from each sorted_ac_regions.txt
# and then missingAreaDistribution.pl on both the real and the randomized,
# to get mad.txt and mad.txt.rand.

# On Sc data, 39 non-ref strains, this suggests p(random absence) ~ 1
# for areas up to about 2000, linearly (with area) down to 0.8 at
# about 21000, close to zero (0.05-ish) up to 26000, then 0
# thereafter.

# On Sp data, the pattern looks different: p(random absence) ~ 1 for
# areas up to 1000, then linearly down to 0 at 15000. This is probably
# because of structuring, but we don't want to go into that now.

# Thus what we should do to sorted_ac.gz is: use T to represent
# "totally different". When p(not random absence) >= say 0.25 for
# _any_ strain, replace all spaces for _any_ such strain with T
# and appropriate quality char. After imputation, be cautious:
# where decision was not T, cap quality score by p(WAS random absence).
# And where decision was T, we want nucleotide to be N.

use Utility;
use strict;

my $BIN_SZ = log(10)/50;

my (@real,@rand);
my (@realFiles,@randFiles);
foreach my $d (glob("p?/chr??")) {
    my $realFile = "$d/mad.txt";
    my $randFile = "$d/mad.txt.rand";
    if (-s $realFile && -s $randFile) {
	push @realFiles,$realFile;
	push @randFiles,$randFile;
    }
}
my $pReal = readAndBin(@realFiles);
my $pRand = readAndBin(@randFiles);

foreach my $i (0 .. $#$pReal) {
    my $sz = int(0.5+exp(($i+0.5)*$BIN_SZ));
    if ($pReal->[$i]) {
	# 0.5 here is from looking at lowest 1% of real,
	# which ought to be all genuine. Works for both Sc and Sp.
	my $p = min(1,0.5*$pRand->[$i]/$pReal->[$i]);
	printf("%6d %8.6f %8d %8d\n",
	       $sz,$p,
	       $pRand->[$i],$pReal->[$i]);
    }
}	   

sub readAndBin {
    my ($f) = @_;
    my $fh = openToRead($f);
    my @count;
    while (<$fh>) {
	my ($sz,$n) = split;
	if ($n > 0 && $sz > 0) {
	    my $b = int(log($sz)/$BIN_SZ);
	    $count[$b] += $n;
	}
    }
    return \@count;
}
