#!/usr/bin/env perl

# Run inside $SGR/ref/cere directory.

use Utility;
use strict;

extendWithTransposons("augmented.fa","augmented.map",
		      "te/te.fa","te/CERE.ptis2",
		      "extended.fa","extended.map");

sub extendWithTransposons {
    my ($inFA,$inMap,$teFA,$teSpec,$outFA,$outMap) = @_;

    my $pSpec = readTESpecs($teSpec);
    my $pSeqs = readFasta($teFA);

    extendWithTransposons1($inFA,$inMap,$pSpec,$pSeqs,$outFA,$outMap);

}

sub readTESpecs {
    my ($f) = @_;
    my $fh = openToRead($f);
    my %spec;
    while (<$fh>) {
	chomp;
	my ($chr,$off,$te,$fc) = split;
	my (undef,$teLo,$teHi) = split(/\./,$te);
	# We don't subtract 1 here, even though .map files are
	# zero-offset, so this means "insert transposon BEFORE
	# current position".
	$spec{$chr}{$off} = [$te,$fc,$teHi-$teLo+1];
    }
    return \%spec;
}

sub readFasta {
    my ($f) = @_;
    my $fh = openToRead($f);
    my %seq;
    my ($chr,$buf);
    while (<$fh>) {
	chomp;
	if (/^>(\S+)/) {
	    $seq{$chr} = $buf if ($buf);
	    $buf = "";
	    $chr = $1;
	} else {
	    $buf .= $_;
	}
    }
    $seq{$chr} = $buf if ($buf);
    return \%seq;
}

sub extendWithTransposons1 {
    my ($inFA,$inMap,$pSpec,$pSeqs,$outFA,$outMap) = @_;
    my $mfh = openToRead($inMap);
    my $mgh = openToWrite($outMap);
    my $afh = openToRead($inFA);
    chomp (my $afhLine = <$afh>);
    my $agh = openToWrite($outFA);
    my $oldChr;
    while (<$mfh>) {
	chomp;
	my $mapLine = $_;
	my ($chr,$lo1,$lo2,$chrLen) = split(" ",$mapLine);
	my $inSeq;
	unless ($chr eq $oldChr) {
	    # Get header and sequence for new chr or chunk
	    $oldChr = $chr;
	    if ($afhLine eq ">$chr") {
		($inSeq,$afhLine) = readSequence($afh);
		print $agh ">$chr\n";
	    }
	    #print "PROCESSING $chr\n";
	}
	(my $chrBase = $chr) =~ s/^REF\.//;
	my $pH = $pSpec->{$chrBase};
	if (defined $pH) {
	    #print "COMPLEX: $chrBase\n";
	    my $outSeq;
	    my $off = 0;
	    my $extOff = 0;
	    my $nAux = 500; # Assume at most 500 aux sequences per chr already!
	    my (%auxMap,%auxSeq);
	    foreach my $off2 (sort {$a <=> $b} keys %$pH) {
		# Sort out aux for ref
		my $key = sprintf("%s.%3d",$chr,$nAux);
		$key =~ tr/ /0/;
		$auxMap{$key}  = sprintf("%s %d %d %d\n",$key,0,$extOff-1200,1200);
		$auxMap{$key} .= sprintf("%s %d %d %d\n",$key,1200,$extOff,1200);
		$auxSeq{$key} = substr($inSeq,$off2-1200,2400);
		# copy up to off2 ...
		my $len = $off2-$off;
		$outSeq .= substr($inSeq,$off,$len);
		print $mgh "$chr $off $extOff $len\n";
		$outSeq .= transposonSequence($pSeqs,$pH->{$off2});
		#printf("LEN %d %d\n",length($outSeq),$len);
		$extOff += $len+$pH->{$off2}[2];
		$off += $len;
		$nAux ++;
	    }
	    if ($off < $chrLen-1) {
		my $len = $chrLen-1-$off;
		print $mgh "$chr $off $extOff $len\n";
		$outSeq .= substr($inSeq,$off);
	    }
	    printFasta(undef,$outSeq,$agh);
	    # Print aux map and sequences
	    foreach my $key (sort keys %auxMap) {
		print $mgh $auxMap{$key};
		printFasta($key,$auxSeq{$key},$agh);
	    }
	} else {
	    print $mgh "$mapLine\n";
	    if ($inSeq) {
		#print "SIMPLE: $chrBase\n";
		# probably just an aux sequence; copy ...
		printFasta(undef,$inSeq,$agh);
		$inSeq = undef;
	    }
	}
    }
}

sub printFasta {
    my ($key,$seq,$gh) = @_;
    print $gh ">$key\n" if ($key);
    my $iMax = length($seq)-1;
    for (my $i=0; $i<=$iMax; $i+=60) {
	print $gh substr($seq,$i,60) . "\n";
    }
}

sub readSequence {
    my ($fh) = @_;
    my $seq;
    while (<$fh>) {
	chomp;
	if (/^>/) {
	    return ($seq,$_);
	} else {
	    $seq .= $_;
	}
    }
    return ($seq,undef);
}

sub transposonSequence {
    my ($pSeqs,$pSpec) = @_;
    my ($te,$fc,$len) = @$pSpec;
    my $seq = $pSeqs->{$te};
    unless (length($seq) == $len) {
	die("Length mismatch");
    }
    if ($fc eq 'C') {
	$seq = reverse($seq);
	$seq =~ tr/ACGT/TGCA/;
    }
    return $seq;
}
