#!/usr/bin/env perl

use Utility;
use strict;

autoflush STDOUT 1;

# Args: dirs to operate in.

foreach my $d (@ARGV) {
    my $f1 = "$d/sorted.gz";
    my $f2 = "$d/hgr.txt";
    my $pRegs = extractHardGapRegions($f1,$f2);
    my $f3 = "$d/sorted_ac.gz";
    createACFile($f1,$pRegs,$f3);
}

sub extractHardGapRegions{
    my ($f,$g) = @_;
    if (-s $g) {
	return readHardGapRegions($g);
    }
    report("finding hard gap regions in $f");
    my $fh = openToRead($f);
    <$fh>; # header
    my $off = 0;
    my (@hgrStart,@hgrFiller); # per column
    my %hgrFillers;
    while (<$fh>) {
	my $len = length($_);
	for (my $i=0; $i<$len; $i+=2) {
	    my $v = substr($_,$i,1);
	    if ($v eq '=') {
		unless (defined $hgrStart[$i]) {
		    $hgrStart[$i] = $off;
		}
	    } else {
		if (defined $hgrStart[$i]) {
		    my @fillers;
		    if (defined $hgrFiller[$i]) {
			@fillers = sort {$a <=> $b} keys %{$hgrFiller[$i]};
		    }
		    $hgrFillers{$hgrStart[$i]}{$off} = \@fillers;
		    $hgrStart[$i] = undef;
		    $hgrFiller[$i] = undef;
		}
		if ($v =~ /[A-Za-z]/) {
		    for (my $j=0; $j<$len; $j+=2) {
			if (defined $hgrStart[$j] && $j != $i) {
			    $hgrFiller[$j]{$i} = 1;
			}
		    }
		}
	    }
	}
	$off++;
	print "." if ($off%10000 == 0);
    }
    foreach my $i (0 .. $#hgrStart) {
	if (defined $hgrStart[$i]) {
	    my @fillers;
	    if (defined $hgrFiller[$i]) {
		@fillers = sort {$a <=> $b} keys %{$hgrFiller[$i]};
	    }
	    $hgrFillers{$hgrStart[$i]}{$off} = \@fillers;
	}
    }
    my @regs;
    report();
    report("writing $g");
    my $gh = openToWrite($g);
    foreach my $lo (sort {$a <=> $b} keys %hgrFillers) {
	my $pH = $hgrFillers{$lo};
	foreach my $hi (sort {$a <=> $b} keys %$pH) {
	    my @a = ($lo,$hi,@{$pH->{$hi}});
	    print $gh "@a\n";
	    push @regs,\@a;
	}
    }
    $gh->close();
    report();
    return \@regs;
}

sub readHardGapRegions {
    my ($f) = @_;
    my @regs;
    report("reading $f");
    my $fh = openToRead($f);
    while (<$fh>) {
	chomp;
	my @a = split;
	push @regs,\@a;
    }
    report();
    return \@regs;
}

sub createACFile {
    my ($f,$pRegs,$g) = @_;
    my $fh = openToRead($f);
    my $hdr = <$fh>;
    report("creating $g");
    my $gh = openToWrite($g);
    #autoflush $gh 1; # for debugging only
    print $gh $hdr;
    chomp $hdr;
    my @strains = split(" ",substr($hdr,1));
    my $nCols = 2*(scalar(@strains)+1);
    my $off = 0;
    my $k = 0; # current offset in $pRegs
    while (<$fh>) {
	chomp;
	$off ++;
	while ($k <= $#$pRegs && $pRegs->[$k][1] < $off) {
	    $k++;
	}
	my $line;
	my %seen;
	for (my $i=0; $i<$nCols; $i+=2) {
	    my $v = substr($_,$i,1);
	    my $ac;
	    if ($v eq ' ') {
		$ac = canBeInHardGap($i,$off,$k,$pRegs) ? ' ' : 'C';
	    } elsif ($v eq '=') {
		$ac = 'A';
	    } else {
		$ac = 'C';
	    }
	    $line .= "$ac ";
	    $seen{$ac} = 1 if ($ac ne ' ');
	}
	# If all same, fill in
	my @seen = keys %seen;
	if (@seen == 1) {
	    $line = "$seen[0] " x ($nCols/2);
	}
	print $gh "$line\n";
    }
    report();
}

sub canBeInHardGap {
    my ($i,$off,$k,$pRegs) = @_;
    while ($k <= $#$pRegs && $pRegs->[$k][0] <= $off) {
	if ($pRegs->[$k][1] >= $off) {
	    # then we're in this region
	    my (undef,undef,@fillers) = @{$pRegs->[$k]};
	    my $filled;
	    foreach my $j (@fillers) {
		if ($j == $i) {
		    # can't have this hard gap
		    $filled = 1;
		    last;
		}
	    }
	    return 1 unless ($filled);
	}
	$k ++;
    }
    return 0; # can't have hard gap here
}
