#!/usr/bin/env perl

=head1 NAME

extractLayoutRegion.pl

=head1 SYNOPSIS

extractLayoutRegion.pl loOffset hiOffset (layoutFile|sortedFile|imputedFile)

=head1 DESCRIPTION

Prints to STDOUT layout (or sorted/imputed) data corresponding only to
offsets loOffset to hiOffset inclusive (one-offsets) in the
file. Written for NCYC people.

If used on sorted/imputed file, can pass result through qualitiesToNumbers.pl.

=head1 AUTHOR

David Carter, dmc@sanger.ac.uk , August 2006.

=head1 SEE ALSO

    splitLayoutByStrain.pl
    pasteLayoutRegions.pl
    bin/createRDNAFiles

=cut

use Utility;
use strict;

if (@ARGV != 3) {
    die("Usage: $0 lo hi layout.gz|sorted.gz|imputed.gz");
}

my ($lo,$hi,$f) = @ARGV;
my $fh = openToRead($f);
my $off = 0;
my @readAt;
my $active = 0;
my $chr;
if ($f =~ /sorted/ || $f =~ /imputed/) {
    my $hdr = <$fh>;
    print $hdr;
}
while (<$fh>) {
    if ($active) {
	if (/^[A-Z]/) {
	    $off ++;
	    last if ($off > $hi);
	} 
	print;
    } elsif (/^>REF (\S+)/) {
	$chr = $1;
    } elsif (/^>(\S+) (\d+)/) {
	my ($read,$pos) = ($1,$2);
	unless ($read eq 'REF') {
	    $readAt[$pos] = $read;
	}
    } elsif (/^[A-Z]/) {
	$off ++;
	last if ($off > $hi);
        if ($off == $lo) {
	    $active = 1;
	    if (defined $chr) {
		print ">REF $chr $off\n";
	    }
	    foreach my $k (0 .. $#readAt) {
		if (substr($_,2*$k+2,1) =~ /\S/) {
		    printf(">%s %d\n",$readAt[$k],$k);
		}
	    }
	    print;
	}
    } elsif ($active && /^[^>]/) {
	print;
    }
}

