#!/usr/bin/env perl

use Utility;
use strict;

# Input: sorted.gz

my $VALS_ONLY = 0;

my $fh = openToRead($ARGV[0]);
my @buf;
my $ln=1;
<$fh>; # header
while (<$fh>) {
    $ln ++;
    chomp;
    s/(.)./\1/g if ($VALS_ONLY); # for values only
    #tr/a-z/A-Z/;
    #s/[0-9=]/-/g;
    if (/^>/) {
	# ignore
    } elsif (isPolyLine($_)) {
	push @buf,$_;
    } elsif (@buf) {
	printBuffer($ln,@buf);
	@buf = ();
    }
}

sub printBuffer {
    my ($line,@buf) = @_;
    my $len = scalar(@buf);
    return unless ($len > 1);
    foreach my $line (@buf) {
	my $hasGap = 0;
	my $llen = length($line);
	for (my $i=0; $i<$llen; $i+=2) {
	    if (substr($line,$i,1) !~ /[ A-Za-z]/) {
		printf("%d %s\n%s\n\n",$ln-$len,$len,join("\n",@buf));
		return;
	    }
	}
    }
}

sub isPolyLine {
    my ($line) = @_;
    my $r;
    my $len = length($line);
    my $inc = $VALS_ONLY ? 1 : 2;
    for (my $i=0; $i<$len; $i+=$inc) {
	my $v = substr($line,$i,1);
	# Note: hard gap does NOT qualify us as polymorphic,
	# else we get huge regions that it's slow and unsafe
	# to realign.
	if ($v ne ' ' && $v ne '=') { 
	    $v =~ tr/a-z/A-Z/;
	    $v = '-' if ($v !~ /[A-Z]/);
	    if (defined $r) {
		return 1 unless ($v eq $r);
	    } else {
		$r = $v;
	    }
	}
    }
    return 0;
}

    
