#!/usr/bin/env perl

use Utility;
use strict;

my $TOP_DIR = '/lustre/cbi4/work1/sgrp/g1k';
my $DATA_DIR = "$TOP_DIR/DATA";

my (%size,%count,%minAge,%maxAge);
foreach my $f (glob("$DATA_DIR/*/*/*/*/*.fastq")) {
    my (undef,undef,$lib,$indiv,$proj) = reverse(split("/",$f));
    my $sz = -s $f;
    my $key = "$proj/$indiv/$lib";
    $size{$key} += $sz;
    $count{$key} ++;
    my $age = -M $f;
    $minAge{$key} = min($age,$minAge{$key});
    $maxAge{$key} = max($age,$maxAge{$key});
}
my $gig = 1048576*1024;
my $fac = 0.42; # estimated bases per byte of fastq file
print "FastqGB = gigabases of sequences, estimated at 0.42 bases per byte\n";
print "      N = number of lanes\n";
print "FastqAv = FastqGB/N\n";
print "MaxAge, MinAge = max/min fastq file ages in days\n\n";
printf("%7s %3s %7s %5s %7s %3s %7s %6s %6s\n",
       "Project","Pop","Indiv","Lib",
       "FastqGB",
       "N",
       "FastqAv",
       "MaxAge","MinAge");
foreach my $key (sort keys %count) {
    my $n = $count{$key};
    my $sum = $fac*$size{$key}/$gig;
    my $av = $sum/$n;
    printf("%7s %3s %7s %5s %7.3f %3d %7.3f %6.2f %6.2f\n",
	   split(/[-\/]/,$key),$sum,$n,$sum/$n,$maxAge{$key},$minAge{$key});
}
