#!/usr/bin/env perl

use Utility;
use File::Basename;
use strict;

filterGBDB(@ARGV);

sub filterGBDB {
    my ($chr,$srcD,$tgtD) = @_;
    if (@_ != 3 || ! -d $srcD || -d $tgtD) {
	die("Usage: $0 chr srcD tgtD, where srcD exists and tgtD does not");
    }
    my $lsrcD = length($srcD);
    my $n = 0;
    foreach my $f (glob("$srcD/*")) {
	next if (-d $f);
	my $g = $tgtD . substr($f,$lsrcD);
	$n += filterGBDB1($chr,$f,$g);
    }
    print "Created $n files in $tgtD\n";
    print "Consider: sgrp.pl cmgb $tgtD\n";
}

sub filterGBDB1 {
    my ($chr,$f,$g) = @_;
    if ($f =~ /\.fa/) {
	filterFA($chr,$f,$g);
    } elsif ($f =~ /\.conf/) {
	copyFile($f,$g);
    } elsif ($f =~ /\.gff/) {
	if ($f =~ /_snp/) {
	    my $b = basename($f);
	    if ($b eq "${chr}_snps.gff" || $b eq "${chr}_snpdens.gff") {
		copyFile($f,$g);
	    }
	} else {
	    filterGFF($chr,$f,$g);
	}
    } else {
	print STDERR "WARNING: ignoring $f\n";
    }
    if (-s $g) {
	return 1;
    }
    if (-f $g) {
	print STDERR "WARNING: deleting empty $g\n";
	unlink($g);
    }
    return 0;
}

sub filterFA {
    my ($chr,$f,$g) = @_;
    my $fh = openToRead($f);
    my $gh = openToWrite($g);
    my $pat = ">$chr";
    my $lpat = length($pat);
    my $active;
    while (<$fh>) {
	if (/^>/) {
	    $active = (substr($_,0,$lpat) eq $pat);
	}
	print $gh $_ if ($active);
    }
}

sub filterGFF {
    my ($chr,$f,$g) = @_;
    my $fh = openToRead($f);
    my $gh = openToWrite($g);
    my $lchr = length($chr);
    while (<$fh>) {
	if (substr($_,0,$lchr) eq $chr) {
	    print $gh $_;
	}
    }
}
	    
sub copyFile {
    my ($f,$g) = @_;
    my $fh = openToRead($f);
    my $gh = openToWrite($g);
    while (<$fh>) {
	print $gh $_;
    }
}
    
