#!/usr/bin/env perl

use Utility;
use FileHandle;
use strict;

autoflush STDOUT 1;

# From readsToDiscard.pl:
my $discardFile = shift @ARGV;
my $fh = openToRead($discardFile);
my %DISCARD;
while (<$fh>) {
    chomp;
    $DISCARD{$_} = 1;
}

foreach my $d (@ARGV) {
    my $tgt = "$d/layout.gz";
    my $src = "$d/orig_layout.gz";
    if (-s $tgt && ! -s $src) {
	rename($tgt,$src);
    }
    if (-s $src && ! -s $tgt) {
	filterLayoutByReads($src,$tgt);
    }
}

sub filterLayoutByReads {
    my ($f,$g) = @_;
    my $fh = openToRead($f);
    my %posRead; # current read at each position
    my %dependsOn;
    my %posIter; # iteration no. for read at each position
    my %discard;
    my $recalc;
    while (<$fh>) {
	chomp;
	if (/^>REF /) {
	    print "$_\n" if (/0000$/);
	    1; # nothing
	} elsif (/^>(\S+) (\d+) (\d+)/) {
	    my ($r,$pos,$iter) = ($1,$2,$3);
	    $r =~ s/:.*//;
	    $posRead{$pos} = $r;
	    $posIter{$pos} = $iter;
	    if ($DISCARD{$r} &&  !$discard{$r}) {
		$discard{$r} = 1;
		print "Original discard: $r\n";
	    }
	    $recalc = 1;
	} elsif ($recalc && /^[^-A-Z]/ && length($_) <= 100) {
	    $recalc = 0;
	    my $minIter = 1000;
	    my @minIterReads;
	    foreach my $pos (keys %posIter) {
		my $v = substr($_,($pos+1)*2,1);
		if ($v eq ' ' || $v eq '' || !defined $v) {
		    delete $posIter{$pos};
		    delete $posRead{$pos};
		} else {
		    my $minIter2 = min($minIter,$posIter{$pos});
		    if ($minIter2 < $minIter) {
			$minIter = $minIter2;
			@minIterReads = ($posRead{$pos});
		    } elsif ($minIter2 == $minIter) {
			push @minIterReads,$posRead{$pos};
		    }
		}
	    }
	    foreach my $pos (keys %posIter) {
		foreach my $pos (keys %posIter) {
		    unless ($posIter{$pos} == $minIter) {
			my $r = $posRead{$pos};
			foreach my $rr (@minIterReads) {
			    $dependsOn{$r}{$rr} = 1 unless ($r eq $rr);
			}
		    }
		}
	    }
	}
    }

    # Propagate discards
    my $going = 1;
    while ($going) {
	$going = 0;
      TRY:
	foreach my $r (keys %dependsOn) {
	    unless ($discard{$r}) {
		foreach my $rr (keys %{$dependsOn{$r}}) {
		    next TRY unless ($discard{$rr});
		}
		$discard{$r} = 1;
		$going = 1;
		my @from = keys %{$dependsOn{$r}};
		print "Propagated discard: $r from @from\n";
	    }
	}
    }
    
    # Re-read and print non-discarded
    $fh = openToRead($f);
    my $gh = openToWrite($g);
    my %discardPos;
    while (<$fh>) {
	chomp;
	if (/^>REF /) {
	    print $gh "$_\n";
	} elsif (/^>(\S+) (\d+)/) {
	    my ($r,$pos) = ($1,$2);
	    $r =~ s/:.*//;
	    if ($discard{$r}) {
		$discardPos{$pos} = 1;
	    } else {
		print $gh "$_\n";
		delete $discardPos{$pos};
	    }
	} else {
	    my $len = length($_);
	    my $posMax = $len/2-2;
	    foreach my $pos (keys %discardPos) {
		if ($pos <= $posMax) {
		    substr($_,($pos+1)*2,2) = '  ';
		}
	    }
	    my $hasNuc;
	    for (my $col=0; !$hasNuc && $col<$len; $col+=2) {
		my $v = substr($_,$col,1);
		# We do want to keep lines starting a-z, so
		# we have a complete record of transposon insertions
		if ($v =~ /[A-Za-z]/) {
		    print $gh "$_\n";
		    last;
		}
	    }
	}
    }
    
}
	    
