#!/usr/bin/env perl

use Utility;
use strict;

foreach my $d (@ARGV) {
    fixYGPMInsertions($d);
}

sub fixYGPMInsertions {
    my ($d) = @_;
    my $f = "$d/layout.gz";
    my $g = "$d/layout2.gz";
    return unless (-s $f && stamp($g));
    my $pRegs = ygpmInsertions($f);
    fixYGPMInsertions1($f,$pRegs,$g);
    unstamp($g);
}

sub ygpmInsertions {
    my ($f) = @_;
    my $fh = openToRead($f);
    my $ln = 0;
    my %isYGPM;
    my ($lo,$hi,@regs);
  LINE:
    while (<$fh>) {
	$ln ++;
	if (/^>(\S+) (\d+)/) {
	    my ($read,$pos) = ($1,$2);
	    unless ($read eq 'REF') {
		my $col = ($pos+1)*2;
		if ($read =~ /^YGPM/) {
		    $isYGPM{$col} = 1;
		} else {
		    delete $isYGPM{$col};
		}
	    }
	} else {
	    my $found = 0;
	    foreach my $col (keys %isYGPM) {
		if (substr($_,$col,1) =~ /[A-Za-z]/) {
		    $found = 1;
		    last;
		}
	    }
	    next LINE unless ($found);
	    my $len = length($_);
	    for (my $col=2; $col<$len; $col+=2) {
		unless ($isYGPM{$col}) {
		    if (substr($_,$col,1) =~ /[A-Za-z]/) {
			if (defined $hi) {
			    push @regs,[$lo,$hi] if ($hi-$lo < 100 && $hi-$lo>4);
			    $lo = $hi = undef;
			}
			next LINE; # NOT YGPM-only
		    }
		}
	    }
	    $lo = $ln unless (defined $lo);
	    $hi = $ln;
	}
    }
    if (defined $hi) {
	push @regs,[$lo,$hi] if ($hi-$lo < 100 && $hi-$lo>4);
    }
    return \@regs;
}	

sub fixYGPMInsertions1 {
    my ($f,$pRegs,$g) = @_;
    my $fh = openToRead($f);
    my $gh = openToWrite($g);
    my $ln = 0;
    while (<$fh>) {
	$ln++;
	while (@$pRegs && $pRegs->[0][1] < $ln) {
	    shift @$pRegs;
	}
	print $gh $_ unless (@$pRegs && $pRegs->[0][0] <= $ln);
    }
}
