#!/usr/bin/env perl

use Utility;
use strict;

my ($pCereNames,$pCere) = readMatrix("cere.alpha");
my ($pParaNames,$pPara) = readMatrix("para.alpha");

# Order is: cere para mika kudr baya (so actually five species now!)
my @spDist = ([0,0.145,0.241,0.284,0.329],
	      [0.145,0,0.220,0.263,0.308],
	      [0.241,0.220,0,0.276,0.318],
	      [0.284,0.263,0.276,0.0.318],
	      [0.329,0.308,0.318,0.318,0]);

my $nc = scalar(@$pCere);
my $np = scalar(@$pPara);
my @matrix;
my $sz = $nc+$np+scalar(@spDist)-2;
for (my $i=0; $i<$sz; $i++) {
    my $spi = $i < $nc ? 0 : $i < $nc+$np ? 1 : $i - ($nc+$np) + 2;
    for (my $j=0; $j<$sz; $j++) {
	my $spj = $j < $nc ? 0 : $j < $nc+$np ? 1 : $j - ($nc+$np) + 2;
	my $d;
	if ($spi != $spj || $spi > 1) {
	    $d = $spDist[$spi][$spj];
	} elsif ($spi == 0) {
	    $d = $pCere->[$i][$j];
	} else {
	    $d = $pPara->[$i-$nc][$j-$nc];
	}
	$matrix[$i][$j] = $d;
    }
}
my $gh = openToWrite("all.alpha");
print $gh "$sz\n";
my @spList = qw(C P M K B);
for (my $i=0; $i<$sz; $i++) {
    my $spi = $i < $nc ? 0 : $i < $nc+$np ? 1 : $i - ($nc+$np) + 2;
    my $name = $spList[$spi];
    if ($spi == 0) {
	#$name .= $pCereNames->[$i];
	$name = '.';
    } elsif ($spi == 1) {
	#$name .= $pParaNames->[$i-$nc];
	$name = '.';
    }
    print $gh $name;
    for (my $j=0; $j<$sz; $j++) {
	printf($gh " %12.10f",$matrix[$i][$j]);
    }
    print $gh "\n";
}

sub readMatrix {
    my ($f) = @_;
    my $fh = openToRead($f);
    <$fh>;
    my (@names,@mat);
    while (<$fh>) {
	chomp;
	my ($name,@row) = split;
	push @names,$name;
	push @mat,\@row;
    }
    return (\@names,\@mat);
}
