#!/usr/bin/env perl

=head1 NAME

gbrowseColourKey.pl

=head1 SYNOPSIS

gbrowseColourKey.pl [-p paletteFile] [-o outputPNG] [specFile]

=head1 DESCRIPTION

Given a paletteFile (default:
~/sgroot/misc/colours/gbrowsePalette.txt) and a specFile (default:
~/sgroot/misc/colours/gbrowseSpec.txt), outputs a .png file (default:
~/sgroot/misc/colours/gbrowseColours.png).

=head1 AUTHOR

David Carter, C<dmc@sanger.ac.uk>

=head1 SEE ALSO

       

=cut

use Utility;
use Getopt::Std;
use GD;;
use GD::Simple;
use strict;

my %OPT;
getopts('p:o:',\%OPT);

my $dir = "$ENV{'HOME'}/sgroot/misc/colours";

my $paletteFile = $OPT{p} || "$dir/gbrowsePalette.txt";
my $specFile = $ARGV[0] || "$dir/gbrowseSpec.txt";
my $gh = openToWrite($OPT{o} || "$dir/gbrowseColours.png");
binmode($gh);

my %rgb;
my $fh = openToRead($paletteFile);
while (<$fh>) {
    chomp;
    my ($name,$r,$g,$b) = split;
    $rgb{$name} = [hex($r),hex($g),hex($b)];
}

$fh = openToRead($specFile);
my @lines;
my ($xsz,$ysz) = (60,10);
while (<$fh>) {
    chomp;
    push @lines,$_;
    my ($name,$text) = split(/\s+/,$_,2);
    if ($name) {
	die("Colour $name is not defined in $paletteFile") unless ($rgb{$name});
	$xsz = max($xsz,120+10*length($text));
	$ysz += 50;
    } elsif ($text) {
	$ysz += 50;
	$xsz = max($xsz,70+10*length($text));
    } else {
	$ysz += 20;
    }
}
$ysz += 50; # to allow for bottom of final rectangle

my $im = new GD::Simple($xsz,$ysz);

my $y = 10;
$im->font('Tahoma:bold');
$im->fontsize(12);
foreach my $line (@lines) {
    my ($name,$text) = split(/\s+/,$line,2);
    if ($name) {
	my ($r,$g,$b) = @{$rgb{$name}};
	$im->bgcolor($im->colorAllocate($r,$g,$b));
	$im->rectangle(60,$y,110,$y+50);
	$im->moveTo(120,$y+30);
	$im->string($text);
	$y += 50;
    } elsif ($text) {
	$im->moveTo(60,$y+30);
	$im->string($text);
	$y += 50;
    } else {
	$y += 20;
    }
}

print $gh $im->png;
