#!/usr/bin/env perl

use Utility;
use strict;

# Run from $SGRW/release3/cere

geneCoverageTable();

sub geneCoverageTable {
    my $pGenes = readGenes("ref/genomeAligned.gff");
    my $gh = openToWrite("match/geneCoverageTable.txt");
    foreach my $chr (sort keys %$pGenes) {
	my @genes = @{$pGenes->{$chr}};
	my $f = "match/$chr/coverageABI.gz";
	report("$f: reading");
	my $fh = openToRead($f);
	chomp (my $hdr = <$fh>);
	my @strains = split(" ",$hdr);
	my (@sum,@min,@max);
	my $jMin = 0;
      LOOP:
	while (<$fh>) {
	    chomp;
	    my ($lo,$hi,@depths) = split;
	    my $newJMin;
	    foreach my $j ($jMin .. $#genes) {
		my ($gLo,$gHi,$gName) = @{$genes[$j]};
		last if ($gLo > $hi);
		if ($gHi < $lo) {
		    $newJMin = $j+1;
		} else {
		    my $sz = min($hi,$gHi) - max($lo,$gLo) + 1;
		    if ($sz > 0) {
			my $len = $gHi-$gLo+1;
			foreach my $i (0 .. $#strains) {
			    my $d = $depths[$i];
			    $min[$j][$i] = min($min[$j][$i],$d);
			    $max[$j][$i] = max($max[$j][$i],$d);
			    $sum[$j][$i] += $d*$sz;
			}
		    }
		}
	    }
	    $jMin = $newJMin;
	}
	foreach my $j (0 .. $#genes) {
	    my ($gLo,$gHi,$gName) = @{$genes[$j]};
	    my $len = $gHi-$gLo+1;
	    foreach my $i (0 .. $#strains) {
		printf($gh "%s %s %d %4.2f %d\n",
		       $gName,$strains[$i],$min[$j][$i],$sum[$j][$i]/$len,$max[$j][$i]);
	    }
	}
	report();
    }
}

sub readGenes {
    my ($f) = @_;
    my %genes;
    my $fh = openToRead($f);
    while (<$fh>) {
	chomp;
	my ($chr,undef,$type,$lo,$hi,undef,undef,undef,$anno) = split;
	if ($type eq 'gene' && $chr =~ /^chr..$/) {
	    my @fields = split(/;/,$anno);
	    my %field;
	    foreach my $eqn (@fields) {
		my ($key,$val) = split(/=/,$eqn,2);
		$field{$key} = $val;
	    }
	    my $name;
	    foreach my $key (qw(gene Name ID)) {
		if (defined $field{$key}) {
		    $name = $field{$key};
		    last;
		}
	    }
	    if ($name) {
		push @{$genes{$chr}},[$lo,$hi,$name];
	    }
	}
    }
    return \%genes;
}
	    
