#!/usr/bin/env perl

# Run in e.g. release dir

my $CERE = $ENV{CERE} || die("CERE not set");
my $PARA = $ENV{PARA} || die("PARA not set");
my $MORE = $ENV{MORE} || die("MORE not set");

my %isStrain = map {$_ => 1} ('REF',split(" ",$CERE),split(" ",$PARA),split(" ",$MORE));

my %isSp = map {$_ => 1} qw(cere para baya);

my %isChr = map {$_ => 1} qw(chrm scplasm1);
foreach my $n (1 .. 16) {
    my $chr = sprintf("chr%2d",$n);
    $chr =~ tr/ /0/;
    $isChr{$chr} = 1;
}

my %genName;
foreach my $d (@ARGV) {
    generalizeFileNames($d);
}
foreach my $name (sort keys %genName) {
    printf("%4d %s\n",$genName{$name},$name);
}

sub generalizeFileNames {
    my ($f) = @_;
    if (-d $f) {
	foreach my $g (glob("$f/*")) {
	    generalizeFileNames($g);
	}
    } else {
	my @comps = split("/",$f);
	foreach my $i (0 .. $#comps) {
	    my $comp = $comps[$i];
	    my ($pfx) = split(/[-\._]/,$comp);
	    my $lp = length($pfx);
	    if ($isStrain{$comp}) {
		$comp = '$STR';
	    } elsif ($isSp{$comp}) {
		$comp = '$SP';
	    } elsif ($isChr{$comp}) {
		$comp = '$CHR';
	    } elsif ($pfx eq 'translations') {
		$comps[$i+1] =~ s/^(.*)\.gz/\$GENE.gz/;
	    } elsif ($pfx eq $comp) {
		# nothing, to save time
	    } elsif ($isStrain{$pfx}) {
		$comp = '$STR' . substr($comp,$lp);
	    } elsif ($isSp{$pfx}) {
		$comp = '$SP' . substr($comp,$lp);
	    } elsif ($isChr{$pfx}) {
		$comp = '$CHR' . substr($comp,$lp);
	    }
	    $comps[$i] = $comp;
	}
	$genName{join("/",@comps)} ++;
    }
}
