#!/usr/bin/env perl

# Args: e.g. cere methc CERE

use Utility;
use strict;

if (@ARGV > 3) {
    goodReadPairFates(@ARGV);
} else {
    die("Usage: $0 cere methc CERE str1 str2 ... \n");
}

sub goodReadPairFates {
    my ($sp,$meth,$strSet,@strains) = @_;
    die("No value found for \$$strSet\n") unless (@strains);
    my $topDir = "$ENV{SGR}/match/$strSet/$sp/$meth";
    my @mDirs = glob("$topDir/[0-9][0-9][0-9]");
    my $mDir = $mDirs[-1];
    my $md0 = "$ENV{SGR}/match/$strSet/$sp/$meth/000";
    my $pAccepted0 = {};
    my $pAccepted1 = {};
    foreach my $md (@mDirs) {
	my $pAccepted = $md eq $md0 ? $pAccepted0 : $pAccepted1;
	foreach my $f (glob("$md/p?/*/alilines.nola.txt")) {
	    my $pAcc = getReads($f);
	    foreach my $r (keys %$pAcc) {
		$pAccepted->{$r} = 1;
	    }
	}
    }
    print <<EOF;
# Status is:
#   0 for not aligned,
#   1 for aligned but not passed
#   2 for passed but not accepted
#   3 for accepted on 000
#   4 for accepted subsequently
EOF
    printf("\n%13s","Strain");
    foreach my $i (0 .. 4) {
	foreach my $j (0 .. $i) {
	    printf(" %5s","$i:$j");
	}
    }
    print " Total";
    foreach my $i (0 .. 4) {
	printf(" %5d",$i);
    }
    print "\n";
    my %propLine;
    foreach my $str (@strains) {
	my $pPassed = getReads("$mDir/align/$str/alilines.pass.txt");
	my $pAligned = getReads("$mDir/align/$str/alilines.all.txt");
	my $pPairs = goodReadPairs("$ENV{SGR}/read/$str/renamed.wm");
	my @count;
	my $gh = openToWrite("$topDir/grpf/$str.txt");
	foreach my $name (sort @$pPairs) {
	    my $stP = getStatus("$name.p1k",$pAccepted1,$pAccepted0,$pPassed,$pAligned);
	    my $stQ = getStatus("$name.q1k",$pAccepted1,$pAccepted0,$pPassed,$pAligned);
	    my @pair = sort {$b <=> $a} ($stP,$stQ);
	    $count[$pair[0]][$pair[1]] ++;
	    print $gh "$name $stP $stQ\n";
	}
	$gh->close();
	printf("%13s",$str);
	my $sum = 0;
	my @count1;
	foreach my $i (0 .. 4) {
	    foreach my $j (0 .. $i) {
		my $c = $count[$i][$j];
		printf(" %5d",$c);
		$sum += $c;
		$count1[$i] += $c;
		$count1[$j] += $c;
	    }
	}
	printf(" %5d",$sum);
	foreach my $i (0 .. 4) {
	    printf(" %5d",$count1[$i]);
	}
	print "\n";
	my $line = sprintf("%13s",$str);
	foreach my $i (0 .. 4) {
	    foreach my $j (0 .. $i) {
		my $c = $count[$i][$j];
		$line .= sprintf(" %5.3f",$c/$sum);
	    }
	}
	$line .= sprintf(" %5s","");
	foreach my $i (0 .. 4) {
	    $line .= sprintf(" %5.3f",$count1[$i]/(2*$sum));
	}
	$line .= "\n";
	$propLine{$str} = $line;
    }
    printf("\n%13s","Strain");
    foreach my $i (0 .. 4) {
	foreach my $j (0 .. $i) {
	    printf(" %5s","$i:$j");
	}
    }
    print "      ";
    foreach my $i (0 .. 4) {
	printf(" %5d",$i);
    }
    print "\n";
    foreach my $str (@strains) {
	print $propLine{$str};
    }
}

sub getReads {
    my ($f) = @_;
    my %read;
    my $fh = openToRead($f);
    while (<$fh>) {
	chomp;
	my (undef,undef,$r) = split;
	$read{$r} = 1;
    }
    return \%read;
}
       
sub goodReadPairs {
    my ($f) = @_;
    my @good;
    my $fh = openToRead($f);
    my %nGood;
    while (<$fh>) {
	chomp;
	my ($read,undef,@spans) = split;
	if (@spans) {
	    $read =~ s/\.[pq]1k$//;
	    $nGood{$read} ++;
	}
    }
    foreach my $r (sort keys %nGood) {
	push @good,$r if ($nGood{$r} == 2);
    }
    return \@good;
}

sub getStatus {
    my ($r,$pAcc1,$pAcc0,$pPass,$pAli) = @_;
    return 4 if ($pAcc1->{$r});
    return 3 if ($pAcc0->{$r});
    return 2 if ($pPass->{$r});
    return 1 if ($pAli->{$r});
    return 0;
}
