#!/usr/bin/env perl

# Run in NNN dir OR in release dir. When done, move snps dir to
# e.g. ~/myperl/htdocs/snps. Args: species name(s)

use Utility;
use File::Basename;
use Cwd;
use strict;

my $CHR_PATTERN = '*';

my $IS_RELEASE_DIR = 1; # adjust as needed

foreach my $sp (@ARGV) {
    my $pat = $IS_RELEASE_DIR ? "$sp/match/$CHR_PATTERN" : "p?/$CHR_PATTERN";
    foreach my $d (glob($pat)) {
	if ($IS_RELEASE_DIR) {
	    my $fi = "$d/imputed.gz";
	    my $fs = "$d/sequenced.gz";
	    my $tGFFDir = "$sp/ref/translatedGFF"; # create with translateGFF.pl
	    imputed2html($sp,$fi,$fs,basename($d),$tGFFDir);
	} else {
	    my $tGFFDir = "$ENV{SGR}/ref/$sp/translatedGFF";  # create with translateGFF.pl
	    foreach my $suf (qw(z y m)) {
		my $fi = "$d/imputed_$suf.gz";
		if (-s $fi) {
		    my $fs = "$d/sorted_$suf.gz";
		    imputed2html($sp,$fi,$fs,basename($d),$tGFFDir);
		    last;
		}
	    }
	}
    }
}

sub imputed2html {
    my ($sp,$fi,$fs,$chr,$tGFFDir) = @_;
    my $tGFF = "$tGFFDir/$chr.txt";
    my $tGFFSw = -s $tGFF ? "-G $tGFF" : "";
    my $ih = openFromPipe("alicat.pl $tGFFSw -ys $fi");
    my $sh = openFromPipe("alicat.pl $tGFFSw -ys $fs");
    my (@ipad,@spad,@ibuf,@sbuf);
    my $hdr = <$ih>;
    <$sh>; # also header
    my $nFile = 0;
    my $dir = "snps/$sp/$chr";
    my $x = "$dir/index.html";
    if (stamp($x)) {
	#report("$dir: creating");
	mkdir_p($dir);
	my $xh = openToWrite($x);
	print $xh "<html><head><title>$chr</title></head><body><ul>\n";
	while (<$ih>) {
	    push @ibuf,$_;
	    my $line = <$sh>;
	    push @sbuf,$line;
	    if (scalar(@ibuf) == 1000) {
		my $g = createOutputFile($nFile,\@ipad,\@spad,\@ibuf,\@sbuf,$dir,$chr,$hdr,$xh);
		$nFile ++;
		@ipad = @ibuf[-10..-1];
		@ibuf = ();
		@spad = @sbuf[-10..-1];
		@sbuf = ();
	    }
	}
	if (@ibuf > 10) {
	    my $g = createOutputFile($nFile,\@ipad,\@spad,\@ibuf,\@sbuf,$dir,$chr,$hdr,$xh);
	}
	print $xh "</ul></body></html>\n";
	$xh->close();
	unstamp($x);
	#report();
    }
}

sub createOutputFile {
    my ($nFile,$pIPad,$pSPad,$pIBuf,$pSBuf,$dir,$chr,$hdr,$xh) = @_;
    my $g = createOutputFileName($dir,$nFile);
    report("$g: creating");
    mkdir_p(dirname($g));
    my $gh = openToPipe("alicat2html.pl -n 2 -c $chr -o $g");
    #my $gh = openToWrite("$g.tmp");
    printPreviousAndNext($dir,$nFile,$gh);
    print $gh $hdr;
    foreach my $i (0 .. $#$pIPad) {
	print $gh $pSPad->[$i];
	print $gh $pIPad->[$i];
    }
    foreach my $i (0 .. $#$pIBuf) {
	print $gh $pSBuf->[$i];
	print $gh $pIBuf->[$i];
    }
    printPreviousAndNext($dir,$nFile,$gh);
    $gh->close();
    #`alicat2html.pl -n 2 -c $chr -o $g $g.tmp`;
    report();
    printf($xh "<li><a href=%s>%s:%s .. %s</a>\n",
	   substr($g,-13),$chr,substr($pIBuf->[0],0,11),substr($pIBuf->[-1],0,11));
    return $g;
}

sub createOutputFileName {
    my ($dir,$nFile) = @_;
    my $quad = sprintf("%4d",$nFile);
    $quad =~ tr/ /0/;
    return sprintf("%s/%2s/%2s000.html",$dir,substr($quad,0,2),substr($quad,2));
}

sub printPreviousAndNext {
    my ($dir,$nFile,$gh) = @_;
    # If nFile ends in 00, need to go up a dir.
    my $g = createOutputFileName($dir,$nFile);
    if ($nFile > 0) {
	my $gPrev = relativizePath(createOutputFileName($dir,$nFile-1),$g);
	print $gh "<a href=$gPrev>Previous chunk</a>\n";
    }
    my $gNext = relativizePath(createOutputFileName($dir,$nFile+1),$g);
    print $gh "<a href=$gNext>Next chunk</a>\n";
}

sub relativizePath {
    my ($path,$wrt) = @_;
    my @comps1 = split(/\//,$path);
    my @comps2 = split(/\//,$wrt);
    while (@comps1 && @comps2 && $comps1[0] eq $comps2[0]) {
	shift @comps1;
	shift @comps2;
    }
    my $ans = ("../" x (scalar(@comps2)-1)) . join("/",@comps1);
}

