#!/usr/bin/env perl

# Run in writing/imputing/tabs12
# Take input from output of imputingTable12.pl, where we have run:

# imputingTable12.pl > ~/writing/imputing/tabs12/abi_table
# imputingTable12.pl Solexa > ~/writing/imputing/tabs12/solexa_table

use Utility;
use strict;

createGraphs('abi_table','');
createGraphs('solexa_table','sol');


# For every allele-freq range midpoint, number of
# instances (both impu and conf/corr) of that range.
# Should sum to 200 million.
my $gh0 = openToWrite("counts12.xy");

# For every range midpoint, log_10 of MF imputed error rate
# (as a proportion, not per million)
my $gh1 = openToWrite("rates12_mf_i.xy");

# For allele-freq = 0 and 1, log_10 of null error rate over
# all instances, regardless of allele freq.
my $gh2 = openToWrite("rates12_null_c.xy");

# As gh1, but conf/corr not imputed
my $gh3 = openToWrite("rates12_mf_c.xy");

# For every range midpoint, cumulative number of MF errors
# (both impu and conf/corr) up to and including this range.
my $gh4 = openToWrite("cumul_errs12.xy");

my $ce = 0;
my $ncSum = 0;
my $nullSum = 0;
while (<>) {
    chomp;
    last if (/caption/);
    if (/^ +1 \& /) {
	substr($_,0,5) = "1 to ";
    }
    if (/ to /) {
	s/\&//g;
	#print "$_\n";
	my @a = split;
	my ($lo,undef,$hi,$ni,$mfi,$nc,$null,$mfc) = @a;
	my $af = ($lo+$hi)/2; # allele freq is midpoint of range
	my $n = $ni+$nc; # count is impuŧed count + conf/corr count
	#print "$n $mfi $null $mfc\n";
	printf($gh0 "%8.6f %8.6f\n",$af,$n);
	printf($gh1 "%8.6f %8.6f\n",$af,log(($mfi || 1)/1000000)/log(10));
	#printf($gh2 "%8.6f %8.6f\n",$af,log(($null || 1)/1000000)/log(10));
	$ncSum += $nc;
	$nullSum += $null*$nc;
	printf($gh3 "%8.6f %8.6f\n",$af,log(($mfc || 1)/1000000)/log(10));
	$ce += ($mfi*$ni+$mfc*$nc)/1000000;
	printf($gh4 "%8.6f %d\n",$af,$ce);
    }
}
foreach my $x (0,1) {
    printf($gh2 "%8.6f %8.6f\n",$x,log($nullSum/($ncSum*1000000))/log(10));
}

