#!/usr/bin/env perl

use Utility;
use strict;

# Create files in ~/writing/imputing/

# This output is now just used for imputingGraphs12.pl.

#  mkdir -p ~/writing/imputing/tabs12/abi ~/writing/imputing/tabs12/solexa
#  imputingTable12.pl > ~/writing/imputing/tabs12/abi/table
#  imputingTable12.pl Solexa > ~/writing/imputing/tabs12/solexa/table

# Need sum.txt in each cia* directory to have been created with
# sumCompareImputedAcross.pl

my $IS_SOLEXA = ($ARGV[0] eq 'Solexa');

if ($IS_SOLEXA) {
    chdir("$ENV{SGR}/match/FREG200_1S/freg");
} else {
    chdir("$ENV{SGR}/match/FREG200_1/freg"); # /FOR_TABLES_1_AND_2");
}

my @RANGES = qw(0:0.001 0.001:0.005 0.005:0.01 0.01:0.02 0.02:0.03
0.03:0.05 0.05:0.10 0.10:0.20 0.20:0.30 0.30:0.40 0.40:0.50 0.50:0.60
0.60:0.70 0.70:0.80 0.80:0.90 0.90:0.95 0.95:0.97 0.97:0.98 0.98:0.99
0.99:0.995 0.995:0.999 0.999:1 1);

my %DETECTABLE = qw(0:0.001 5.1
		    0.001:0.005 36.09
		    0.005:0.01 71.36
		    0.01:0.02 95.94
		    0.02:0.03 100
		    0.03:0.05 100
		    0.05:0.10 100
		    0.10:0.20 100
		    0.20:0.30 100
		    0.30:0.40 100
		    0.40:0.50 100);

printImputingTable(1);
printImputingTable(2);

sub printImputingTable {
    my ($id) = @_;
    print "\\begin{table}\n";
    if ($id == 1) {
	print <<\EOF;
\begin{tabular}{|c|c|c|c|c|c|c|c|c|} \hline
     MAF       &  \multicolumn{2}{c|}{Imputation} & \multicolumn{3}{c|}{Confirmation/correction} & \multicolumn{3}{c|}{SNP discovery} \\
    range      &   Count   &     MF &    Count  &  Null  &     MF &  Count & Present &      MF \\ \hline
EOF
    } else {
	print <<\EOF;
\begin{tabular}{|c|c|c|c|c|} \hline
     MAF range &   Count   &   Star & Window &     MF \\ \\hline
EOF
    }
    my @sum;
    my $fac = 1; # could be 1000
    foreach my $r (@RANGES) {
	my $sf = "cia_chrs_errs_complete_mac$r/sum.txt";
	unless (-s $sf) {
	    print "% WARNING: missing $sf\n";
	    next;
	}
	my $fh = openToRead($sf);
	my $det = $DETECTABLE{$r};
	my @val = ($r);
	$val[0] =~ s/:/ to /;
	$val[7] = $det if ($id == 1);
	my $cond = 1; # $r ne '0'
	while (<$fh>) {
	    chomp;
	    # Fields of @a are:
            # When "p" is final character of $a[0]:
	    #   0=type 1=count 2=proportion 3=null 4=star 5=window 6=MF 7=oracle
	    # When it isn't:
	    #   0=type 1=count 2=proportion        3=star 4=window 5=MF 6=oracle
            # But star and window fields may not be present ...
	    my @a = split;
	    if ($id == 1) {
		if ($a[0] eq 'N.m') {
		    $val[1] = int(0.5+$a[1]/$fac); # count of missing, per k
		    $sum[1] += $a[1] if ($cond);
		    $val[2] = int($a[-2]*1000000+0.5); # MF error rate per million
		    $sum[2] += $a[1]*$a[-2] if ($cond);
		} elsif ($a[0] eq 'N.p') {
		    $val[3] = int(0.5+$a[1]/$fac); # count of present, per k
		    $sum[3] += $a[1] if ($cond);
		    $val[4] = int($a[3]*1000000+0.5); # Null error rate per million
		    $sum[4] += $a[1]*$a[3] if ($cond); # error COUNT
		    $val[5] = int($a[-2]*1000000+0.5); # MF error rate per million
		    $sum[5] += $a[1]*$a[-2] if ($cond);
		} elsif ($a[0] eq 'm') { # SNP discovery
		    $val[6] = $a[1]+$a[-2];
		    $sum[6] += $val[6] if ($r ne '0');
		    $sum[7] += $val[6]*$val[7] if ($r ne '0');
		    $val[8] = 100*$a[-2]/$val[6];
		    $sum[8] += $val[6]*$val[8] if ($r ne '0');
		}
	    } elsif ($id == 2 && $a[0] eq 'N..') {
		$val[1] = int(0.5+$a[1]/$fac); # count of missing, per k
		$sum[1] += $a[1] if ($cond);
		unless ($IS_SOLEXA) {
		    $val[2] = int($a[3]*1000000+0.5); # Star error rate per million
		    $sum[2] += $a[1]*$a[3] if ($cond);
		    $val[3] = int($a[4]*1000000+0.5); # Window(100) error rate per million
		    $sum[3] += $a[1]*$a[4] if ($cond);
		}
		$val[4] = int($a[-2]*1000000+0.5); # MF error rate per million
		$sum[4] += $a[1]*$a[-2] if ($cond);
	    }
	}
	if ($id == 1) {
	    # Table 1: fields of @val are
	    # 0=MAFRange 1=ImpuCount 2=ImpuMFErrs 3=ConfCount 4=ConfNullErrs 5=ConfMFErrs
	    # 6=SNPCount 7=SNPPres 8=SNPMF
	    if ($r eq '0') {
		# we don't do this because 0 isn't a range now: we get 1 instead.
		printf("%14s & %9d & %6d & %9d & %6d & %6d &  \\multicolumn{3}{c|}{N/A}  \\\\ \\hline\n",@val);
	    } else {
		printf("%14s & %9d & %6d & %9d & %6d & %6d & %6d & %7.2f & %7.2f \\\\ \\hline\n",@val);
	    }
	} elsif ($id == 2) {
	    # Table 2 (nuc error rates per million): fields of @val are
	    # 0=MAFRange 1=Count 2=Star 3=Window 4=MF
	    printf("%14s & %9d & %6d & %6d & %6d \\\\ \\hline\n",@val);
	}
    }
    if ($id == 1) {
	printf("%14s & %9d & %6d & %9d & %6d & %6d & %6d & %7.2f & %7.2f \\\\ \\hline\n",
	       "All",
	       int(0.5+$sum[1]/$fac),int(0.5+1000000*$sum[2]/$sum[1]),
	       int(0.5+$sum[3]/$fac),int(0.5+1000000*$sum[4]/$sum[3]),int(0.5+1000000*$sum[5]/$sum[3]),
	       $sum[6],$sum[7]/$sum[6],$sum[8]/$sum[6]);
    } elsif ($id == 2) {
	printf("%14s & %9d & %6d & %6d & %6d \\\\ \\hline\n",
	       "All",
	       int(0.5+$sum[1]/$fac),int(0.5+1000000*$sum[2]/$sum[1]),
	       int(0.5+1000000*$sum[3]/$sum[1]),int(0.5+1000000*$sum[4]/$sum[1]));
    }
    print "\\end{tabular}\n\\caption{";
    if ($id == 1) {
	print "Nucleotide imputation and confirmation/correction error rates (per million) and SNP discovery rates (per cent)";
    } else {
	print "Nucleotide error rates per million for Star, Window and MF methods";
    }
    print "}\n\\end{table}\n\n";
}
