#!/usr/bin/perl

use Utility;
use File::Basename;
use Getopt::Std;
use strict;

my %OPT;
getopts('o',\%OPT);

chdir("$ENV{SGR}/match");

my @N_LIST = (3,6,12,24,50,100,200);
my @C_LIST = (0,1,2,4,8);

my $OMIT_SINGLETONS = $OPT{o};
if ($OMIT_SINGLETONS) {
    print "Omitting singletons\n";
} else {
    print "NOT omitting singletons\n";
}

my %sn;
printf("%3s %2s %6s %6s %6s %6s %6s %6s\n",
       qw(N C TP TN FP FN Sn Sp));
foreach my $n (@N_LIST) {
    foreach my $c (@C_LIST) {
	$sn{$n}{$c} = findSNPDiscoveryRates($n,$c);
    }
}
print "\n C ";
foreach my $n (@N_LIST) {
    printf(" & %6d",$n);
}
print " \\\\ \\hline\n";
foreach my $c (@C_LIST) {
    printf("%3.1f",$c || 0.5);
    foreach my $n (@N_LIST) {
	my $v = $sn{$n}{$c};
	if (defined $v) {
	    printf(" & %6.2f",$v);
	} else {
	    printf(" & %6s","");
	}
    }
    print " \\\\ \\hline\n";
}

sub findSNPDiscoveryRates {
    my ($n,$c) = @_;
    my $d = "FREG${n}_$c/freg";
    my $f0 = "$d/cia_chrs_errs_complete_mac0/cia.chr00.bout";
    my ($tn,$fp) = sumMFSNPs($n,$f0);

    my ($tp,$fn);
    foreach my $f (glob("$d/cia_chrs_errs_complete_mac*:*/cia.chr00.bout")) {
	my ($neg,$pos) = sumMFSNPs($n,$f);
	$tp += $pos;
	$fn += $neg;
    }
    my $sn = $tp/($tp+$fn || 1);
    my $sp = $tp/($tp+$fp || 1);
    printf("%3d %2d %6d %6d %6d %6d %6.2f %6.2f\n",
	   $n,$c,$tp,$tn,$fp,$fn,
	   100*$sn,100*$sp);
    return $sn > 0 ? 100*$sn : undef;
	
}

sub sumMFSNPs {
    my ($nHaps,$f) = @_;
    return (0,0) unless (-s $f);
    my $fh = openToRead($f);
    my $neg = 0;
    my $pos = 0;
    my $iPosMin = 2;
    while (<$fh>) {
	chomp;
	if ($OMIT_SINGLETONS && /^Method /) {
	    my @rangeTops = split;
	    foreach my $j (2 .. $#rangeTops) {
		if ($rangeTops[$j] >= 1/($nHaps+1)) {
		    $iPosMin = $j+1;
		    last;
		}
	    }
	}
	if (/^   m /) {
	    my @a = split;
	    foreach my $i (1 .. $iPosMin-1) {
		$neg += $a[$i];
	    }
	    foreach my $i ($iPosMin .. $#a) {
		$pos += $a[$i];
	    }
	}
    }
    return ($neg,$pos);
}
