#!/usr/bin/env perl

# Run on e.g. imputed_y.gz

use Utility;
use File::Basename;
use strict;

my $LMIN = 6; # to be worth printing

foreach my $f (@ARGV) {
    indelLengths($f);
}

sub indelLengths {
    my ($f) = @_;
    my $fh = openToRead($f);
    my $chr = basename(dirname($f));
    my $hdr = <$fh>;
    chomp $hdr;
    my @strains = ('REF',split(" ",substr($hdr,1)));
    my $refWasGap; # 0 for sequence, 1 for gap
    my @len; # length of indel so far
    my $ln = 0;
    while (<$fh>) {
	chomp;
	my @curGap;
	for (my $i=0; $i<=$#strains; $i++) {
	    my $v = substr($_,$i*2,1);
	    $curGap[$i] =  ($v eq '-' || $v eq '_' || $v eq '=');
	}
	$ln ++ unless ($curGap[0]);
	if ($curGap[0]) {
	    if ($refWasGap) {
		# Extending reference deletion
		foreach my $i (1 .. $#curGap) {
		    $len[$i] ++ unless ($curGap[$i]);
		}
	    } else {
		# Start reference sequence
		foreach my $i (1 .. $#len) {
		    if ($len[$i] >= $LMIN) {
			print "$chr $ln D $len[$i] $strains[$i]\n";
		    }
		    $len[$i] = $curGap[$i]; # 0 or 1
		}
	    }
	} elsif ($refWasGap) {
	    # Start reference gap
	    foreach my $i (1 .. $#len) {
		if ($len[$i] >= $LMIN) {
		    print "$chr $ln I $len[$i] $strains[$i]\n";
		}
		$len[$i] = 1-$curGap[$i]; # 0 or 1
	    }
	} else {
	    # Continue reference sequence
	    foreach my $i (1 .. $#curGap) {
		if (!$curGap[$i]) {
		    # End of strain deletion if any
		    if ($len[$i] >= $LMIN) {
			print "$chr $ln D $len[$i] $strains[$i]\n";
		    }
		    $len[$i] = 0;
		} else {
		    # Extension of strain deletion
		    $len[$i] ++;
		}
	    }
	}
	$refWasGap = $curGap[0];
    }
    my $sym = $refWasGap ? 'I' : 'D';
    foreach my $i (1 .. $#len) {
	if ($len[$i] >= $LMIN) {
	    print "$chr $ln $sym $len[$i] $strains[$i]\n";
	}
    }
}
	    
