#!/usr/bin/env perl

use Utility;
use strict;

autoflush STDOUT 1;

# If set, don't output lines consisting only of gaps.

my $IGNORE_GAP_LINES = 1;

foreach my $d (@ARGV) {
    joinImputed($d);
}

sub joinImputed {
    my ($d) = @_;
    my $fm = "$d/imputed_m.gz";
    my $fx = "$d/imputed_x.gz";
    my $g = "$d/imputed_y.gz";
    foreach my $f ($fm,$fx) {
	unless (-s $f) {
	    report("$f: missing or empty",1);
	    return;
	}
    }
    unless (stamp($g)) {
	report("$g: cannot stamp",1);
	return;
    }
    my $mh = openToRead($fm);
    my $xh = openToRead($fx);
    my $gh;
    my $mHdr = <$mh>;
    my $xHdr = <$xh>;
    if ($mHdr eq $xHdr) {
	$gh = openToWrite($g);
	print $gh $mHdr;
    } else {
	report("headers differ in $d/imputed_{m,x}.gz",1);
	return;
    }
    report("$g: creating");
    joinImputed1($mh,$xh,$gh);
    unstamp($g);
    report();
}

sub joinImputed1 {
    my ($mh,$xh,$gh) = @_;
    while (1) {
	my $mLine = <$mh>;
	my $xLine = <$xh>;
	if (!$mLine) {
	    if ($xLine) {
		print "WARNING: more lines in imputed_x than imputed_m\n";
	    }
	    last;
	} elsif (!$xLine) {
	    print "WARNING: more lines in imputed_m than imputed_x\n";
	    last;
	}
	chomp $mLine;
	chomp $xLine;
	my $mLen = length($mLine);
	my $xLen = length($xLine);
	unless ($mLen == $xLen) {
	    print "WARNING: lengths differ\n$mLen: $mLine\n$xLen: $xLine\n";
	    last;
	}
	my $gLine;
	my $hasNuc;
	for (my $i=0; $i<$mLen; $i+=2) {
	    my $xv = substr($xLine,$i,1);
	    my $xq = substr($xLine,$i+1,1);
	    if ($xv eq 'A' || $xv eq 'a') {
		$gLine .= "=$xq";
	    } else {
		my $mv = substr($mLine,$i,1);
		my $mq = substr($mLine,$i+1,1);
		my $q = $xq lt $mq ? $xq : $mq;
		$gLine .= "$mv$q";
		$hasNuc = 1 unless ($hasNuc || $mv !~ /[A-Za-z]/);
	    }
	}
	unless ($IGNORE_GAP_LINES && !$hasNuc) {
	    print $gh "$gLine\n";
	}
    }
}
