#!/usr/bin/perl

# Args: dirs containing layout file(s)
# Output: layout_disagreements.txt in each file, detailing
# reads with >= 30% non-consensus nucleotides.

use Utility;
use strict;

foreach my $d (@ARGV) {
    layoutDisagreements($d);
}

sub layoutDisagreements {
    my ($d) = @_;
    my $f = "$d/layout.gz";
    my $g = "$d/layout_disagreements.txt";
    if (! -s $f) {
	report("$f: missing",1);
    } elsif (-f $g) {
	report("$g: already exists",1);
    } else {
	report("$g: creating");
	layoutDisagreements1($f,$g);
	report();
    }
}

sub layoutDisagreements1 {
    my ($f,$g) = @_;
    my $fh = openToRead($f);
    my @readAt;
    my (%good,%bad,%startLine);
    my $line = 1;
    while (<$fh>) {
	chomp;
	if (/^>REF/) {
	} elsif (/^>(\S+) (\d+)/) {
	    my ($read,$col) = ($1,$2+1);
	    #$read =~ s/:.*//;
	    $readAt[$col] = $read;
	    unless (defined $startLine{$read}) {
		$startLine{$read} = $line;
	    }
	} else {
	    s/(.)./\1/g; # vals only
	    tr/a-z/A-Z/;
	    my $iMax = length($_)-1;
	    my ($sum,%count);
	    for (my $i=0; $i<=$iMax; $i++) {
		my $v = substr($_,$i,1);
		if ($v =~ /[ACGT]/) {
		    $count{$v} ++;
		    $sum++;
		}
	    }
	    my $th = $sum/2;
	    for (my $i=1; $i<=$iMax; $i++) {
		my $v = substr($_,$i,1);
		if ($v =~ /[ACGT]/) {
		    if ($count{$v} < $th) {
			$bad{$readAt[$i]} ++;
		    } else {
			$good{$readAt[$i]} ++;
		    }
		}
	    }
	}
	$line ++;
    }
    my $gh = openToPipe("sort -nr > $g");
    foreach my $read (keys %bad) {
	my $b = $bad{$read};
	my $g = $good{$read};
	my $r = $b/($b+$g);
	if ($r >= 0.3) {
	    printf($gh "%6.4f %4d %4d %7d %s\n",$r,$b,$g,$startLine{$read},$read);
	}
    }
}
