#!/usr/bin/env perl

use Utility;
use File::Basename;
use strict;

my $key = $ARGV[0] || '200_1';

my $dir = "$ENV{SGR}/match/FREG$key/freg";

unless (-d $dir) {
    die("$dir: no such directory\n");
}

chdir($dir);

my %CR = (p => 'Confirmation/rejection', m => 'Imputation', '.' => 'All locations');

my @METHODS = qw(Null Star Static Chrom W500k W200k W100k W50k W20k W10k MF Oracle);
my %DISCARD = map {$_ => 1} qw(Null Chrom W500k W200k W50k W20k W10k);

foreach my $t (qw(noerrs errs)) {
    foreach my $u (qw(m p .)) {
	next if ($t eq 'noerrs' && $u ne 'm'); # nothing to do
	#$DISCARD{Null} = $u ne 'p';
	my %line;
	foreach my $d (glob("cia_chrs_${t}_complete_mac*")) {
	    my $f = "$d/sum.txt";
	    my $b = "$d/cia.chr00.bout";
	    if (-s $b && (! -s $f || newer($b,$f))) {
		`sumCompareImputedAcross.pl $d/cia.chr00.bout > $f`;
	    }
	    next unless (-s $f);
	    my $range = $d;
	    $range =~ s/.*mac//;
	    if ($range =~ /^0:/) {
		$range = '0.00' . $range;
	    }
	    my $fh = openToRead($f);
	    while (<$fh>) {
		chomp;
		if (substr($_,0,4) eq "..${u} ") {
		    $line{$range} = $_;
		    last;
		}
	    }
	}
	next unless (%line);
	print "\\begin{tabular}{|c|c|";
	foreach my $meth (@METHODS) {
	    print "c|" unless ($DISCARD{$meth});
	}
	print "} \\hline\n";
	print "  MAF range    &     Count";
	foreach my $meth (@METHODS) {
	    printf(" & %9s",$meth) unless ($DISCARD{$meth});
	}
	print " \\\\ \\hline \n";
	my ($locs,@errs);
	foreach my $range (sort keys %line) {
	    my ($dots,$count,undef,@rest) = split(" ",$line{$range});
	    if (scalar(@rest) == 11) {
		@rest = (0,@rest);
	    }
	    $locs += $count;
	    foreach my $k (0 .. $#rest) {
		$errs[$k] += $rest[$k]*$count;
	    }
	    $range =~ s/:/ to /;
	    printf("%14s & %9d",$range,$count);
	    foreach my $k (0 .. $#rest) {
		unless ($DISCARD{$METHODS[$k]}) {
		    if ($u ne 'p' && $k == 0) {
			printf(" & %9s","");
		    } elsif ($rest[$k] > 0) {
			my $e = $rest[$k]*100;
			my $nd = $e < 0.00001 ? 6 : 5;
			if ($METHODS[$k] eq 'Oracle') {
			    printf(" & \{\\it %7.${nd}f\}",$e);
			} else {
			    printf(" & %9.${nd}f",$e);
			}
		    } elsif ($METHODS[$k] eq 'Oracle') {
			printf(" & \{\\it %7s\}",0);
		    } else {
			printf(" & %9s",0);
		    }
		}
	    }
	    print " \\\\ \\hline \n";
	}
	printf("%14s & %9d", "All values",$locs);
	foreach my $k (0 .. $#errs) {
	    unless ($DISCARD{$METHODS[$k]}) {
		if ($u ne 'p' && $k == 0) {
		    printf(" & %9s","");
		} elsif ($METHODS[$k] eq 'Oracle') {
		    printf(" & \{\\it %7.5f\}",100*$errs[$k]/$locs);
		} else {
		    printf(" & %9.5f",100*$errs[$k]/$locs);
		}
	    }
	}
	print " \\\\ \\hline \n";
	print "\\end{tabular}\n";
	printf("\\caption{%s percentage error rates (per cent) for simulated human data, 1x coverage, %s sequencing errors}\n\n",
	       $CR{$u},
	       $t eq 'errs' ? 'with' : 'without');
    }
}
	
