#!/usr/bin/env perl

# Args: liti_list.csv

use Utility;
use strict;

my $FTP_DIR = 'ftp://ftp.sanger.ac.uk/pub/dmc/yeast/read';

my $FTP_LOCAL = $ENV{'FTP'} . '/yeast/read';

my %SZ = qw(cerevisiae 12162911 paradoxus 11872617);

my %SOLEXA = map {$_ => 1} qw(S288c SK1 W303 Y55 Q31.4 Q69.8 Q74.4 W7 Y8.1 Y8.5
		     Y9.6 Z1);

my (%COV,%LOCAL);
opendir(D,$FTP_LOCAL);
foreach my $str (readdir(D)) {
    next unless ($str =~ /^[A-Z0-9]/);
    my $str2 = $str;
    $str2 =~ s/[-_](\d)$/.\1/;
    $str2 =~ tr/_/-/;
    $LOCAL{$str2} = $str;
    my $cf = "$FTP_LOCAL/$str/renamed.fastq.gz";
    if (-s $cf) {
	my $fh = openToRead($cf);
	my $sum = 0;
	while (<$fh>) {
	    if (/^\@.*bases (\d+) to (\d+)/) {
		my ($n1,$n2) = ($1,$2);
		unless ($n1 == 0 && $n2 == 100) { # garbage
		    $sum += $n2-$n1;
		}
	    }
	}
	$COV{$str2} = $sum;
    } else {
	print "WARNING: cannot find $cf<br>\n";
    }
}

my $species;
while (<>) {
    next if (/^\#/);
    chomp;
    if (/S\. (.+) strains/) {
	print "</TABLE>\n" if ($species);
	$species = $1;
	die("Missing size for species $species") unless ($SZ{$species});
	print "<B>S. $species strains</B>\n";
	print "<TABLE frame=box rules=all cellspacing=3 cellpadding=3>\n";
	print "<TR><TD>Name</TD><TD>ABI</TD><TD>Sol/454</TD><TD>Location</TD><TD>Source</TD><TD>Notes</TD></TR>\n";
    } else {
	my @a = splitCSV($_);
	my ($name,$geo,$source,@notes) = @a;
	next unless ($source);
	printf("<TR><TD>%s</TD>",$name);
	if (defined $COV{$name}) {
	    printf("<TD align=center><A HREF='%s/%s/renamed.fastq'>%3.1fx</A></TD>",
		   $FTP_DIR,$LOCAL{$name},$COV{$name}/$SZ{$species});
	    delete $COV{$name};
	} else {
	    print "<TD></TD>";
	}
	my $sol = $SOLEXA{$name} ? '+' : '';
	printf("<TD align=center>$sol</TD>");
	printf("<TD>%s</TD><TD>%s</TD><TD>%s</TD></TR>\n",
	       $geo,$source,join(", ",@notes));
    }
}
print "</TABLE>\n" if ($species);
print "<p>\n";
foreach my $name (sort keys %COV) {
    print "MISSING line for $name<br>\n";
}

sub splitCSV {
    my ($line) = @_;
    $line =~ s/^\s*\"//;
    $line =~ s/\"[ ,]*$//;
    return split(/","/,$line);
}
