#!/usr/bin/env perl

use Utility;
use File::Basename;
use strict;

my %CR = (p => 'Confirmation/rejection', m => 'Imputation', '.' => 'All locations');

my @METHODS = qw(Null Star Static Chrom W500k W200k W100k W50k W20k W10k MF Union Oracle);
my %DISCARD = map {$_ => 1} qw(Chrom W500k W200k W50k W20k W10k);

foreach my $key qw(CERU/cere PARS/para) {
    makeYeastTables($key);
}

sub makeYeastTables {
    my ($key) = @_;
    my $dir = "$ENV{SGR}/match/$key/cia_chrs";
    unless (-d $dir) {
	die("$dir: no such directory\n");
    }
    chdir($dir);
    my %line;
    foreach my $u (qw(p m .)) {
	foreach my $op (qw(0 \# .)) {
	    my $f = "sum.txt";
	    if (! -s $f) {
		`sumCompareImputedAcross.pl cia*bout > $f`;
	    }
	    next unless (-s $f);
	    my $fh = openToRead($f);
	    while (<$fh>) {
		chomp;
		# Change 1 and 2 to #
		substr($_,1,1) =~ s/[12]/\#/;
		my $pat = "N$op$u";
		if (substr($_,0,4) eq "$pat ") {
		    $line{$pat} = $_;
		}
	    }
	}
    }
    print "\\begin{tabular}{|c|c|";
    foreach my $meth (@METHODS) {
	print "c|" unless ($DISCARD{$meth});
    }
    print "} \\hline\n";
    # Operation & Apparent & Cases &  Null &  Star & Static & Window &   MF  &  S+MF & Oracle \\ 
    #           &   type   &       &       &       &        &        &       &       &        \\ \hline
    # 
    #           &   Mono   &       & \multicolumn{7}{c|}{}                           \\ \cline{2-10}
    # Conf/corr &   Poly   &       &       &       &        &        &       &       &        \\ \cline{2-10}
    #           &   Both   &       &       &       &        &        &       &       &        \\ \hline
    # 
    #           &   Mono   &       &       & \multicolumn{6}{c|}{}                   \\ \cline{2-3} \cline{5-10}
    # Impute    &   Poly   &       &  n/a  &       &        &        &       &       &        \\ \cline{2-3} \cline{5-10}
    #           &   Both   &       &       &       &        &        &       &       &        \\ \hline
    # 
    #           &   Mono   &       &       & \multicolumn{6}{c|}{}                   \\ \cline{2-3} \cline{5-10}
    # Both      &   Poly   &       &  n/a  &       &        &        &       &       &        \\ \cline{2-3} \cline{5-10}
    #           &   Both   &       &       &       &        &        &       &       &        \\ \hline
    print "Operation    & Apparent";
    foreach my $meth (@METHODS) {
	printf(" & %9s",$meth) unless ($DISCARD{$meth});
    }
    print "\\\\\n";
    print "             & type    ";
    foreach my $meth (@METHODS) {
	printf(" & %9s","") unless ($DISCARD{$meth});
    }
    print "\\\\ \\hline \n";
    my ($locs,@errs);
    foreach my $range (sort keys %line) {
	my ($dots,$count,undef,@rest) = split(" ",$line{$range});
	if (scalar(@rest) == 11) {
	    @rest = (0,@rest);
	}
	$locs += $count;
	foreach my $k (0 .. $#rest) {
	    $errs[$k] += $rest[$k]*$count;
	}
	$range =~ s/:/ to /;
	printf("%14s & %9d",$range,$count);
	foreach my $k (0 .. $#rest) {
	    unless ($DISCARD{$METHODS[$k]}) {
		if ($u ne 'p' && $k == 0) {
		    printf(" & %9s","");
		} elsif ($rest[$k] > 0) {
		    my $e = $rest[$k]*100;
		    my $nd = $e < 0.00001 ? 6 : 5;
		    printf(" & %9.${nd}f",$e);
		} else {
		    printf(" & %9s",0);
		}
	    }
	}
	print "\\\\ \\hline \n";
    }
    printf("%14s & %9d", "All values",$locs);
    foreach my $k (0 .. $#errs) {
	unless ($DISCARD{$METHODS[$k]}) {
	    if ($u ne 'p' && $k == 0) {
		printf(" & %9s","");
	    } else {
		printf(" & %9.5f",100*$errs[$k]/$locs);
	    }
	}
    }
    print "\\\\ \\hline \n";
    print "\\end{tabular}\n";
    printf("\\caption{%s percentage error rates (per cent) for simulated human data, 1x coverage, %s sequencing errors}\n\n",
	   $CR{$u},
	   $t eq 'errs' ? 'with' : 'without');
}
}
	
