#!/usr/bin/env perl

# Args: Run from e.g. $SGR/match/CERT/cere/methc/015

use Utility;
use File::Basename;
use strict;

my $BLOCK_SZ = 10000;
my $FIX_SZ = 800000;
my $TEL_SZ =  30000;
my $MID_SZ = $FIX_SZ-2*$TEL_SZ;

my %vals;
foreach my $cFile (glob("align/*/clusters.txt")) {
    my $str = basename(dirname($cFile));
    my $sFile = "assembly/$str/genome.sz";
    
    my %sz;
    my $fh = openToRead($sFile);
    while (<$fh>) {
	chomp;
	my ($chr,$s) = split;
	$sz{$chr} = $s;
    }

    my $fh = openToRead($cFile);
    while (<$fh>) {
	chomp;
	my (undef,$cid,$reas,undef,$region) = split;
	if ($cid eq 'c0') {
	    my ($chr,$lo,$hi) = ($region =~ /^(.*):(\d+)\.\.(\d+)$/);
	    unless ($chr =~ /\.\d+$/) {
		my $x = ($lo+$hi)/2;
		my $s = $sz{$chr};
		if ($x > $TEL_SZ) {
		    if ($x < $s-$TEL_SZ) {
			# then squeeze it ...
			my $frac = ($x-$TEL_SZ)/($s-2*$TEL_SZ);
			$x = $TEL_SZ+$frac*$MID_SZ;
		    } else {
			$x = $FIX_SZ+$x-$s;
		    }
		}
		$chr =~ s/^$str\.//;
		$vals{$reas}{$chr}[int($x/$BLOCK_SZ)] ++;
		#printf($gh "%s %8s %2d %s\n",$reas,$chr,int($x/$BLOCK_SZ),$str);
	    }
	}
    }
}
foreach my $r (keys %vals) {
    my $pR = $vals{$r};
    foreach my $c (keys %$pR) {
	my $pA = $pR->{$c};
	my $gh = openToWrite("decisions/$r/$c.xy");
	foreach my $i (0 .. $FIX_SZ/$BLOCK_SZ-1) {
	    printf($gh "%d %d\n",$i,$pA->[$i]);
	}
    }
}
