#!/usr/bin/env perl

use Utility;
use SGRP::OffsetMap;
use strict;

my $pOM = new OffsetMap('?',shift @ARGV);

autoflush STDOUT 1;

while (<>) {
    chomp;
    my @a = split;
    if (defined $pOM->{$a[0]}) {
	# tRNA line: offsets are items 2 and 3 and maybe 6 and 7
	foreach my $i (2,3,6,7) {
	    if ($a[$i] > 0) {
		$a[$i] = $pOM->translation($a[0],$a[$i]);
	    }
	}
	$a[0] =~ s/.*\.//;
	print equivalentLine($_,@a) . "\n";
    } elsif (defined $pOM->{$a[3]}) {
	# Alignment line
	foreach my $i (6,7) {
	    if ($a[$i] > 0) {
		$a[$i] = $pOM->translation($a[3],$a[$i]);
	    }
	}
	$a[3] =~ s/.*\.//;
	print equivalentLine($_,@a) . "\n";
    } else {
	print "$_\n";
    }
}

sub equivalentLine {
    my ($line,@a) = @_;
    my @x = split(" ",$line);
    die("Mismatching arg counts") unless ($#x == $#a);
    my $off = 0;
    foreach my $i (0 .. $#x) {
	$off = index($line,$x[$i],$off);
	unless ($a[$i] eq $x[$i]) {
	    my $len = length($x[$i]);
	    my $fmt;
	    if ($x[$i] =~ /^-?\d+$/ && $a[$i] =~ /^-?\d+$/) {
		$fmt = sprintf("%%%dd",$len);
	    } elsif ($x[$i] =~ /^-?\d+\.\d+$/ && $a[$i] =~ /^-?[\d\.]+$/) {
		my $dotOff = index($x[$i],".");
		$fmt = sprintf("%%%d.%df",$len,$len-$dotOff-1);
	    } else {
		my $dl = $len-length($a[$i]);
		$fmt = "%s";
		$fmt .= ' ' x $dl if ($dl>0);
	    }
	    my $to = sprintf($fmt,$a[$i]);
	    while ($len < length($to) && 
		   $off > 1 && 
		   substr($line,$off-2,1) eq ' ') {
		$len ++;
		$off --;
	    }
	    substr($line,$off,$len) = sprintf($fmt,$a[$i]);
	}
    }
    return $line;
}
