
1;

sub maqConsensusPair {
    my ($pPairs) = @_;
    my (%eps,%sumQ);
    foreach my $pair (@$pPairs) {
	my ($v,$qc) = split("",$pair);
	my $q = ($qc eq ' ' ? 94 : ord($qc)-33);
	my $ep = exp(-log(10)*0.1*$q);
	push @{$eps{$v}},$ep;
	$sumQ{$v} += $q;
    }
    my @sorted = sort {$sumQ{$b} <=> $sumQ{$a}} keys %sumQ;
    my $pEPA = [sort {$a <=> $b} @{$eps{$sorted[0]}}];
    my $pEPB = @sorted > 1 ? [sort {$a <=> $b} @{$eps{$sorted[1]}}] : [];
    my $ep = maqConsensus($pEPA,$pEPB);
    my $q = int(-log($ep)*10/log(10));
    return $sorted[0] . ($q >= 94 ? ' ' : chr($q+33));
}

sub maqConsensus {
    my ($pEPA,$pEPB) = @_;
    my $postA = findAlphaNK($pEPA,$pEPB);
    my $postB = findAlphaNK($pEPB,$pEPA);
    return $postA/($postA+$postB);
}

sub findAlphaNK {
    my ($pEPA,$pEPB) = @_;
    if (@$pEPA) {
	my $eb = epsilonBar($pEPA);
	my $pABI = alphaBarI($pEPA,$pEPB,$eb);
	my $pBBI = betaBarI($pABI);
	my $cNK = cNK($pBBI,$eb);
	return alphaNK($cNK,$pEPA);
    } else {
	return 1;
    }
}

sub epsilonBar {
    my ($pEPA) = @_;
    my ($num,$den) = (0,0);
    my $fi = 1;
    foreach my $i (0 .. $#$pEPA) {
	$num += $fi*log($pEPA->[$i]);
	$den += $fi;
	$fi *= 0.8;
    }
    return exp($num/$den);
}
    
sub alphaBarI {
    my ($pEPA,$pEPB,$eb) = @_;
    my @ans;
    my $n = scalar(@$pEPA)+scalar(@$pEPB);
    my $term;
    foreach my $i (0 .. scalar(@$pEPA)) {
	if ($i == 0) {
	    $term = power(1-$eb,$n);
	} else {
	    $term *= (($n-$i+1)/$i) * ($eb/(1-$eb));
	}
	push @ans,$term;
    }
    return \@ans;
}

sub betaBarI {
    my ($pAlphaBarI) = @_;
    my ($num,$den) = (1,1);
    my @ans;
    foreach my $i (0 .. $#$pAlphaBarI) {
	$num -= $pAlphaBarI->[$i];
	$den -= $pAlphaBarI->[$i-1] if ($i > 0);
	push @ans,($num <= 0 ? 0 : $num/$den);
    }
    return \@ans;
}

sub cNK {
    my ($pBetaBarI,$eb) = @_;
    my @ans;
    my $prod = 1;
    my $fi = 1;
    foreach my $i (0 .. $#$pBetaBarI-1) {
	$fi *= 0.8;
	$prod *= power($pBetaBarI->[$i]/$eb,$fi);
    }
    return (1-power($pBetaBarI->[-1],$fi))*$prod;
}

sub power {
    my ($x,$n) = @_;
    if ($x <= 0) {
	return 0;
    } else {
	return exp($n*log($x));
    }
}

sub alphaNK {
    my ($cNK,$pEPA) = @_;
    my $prod = 1;
    my $fi = 1;
    foreach my $i (0 .. $#$pEPA) {
	$prod *= $pEPA->[$i]*$fi;
	$fi *= 0.8;
    }
    return $cNK*$prod;
}
