#!/usr/bin/env perl

# Run "maq mapvalidate" on aln.map files for all lanes under $G1K/v1, creating
# aln.map.valid files (takes a bif of time). Uses the touch file mechanism, so
# you safely can submit multiple identical jobs in parallel.
#
# With "-q" switch, just look for errors in existing aln.map.valid files.
# With "-s" switch, look at lane*/sample/aln.map files instead of lane*/aln.map files.
#
# Really this code should be incorporated as a "g1k.pl val" action...

use Utility;
use File::Basename;
use Getopt::Std;
use strict;

my %OPT;
getopts('qs',\%OPT);

autoflush STDOUT 1;

my $MAQ = "$ENV{G1K}/bin/maq";

chdir("$ENV{G1K}/v1");

my $PAT = $OPT{s} ? "*/*/*/lane*/sample/aln.map" : "*/*/*/lane*/aln.map";

unless ($OPT{q}) {
    foreach my $f (glob($PAT)) {
	unless (-f "$f.touch") {
	    my $g = "$f.valid"; # we'll create this
	    if (stamp($g)) { # take possession of $g (create $g.touch)
		report("$g: creating");
		`$MAQ mapvalidate $f > $g`;
		unstamp($g); # relinquish $g
		report();
	    }
	}
    }
}

# Print out all X chromosome lines, as X is the last chromosome (that
# everyone has) to be filled in an aln.map file. Shout to stderr if
# there are 0 reads assigned. Also shout any non-"message" lines.

foreach my $f (glob("$PAT.valid")) {
    next if (-f "$f.touch");
    my $fh = openToRead($f);
    my $d = dirname($f);
    my $seenX;
    while (<$fh>) {
	if (/^\[message\]/) {
	    my (undef,$chr,undef,$n) = split;
	    if ($chr eq 'X') {
		print "$d $_";
		if ($n==0) {
		    print STDERR "$d $_";
		}
		$seenX = 1;
	    }
	} else {
	    print STDERR "$d $_";
	}
    }
    print STDERR "$d [fatal error] has no X line\n" unless ($seenX);
}
