#!/usr/bin/env perl

=head1 NAME

matrix2phylip.pl

=head1 SYNOPSIS

matrix2phylip.pl matrixFile

=head1 DESCRIPTION

matrixFile should be created by "sgrp.pl smat". Outputs a version of
matrixFile with each line preceded by A, B, C etc (so there had better
not be more than a few dozen lines or we will run out of characters).

The resulting output can then by processed by quicktree, e.g.

    ~/software/quicktree_1.1/bin/quicktree -in m -out t rates.sorted.q40.phylip 
        > rates.sorted.q40.tree

and a tree created in postscript format by one of

    unrooted rates.sorted.q40.tree
    njplot rates.sorted.q40.tree

Trees for cerevisiae and paradoxus can be combined by e.g.

    scalePostscript.pl -s .210526 -y 400 -x 40 -q rates.sorted.q40.ps > rates.sorted.q40.scaled.ps

The reason for using strain names A, B, C ... is that "unrooted" does
horrible things with strain names in the postscript, so it's better to
post-edit it. If we wanted a rooted tree instead, then an alternative
to using matrix2phylip.pl is

    paste rates.sorted.strains rates.sorted.q40.tree > rates.sorted.q40.phylip

Strain names can be put back into a PS file using

    reinstateStrainNames.pl foo.strains bar.ps

Can do postscript output from unrooted, then measure |----| scale in both
cere and para plots and work out scale factor, then rescale cere tree to
go on para diagram, e.g.

scalePostscript.pl -s .210526 -y 400 -x 40 -q rates.sorted.q40.ps > rates.sorted.q40.noanno.ps

and insert that into para plot with emacs.

=head1 AUTHOR

David Carter, C<dmc@sanger.ac.uk> , August 2006 and January 2007

=head1 SEE ALSO

    snpMatrix.pl       
    quicktree
    unrooted
    njplot
    scalePostscript.pl

=cut

use strict;
my @mat = <>;
print scalar(@mat) . "\n";
my $idx = ord('A');
my $bigZ = ord('Z');
foreach my $line (@mat) {
    printf("%s %s",chr($idx),$line);
    if ($idx == $bigZ) {
       $idx = ord('a');
    } else {
       $idx ++;
    }
}
