#!/usr/bin/env perl

# Run in NNN dir. Creates maximal.fa.

use Utility;
use File::Basename;
use strict;

my $gh = openToWrite("maximal.fa");

foreach my $suf (qw(z y m)) {
    my @files = glob("p?/*/imputed_$suf.gz");
    if (@files) {
	foreach my $f (@files) {
	    my $chr = basename(dirname($f));
	    report("processing $f");
	    print $gh ">MAX.$chr\n";
	    maximalAssembly(openToRead($f),$gh);
	    report();
	}
	last;
    }
}

sub maximalAssembly {
    my ($fh,$gh) = @_;
    <$fh>; # header
    my $buf;
    my $len=0;
    while (<$fh>) {
	chomp;
	$buf .= bestNucleotide($_);
	$len++;
	if ($len == 60) {
	    print $gh "$buf\n";
	    $buf="";
	    $len=0;
	}
    }
    print $gh "$buf\n" if ($buf);
}

sub bestNucleotide {
    my ($line) = @_;
    my $iLim = length($line);
    my %sum;
    for (my $i=0; $i<$iLim; $i+=2) {
	my $v = substr($line,$i,1);
	$v =~ tr/a-z/A-Z/;
	if ($v =~ /[ACGT]/) {
	    my $qc = substr($line,$i+1,1);
	    my $q = $qc eq ' ' ? 94 : (ord($qc)-33);
	    $sum{$v} += $q;
	}
    }
    my ($best) = sort {$sum{$b} <=> $sum{$a}} keys %sum;
    return $best || 'N';
}
