#!/usr/bin/perl

use Getopt::Std;

use strict;

use vars qw($opt_n $opt_p);

# -p means output correlations as pairs "R" I J R(I,J) for surface plotting
getopts('n:p');

my $NFCORREL = 15;
$NFCORREL = $opt_n if (defined $opt_n);

meansd();

sub meansd {

    my ($nFields,$n,%ss,@s);

    while (<>) {
	chomp;
	next if (/[^\s-+\d\.eE]/);
	my @a = split;
	next unless (@a);
	if (scalar(@a) != $nFields) {
	    if ($nFields) {
		printf(STDERR "Field count mismatch: old %d, new %d\n\%s\n",
		       $nFields,scalar(@a),$_);
		last;
	    } else {
		$nFields = scalar(@a);
	    }
	}
	foreach my $k (0 .. $#a) {
	    $s[$k] += $a[$k];
	    foreach my $l ($k .. $#a) {
		$ss{"$k $l"} += $a[$k]*$a[$l];
		last unless ($k == 0 || $#a < $NFCORREL);
	    }
	}
	$n++;
    }

    my $LINE;
    
    if ($n > 1) {
	print "$n samples\n";
	printf("Field      Mean         SD    SD/Mean  r(f,0)\n");
	my (@mu,@sd);
	foreach my $k (0 .. $nFields-1) {
	    $mu[$k] = $s[$k]/$n;
	    my $var = $ss{"$k $k"}/$n-$mu[$k]*$mu[$k];
	    if ($var > 0) {
		$sd[$k] = sqrt($var);
	    } else {
		$sd[$k] = 0; # rounding error
	    }
	    my $r = sprintf("%8s");
	    if ($k > 0 && $sd[$k] > 0 && $sd[0] > 0) {
		my $cov = $ss{"0 $k"}/$n - $mu[$k]*$mu[0];
		$r = sprintf(" %7.4f",$cov/($sd[$k]*$sd[0]));
	    }
	    if ($mu[$k] == 0) {
		printf("%3d %11.6f %11.6f %8s %8s\n",$k+1,$mu[$k],$sd[$k],"",$r);
	    } else {
		printf("%3d %11.6f %11.6f %8.3f %8s\n",$k+1,$mu[$k],$sd[$k],$sd[$k]/abs($mu[$k]),$r);
	    }
	}
	if ($nFields <= $NFCORREL) {
	    print "\nCorrelation coefficients:\n  ";
	    unless ($opt_p) {
		foreach my $l (1 .. $#mu) {
		    printf(" %7d",$l+1);
		}
		print "\n";
	    }
	    foreach my $k (0 .. $#mu-1) {
		printf("%3d",$k+1) unless ($opt_p);
		foreach my $l (0 .. $#mu) {
		    if ($sd[$k] > 0 && $sd[$l] > 0) {
			my $sskl;
			if ($k <= $l) {
			    $sskl = $ss{"$k $l"};
			} else {
			    $sskl = $ss{"$l $k"};
			}
			my $cov = $sskl/$n - $mu[$k]*$mu[$l];
			my $r = $cov/($sd[$k]*$sd[$l]);
			if ($opt_p) {
			    printf("%d %d %7.4f\n",$k,$l,$r);
			} elsif ($l > 0) {
			    printf(" %7.4f",$r);
			}
			if ($k == 0 && $l == 1 && $#mu == 1) {
			    $LINE=sprintf("x2 = %5.5f x1 + %5.5f\n",
					  $r*$sd[1]/$sd[0], $mu[1]-$mu[0]*$r*$sd[1]/$sd[0]);
			    $LINE =~ s/\+ -/-/;
			}
		    } elsif (!$opt_p) {
			print "        ";
		    }
		}
		print "\n";
	    }
	}
    }

    if ($LINE) {
	print "\nLine: $LINE\n";
    }
}
