#!/usr/bin/env perl

# Args: PALAS fastq file (from assembly dir), Solexa fastq file (from solexaAssembly.pl), output file.

use Utility;
use strict;

if (@ARGV == 3) {
    mergeFastqFiles(@ARGV);
} else {
    die("Usage: $0 palas.fastq.gz solexa.fastq.gz merged.fastq.gz\n");
}

sub mergeFastqFiles {
    my ($f,$s,$g) = @_;
    if (! -s $f) {
	report("$f: missing or empty",1);
	return;
    } elsif (! -s $s) {
	report("$s: missing or empty",1);
	return;
    } elsif (! stamp($g)) {
	report("$g: cannot stamp",1);
    }
    my $fh = openToRead($f);
    my $sh = openToRead($s);
    my $gh = openToWrite($g);
    mergeFastqFiles1($fh,$sh,$gh);
    unstamp($g);
}

sub mergeFastqFiles1 {
    my ($fh,$sh,$gh) = @_;
    my $pB1 = readFastqBlock($sh);
    while (1) {
	my $pB0 = readFastqBlock($fh);
	last unless ($pB0);
	while (integrateFastqBlock($pB1,$pB0)) {
	    $pB1 = readFastqBlock($sh);
	}
	foreach my $line (@$pB0) {
	    print $gh "$line\n";
	}
    }
    $gh->close();
}

sub integrateFastqBlock {
    my ($pB1,$pB0) = @_;
    return undef unless (defined @$pB1);
    my ($hdr,$seq,undef,$quals) = @$pB0;
    my ($name,$refChr,$lo,$hi) = split(" ",$hdr);
    my ($hdr1,$seq1,undef,$quals1) = @$pB1;
    my $name1 = substr($hdr1,1);
    my ($chr1,$lo1) = split(/\./,$name1);
    my $hi1 = $lo1+length($seq1)-1;
    if ($chr1 lt $refChr || ($chr1 eq $refChr && $hi1 < $lo)) {
	die("Bad coordinates");
    } elsif ($chr1 gt $refChr || $lo1 > $hi) {
	return undef; # nothing to do
    }
    my $hi1 = $lo1+length($seq1)-1;
    my $wantMore = $hi1 <= $hi;
    # Do the integrating
    my $d = $lo1-$lo;
    my $iMin = max($d,0);
    my $iLim = min($hi1,$hi)-$lo+1;
    for (my $i=$iMin; $i<$iLim; $i++) {
	my $v1 = substr($seq1,$i-$d,1);
	next if ($v1 eq 'N');
	my $q1 = substr($quals1,$i-$d,1);
	my $v = substr($seq,$i,1);
	$v =~ tr/a-z/A-Z/;
	my $q = substr($quals,$i,1);
	next if ($v1 eq $v);
	my $qn;
	if ($q1 gt $q) {
	    substr($seq,$i,1) = $v1;
	    $qn = 33+ord($q1)-ord($q);
	} else {
	    $qn = 33+ord($q)-ord($q1);
	}
	substr($quals,$i,1) = chr($qn);
    }
    $pB0->[1] = $seq;
    $pB0->[3] = $quals;
    return $wantMore;
}

sub readFastqBlock {
    my ($fh) = @_;
    my @a;
    my $i=0;
    while (<$fh>) {
	chomp;
	push @a,$_;
	$i++;
	return \@a if ($i == 4);
    }
    return undef;
}

	
