#!/usr/bin/env perl

use Utility;
use strict;

# Chromosome ordering is 1-22 then X Y; we bail out when we find N...

my $suf = $ARGV[0]; # flt or final.snp

my $root = "$ENV{G1K}/v1/Trio-CEU";
my @files = glob("$root/NA?????/maq_post/cns.$suf");

my @in = map {openToRead($_)} @files;

my @buf;
foreach my $fh (@in) {
    chomp (my $line = <$fh>);
    if ($line =~ /^N/) {
	push @buf,undef;
    } else {
	my @a = split(" ",$line);
	push @buf,\@a;
    }
}

while (1) {
    my @isNext;
    my ($minChr,$minOff);
    my $ref;
    foreach my $pA (@buf) {
	if ($pA) {
	    my ($chr,$off,$r) = @$pA;
	    if (chrLt($chr,$off,$minChr,$minOff)) {
		$ref = $r;
		$minChr = $chr;
		$minOff = $off;
	    }
	}
    }
    last unless (defined $minChr);
    foreach my $pA (@buf) {
	my ($chr,$off,$r) = @$pA;
	push @isNext,($chr eq $minChr && $off eq $minOff) ? 1 : 0;
    }
    printf("%s %9d %s  ",$minChr,$minOff,$ref);
    foreach my $i (0 .. $#buf) {
	printf(" %s",$isNext[$i] ? $buf[$i][3] : $ref);
    }
    foreach my $i (0 .. $#buf) {
	printf(" %20s",$isNext[$i] ? join(",",@{$buf[$i]}[4..8]) : "---");
    }
    print "\n";
    foreach my $i (0 .. $#buf) {
	if ($isNext[$i]) {
	    my $fh = $in[$i];
	    chomp (my $line = <$fh>);
	    $buf[$i] = ($line && $line !~ /^N/) ? [split(" ",$line)] : undef;
	}
    }
}

sub chrLt {
    my ($c1,$o1,$c2,$o2) = @_;
    return 1 unless (defined $c2);
    if ($c1 eq $c2) {
	return $o1 < $o2;
    }
    if ($c1 > 0) {
	if ($c2 > 0) {
	    return $c1 < $c2;
	} else {
	    return 1;
	}
    } elsif ($c2 > 0) {
	return 0;
    } else {
	return ($c1 lt $c2);
    }
}
