#!/usr/bin/env perl

# Switch: -q quality threshold (default 0)
# Input: output of mergeTrioCnsFlt.pl

use Getopt::Std;
my %OPT;
getopts('q:',\%OPT);
my $qTh = $OPT{q};

use strict;

my %CONS1;
my %SPLIT;
foreach my $trip (qw(MAC RAG WAT SCG YCT KGT)) {
    my ($v,$v1,$v2) = split(//,$trip);
    $CONS1{$v1}{$v} = $CONS1{$v2}{$v} = 1;
    $SPLIT{$v} = [$v1,$v2];
}

my @count;
my @countPat;
LINE:
while (<>) {
    chomp;
    my (undef,undef,$ref,$child,$mthr,$fthr,$ctup,$mtup,$ftup) = split;
    if ($qTh) {
	foreach my $tup ($ctup,$mtup,$ftup) {
	    if ($tup ne '---') {
		# (undef,$q) ?
		my ($q) = split(/,/,$tup);
		next LINE if ($q < $qTh);
	    }
	}
    }
    my $het = $child eq $ref ? 0 : $child =~ /[ACGT]/ ? 1 : 2;
    my $cons = consistent($child,$mthr,$fthr);
    $count[$cons][$het] ++;
    $countPat[$cons][$het]{"$ref $child $mthr $fthr"} ++;
}
my @rowSum;
foreach my $pA (@count) {
    $rowSum[0] += $pA->[0];
    $rowSum[1] += $pA->[1];
    $rowSum[2] += $pA->[2];
}
# ref and var are homs that do and do not agree with ref
my @word0 = qw(ref var het);
my @word1 = qw(neither one both);
printf("%4s %7s %7s %7s   Total\n",'',reverse(@word1));
my @csum;
foreach my $j (0..2) {
    printf("%4s",$word0[$j]);
    my $rsum = 0;
    foreach my $i (2,1,0) {
	my $c = $count[$i][$j];
	$rsum += $c;
	$csum[$i] += $c;
	printf(" %7d",$c);
    }
    printf(" %7d\n",$rsum);
    $csum[3] += $rsum;
}
print "Totl";
foreach my $i (2,1,0,3) {
    printf(" %7d",$csum[$i]);
}
print "\n\n";
printf("%4s %7s %7s  %7s\n",'',reverse(@word1));
foreach my $j (0..2) {
    printf("%4s ",$word0[$j]);
    foreach my $i (2,1,0) {
	my $c = $count[$i][$j];
	printf(" %6.2f%%",100*$c/$rowSum[$j]);
    }
    print "\n";
}

print "\nChld  Agrees R C F M  Count  Perc\n";
foreach my $j (0..2) {
    foreach my $i (2,1,0) {
	my $pH = $countPat[$i][$j];
	my $c = $count[$i][$j];
	foreach my $quad (sort keys %$pH) {
	    printf("%4s %7s %s %6d %5.1f\n",$word0[$j],$word1[$i],$quad,$pH->{$quad},
		   100*$pH->{$quad}/$c);
	}
    }
}

sub consistent {
    my ($child,$mthr,$fthr) = @_;
    if ($child =~ /[ACGT]/) {
	return consistent1($child,$mthr) + consistent1($child,$fthr);
    } else {
	my ($v1,$v2) = @{$SPLIT{$child}};
	return 
	    max(consistent1($v1,$mthr)+consistent1($v2,$fthr),
		consistent1($v2,$mthr)+consistent1($v1,$fthr));
    }
}

sub max {
    my ($x,$y) = @_;
    return ($x > $y) ? $x : $y;
}

sub consistent1 {
    my ($child,$parent) = @_;
    return ($child eq $parent || $CONS1{$child}{$parent}) ? 1 : 0;
}
    
