#!/usr/bin/env perl

use Utility;
use Getopt::Std;
use strict;

autoflush STDOUT 1;

# Args: sorted_ac_regions.txt file(s)

my $FACTOR = log(10)/10;

my $NS;

my %OPT;
getopts('o:',\%OPT);

my %COUNT;
foreach my $f (@ARGV) {
    addCounts($f);
}
my $gh = $OPT{o} ? openToWrite($OPT{o}) : *STDOUT;

foreach my $v (sort {$a <=> $b} keys %COUNT) {
    printf($gh "%6d",$v);
    my $pA = $COUNT{$v};
    foreach my $i (0 .. $NS-1) {
	printf($gh " %6d",$pA->[$i]);
    }
    print $gh "\n";
}

sub addCounts {
    my ($f) = @_;
    report("$f: reading");
    my $fh = openToRead($f);
    my $dh = openToWrite("$f.area");
    my (@low,@high,@mat);
    while (<$fh>) {
	chomp;
	my ($lo,$hi,$pat) = split(" ",$_,3);
	my @pres;
	my $iMax = length($pat)-2;
	for (my $i=2; $i<=$iMax; $i+=2) {
	    push @pres,(substr($pat,$i,1) =~ /[AC]/ ? 1 : 0);
	}
	$NS = max($NS,scalar(@pres));
	push @low,$lo;
	push @high,$hi;
	push @mat,\@pres;
    }
    report();
    report("$f: finding nextFilled and prevFilled");
    my (@nextFilled,@prevFilled);
    foreach my $j (0 .. $NS-1) {
	my $pf = 1;
	foreach my $i (0 .. $#mat) {
	    if ($mat[$i][$j]) {
		$pf = $high[$i];
	    } else {
		$prevFilled[$i][$j] = $pf;
	    }
	}
    }
    foreach my $j (0 .. $NS-1) {
	my $nf = $high[$#mat];
	for (my $i = $#mat; $i>=0; $i--) {
	    if ($mat[$i][$j]) {
		$nf = $low[$i];
	    } else {
		$nextFilled[$i][$j] = $nf;
	    }
	}
    }
    report();
    report(sprintf("%s: finding areas at %d loci",$f,scalar(@mat)));
    foreach my $i (0 .. $#mat) {
	my (@nBefore,@nAfter);
	foreach my $j (0 .. $NS-1) {
	    unless ($mat[$i][$j]) {
		$nBefore[$j] = $low[$i] - $prevFilled[$i][$j];
		$nAfter[$j] = $nextFilled[$i][$j] - $high[$i];
	    }
	}
	my $sz = $high[$i]-$low[$i]+1;
	my $maxA = 0;
	foreach my $j (0 .. $NS-1) {
	    unless ($mat[$i][$j]) {
		my $a = maximumArea(\@nBefore,\@nAfter,$j,$sz);
		$maxA = max($a,$maxA);
		$a = assignToBin($a);
		$COUNT{$a}[$j] += $sz;
	    }
	}
	print "#" if ($i =~ /00$/);
	print $dh "$low[$i] $maxA\n";
	print $dh "$high[$i] $maxA\n";
    }
    report();
}

sub assignToBin {
    my ($a) = @_;
    my $b = int(log($a)/$FACTOR);
    return exp(($b+0.5)*$FACTOR);
}

sub countAbsences {
    my ($pMat,$pOff,$i,$j,$di) = @_;
    my $k = $i+$di;
    while (1) {
	last if ($k < 0 || $k > $#$pMat || $pMat->[$k][$j]);
	$k += $di;
    }
    $k -= $di;
    my $len = $pOff->[$k]-$pOff->[$i];
    return $len*$di;
}
	
sub maximumArea {
    my ($pBef,$pAft,$j,$sz) = @_;
    # Try various lBef and lAft thresholds;
    # 0 < lBef <= $pBef->[$j] and 0 < lAft <= $pAft->[$j] to qualify.
    my %ngeBef;
    $ngeBef{0} = 0;
    my $nBJ = $pBef->[$j];
    my @befDecr = sort {$b <=> $a} @$pBef;
    foreach my $k (0 .. $#befDecr) {
	my $v = $befDecr[$k];
	last if ($v == 0);
	if ($v <= $nBJ) {
	    $ngeBef{$v} = $k+1;
	}
    }
    my %ngeAft;
    $ngeAft{0} = 0;
    my $nAJ = $pAft->[$j];
    my @aftDecr = sort {$b <=> $a} @$pAft;
    foreach my $k (0 .. $#aftDecr) {
	my $v = $aftDecr[$k];
	last if ($v == 0);
	if ($v <= $nAJ) {
	    $ngeAft{$v} = $k+1;
	}
    }
    my ($maxArea,$bestB,$bestA,$bestQ);
    my $ns = scalar(@$pBef);
    foreach my $b (keys %ngeBef) {
	foreach my $a (keys %ngeAft) {
	    my $qual1 = $ngeBef{$b}+$ngeAft{$a};
	    my $area1 = $qual1*($a+$b+$sz);
	    next unless ($area1 > $maxArea);
	    # Now really calculate ...
	    my $qual = 0;
	    foreach my $k (0 .. $#$pBef) {
		$qual ++ if ($pBef->[$k] >= $b && $pAft->[$k] >= $a);
	    }
	    my $area = $qual*($a+$b+$sz);
	    if ($area > $maxArea) {
		$maxArea = $area;
		$bestB = $b;
		$bestA = $a;
		$bestQ = $qual;
	    }
	}
    }
    return $maxArea;
}
