#!/usr/bin/env perl

use lib qw(/software/solexa/lib/site_perl/5.8.8);
use st::api::project;
use npg::api::run;

# Documentation is in:
# http://svn.internal.sanger.ac.uk/cgi-bin/viewcvs.cgi/npg-tracking/trunk/scripts/?root=new-pipeline-dev

use strict;

autoflush STDOUT 1;

foreach my $id (@ARGV) {
    #npgQuery($id);
    lanesForProject($id);
}

# Gorilla = 33
# 10000Genomes-A1-LowCoveragePilot = 50
# 10000Genomes-A2-TriosPilot = 10

sub npgQuery {
    my ($id) = @_;
    foreach my $pA (lanesForProject($id,1)) {
	foreach my $v (@$pA) {
	    printf(" %12s",$v);
	}
	print "\n";
    }
}

sub lanesForProject {
    my ($id,$silent) = @_;

    my @ans;
    # Way in to NPG database
    my $root_run = npg::api::run->new();
    # Handle for my project; $id should be its project number.
    my $proj = st::api::project->new({id => $id});

    # A batch just has an id(), which is a number. It's a set
    # of runs, not lanes, so it can't correspond to a sample.
    # For that, see $sample below.
    for my $batch (@{$proj->batches()}) {
	my $batchId = $batch->id();

	# A run has: id_run, id_instrument, id_run_pair,
	# name, is_paired and batch_id (among other fields).
	for my $run (@{$root_run->runs_on_batch($batch->id())}) {
	    #my $runName = $run->name();
	    my $runId = $run->id_run();

	    # A run_lane has: id_run_lane, id_run, tile_count,
	    # tracks, id_project (the last is not the same as $proj->id
	    # so it must be a different type of project. Instead,
	    # use $sample->project_id below); and position.
	    for my $run_lane (@{$run->run_lanes()}) {
		my $pos = $run_lane->position();
		my $casproject = $run_lane->project();
		if ($casproject) {
		    # A sample has: id, name, count, user, project_id, ended
		    my $sample  = $run_lane->sample();
		    if ($sample && $sample->project_id() == $id) {
			push @ans,[$id,$batchId,$runId,$pos,$sample->name(),$casproject->repository()];
			unless ($silent) {
			    foreach my $v (@{$ans[-1]}) {
				printf(" %12s",$v);
			    }
			    print "\n";
			}
		    }
		}
	    }
	}
    }
    return @ans;
}
    


