#!/usr/bin/env perl

# Args: species method SET str1 str2 ...

use Utility;
use File::Basename;
use strict;

my ($spec,$meth,$set,@strains) = @ARGV;

my $SGR = $ENV{'SGR'};

my $rDir = "$SGR/read";
my $mDir = "$SGR/match/$set/$spec/$meth";

foreach my $str (@strains) {
    print "       Strain Itn  Reads Unalig Alignd  Fail1  Pass1  Fail2  Pass2  Done%  DoneA%\n";
    my ($todo,$done);
    foreach my $sub (glob("$mDir/[0-9][0-9][0-9]")) {
	($todo,$done) = placementTableLine($str,$sub,$todo,$done);
    }
    print "\n";
}

sub placementTableLine {
    my ($str,$sub,$todo,$done) = @_;
    my $nnn = basename($sub);
    printf("%13s %3s",$str,$nnn);
    my $nReads = readCount($nnn,$rDir,$sub,$str);
    printf(" %6d",$nReads);
    my $nAligned = alignedCount($nnn,$rDir,$sub,$str,$spec);
    my $nUnaligned = $nReads-$nAligned;
    printf(" %6d %6d",$nUnaligned,$nAligned);
    my $nPass1 = pass1Count($nnn,$rDir,$sub,$str,$spec,$meth);
    my $nFailed1 = $nAligned-$nPass1;
    printf(" %6d %6d",$nFailed1,$nPass1);
    my $nPass2 = pass2Count($sub,$str);
    my $nFailed2 = $nPass1-$nPass2;
    $todo = $nReads unless ($todo);
    $done += $nPass2;
    printf(" %6d %6d %5.1f%% %5.1f%%\n",$nFailed2,$nPass2,100*$done/$todo,100*$done/($todo-$nUnaligned));
    return ($todo,$done);
}

sub readCount {
    my ($n,$rDir,$sub,$str) = @_;
    my $dir = $n == 0 ? "$rDir/$str" : "$sub/align/$str";
    my $c = 0;
    if (-s "$dir/renamed.wm") {
	my $fh = openToRead("$dir/renamed.wm");
	while (<$fh>) {
	    $c++;
	}
    } else {
	foreach my $f (glob("$dir/renamed.?.fastq")) {
	    my $fh = openToRead($f);
	    while (<$fh>) {
		$c ++ if (substr($_,0,1) eq '@');
	    }
	}
    }
    return $c;
}

sub alignedCount {
    my ($n,$rDir,$sub,$str,$spec) = @_;
    my $dir = $n == 0 ? "$rDir/$str/$spec" : "$sub/align/$str";
    my $fh = openFromPipe("pf 3 $dir/alilines.all.txt 2>/dev/null | sort -u");
    my $c = 0;
    while (<$fh>) {
	$c++;
    }
    return $c;
}

sub pass1Count {
    my ($n,$rDir,$sub,$str,$spec,$meth) = @_;
    my $dir = $n == 0 ? "$rDir/$str/$spec/$meth" : "$sub/align/$str";
    my $fh = openFromPipe("pf 3 $dir/alilines.pass.txt 2>/dev/null | sort -u");
    my $c = 0;
    while (<$fh>) {
	$c++;
    }
    return $c;
}

sub pass2Count {
    my ($sub,$str) = @_;
    my %seen;
    my $pfx = $str . '-';
    my $lpf = length($pfx);
    foreach my $f (glob("$sub/p?/*/alilines.nola.txt")) {
	my $fh = openToRead($f);
	while (<$fh>) {
	    my (undef,undef,$r) = split;
	    $seen{$r} = 1 if (substr($r,0,$lpf) eq $pfx);
	}
    }
    return scalar(keys %seen);
}
    
