#!/usr/bin/env perl

# Input: output of readFatesTable.pl

# Input fates:
#   0 first iter
#   1 subsequent but pre-final
#   2 final
#   3 never (or "extra")

# Output fates:
#   0 first or other pre-final iter
#   1 final
#   2 never (or "extra")

use strict;

my $MAX_FATE = 2;

my (@fate,%strFate,%plateFate,%pqFate);
while (<>) {
    chomp;
    next if (/bad_read/ || /mediocre_read/); # don't want to count these
    my ($r,@a) = split
    my $f = $a[5];
    $f-- if ($f > 0);
    if ($r =~ /^(\S+)-(\d+)[a-z]\d+\.([pq])1k$/) {
	$fate[$f] ++;
	my ($str,$plate,$pq) = ($1,$2,$3);
	$strFate{$str}[$f] ++;
	$plateFate{$str}{$plate}[$f] ++;
	$pqFate{$str}{$plate}{$pq}[$f] ++;
    }
}

printFates(0,'ALL',\@fate,[0],\%strFate,\%plateFate,\%pqFate);

sub printFates {
    my ($lev,$name,$pTotal,$csAbove,$pFate,@rest) = @_;
    printFate($lev,$name,$pTotal,@$csAbove);
    my %cs;
    my %isKey = map {$_ => 1} (keys %$pFate);
    while (%isKey) {
	my %cs;
	my @keys = keys %isKey;
	my $nk = scalar(@keys);
	my @winners;
	if ($nk == 1) {
	    $cs{$keys[0]} = [0];
	    @winners = @keys;
	} elsif ($nk == 2) {
	    $cs{$keys[0]} = $cs{$keys[1]} = chiSquared($pFate->{$keys[0]},$pTotal);
	    @winners = sort @keys;
	} else {
	    foreach my $key (@keys) {
		$cs{$key} = chiSquared($pFate->{$key},$pTotal);
	    }
	    ($winners[0]) = sort {$cs{$b}->[0] <=> $cs{$a}->[0] || $a cmp $b} keys %cs;
	}
	foreach my $w (@winners) {
	    my @keyRest;
	    foreach my $pH (@rest) {
		push @keyRest,$pH->{$w};
	    }
	    my $newName = $name eq 'ALL' ? $w : "${name}-$w";
	    my $pW = $pFate->{$w};
	    printFates($lev+1,$newName,$pW,$cs{$w},@keyRest);
	    delete $isKey{$w};
	    foreach my $i (0 .. $#$pW) {
		$pTotal->[$i] -= $pW->[$i];
	    }
	}
    }
}

sub printFate {
    my ($lev,$name,$pA,$cs,@above) = @_;
    printf("%d %s",$lev,$name . (' ' x (19-length($name))));
    my $sum = 0;
    foreach my $v (@$pA) {
	$sum += $v;
    }
    foreach my $i (0 .. $MAX_FATE) {
	if ($i <= $#above) {
	    printf(" | %5d -> %5d",$above[$i]*$sum,$pA->[$i]);
	} else {
	    printf(" | %5s    %5d","",$pA->[$i]);
	}
    }
    if ($cs > 0) {
	printf(" | %6d %8.2f\n",$sum,$cs);
    } else {
	printf(" | %6d\n",$sum);
    }
}

sub chiSquared {
    my ($pA,$pS) = @_;
    return 0 unless defined @$pS;
    my (@rowSum,$allSum,$pB);
    foreach my $i (0 .. $MAX_FATE) {
	$rowSum[0] += $pA->[$i];
	$rowSum[1] += ($pB->[$i] = $pS->[$i]-$pA->[$i]);
	$allSum += $pS->[$i];
    }
    my $cs;
    my @eList;
    foreach my $i (0 .. $MAX_FATE) {
	my $e0 = $eList[$i] = $rowSum[0]*$pS->[$i]/$allSum;
	my $e1 = $rowSum[1]*$pS->[$i]/$allSum;
	if ($e0 >= 5) {
	    my $d = $pA->[$i]-$e0; 
	    $cs += $d*$d/$e0;
	}
	if ($e1 >= 5) {
	    my $d = $pB->[$i]-$e1; 
	    $cs += $d*$d/$e1;
	}
    }
    foreach my $i (0 .. $#eList) {
	$eList[$i] /= $rowSum[0];
    }
    return [$cs,@eList];
}
