#!/usr/bin/env perl

=head1 NAME

plotAlignments.pl

=head1 SYNOPSIS

plotAlignments.pl [-s sizeFile] ins1 ins2 ins3 ... alilines.txt

=head1 DESCRIPTION



=head1 AUTHOR

David Carter, C<dmc@sanger.ac.uk>

=head1 SEE ALSO

       

=cut

use GD;
use GD::Simple;
use Utility;
use Getopt::Std;
use strict;

my %OPT;
my %SZ;
getopts('s:',\%OPT);
if ($OPT{s}) {
    my $fh = openToRead($OPT{s});
    while (<$fh>) {
	chomp;
	my ($chr,$sz) = split;
	$SZ{$chr} = $sz;
    }
}

my %insToGet;
my @files;

foreach my $a (@ARGV) {
    if ($a =~ /txt$/ || $a =~ /ali$/) {
	push @files,$a;
    } else {
	$insToGet{$a} = 1;
    }
}

@ARGV = @files;

my %ali;
while (<>) {
    chomp;
    my @a = split;
    if ($a[2] =~ /^(.*)\.([pq])1k$/) {
	my ($ins,$pq) = ($1,$2);
	$a[1] /= $a[-1]; # normalize score by length of read
	if ($insToGet{$ins}) {
	    push @{$ali{$ins}{$pq}},\@a;
	}
    }
}

foreach my $ins (sort keys %ali) {
    plotAlignments($ins,$ali{$ins}{p},$ali{$ins}{q});
}

sub plotAlignments {
    my ($ins,$pAlis,$qAlis) = @_;
    my $pLen = $pAlis ? $pAlis->[0][-1] : 0;
    my $qLen = $qAlis ? $qAlis->[0][-1] : 0;
    my $pMax = $pAlis ? $pAlis->[0][1] : 0;
    my $qMax = $qAlis ? $qAlis->[0][1] : 0;
    my $pMin = $pAlis ? $pAlis->[-1][1] : undef;
    my $qMin = $qAlis ? $qAlis->[-1][1] : undef;
    my $max = 1;
    my $min = 0;
    my ($xsz,$ysz) = (800,800);
    my $width = $pLen+$qLen;
    return unless ($width);
    my @alis = @$pAlis if ($pAlis);
    push @alis,@$qAlis if ($qAlis);
    @alis = sort {$b->[1] <=> $a->[1]} @alis;
    # Decrease scores where necessary so separation always >= 0.01
    my %lowest;
    foreach my $pA (@alis) {
	if (defined $lowest{$pA->[2]} &&
	    $lowest{$pA->[2]} < $pA->[1]) {
	    $pA->[1] = $lowest{$pA->[2]};
	}
	$lowest{$pA->[2]} = $pA->[1]*0.99;
    }
    my $xMult = $xsz/$width;
    my $yMult = $ysz/($max-$min);
    my $im = new GD::Simple($xsz,$ysz);
    if ($pLen && $qLen) {
	$im->moveTo($pLen*$xMult,0);
	$im->lineTo($pLen*$xMult,$ysz);
    }
    my @colours = qw(red green blue brown orange purple);
    my @col;
    my $colK = 0;
    foreach my $i (0 .. $#alis) {
	my $pAI = $alis[$i];
	my $rI = $pAI->[2];
	foreach my $j (0 .. $#alis) {
	    my $pAJ = $alis[$j];
	    my $rJ = $pAJ->[2];
	    if ($rI lt $rJ && consistentAlignments($pAI,$pAJ)) {
		my $cc = 
		    defined $col[$i] ? $col[$i] :
		    defined $col[$j] ? $col[$j] :
		    $colK ++;
		$col[$i] = $col[$j] = $cc;
	    }
	}
    }
    1;
    foreach my $i (0 .. $#alis) {
	my $pA = $alis[$i];
	my $colour = defined $col[$i] ? $colours[$col[$i]] : undef;
	my $sh = ($pA->[2] =~ /q1k/) ? $pLen : 0;
	my $x1 = $sh+$pA->[4];
	my $x2 = $sh+$pA->[5];
	my $y = $pA->[1];
	if (defined $colour) {
	    $im->penSize(3);
	} else {
	    $colour = 'black';
	    $im->penSize(1);
	}
	my $mid = ($pA->[6]+$pA->[7])/2;
	my $telo = (%SZ && ($mid < 20000 || 
			    $mid > $SZ{$pA->[3]}-20000));
	$im->fgcolor($colour);
	$im->moveTo($x1*$xMult,$ysz-($y-$min)*$yMult);
	if ($telo) {
	    $im->lineTo(($x1*.51+$x2*.49)*$xMult,$ysz-($y-$min)*$yMult);
	    $im->moveTo(($x1*.49+$x2*.51)*$xMult,$ysz-($y-$min)*$yMult);
	}
	$im->lineTo($x2*$xMult,$ysz-($y-$min)*$yMult);
    }
    my $gh = openToWrite("$ins.png");
    binmode $gh;
    print $gh $im->png;
    $gh->close();
}

sub consistentAlignments {
    my ($pA1,$pA2) = @_;
    return if ($pA1->[8] eq $pA2->[8]); # same orientation no good
    return unless ($pA1->[3] eq $pA2->[3]); # diff chrs no good
    if ($pA1->[8] eq 'F') {
        return consistentAlignments1($pA1,$pA2);
    } else {
        return consistentAlignments1($pA2,$pA1);
    }
}

sub consistentAlignments1 {
    my ($pAF,$pAC) = @_;
    my $len = $pAC->[6] - $pAF->[6] + 1;
    return ($len >= 1000 && $len <= 30000);
}
