#!/usr/bin/env perl

# Given imputed file, plot presence of strains across chromosome:
# black for present, grey for imputed (or space), white for absent ("=")

use GD::Simple;
use Utility;
use strict;

my $fh = openToRead($ARGV[0]);
my $off = 0; # offset in alignment
my $ref = 0; # offset in reference
chomp (my $hdr = <$fh>);
my @strains = ('REF',split(" ",substr($hdr,1)));

my @status; # 0 absent, 1 imputed or space, 2 present
my @start; # where status started
my @blocks;
my %ali2ref;
while (<$fh>) {
    chomp;
    foreach my $i (0 .. $#strains) {
	my $v = substr($_,$i*2,1);
	my $s = $v eq '=' ? 0 : ($v =~ /[a-z_]/ || !$v) ? 1 : 2;
	my $p = $status[$i];
	next if (defined $p && $s == $p);
	if (defined $p) {
	    push @blocks,[$i,$start[$i],$off-1,$p];
	    if ($i == 0) {
		$ali2ref{$off} = $ref;
	    }
	}
	$status[$i] = $s;
	$start[$i] = $off;
    }
    $off ++;
    $ref ++ if (/^[A-Z]/);
    if ($off%10000 == 0) {
	report("Alignment $off, reference offset $ref",1);
    }
}
foreach my $i (0 .. $#strains) {
    if (defined $status[$i]) {
	push @blocks,[$i,$start[$i],$off-1,$status[$i]];
    }
}

my @colours = qw(white gray black);

my $dimX = 8000;
my $leftX = 100;
my $dimY = 500;
my $botY = 50;
my $im = new GD::Simple($leftX+$dimX,$dimY+$botY);
my $sx = $dimX/$off;
my $sy = $dimY/scalar(@strains);
$im->bgcolor('black');
$im->fgcolor('black');
for (my $i=0; $i<=$#strains; $i++) {
    $im->moveTo(5,$sy*($i+0.5));
    $im->string($strains[$i]);
}
foreach my $pA (@blocks) {
    my ($i,$lo,$hi,$s) = @$pA;
    $im->bgcolor($colours[$s]);
    $im->rectangle($lo*$sx+$leftX,$sy*$i,$hi*$sx+$leftX,$sy*($i+1));
}
$im->bgcolor('red');
$im->fgcolor('red');
my $prev = -10000;
foreach my $off (sort {$a <=> $b} keys %ali2ref) {
    next if ($sx*($off-$prev) < 30);
    $prev = $off;
    my $ref = $ali2ref{$off};
    my $x = $leftX+$sx*$off;
    my $y = $dimY+5;
    $im->moveTo($x,0);
    $im->lineTo($x,$y+10);
    my $dx = 3.5*length($off);
    $im->moveTo($x-$dx,$y+25);
    $im->string($off);
    $im->moveTo($x-$dx,$y+40);
    $im->string($ref);
}
    
my $gh = openToWrite("tmp.png");
binmode $gh;
print $gh $im->png;
$gh->close();
