#!/usr/bin/env perl

use GD;
use GD::Simple;

use Utility;

use GD;
use GD::Simple;

use strict;

unless (@ARGV > 3) {
    die("Usage: $0 out.png renamed.fastq renamed.wm read1 read2 ...");
}

plotQualities(@ARGV);

sub plotQualities {
    my ($out,$fq,$wm,@reads) = @_;
    my $pWM = readWatermarks($wm,@reads);
    my $pQ = readQualities($fq,@reads);
    my $maxLen;
    foreach my $pA (values %$pWM) {
	$maxLen = max($pA->[0],$maxLen);
    }
    my $xsz = 1200;
    my $xScale = $xsz/$maxLen;
    my $ysz = @reads < 10 ? 80*scalar(@reads) : 800;
    my $yScale = ($ysz/scalar(@reads))/100;
    my $im = new GD::Simple($xsz,$ysz);
    $im->interlaced('true');
    $im->font('Tahoma');
    $im->fontsize(6);
    foreach my $i (0 .. $#reads) {
	my $r = $reads[$i];
	my $qstr = $pQ->{$r};
	my $len = length($qstr);
	my $yBot = ($i+1)*100*$yScale;
	my $yTop = (($i+1)*100-94)*$yScale;
	my $yNQS = (($i+1)*100-20)*$yScale; # 20 = $th in CorrectNames.pm
	$im->bgcolor('pink');
	my (undef,@pairs) = @{$pWM->{$r}};
	for (my $j=0; $j<$#pairs; $j+=2) {
	    my ($lo,$hi) = ($pairs[$j],$pairs[$j+1]);
	    $im->rectangle($lo*$xScale,$yTop,$hi*$xScale,$yBot);
	}
	$im->fgcolor('gray');
	$im->moveTo(0,$yNQS);
	$im->lineTo($xsz,$yNQS);
	$im->fgcolor('black');
	$im->moveTo($xsz-100,($yBot+3*$yTop)/4);
	$im->string($r);
	$im->moveTo(0,$yBot);
	$im->lineTo($xsz,$yBot);
	$im->moveTo(0,$yTop);
	$im->lineTo($xsz,$yTop);
	foreach my $j (0 .. $len-1) {
	    my $q = ord(substr($qstr,$j,1))-33;
	    my $y = (($i+1)*100-$q)*$yScale;
	    my $x = $j*$xScale;
	    if ($j > 0) {
		$im->lineTo($x,$y);
	    } else {
		$im->moveTo($x,$y);
	    }
	}
    }
    my $gh = openToWrite($out);
    binmode $gh;
    print $gh $im->png;
}

sub readWatermarks {
    my ($f,@reads) = @_;
    my %wm;
    my $fh = openToRead($f);
    my %wanted = map {$_ => 1} @reads;
    while (<$fh>) {
	chomp;
	my ($r,$len,@pairs) = split;
	if ($wanted{$r}) {
	    $wm{$r} = [$len,@pairs];
	}
    }
    return \%wm;
}

sub readQualities {
    my ($f,@reads) = @_;
    my %q;
    my $fh = openToRead($f);
    my %wanted = map {$_ => 1} @reads;
    my $r;
    while (<$fh>) {
	chomp;
	if (/\+(\S+).* bases /) {
	    if ($wanted{$1}) {
		$r = $1;
		delete $wanted{$r};
	    }
	} elsif ($r) {
	    $q{$r} = $_;
	    $r = undef;
	    last unless (%wanted);
	}
    }
    return \%q;
}
