#!/usr/bin/env perl

# Inputs: alilines.all.txt file(s), for one or more strains

use SGRP::Alignment;
use Utility;
use strict;

my ($name,%alis);
while (<>) {
    chomp;
    my $pA = new Alignment($_);
    if ($name && $name ne $pA->name) {
	processAlignments(\%alis);
	%alis = ();
    }
    $name = $pA->name;
    unless ($pA->chr =~ /\..*\./ || $pA->score < 50) { # aux sequence and poor match not of interest
	push @{$alis{$pA->chr}},$pA;
    }
}
processAlignments(\%alis) if (%alis);

sub processAlignments {
    my ($pAlis) = @_;
    foreach my $pV (values %$pAlis) {
	processAlignments1($pV);
    }
}

sub processAlignments1 {
    my ($pAlis) = @_;
    my @alis = sort {$a->rlo <=> $b->rlo} @$pAlis;
    my @dups;
    while (@alis) {
	my $pA1 = shift @alis;
	my $chr = $pA1->chr;
	foreach my $pA2 (@alis) {
	    next if ($pA2->fc ne $pA1->fc);
	    # Check one doesn't contain the other (pA1 can contain pA2, not vv)
	    next if ($pA1->rhi >= $pA2->rhi);
	    # We have an overlap in the chr ...
	    if ($pA1->fc eq 'F') {
		my $rOverlap = $pA1->rhi - $pA2->rlo + 1;
		my @ins;
		my $ms = min($pA1->score,$pA2->score);
		if ($rOverlap < 0) {
		    @ins = ($pA1->name,$pA1->rhi+1,$pA2->rlo-1,$pA1->fc,$ms);
		    $rOverlap = 0;
		} else {
		    @ins = ($pA1->name,$pA1->rhi+1-int($rOverlap/2),$pA2->rlo-1+int(($rOverlap+1)/2),$pA1->fc,$ms);
		}
		my $cOverlap = $pA1->chi - $pA2->clo + 1;
		my $diff = $cOverlap - $rOverlap;
		if ($diff >= 10 && $diff <= 15000) {
		    my $dupLo = $pA2->clo - int(($rOverlap+1)/2);
		    my $dupHi = $pA1->chi + int($rOverlap/2);
		    #printf("# %s\n# %s\n",$pA1->toString,$pA2->toString);
		    #printf("%s %7d %7d %s\n",$chr,$dupLo,$dupHi,$ins);
		    push @dups,[$chr,$dupLo,$dupHi,@ins];
		}
	    }
	}
    }
    # Delete dups whose insertions span others
    @dups = sort {$a->[0] cmp $b->[0] || $a->[1] <=> $b->[1] || $a->[2] <=> $b->[2]} @dups;
    foreach my $i (0 .. $#dups-1) {
	my $pDI = $dups[$i];
	next unless ($pDI);
	foreach my $j ($i+1 .. $#dups) {
	    my $pDJ = $dups[$j];
	    next unless ($pDJ);
	    last if ($pDJ->[0] ne $pDI->[0] || $pDJ->[1] != $pDJ->[1] || $pDJ->[2] != $pDJ->[2]);
	    if ($pDI->[4] <= $pDJ->[4] && $pDI->[5] >= $pDJ->[5]) {
		# delete $pDI
		$dups[$i] = undef;
		last;
	    } elsif ($pDJ->[4] <= $pDI->[4] && $pDJ->[5] >= $pDI->[5]) {
		$dups[$j] = undef;
		next;
	    }
	}
    }
    foreach my $pDup (@dups) {
	next unless ($pDup);
	printf("%s %7d %7d %20s %4d %4d %s %4d %5d %4d\n",@$pDup,$pDup->[2]-$pDup->[1]+1,$pDup->[5]-$pDup->[4]+1);
    }
}
		    
		
		
