#!/usr/bin/env perl

# Args: pat1,pat2,pat3 ... means only include read names matching those patterns.
#       layout.gz

use Utility;
use strict;

if (@ARGV == 2) {
    pruneLayout(@ARGV);
} else {
    die("Usage: $0 pat1,pat2,...patN layout.gz\n");
}

sub pruneLayout {
    my ($pats,$f) = @_;
    my @pats = split(/,/,$pats);
    my $fh = openToRead($f);
    my %wantPos;
    my %used;
    my $off = 0;
    my $printedOff;
    while (<$fh>) {
	chomp;
	if (/^>/) {
	    my ($id,$pos,$iter) = split(" ",substr($_,1));
	    if (wanted($id,\@pats)) {
		my $p = $wantPos{$pos} = nextUnused(\%used);
		print ">REF $off\n";
		print ">$id $p $iter\n";
	    } elsif (defined $wantPos{$pos}) {
		delete $used{$wantPos{$pos}};
		delete $wantPos{$pos};
		print ">REF $off\n";
	    }
	} else {
	    if (/^[A-Z]/) {
		$off ++;
		$printedOff = 0;
	    }
	    next unless (%wantPos);
	    my @a;
	    my $refNuc = substr($_,0,1) =~ /[ACGT]/;
	    foreach my $pos (keys %wantPos) {
		my $pair = substr($_,2*($pos+1),2);
		if ($pair =~ /^[A-Za-z]/ || ($refNuc && $pair =~ /^=/)) {
		    $a[$wantPos{$pos}] = $pair;
		} elsif (!$pair || $pair eq '  ') {
		    delete $used{$wantPos{$pos}};
		    delete $wantPos{$pos};
		}
	    }
	    if (@a) {
		if ($off%100 == 0 && !$printedOff) {
		    print ">REF $off\n";
		    $printedOff = 1;
		}
		my $line = substr($_,0,2);
		foreach my $p (0 .. $#a) {
		    $line .= $a[$p] || '  ';
		}
		print "$line\n";
	    }

	}
    }
}

sub wanted {
    my ($id,$pPats) = @_;
    foreach my $pat (@$pPats) {
	return 1 if ($id =~ /$pat/);
    }
    return 0;
}

sub nextUnused {
    my ($pUsed) = @_;
    my $i = 0;
    while ($pUsed->{$i}) {
	$i++;
    }
    $pUsed->{$i} = 1;
    return $i;
}
