#!/usr/bin/env perl

use Utility;
use Getopt::Std;
use strict;

# Args: sequenced.gz or imputed.gz file(s)

# Switch: -t Th: minimum quality threshold to consider (default: 0)
#         -s str1,str2,... : only look at named strains (default: all)

my %OPT;
getopts('s:t:',\%OPT);

my $qThC;
if (defined $OPT{t}) {
    $qThC = chr(33+$OPT{t});
}

my %wanted;
if (defined $OPT{s}) {
    %wanted = map {$_ => 1} split(",",$OPT{s});
}

my (%count,$nUsed,$nNotUsed);
foreach my $f (@ARGV) {
    report("$f: reading");
    my $fh = openToRead($f);
    chomp (my $hdr = <$fh>);
    my @wanted;
    if (defined %wanted) {
	my @strList = split(" ",substr($hdr,1));
	foreach my $k (0 .. $#strList) {
	    if ($wanted{$strList[$k]}) {
		$wanted[2*($k+1)] = 1;
	    }
	}
    }
    while (<$fh>) {
	next unless (/^[A-Z]/); # ref nuc posns only
	chomp;
	my $iMax = length($_)-2;
	for (my $i=2; $i<=$iMax; $i+=2) {
	    next if (defined @wanted && !$wanted[$i]);
	    my $v = substr($_,$i,1);
	    if ($v =~ /[A-Za-z]/) {
		my $qc = substr($_,$i+1,1);
		if (defined $qThC && $qc lt $qThC) {
		    $nNotUsed ++;
		} else {
		    $count{$qc} ++;
		    $nUsed ++;
		}
	    } else {
		$nNotUsed ++;
	    }
	}
    }
    report();
}

my ($num,$den)=(0,0);
foreach my $qc (sort keys %count) {
    my $n = $count{$qc};
    my $q = ord($qc)-33;
    my $pe = 0.75*exp(log(0.1)*$q/10);
    my $cont = $pe*$n;
    $num += $pe*$n;
    $den += $n;
}
printf("%d nucleotides out of %d (%4.2f%%) were defined",
       $nUsed,$nUsed+$nNotUsed,100*$nUsed/($nUsed+$nNotUsed));
if (defined $OPT{t}) {
    printf(" with quality >= %d",$OPT{t});
}
printf("\nError rate = %d/%d = %8.6f = 1/%d\n",$num,$den,$num/$den,$den/$num);
