#!/usr/bin/env perl

# Run on farm-login

# For each read, prints match count (w.r.t. reference), insertion
# count and read name. "Novel" reads could be defined as those with
# insertion count > match count, for example.

# A few reads come out with insertion count + match count >> 1000.
# This is OK I think, as PALAS places some reads (e.g. rDNA)
# multiple times.

use Utility;
use File::Basename;
use strict;

foreach my $sp (qw(cere para)) {
    my $top = "$ENV{SGRW}/release3/$sp/match";
    foreach my $f (glob("$top/*/layout.gz")) {
	my $g = dirname($f) . "/rac.txt";
	if (stamp($g)) {
	    report("$g: creating");
	    printReadAlignmentCounts($f,$g);
	    report();
	    unstamp($g);
	}
    }
}

sub printReadAlignmentCounts {
    my ($f,$g) = @_;
    my $fh = openToRead($f);
    my (@readAt,%matches,%insertions);
    while (<$fh>) {
	chomp;
	if (/^>/) {
	    my ($spec,$pos,$iter) = split(" ",substr($_,1));
	    if ($iter =~ /^\d+$/) {
		my ($read) = split(/:/,$spec);
		$readAt[$pos] = $read;
		$matches{$read} += 0; # to ensure a key
	    } else {
		$readAt[$pos] = undef;
	    }
	} else {
	    my $iLim = length($_);
	    my $isNuc = substr($_,0,1) =~ /[A-Z]/;
	    my $pos = 0;
	    for (my $col=2; $col<$iLim; $col += 2) {
		my $v = substr($_,$col,1);
		if ($v =~ /[A-Za-z]/) {
		    my $r = $readAt[$pos];
		    if ($r) {
			if ($isNuc) {
			    $matches{$r} ++;
			} else {
			    $insertions{$r} ++;
			}
		    }
		}
		$pos ++;
	    }
	}
    }
    my $gh = openToWrite($g);
    foreach my $r (sort keys %matches) {
	printf($gh "%4d %4d %s\n",$matches{$r},$insertions{$r},$r);
    }
    $gh->close();
}
		
