#!/usr/bin/env perl

# Args: file containing list of reads (or clones) wanted, and list of
# .fastq files. If "-d dir" is given, create a separate file "read.fa"
# for each read in that dir.

use Utility;
use Getopt::Std;
use strict;

my %OPT;
getopts('d:',\%OPT);
my $outDir = $OPT{d};

my %want;
my $f = shift @ARGV;
my $fh = openToRead($f);
while (<$fh>) {
    chomp;
    if (/\.[pq]1k/) {
	$want{$_} = 1;
    } else {
	$want{"$_.p1k"} = 1;
	$want{"$_.q1k"} = 1;
    }
}

my $read;
my $active;
while (<>) {
    chomp;
    if (/^\@(\S+) /) {
	$read = $1;
	$active = $want{$read};
    } elsif ($active) {
	if ($outDir) {
	    my $rd = $read;
	    $rd =~ s/-/\//;
	    my $gh = openToWrite("$outDir/$rd.fa");
	    print $gh ">$read\n$_\n";
	    $gh->close();
	} else {
	    print ">$read\n$_\n"; # to stdout
	}
	$active = 0;
    }
}
