#!/usr/bin/env perl

# Run this in the NNN_solexa dir!

use Utility;
use SGRP::OffsetMap;
use File::Basename;
use strict;

my $MARKER_OVERLAP_SIZE = 20; # From Margarita.pm

my $NARGS = 25;

my $mapFile = $ARGV[0] || die("Need to supplied reference genome.map file");
my $pOM = new OffsetMap('?',$mapFile);

foreach my $d (glob("p?/*")) {
    my $tgt = "$d/recombinationFrequencies.txt";
    if (stamp($tgt)) {
	my %freq;
	my @dirs = glob("$d/m[0-9][0-9][0-9]");
	my $chr = basename($d);
	my $circ = $chr !~ /^chr..$/;
	foreach my $k (0 .. $#dirs) {
	    my $d2 = $dirs[$k];
	    report("$d2: processing");
	    my $pOffs = readMarkerOffsets("$d2/mgr_in");
	    my @fr;
	    my @files = glob("$d2/arg???.mgr");
	    unless (@files == $NARGS) {
		die(sprintf("%s: we have %d arg???.mgr files, expecting %d",
			    $d2,scalar(@files),$NARGS));
	    }
	    foreach my $f (@files) {
		augmentFrequencies($f,\@fr);
	    }
	    my $iMin = ($k == 0 && !$circ) ? 0 : $MARKER_OVERLAP_SIZE;
	    my $iMax = ($k == $#dirs && !$circ) ? $#fr : $#fr-$MARKER_OVERLAP_SIZE;
	    foreach my $i ($iMin .. $iMax) {
		# "|| 0" is to ensure we get zero counts, so final result shows
		# where markers are.
		$freq{$pOffs->[$i]} += $fr[$i] || 0;
	    }
	    report();
	}
	report("$tgt: writing");
	my $gh = openToWrite($tgt);
	my @offs = sort {$a <=> $b} keys %freq;
	my $refChr = "REF.$chr";
	foreach my $i (0 .. $#offs-1) {
	    my $off = $offs[$i];
	    my $n = $freq{$off};
	    if ($n) {
		my $lo = $off;
		my $hi = $offs[$i+1]-1;
		my $refLo = $pOM->backTranslation($refChr,$lo,2);
		my $refHi = $pOM->backTranslation($refChr,$hi,1);
		printf($gh "%7d %7d %7d %7d %6.2f\n",$lo,$hi,$refLo,$refHi,$n/$NARGS);
	    }
	}
	report();
	unstamp($tgt);
    }
}

sub readMarkerOffsets {
    my ($f) = @_;
    my $fh = openToRead($f);
    chomp (my $hdr = <$fh>);
    my (undef,$nm) = split(" ",$hdr);
    my @offs;
    foreach my $i (1 .. $nm) {
	chomp (my $off = <$fh>);
	push @offs,$off;
    }
    return \@offs;
}

sub augmentFrequencies {
    my ($f,$pFreq) = @_;
    my $fh = openToRead($f);
    while (<$fh>) {
	chomp;
	my @a = split;
	$pFreq->[$a[5]] ++ if ($a[1] eq 're');
    }
}
