#!/usr/bin/env perl

# Given a list of layout (etc) files, print lengths and locations of
# gaps in reference; OR given a directory, call refGapLengths intelligently
# on the sorted.gz files in all its NNN subdirs.

use Utility;
use strict;

my $MIN_GAP_LENGTH = 10;

foreach my $f (@ARGV) {
    if (-d $f) {
	refGapLengthsDir($f);
    } else {
	refGapLengths($f,*STDOUT);
    }
}

sub refGapLengthsDir {
    my ($dir) = @_;
    my @subs = glob("$dir/[0-9][0-9][0-9]");
    my @exts;
    my $idx = length($subs[0])+1;
    foreach my $d (glob($subs[0] . "/p?/*")) {
	if (-s "$d/sorted.gz") {
	    push @exts,substr($d,$idx);
	}
    }
    foreach my $i (0 .. $#subs) {
	my $g = "$subs[$i]/rgl.txt";
	next if (-s $g);
	report("creating $g");
	my @files;
	foreach my $ext (@exts) {
	    for (my $j=$i; $j>=0; $j--) {
		my $f = "$subs[$j]/$ext/sorted.gz";
		if (-s $f) {
		    push @files,$f;
		    last;
		}
	    }
	}
	my $gh = openToPipe("sort +1 -nr > $g");
	foreach my $f (@files) {
	    refGapLengths($f,$gh);
	}
	$gh->close();
	report();
    }
}

sub refGapLengths {
    my ($f,$gh) = @_;
    my $fh = openToRead($f);
    my $len;
    my $off;
    while (<$fh>) {
	if (/^[A-Za-z]/) {
	    $off ++ if (/^[A-Z]/);
	    if ($len>0) {
		print $gh "$f $len $off\n" if ($len >= $MIN_GAP_LENGTH);
		$len=0;
	    }
	} elsif (/^[-_=]/) {
	    $len ++;
	}
    }
    if ($len>=100) {
	print $gh "$f $len $off\n";
    }
}
