#!/usr/bin/perl

# Args: kell.fa, and output of ssaha2AlignmentCoverage.pl

# Outputs: notcov.fasta, notcov.supercontigs, notcov.aliases

use Utility;
use strict;
my ($faFile,$covFile) = @ARGV;

my $MIN_SZ = 1000;

my $pSeq = readFastaFile($faFile);
my $fh = openToRead($covFile);

my $gh1 = openToWrite("notcov.fasta");
my $gh2 = openToWrite("notcov.supercontigs");
my $gh3 = openToWrite("notcov.aliases");

my $NN = 616; # where Yong's leave off

while (<$fh>) {
    chomp;
    my ($name,undef,$cov,undef,undef,@regs) = split;
    if ($cov == 0) {
	mayPrintSequence($name,$pSeq->{$name},$gh1,$gh2,$gh3);
    } else {
	foreach my $i (0 .. $#regs) {
	    $regs[$i] = [split(/\.\./,$regs[$i])];
	}
	my $i = 0;
	while ($i <= $#regs) {
	    my $j=$i;
	    while ($j < $#regs 
		   && $regs[$j+1][0] <= $regs[$j][1]+$MIN_SZ+1) {
		$j++;
	    }
	    my $lo = $regs[$i][0];
	    my $hi = $regs[$j][1];
	    my $seq = substr($pSeq->{$name},$lo-1,$hi-$lo+1);
	    mayPrintSequence(sprintf("%s_%d_%d\n",$name,$lo,$hi),$seq,$gh1,$gh2,$gh3);
	    $i = $j+1;
	}
    }
}

sub mayPrintSequence {
    my ($name,$seq,$gh1,$gh2,$gh3) = @_;
    my $iLim = length($seq);
    if ($iLim >= $MIN_SZ) {
	$NN ++;
	my $alias = sprintf("Contig_%7d",$NN);
	$alias =~ tr/ /0/;
	print $gh1 ">$alias\n";
	print $gh2 "supercontig Superc" . substr($alias,1) . "\n";
	print $gh2 "contig $alias\n\n";
	print $gh3 "$alias $name\n";
	for (my $i=0; $i<$iLim; $i+=60) {
	    print $gh1 substr($seq,$i,60) . "\n";
	}
    }
}
