#!/usr/bin/env perl

use Utility;
use strict;

# Run from "read" dir

if (@ARGV == 2) {
    renameStrain(@ARGV);
} else {
    die("Usage: $0 fromName toName\n");
}

sub renameStrain {
    my ($from,$to) = @_;
    if (! -d $from) {
	die("Missing directory: $from\n");
    } elsif (-d $to || -f $to) {
	die("Directory already exists: $to\n");
    }
    renameStrain1($from,$to,$from,$to);
}

sub renameStrain1 {
    my ($from,$to,$fromName,$toName) = @_;
    if (-d $from) {
	opendir(D,$from);
	my @bases = readdir(D);
	closedir(D);
	foreach my $b (@bases) {
	    if ($b =~ /^[^\.]/) {
		renameStrain1("$from/$b","$to/$b",$fromName,$toName);
	    }
	}
    } elsif (-s $from) {
	my $fh = openToRead($from);
	my $gh = openToWrite($to);
	my $lf = length($fromName);
	my $ns = 0;
	while (<$fh>) {
	    chomp;
	    my $posMax = length($_)-$lf;
	    my $pos = 0;
	    my @posList;
	    while (($pos = index($_,$fromName,$pos)) >= 0) {
		unshift @posList,$pos; # so we can substitute from the end
		$pos += $lf;
	    }
	    if (@posList) {
		foreach my $pos (@posList) {
		    next if ($pos > 0 && 
			     substr($_,$pos-1,1) =~ /[A-Za-z0-9]/);
		    next if ($pos <= $posMax && 
			     substr($_,$pos+$lf,1) =~ /[A-Za-z0-9]/);
		    substr($_,$pos,$lf) = $toName;
		    $ns ++;
		}
	    }
	    print $gh "$_\n";
	}
	printf("%7d substitutions in %s\n",$ns,$to);
    } elsif (-f $from) {
	my $gh = openToWrite($to);
	$gh->close();
    }
}
