#!/usr/bin/env perl

use Utility;
use SGRP::TranslateGenes;
use strict;

reportCorrectionEffects();

sub reportCorrectionEffects {

    my $top = '/lustre/work1/sanger/dmc/sgroot/release3';

    my $faFile = "$top/cere/ref/genome.fa";
    my $gffFile = "$top/cere/ref/genome.gff";
    
    my $corrFile = "$ENV{HOME}/forSGD.txt";
    
    my $pSeq = readFastaFile($faFile);
    
    my $pCorrs = readCorrectionFile($corrFile);

    my $fh = openToRead($gffFile);
    my ($currentChr,@corrList);
    while (<$fh>) {
	chomp;
	my ($chr,undef,$type,$lo,$hi,undef,$strand,$ph,$anno) = split;
	next unless ($type eq 'CDS');
	my ($gene) = ($anno =~ /Parent=([^;]+);/);
	my $efst; # effective start of codon
	if ($strand eq '+') {
	    $efst = $ph > 0 ? $lo+$ph-3 : $lo;
	    #$efst ++;
	} else {
	    $efst = $ph > 0 ? $hi-$ph+3 : $hi;
	}
	unless ($currentChr eq $chr) {
	    $currentChr = $chr;
	    if (defined $pCorrs->{$chr}) {
		@corrList = @{$pCorrs->{$chr}};
	    #} else {
		#print "No corrections for $chr\n";
	    }
	}
	while (@corrList && $corrList[0][0] < $lo) {
	    shift @corrList;
	}
	foreach my $i (0 .. $#corrList) {
	    my $pA = $corrList[$i];
	    my ($off,$v1,$v2,$q) = @$pA;
	    last if ($off > $hi || ($off == $hi && $v1 eq '-'));
	    if ($v1 eq '-' || $v2 eq '-') {
		print "$chr @$pA in $gene $lo $hi $strand = frame shift\n";
	    } else {
		my ($codon1,$codon2,$codonPosSet);
		if ($strand eq '+') {
		    my $codonStart = $efst+3*int(($off-$efst)/3);
		    if ($hi-$codonStart < 2) {
			print "$chr @$pA in $gene = incomplete codon\n";
			next;
		    }
		    $codon1 = substr($pSeq->{$chr},$codonStart-1,3);
		    $codon2 = $codon1;
		    # Try all positions in the codon...
		    foreach my $d (-2 .. 2) {
			my $j = $i+$d;
			next unless ($j >= 0 && $j <= $#corrList);
			my ($offD,$v1D,$v2D) = @{$corrList[$i+$d]};
			my $codonPosD = $offD-$codonStart;
			if ($codonPosD >= 0 && $codonPosD <= 2) {
			    substr($codon2,$codonPosD,1) = $v2D;
			    $codonPosSet .= $codonPosD;
			}
		    }
		} else {
		    # Rightmost base of codon ...
		    my $codonStart = $efst-3*int(($efst-$off)/3);
		    if ($codonStart-$lo < 2) {
			print "$chr @$pA in $gene = incomplete codon\n";
			next;
		    }
		    $codon1 = substr($pSeq->{$chr},$codonStart-3,3);
		    $codon2 = $codon1;
		    foreach my $d (-2 .. 2) {
			my $j = $i+$d;
			next unless ($j >= 0 && $j <= $#corrList);
			my ($offD,$v1D,$v2D) = @{$corrList[$i+$d]};
			my $codonPosD = $codonStart-$offD;
			if ($codonPosD >= 0 && $codonPosD <= 2) {
			    substr($codon2,2-$codonPosD,1) = $v2D;
			    $codonPosSet .= $codonPosD;
			}
		    }
		    ($codon1 = reverse($codon1)) =~ tr/ACGT/TGCA/;
		    ($codon2 = reverse($codon2)) =~ tr/ACGT/TGCA/;
		}
		my $aa1 = translateCodonSafely($codon1);
		my $aa2 = translateCodonSafely($codon2);
		print "$chr @$pA in $gene $strand = ";
		if ($aa1 eq $aa2) {
		    print "synonymous     $aa1 > $aa2 $codonPosSet\n";
		} else {
		    print "nonsynonymous  $aa1 > $aa2 $codonPosSet\n";
		}
	    }
	}
    }
}

sub translateCodonSafely {
    my ($codon) = @_;
    if ($codon =~ /-/) {
	return '*';
    } else {
	return translateCodon($codon);
    }
}

sub readCorrectionFile {
    my ($corrFile) = @_;
    my $fh = openToRead($corrFile);
    my %corrs;
    while (<$fh>) {
	chomp;
	my ($chr,$off,$v1,$v2,$q) = split;
	push @{$corrs{$chr}},[$off,$v1,$v2,$q];
    }
    return \%corrs;
}

