#!/usr/bin/env perl

use Utility;
use Getopt::Std;
use strict;

# Input: e.g. s288cDis2q35.txt
# Switches:
#   -s (0,1,2): 0 = all, 1 = non-subtelomeres, 2 = subtelomeres

my %OPT;
getopts('s:',\%OPT);

my $where = $OPT{s} || 0;

my %chrLen;
my $fh = openFromPipe("chrsz chr..");
while (<$fh>) {
    chomp;
    my ($chr,$sz) = split;
    $chrLen{$chr} = $sz;
}

my %count;
while (<>) {
    chomp;
    my ($chr,$off,$v1,$v2,$q,$vote1,$vote2,$voteOther) = split;
    next if ($v1 eq '=' || $v2 eq '=');
    if (locationOK($chr,$off,\%chrLen,$where)) {
	my $i=0;
	if ($v2 eq '-') {
	    $i=3;
	} elsif ($v1 eq '-') {
	    $i=6;
	}
	my $j;
	if ($vote1 > 0 && $vote2 == 0) {
	    $j = 0;
	} elsif ($vote2 > 0 && $vote1 == 0) {
	    $j = 1;
	} else {
	    $j = 2;
	}
	$count{$chr}[$i+$j] ++;
	$count{'whole'}[$i+$j] ++;
    }
}

print "         subst        ins_v1       ins_v2\n";
print "       v1  v2  un   v1  v2  un   v1  v2  un\n";
foreach my $chr (sort keys %count) {
    print $chr;
    foreach my $i (0 .. 8) {
	printf(" %3d",$count{$chr}[$i]);
	print " " if ($i%3 == 2);
    }
    print "\n";
}

sub locationOK {
    my ($chr,$off,$pLen,$where) = @_;
    return 1 unless ($where);
    my $inST = $off <= 30000 || $off > $pLen->{$chr}-30000;
    return $where == ($inST ? 2 : 1);
}
