#!/usr/bin/env perl

# Input: ssaha2 output file (or similar) with ALIGNMENT lines
# Output: RBH subset thereof, no overlaps.

use strict;
my %scoreOver;
my @lines;
while (<>) {
    chomp;
    push @lines,$_ if (/^ALIGNMENT/);
}
foreach my $line (@lines) {
    my (undef,$score,$query,$sbjct,$qlo,$qhi,$slo,$shi) = split(" ",$line);
    ($qlo,$qhi) = sort {$a <=> $b} ($qlo,$qhi);
    ($slo,$shi) = sort {$a <=> $b} ($slo,$shi);
    push @{$scoreOver{$query}},[$qlo,$qhi,$score];
    push @{$scoreOver{$sbjct}},[$slo,$shi,$score];
}

foreach my $key (keys %scoreOver) {
    $scoreOver{$key} = pruneScoresOver($scoreOver{$key});
}

foreach my $line (@lines) {
    my (undef,$score,$query,$sbjct,$qlo,$qhi,$slo,$shi) = split(" ",$line);
    if (scoresTop($score,$query,$qlo,$qhi) &&
	scoresTop($score,$sbjct,$slo,$shi)) {
	print "$line\n";
    }
}

sub pruneScoresOver {
    my ($pA) = @_;
    my @ans;
    my @arr = sort {$b->[2] <=> $a->[2]} @$pA;
    my @ans = shift @arr;
    foreach my $pTrip (@arr) {
	my ($lo,$hi,$score) = @$pTrip;
	if ($ans[-1][1] < $lo) {
	    push @ans,$pTrip;
	} elsif ($ans[0][0] > $hi) {
	    unshift @ans,$pTrip;
	} else {
	    foreach my $i (1 .. $#ans) {
		if ($ans[$i][0] > $hi) {
		    @ans = (@ans[0..$i-1],$pTrip,@ans[$i..$#ans]); # use it
		    last;
		} elsif ($ans[$i][1] >= $lo) {
		    last; # lose it
		}
	    }
	}
	#print scalar(@ans) . "\n";
    }
    return \@ans;
}

sub scoresTop {
    my ($score,$name,$lo,$hi) = @_;
    ($lo,$hi) = sort {$a <=> $b} ($lo,$hi);
    foreach my $pTrip (@{$scoreOver{$name}}) {
	unless ($pTrip->[1] < $lo) {
	    return $score == $pTrip->[2];
	}
    }
    die("scoresTop failed on @_");
}
    
    
