#!/usr/bin/env perl

use Utility;
use strict;

# Six numbers after strain name are:
# number of alignment locations with:
#   -- ref nuc, strain nuc
#   -- ref nuc, strain gap
#   -- ref nuc, strain space
#   -- ref gap, strain nuc
#   -- ref gap, strain gap
#   -- ref gap, strain space

foreach my $f (@ARGV) {
    my $g = "$f.seqcounts";
    $g =~ s/\.gz//;
    if (stamp($g)) {
	my %count;
	report("$f: reading");
	sequenceCounts($f,\%count);
	report();
	report("$g: writing");
	my $gh = openToWrite($g);
	foreach my $str (sort keys %count) {
	    printf($gh "%13s",$str);
	    foreach my $k (0 .. 5) {
		printf($gh " %8d",$count{$str}[$k]);
	    }
	    print $gh "\n";
	}
	$gh->close();
	unstamp($g);
	report();
    }
}

sub sequenceCounts {
    my ($f,$pC) = @_;
    my $fh = openToRead($f);
    my $hdr = <$fh>;
    chomp $hdr;
    my @strains = split(" ",substr($hdr,1));
    my @count;
    while (<$fh>) {
	chomp;
	my $r = substr($_,0,1);
	my $vals = substr($_,2);
	$vals =~ s/(.)./\1/g;
	my $iLim = length($vals);
	my $rGap = ($r =~ /[A-Z]/) ? 0 : 3;
	for (my $i=0; $i<=$#strains; $i++) {
	    my $vn = 2; # space by default
	    if ($i < $iLim) {
		my $v = substr($vals,$i,1);
		if ($v =~ /[A-Za-z]/) {
		    $vn = 0;
		} elsif ($v ne ' ') {
		    $vn = 1;
		}
	    }
	    $count[$i][$rGap+$vn] ++;
	}
    }
    foreach my $i (0 .. $#strains) {
	my $str = $strains[$i];
	foreach my $k (0 .. 5) {
	    $pC->{$str}[$k] += $count[$i][$k];
	}
    }
}
    
