#!/usr/bin/env perl

# Arg(s): dir(s) with NNN subdirs

use Utility;
use Cwd;
use File::Basename;
use strict;

my $REAL = 1;

foreach my $d (@ARGV) {
    if (-d $d) {
	sgrpClean($d);
    } else {
	mayReport("$d: missing, or not a directory",1);
    }
}

sub mayReport {
    report(@_) if ($REAL);
}

sub sgrpClean {
    my ($dir) = @_;
    my @subs = glob("$dir/[0-9][0-9][0-9]");
    my $iMax;
    if (-f "$subs[-2]/isFinalIteration" && -f "$subs[-1]/isExtraIteration") {
	$iMax = $#subs-1;
    } else {
	print "Not finished: $dir\n";
	$iMax = $#subs-2;
    }
    foreach my $i (0 .. $iMax) {
	sgrpClean1($subs[$i],$subs[$i+1]);
    }
}

sub sgrpClean1 {
    my ($dir,$nextDir) = @_;
    if (-f "$dir/.cleaned") {
	mayReport("$dir: already cleaned",1);
	return;
    }
    touch("$dir/.cleaned");
    #mayReport("$dir/align: cleaning");
    sgrpCleanAlign("$dir/align");
    #mayReport();
    #mayReport("$dir/assembly: cleaning");
    sgrpCleanAssembly("$dir/assembly");
    #mayReport();
    my $len = length($dir);
    my $isFinal = -f "$dir/isFinalIteration";
    foreach my $chrDir (glob("$dir/p?/*")) {
	my $nextChrDir = $nextDir . substr($chrDir,$len);
	if (-d $nextChrDir) {
	    mayReport("$chrDir: cleaning");
	    sgrpCleanChrDir($chrDir,$isFinal);
	    mayReport();
	} else {
	    mayReport("$chrDir: leaving, since no $nextChrDir",1);
	}
    }
}

sub sgrpCleanAlign {
    my ($dir) = @_;
    foreach my $strDir (glob("$dir/*")) {
	if (-d $strDir) {
	    zap("$strDir/ssahaSNP.?.out.gz") if (-s "$strDir/ssahaSNP.all.gz");
	    zap("$strDir/renamed.?.fastq");
	    foreach my $t (glob("$strDir/*.txt")) {
		report("$t: gzipping");
		`gzip $t`;
		report();
	    }
	}
    }
}

# Should gzip all but .ali.gz, which is already gzipped.
sub sgrpCleanAssembly {
    my ($dir) = @_;
    foreach my $strDir (glob("$dir/*")) {
	foreach my $f (glob("$strDir/*")) {
	    unless ($f =~ /\.gz$/) {
		report("$f: gzipping");
		`gzip $f`;
		report();
	    }
	}
    }
}

sub zap {
    my ($pat) = @_;
    if ($REAL) {
	foreach my $f (glob($pat)) {
	    if (-d $f) {
		`rm -rf $f`;
	    } else {
		unlink($f);
	    }
	}
    } else {
	print "rm -rf $pat\n" if (glob($pat));
    }
}
	    
sub sgrpCleanChrDir {
    my ($dir,$isFinal) = @_;
    if ($isFinal) {
	my $cwd = cwd();
	foreach my $mDir (glob("$dir/m0??")) {
	    chdir(dirname($mDir));
	    my $b = basename($mDir);
	    `tar zcf $b.tgz $b`;
	    chdir($cwd);
	}
    } else {
	zap("$dir/rqpw.gz");
	zap("$dir/clbs*");
    }
    zap("$dir/m0??");
    zap("$dir/impm");
}
