#!/usr/bin/env perl

use Utility;
use strict;

my $top = "$ENV{SGRW}/release3";

my $g = "$top/tables/snpDensityTable.txt";
my $gh = openToWrite($g);
autoflush $gh 1;

# Each line of output is:
#   species  strain percentCoveredx4 snpRatePerThoux4
# Each of the "x4"s is four values, for sequenced, quality >=0;
# sequenced, quality >=40; imputed, quality>=0; imputed, quality>=40.
# Note that the SNP rates are calculated by
#    1000 * snpCount / (genomeSz * fractionCovered)
# i.e. it is in principle possible for the q40 values to be more than q0
# because the "fractionCovered" can be less.

printf($gh "%4s %13s",
       "spec","strain");
printf($gh " %6s %6s %6s %6s",
       "seq00","seq40","imp00","imp40");
printf($gh " %8s %8s %8s %8s",
       "seq00","seq40","imp00","imp40");
print $gh "\n";

foreach my $sp (qw(cere para)) {
    my $sz = sumCol(1,"$top/$sp/ref/genome.sz");
    my %forc;
    fractionsOfReferenceCovered("$top/tables/qualdist_${sp}_sequenced.txt",\%forc,0);
    fractionsOfReferenceCovered("$top/tables/qualdist_${sp}_imputed.txt",\%forc,2);
    snpDensityTable($top,$sp,$sz,\%forc,$gh);
}

sub fractionsOfReferenceCovered {
    my ($f,$pForc,$n2) = @_;
    my $fh = openToRead($f);
    while (<$fh>) {
	chomp;
	if (/^  ([ 4])0 /) {
	    my $n = $n2+($1 == 4 ? 1 : 0);
	    my @a = split;
	    $pForc->{$a[-1]}[$n] = $a[-2]-$a[-3];
	}
    }
}

sub sumCol {
    my ($col,$f) = @_;
    my $fh = openToRead($f);
    my $sum = 0;
    while (<$fh>) {
	chomp;
	my @a = split;
	$sum += $a[$col];
    }
    return $sum;
}

sub snpDensityTable {
    my ($top,$sp,$sz,$pForc,$gh) = @_;
    my %count;
    my $n2 = 0;
    foreach my $type (qw(sequenced imputed)) {
	foreach my $f (glob("$top/$sp/match/*/${type}SNPs.txt")) {
	    report("$f: reading");
	    my $fh = openToRead($f);
	    while (<$fh>) {
		chomp (my $line = <$fh>); # line we want
		last unless ($line);
		#print "$line\n";
		<$fh>; # quality line
		<$fh>; # blank line
		my @a = split(" ",$line);
		my $str = $a[-3];
		unless ($str eq 'MRCA') {
		    my $qual = $a[-1];
		    $count{$str}[$n2] ++;
		    $count{$str}[$n2+1] ++ if ($qual >= 40);
		}
	    }
	    report();
	}
	$n2 += 2;
    }
    foreach my $str (sort keys %count) {
	my $pA = $count{$str};
	printf($gh "%4s %13s",$sp,$str);
	foreach my $i (0 .. 3) {
	    my $frac = $pForc->{$str}[$i];
	    printf($gh " %6.2f",100*$frac);
	}
	foreach my $i (0 .. 3) {
	    my $frac = $pForc->{$str}[$i] || die("No frac for $str $i");
	    printf($gh " %8.4f",1000*$pA->[$i]/($frac*$sz));
	}
	print $gh "\n";
    }
    report();
}
