#!/usr/bin/env perl

# Args: file mapping strain names (including REF) to clusters,
# and sequenced.gz or imputed.gz file(s).

# Cluster IDs should be 1,2,..., with a default of 0 for strains
# not mentioned.

use Utility;
use strict;

autoflush STDOUT 1;

my %cluster;
fileToHash(shift @ARGV,\%cluster);

my %count;
foreach my $f (@ARGV) {
    report("$f: reading");
    my $fh = openToRead($f);
    chomp (my $hdr = <$fh>);
    my @strains = ('REF',split(" ",substr($hdr,1)));
    my @cluster;
    my $maxC;
    foreach my $s (@strains) {
	push @cluster,$cluster{$s} || 0;
	$maxC = max($maxC,$cluster{$s});
    }
    while (<$fh>) {
	chomp;
	unless (/^>/) {
	    my $pat = clusterSequence($_,\@cluster,$maxC);
	    $count{$pat} ++ if (defined $pat);
	}
    }
    report();
}
foreach my $pat (sort {$count{$b} <=> $count{$a}} keys %count) {
    printf("%s %5d\n",$pat,$count{$pat});
}

sub clusterSequence {
    my ($line,$pCluster,$maxC) = @_;
    # Zap qualities
    $line =~ s/(.)./\1/g;
    # Upper case everything
    $line =~ tr/a-z/A-Z/;
    my @vals = split("",$line);
    my %nv;
    foreach my $v (@vals) {
	$nv{$v}++ if ($v =~ /[ACGT]/);
    }
    my @voc = sort {$nv{$b} <=> $nv{$a}} keys %nv;
    return unless (@voc == 2);
    my ($maj,$min) = @voc;
    my @seq;
    foreach my $i (0 .. $#vals) {
	my $j;
	my $v = $vals[$i];
	if ($v eq $maj) {
	    $j = 0;
	} elsif ($v eq $min) {
	    $j = 2;
	} else {
	    next;
	}
	my $clus = $pCluster->[$i];
	if (defined $seq[$clus] && $seq[$clus] != $j) {
	    $seq[$clus] = 1;
	} else {
	    $seq[$clus] = $j;
	}
    }
    return unless ($#seq == $maxC);
    my $pat;
    foreach my $v (@seq) {
	if (defined $v) {
	    $pat .= $v;
	} else {
	    return undef;
	}
    }
    return $pat;
}
