#!/usr/bin/env perl

use Utility;
use File::Basename;
use Cwd;
use strict;

autoflush STDOUT 1;

# Args: dirs to process, e.g. p?/chr*

my $BASE = 'sorted_z';

foreach my $d (@ARGV) {
    snpsByWindows($d);
}

sub snpsByWindows {
    my ($dir) = @_;
    #my $f = "$dir/imputed.gz";
    #$f = "$dir/sorted.gz" unless (-s $f);
    my $f = "$dir/$BASE.gz";
    unless (-s $f) {
	print "WARNING: cannot find $dir/$BASE.gz\n";
	return;
    }
    my $g = "$dir/sbw.txt";
    unless (stamp($g)) {
	print "Already exists: $g\n";
	return;
    }
    #report("$g: creating");
    my $chr = basename(absoluteFileName($dir));
    my $fh = openToRead($f);
    my $hdr = <$fh>;
    chomp $hdr;
    my @strains = ('REF',split(" ",substr($hdr,1)));
    my (@num40,@pol40,@den40,@num0,@den0,@pol0);
    my $pBlock;
    my $repName = sprintf("%s/%s",$chr,basename($f));
    while ($pBlock = readBlock($fh)) {
	report("reading block from $repName");
	my (@n40,@d40,@n0,@d0,@p40,@p0);
	foreach my $line (@$pBlock) {
	    my (@val,@hq,$poly40,$poly0,%at0,%at40);
	    for (my $i=0; $i<=$#strains; $i++) {
		my $vi = substr($line,$i*2,1);
		if ($vi =~ /[ACGTacgt]/) {
		    $vi =~ tr/acgt/ACGT/;
		    my $qi = substr($line,$i*2+1,1);
		    my $hi = ($qi eq ' ' || $qi ge 'I');
		    $at0{$vi} = 1;
		    $at40{$vi} = 1 if ($hi);
		    push @val,$vi;
		    push @hq,$hi;
		} else {
		    push @val,undef;
		    push @hq,undef;
		}
	    }
	    my $isPoly0 = keys %at0 > 1;
	    my $isPoly40 = keys %at40 > 1;
	    for (my $i=0; $i<$#strains; $i++) {
		if ($val[$i]) {
		    for (my $j=$i+1; $j<=$#strains; $j++) {
			if ($val[$j]) {
			    my $eq = ($val[$i] eq $val[$j]);
			    $d0[$i][$j] ++;
			    $n0[$i][$j] ++ unless ($eq);
			    $p0[$i][$j] ++ if ($isPoly0);
			    if ($hq[$i] && $hq[$j]) {
				$d40[$i][$j] ++;
				$n40[$i][$j] ++ unless ($eq);
				$p40[$i][$j] ++ if ($isPoly40);
			    }
			}
		    }
		}
	    }
	}
	push @num40,\@n40;
	push @den40,\@d40;
	push @num0,\@n0;
	push @den0,\@d0;
	push @pol40,\@p40;
	push @pol0,\@p0;
	report();
    }
    my $gh = openToWrite($g);
    foreach my $i (0 .. $#strains-1) {
	my $si = $strains[$i];
	report("writing files for $si");
	foreach my $j ($i+1 .. $#strains) {
	    my $sj = $strains[$j];
	    # We omit the first 30k and the last 20-30k
	    foreach my $k (3 .. $#num0-3) {
		printf($gh "%13s %13s %5s %3d     %4d %5d %4d     %4d %4d %5d\n",
		       $si,$sj,$chr,$k,
		       $num0[$k][$i][$j],
		       $pol0[$k][$i][$j],
		       $den0[$k][$i][$j],
		       $num40[$k][$i][$j],
		       $pol40[$k][$i][$j],
		       $den40[$k][$i][$j]);
	    }
	}
	report();
    }
    unstamp($g);
    #report();
}

sub readBlock {
    my ($fh) = @_;
    my @block;
    my $n = 0;
    while ($n < 10000) {
	my $line = <$fh>;
	unless ($line) {
	    return @block ? \@block : undef;
	}
	if ($line =~ /^[ACGT]/) {
	    chomp $line;
	    push @block,$line;
	    $n++;
	}
    }
    return \@block;
}
