#!/usr/bin/perl

use Utility;

use strict;

sortcode(@ARGV);

sub sortcode {
    foreach my $f (@_) {
	my $g = "sorted/$f";
	sortcode1($f,$g,"graph/$f");
	if (-s $f != -s $g) {
	    print STDERR "WARNING: sizes differ: $f and $g\n";
	} elsif (!identical($f,$g)) {
	    print "Sorted $f\n";
	    rename($g,$f);
	} else {
	    #print "Left   $f\n";
	    unlink($g);
	}
    }
}

sub sortcode1 {
    my ($f,$g,$h) = @_;
    my $fh = openToRead($f);
    my @lines = <$fh>;
    my %range;
    my $ln = 0;
    my $caller;
    my $lo = $ln;
    my $top;
    my %calls;
    foreach my $line (@lines) {
	if ($line =~ /^sub\s+([A-Za-z0-9_]+)/) {
	    $caller = $1;
	    $top = $caller unless ($top);
	} elsif ($line =~ /^\}/) {
	    $range{$caller} = [$lo,$ln];
	    $lo = $ln+1;
	    $caller = undef;
	} elsif ($line =~ /^\@EXPORT = qw\((.*)\);/) {
	    my @subs = split(" ",$1);
	    $caller = 'EXPORT';
	    $top = $caller;
	    $calls{$caller} = \@subs;
	    $range{$caller} = [$lo,$ln];
	    $lo = $ln+1;
	    $caller = undef;
	}
	$ln++;
    }
    if ($ln >= $lo) {
	$range{'zztail'} = [$lo,$ln];
    }
    foreach my $line (@lines) {
	if ($line =~ /^sub\s+(\S+)/) {
	    $caller = $1;
	} elsif ($caller) {
	    my $rest = $line;
	    $rest =~ s/\#.*//;
	    while ($rest =~ /[^A-Za-z0-9_]([A-Za-z0-9_]+)(.*)/) {
		if ($range{$1}) {
		    push @{$calls{$caller}},$1;
		}
		$rest = $2;
	    }
	} elsif ($line =~ /^\}/) {
	    $caller = undef;
	}
    }
    my $gh = openToWrite($g);
    my %outAt;
    my $outLn = printSubs($top,\%calls,\%range,\@lines,\%outAt,1,$gh);
    foreach my $sub (sort keys %range) {
	$outLn = printSubs($sub,\%calls,\%range,\@lines,\%outAt,$outLn,$gh);
    }
    my %done;
    my $hh = openToWrite($h);
    printCallGraph($top,\%calls,\%outAt,\%done,'',$hh);
}

sub printSubs {
    my ($caller,$pCalls,$pRange,$pLines,$pOutAt,$ln,$gh) = @_;
    return $ln unless (defined $pRange->{$caller});
    my ($lo,$hi) = @{$pRange->{$caller}};
    foreach my $k ($lo .. $hi) {
	print $gh $pLines->[$k];
	$pOutAt->{$caller} = $ln if ($pLines->[$k] =~ /^sub /);
	$ln ++;
    }
    $pRange->{$caller} = undef;
    foreach my $called (@{$pCalls->{$caller}}) {
	$ln = printSubs($called,$pCalls,$pRange,$pLines,$pOutAt,$ln,$gh);
    }
    return $ln;
}

sub printCallGraph {
    my ($caller,$pCalls,$pOutAt,$pDone,$ind,$hh) = @_;
    my $ln = sprintf("%3d",$pOutAt->{$caller});
    $ln =~ tr/ /0/;
    if ($pDone->{$caller}) {
	printf($hh "   %s %s @ %s\n",$ind,$caller,$ln);
    } else {
	printf($hh "%s %s%s\n",$ln,$ind,$caller);
	$pDone->{$caller} = 1;
	foreach my $called (uniquifyList(@{$pCalls->{$caller}})) {
	    unless ($called eq $caller) {
		printCallGraph($called,$pCalls,$pOutAt,$pDone,"$ind   ",$hh);
	    }
	}
    }
}

sub uniquifyList {
    my (@list) = @_;
    my (%seen,@ans);
    foreach my $x (@list) {
	unless ($seen{$x}) {
	    $seen{$x}=1;
	    push @ans,$x;
	}
    }
    return @ans;
}

sub identical {
    my ($f,$g) = @_;
    my $fh = openToRead($f);
    my $gh = openToRead($g);
    while (<$fh>) {
	my $gLine = <$gh>;
	return 0 unless ($gLine eq $_);
    }
    if (<$gh>) {
	return 0;
    }
    return 1;
}
