#!/usr/bin/env perl

# Input: e.g. alicoords.gz

use Utility;
use strict;

foreach my $f (@ARGV) {
    spanningDistances($f);
}

sub spanningDistances {
    my ($f) = @_;
    my $fh = openToRead($f);
    my (%lo,%hi);
    while (<$fh>) {
	chomp;
	my @a = split;
	if ($a[0] =~ /[pq]1k$/ && @a == 8) {
	    my ($str,$loc,$pq) = ($a[0] =~ /^(\S+)-(.*)\.([pq])1k$/);
	    if ($pq) {
		if ($a[2] == 1) {
		    $lo{$str}{$loc} = $a[4];
		} else {
		    $hi{$str}{$loc} = $a[6];
		}
	    }
	}
    }
    foreach my $str (keys %lo) {
	my $pLo = $lo{$str};
	my $pHi = $hi{$str};
	my %ends;
	foreach my $loc (keys %$pLo) {
	    if (defined $pHi->{$loc}) {
		my $low = $pLo->{$loc};
		my $high = $pHi->{$loc};
		my $len = $high-$low+1;
		push @{$ends{$low}},[$len,1];
		push @{$ends{$high}},[$len,-1];
	    }
	}
	my %current;
	foreach my $v (sort {$a <=> $b} keys %ends) {
	    foreach my $pA (@{$ends{$v}}) {
		my ($len,$delta) = @$pA;
		if ($delta > 0) {
		    $current{$len} ++;
		} else {
		    if ($current{$len} == 1) {
			delete $current{$len};
		    } else {
			$current{$len} --;
		    }
		}
	    }
	    if (keys %current) {
		my ($max,$min,$sum,$n);
		foreach my $len (keys %current) {
		    $n += $current{$len};
		    $sum += $len*$current{$len};
		    $max = max($len,$max);
		    $min = min($len,$min);
		}
		printf("%s %7d %3d %5d %5d %5d\n",
		       $str,$v,$n,$min,$sum/$n,$max);
	    }
	}
    }
}
