#!/usr/bin/env perl

# Args: key, GFF file, input FA file, positive output, negative output.

use Utility;
use strict;

unless (@ARGV == 5) {
    die("Usage: $0 key gffFile inFile posFile negFile\n");
}

splitGenome(@ARGV);

sub splitGenome {
    my ($key,$gff,$inFile,$posFile,$negFile) = @_;
    my $fh = openToRead($gff);
    my %regs;
    while (<$fh>) {
	chomp;
	my ($chr,undef,$type,$lo,$hi) = split;
	if ($type eq $key) {
	    if (defined $regs{$chr} && $regs{$chr}[-1][1] >= $lo-1) {
		# allow for overlapping elements...
		$regs{$chr}[-1][1] = $hi;
	    } else {
		push @{$regs{$chr}},[$lo,$hi];
	    }
	}
    }
    my $fh = openToRead($inFile);
    my $ph = openToWrite($posFile);
    my $nh = openToWrite($negFile);
    my ($chr,$seq);
    while (<$fh>) {
	chomp;
	if (/^>(\S+)/) {
	    outputChr($chr,$seq,$regs{$chr},$ph,$nh);
	    $chr = $1;
	    $seq = '';
	} else {
	    $seq .= $_;
	}
    }
    outputChr($chr,$seq,$regs{$chr},$ph,$nh);
}

sub outputChr {
    my ($chr,$seq,$pRegs,$ph,$nh) = @_;
    return unless ($chr);
    foreach my $pR (@$pRegs) {
	my ($lo,$hi) = @$pR;
	my $name = sprintf("%s.%7d.%7d",$chr,$lo,$hi);
	$name =~ tr/ /0/;
	print $ph ">$name\n";
	my $piece = substr($seq,$lo-1,$hi-$lo+1);
	printPiece($piece,$ph,60);
	substr($seq,$lo-1,$hi-$lo+1) =~ s/./N/g;
    }
    print $nh ">$chr\n";
    printPiece($seq,$nh,60);
}

sub printPiece {
    my ($seq,$gh,$ll) = @_;
    my $iMax = length($seq)-1;
    for (my $i=0; $i<=$iMax; $i+=$ll) {
	print $gh substr($seq,$i,$ll) . "\n";
    }
}
