#!/usr/bin/env perl

# Args: srcDir mainTgtDir heldoutTgtDir [strains to include]
# e.g.:
#
# cd $SGR/match/CERE/cere
# bsubt -n 16 splitLayoutForImputationPaper.pl methc/030 methc09/000 methc91/000
#
# cd $SGR/match/PARA/para
# INC='Q32_3 Q59_1 Q62_5 Q89_8 Q95_3 S36_7 T21_4 Y6_5 Y7 Z1_1'
# bsubt -n 16 splitLayoutForImputationPaper.pl methc/036 methc09/000 methc91/000 $INC

use Utility;
use strict;

splitLayouts(@ARGV);

sub splitLayouts {
    my ($srcDir,$mainDir,$hoDir,@inc) = @_;
    my $ls = length($srcDir);
    my %inc = map {$_ => 1} @inc;
    foreach my $src (glob("$srcDir/p?/chr??/layout_z.gz")) {
	# next if ($src =~ /chr12/); # rDNA too complicated
	my $tgt1 = sprintf("%s/%s",$mainDir,substr($src,$ls+1));
	$tgt1 =~ s/_z.gz$/.gz/;
	my $tgt2 = sprintf("%s/%s",$hoDir,substr($src,$ls+1));
	$tgt2 =~ s/_z.gz$/.gz/;
	if (stamp($tgt1)) {
	    report("$tgt1 and $tgt2: creating");
	    splitLayout($src,$tgt1,$tgt2,%inc && \%inc);
	    report();
	    unstamp($tgt1);
	}
    }
}

sub splitLayout {
    my ($src,$tgt1,$tgt2,$pInc) = @_;
    my $fh = openToRead($src);
    my $gh1 = openToWrite($tgt1);
    my $gh2 = openToWrite($tgt2);
    my @map;
    my ($n0,$n1,$n2)=(0,0,0);
    while (<$fh>) {
	chomp;
	if (/^>REF /) {
	    print $gh1 "$_\n";
	    print $gh2 "$_\n";
	} elsif (/^>(\S+) (\d+) (\S+)/) {
	    my ($spec,$pos,$iter) = ($1,$2,$3);
	    my ($r) = split(/\./,$spec,2);
	    my ($str) = split(/-/,$r);
	    if ($pInc && !$pInc->{$str}) {
		print $gh1 ">\n";
		print $gh2 ">\n";
		$map[$pos] = undef;
		$n0 ++;
	    } else {
		my $tgtIdx = ($r =~ /9$/) ? 2 : 1;
		# Find a target position
		my @used;
		foreach my $pA (@map) {
		    if (defined $pA && $pA->[0] == $tgtIdx) {
			$used[$pA->[1]] = 1;
		    }
		}
		foreach my $i (0 .. $#used+1) {
		    unless (defined $used[$i]) {
			$map[$pos] = [$tgtIdx,$i];
			if ($tgtIdx == 1) {
			    print $gh1 ">$spec $i $iter\n";
			    print $gh2 ">\n";
			    $n1++;
			} else {
			    print $gh1 ">\n";
			    print $gh2 ">$spec $i $iter\n";
			    $n2++;
			}
			last;
		    }
		}
	    }
	} else {
	    # Split data line
	    my $line1 = substr($_,0,2);
	    my $line2 = $line1;
	    # Allocate pieces
	    my $col=2;
	    foreach my $pos (0 .. $#map) {
		my $pA = $map[$pos];
		if ($pA) {
		    my $pair = substr($_,$col,2);
		    if ($pair eq '  ' || $pair eq '' || !defined $pair) {
			$map[$pos] = undef;
		    } else {
			my ($tgtIdx,$tgtPos) = @$pA;
			my $tgtCol = 2*($tgtPos+1);
			if ($tgtIdx == 1) {
			    $line1 = extendLine($line1,$tgtCol,$pair);
			} else {
			    $line2 = extendLine($line2,$tgtCol,$pair);
			}
		    }
		}
		$col += 2;
	    }
	    print $gh1 "$line1\n";
	    print $gh2 "$line2\n";
	}
    }
    print "$n1 to main, $n2 to held-out, $n0 discarded ";
}

sub extendLine {
    my ($line,$col,$pair) = @_;
    if (length($line) < $col+2) {
	$line .= ' ' x ($col+2-length($line));
    }
    substr($line,$col,2) = $pair;
    return $line;
}

