#!/usr/bin/env perl

# Input: output from
# ssaha2 queries.fa subject.fa -tags 1 -align 1 -best 2
# Output: (joint) best alignment line(s) for each query,
# suffixed with shortfall to next.

use strict;
my ($contName,$contLen,@aligned);
while (<>) {
    chomp;
    if (/^Matches For Query \d+ \((\d+) bases\): (\S+)/) {
	printCoverage($contName,$contLen,@aligned);
	($contLen,$contName,@aligned) = ($1,$2);
    } elsif (/^ALIGNMENT /) {
	my (undef,$score,$query,undef,$lo,$hi,undef,undef,undef,undef,$perc,$qLen) = split;
	unless ($query eq $contName && $qLen == $contLen) {
	    die("Mismatch: $query :: $contName and $qLen :: $contLen");
	}
	unless (@aligned && $perc < 90) {
	    push @aligned,[$lo,$hi];
	}
    }
}
printCoverage($contName,$contLen,@aligned);

sub printCoverage {
    my ($contName,$contLen,@aligned) = @_;
    return unless ($contName);
    my @covered;
    foreach my $pA (@aligned) {
	my ($lo,$hi) = @$pA;
	@covered = sort {$a->[0] <=> $b->[0] || $a->[1] <=> $b->[1]} ([$lo,$hi],@covered);
	@covered = mergeCovered(@covered);
    }
    my $cov;
    foreach my $pA (@covered) {
	$cov += $pA->[1]-$pA->[0]+1;
    }
    printf("%s %6d %6d %6.2f %6d",
	   $contName,$contLen,$cov,100*$cov/$contLen,$contLen-$cov);
    # Print uncovered regions
    my @extended = ([0,0],@covered,[$contLen+1,$contLen+1]);
    foreach my $i (0 .. $#extended-1) {
	my $lo = $extended[$i][1]+1;
	my $hi = $extended[$i+1][0]-1;
	print " $lo..$hi" if ($hi >= $lo+100-1);
    }
    print "\n";
}

sub mergeCovered {
    my (@pairs) = @_;
    my @ans;
    while (@pairs) {
	my ($lo,$hi) = @{shift @pairs};
	if (@ans && $ans[-1][1] >= $lo-1) {
	    $ans[-1][1] = $hi if ($hi > $ans[-1][1]);
	} else {
	    push @ans,[$lo,$hi];
	}
    }
    return @ans;
}
	    
		
