#!/usr/bin/env perl

# Inputs: ssaha2 alignment files, from "-best 1 -tag 1"

use Utility;
use strict;

my $gh = openToPipe("sort");
foreach my $f (@ARGV) {
    ssahaPolymorphisms($f,$gh);
}

sub ssahaPolymorphisms {
    my ($f,$gh) = @_;
    my $fh = openToRead($f);
    my ($read,$chr,$q1,$q2,$s1,$s2,$fc,$qSeq,$sSeq,$isTied,@diffs);
    while (<$fh>) {
	chomp;
	if (/^ALIGNMENT/) {
	    my @a = split;
	    if ($a[2] eq $read) {
		$isTied = 1;
	    } elsif ($read && !$isTied) {
		outputPolymorphisms($read,$chr,$q1,$q2,$s1,$s2,$fc,$qSeq,$sSeq,$gh);
	    }
	    $isTied = $a[2] eq $read;
	    $qSeq=$sSeq="";
	    ($read,$chr,$q1,$q2,$s1,$s2,$fc) = @a[2..$#a];
	} elsif (/^Query/) {
	    my @a = split;
	    $qSeq .= $a[2];
	} elsif (/^Sbjct/) {
	    my @a = split;
	    $sSeq .= $a[2];
	}
    }
    if ($read && !$isTied) {
	outputPolymorphisms($read,$chr,$q1,$q2,$s1,$s2,$fc,$qSeq,$sSeq,$gh);
    }
}

sub outputPolymorphisms {
    my ($read,$chr,$q1,$q2,$s1,$s2,$fc,$qSeq,$sSeq,$gh) = @_;
    my $qOff = $q1;
    my $qDir = $fc eq 'F' ? 1 : -1;
    my $sOff = $s1;
    my $off = 0; # in qSeq and sSeq
    my $offMax = length($qSeq)-1;
    my @diffs;
    while ($off <= $offMax) {
	my $len = 0; # length of run of differences
	while (1) {
	    my $pos = $off+$len;
	    last if ($pos > $offMax || substr($qSeq,$pos,1) eq substr($sSeq,$pos,1));
	    $len++;
	}
	if ($len > 0) {
	    my $qSub = substr($qSeq,$off,$len);
	    my $sSub = substr($sSeq,$off,$len);
	    (my $qNucs = $qSub) =~ s/-//g;
	    (my $sNucs = $sSub) =~ s/-//g;
	    my $qEnd = $qOff + $qDir*(length($qNucs)-1);
	    my $sEnd = $sOff + length($sNucs)-1;
	    push @diffs,join(":",$sOff,$sEnd,$qOff,$qEnd,$sSub,$qSub);
	    $qOff = $qEnd+$qDir;
	    $sOff = $sEnd+1;
	    $off += $len;
	} else {
	    $qOff += $qDir;
	    $sOff ++;
	    $off ++;
	}
    }
    my $rsp = ' ' x (20-length($read));
    printf($gh "%2s %9d %9d %s %4d %4d %s %s\n",
	   $chr,$s1,$s2,"$read$rsp",$q1,$q2,$fc,join(" ",@diffs));
}
