#!/usr/bin/env perl

use Utility;
use strict;

# Can generate points for strain-pair plot with strainDivergenceValues.pl

strainDistanceDistributions(@ARGV);

sub strainDistanceDistributions {
    foreach my $d (@_) {
	strainDistanceDistribution("$d/recombs/args_bl","$d/recombs/sdd.txt.gz");
    }
}

sub strainDistanceDistribution {
    my ($f,$g) = @_;
    return unless (-s $f);
    return unless (stamp($g));
    report("$g: creating");
    my $fh = openToRead($f);
    my $gh = openToWrite($g);
    my @dists;
    my $loWater = 0;
    my $tMax;
    while (<$fh>) {
	chomp;
	my ($lo,undef,$hi,$size,@pairs) = split;
	while ($loWater < $lo) {
	    my $pMat = shift @dists;
	    foreach my $i (0 .. $tMax-1) {
		foreach my $j ($i+1 .. $tMax) {
		    my @lst = sort {$a <=> $b} @{$pMat->[$i][$j]};
		    my $sum = 0;
		    foreach my $v (@lst) {
			$sum += $v;
		    }
		    my $n = scalar(@lst);
		    unless ($n == 25) {
			die("NArgs = $n at $loWater");
		    }
		    printf($gh "%5d %2d %2d %12.10f %12.10f %12.10f\n",
			   $loWater,$i,$j,$lst[0],$sum/$n,$lst[-1]);
		}
	    }   
	    # print
	    $loWater ++;
	}
	# $len{$mthr}{$desc} is length from mthr down to desc (1 or more branches)
	my (%len,@dtrs,$nt);
	$tMax = scalar(@pairs)/4; # should always be same, once set
	for (my $ii=0; $ii<$#pairs; $ii+=2) {
	    my $i = $ii/2;
	    my $mthr = $pairs[$ii+1];
	    my $bl = $pairs[$ii];
	    $len{$mthr}{$i} = $bl;
	    my $pHI = $len{$i};
	    if (defined $pHI) {
		foreach my $j (keys %$pHI) {
		    $len{$mthr}{$j} = $bl + $pHI->{$j};
		}
	    }
	    push @{$dtrs[$mthr]},$i;
	}
	my @done;
	foreach my $m ($tMax+1 .. $#dtrs) {
	    my ($dm0,$dm1) = @{$dtrs[$m]};
	    my @descs0 = ($dm0,keys %{$len{$dm0}});
	    my @descs1 = ($dm1,keys %{$len{$dm1}});
	    foreach my $d0 (@descs0) {
		if ($d0 <= $tMax) {
		    foreach my $d1 (@descs1) {
			if ($d1 <= $tMax) {
			    my ($da,$db) = sort {$a <=> $b} ($d0,$d1);
			    my $dist = $size*($len{$m}{$da} + $len{$m}{$db});
			    foreach my $mkr ($lo .. $hi) {
				push @{$dists[$mkr-$loWater][$da][$db]},$dist;
			    }
			    $done[$da][$db] = 1;
			}
		    }
		}
	    }
	}
	1;
    }
    unstamp($g);
    report();
}
