#!/usr/bin/env perl

use Utility;
use strict;

# Run in iter dir

strainSetSupport();

sub strainSetSupport {
    my $g = "sss.txt";
    if (-s $g) {
	report("$g: already exists",1);
    } else {
	my %total;
	foreach my $d (glob("p?/*")) {
	    strainSetSupport1($d,\%total);
	}
	my %prop;
	foreach my $set (keys %total) {
	    my $r = $total{$set}[0]/$total{$set}[1];
	    #if ($r >= 0.01) {
	    $prop{$set} = $r;
	    #}
	}
	report("$g: writing");
	my $gh = openToWrite($g);
	foreach my $set (sort {$prop{$b} <=> $prop{$a}} keys %prop) {
	    printf($gh "%8.6f %s\n",$prop{$set},$set);
	}
	$gh->close();
	report();
    }
}

sub strainSetSupport1 {
    my ($d,$pTotal) = @_;
    my $mf = "$d/mgr_in";
    my $sf = "$d/sorted.gz";
    my $af = "$d/args_bl";
    my $g = "$d/sss.txt"; # target
    foreach my $f ($mf,$sf,$af) {
	unless (-s $f) {
	    report("$d: missing $f",1);
	    return undef;
	}
    }
    return readStrainSetSupport($g,$pTotal) if (-s $g);

    my $pMarker = readMarkers($mf);
    my ($pScore,$sum,$nt) = readArgsBL($af,$pMarker);
    my $pStrains = readStrains($sf);

    my %ans;
    report("writing $g");
    my $gh = openToWrite($g);
  LOOP:
    foreach my $pat (sort {$pScore->{$b} <=> $pScore->{$a}} keys %$pScore) {
	my $r = $pScore->{$pat}/$sum;
	#last if ($r < 0.01); # only want certain ones...
	my $set;
	my $n1 = 0;
	foreach my $i (0 .. $#$pStrains) {
	    if (substr($pat,$i,1)) {
		$n1++;
		next LOOP if ($n1 > 4);
		$set .= " $pStrains->[$i]";
	    }
	}
	$set =~ s/^ //;
	printf($gh "%8d %8.6f %s = %s\n",$pScore->{$pat},$r,$pat,$set);
	if (defined $pTotal->{$set}) {
	    my ($n,$d) = @{$pTotal->{$set}};
	    $pTotal->{$set} = [$n+$pScore->{$pat},$d+$sum];
	} else {
	    $pTotal->{$set} = [$pScore->{$pat},$sum];
	}
    }
    report();
}

sub readStrainSetSupport {
    my ($f,$pTotal) = @_;
    report("$f: reading");
    my $fh = openToRead($f);
    while (<$fh>) {
	chomp;
	my ($score,$r,undef,$set) = split(" ",$_,4);
	my ($n,$d);
	($n,$d) = @{$pTotal->{$set}} if (defined $pTotal->{$set});
	$pTotal->{$set} = [$n+$score,$d+$score/$r];
    }
    report();
}

sub readMarkers {
    my ($f) = @_;
    report("reading $f");
    my $fh = openToRead($f);
    my $hdr = <$fh>;
    chomp $hdr;
    my (undef,$nm) = split(" ",$hdr);
    my @mark;
    foreach my $i (0 .. $nm-1) {
	chomp ($mark[$i] = <$fh>);1
    }
    report();
    return \@mark;
}

sub readStrains {
    my ($f) = @_;
    my $fh = openToRead($f);
    my $hdr = <$fh>;
    chomp $hdr;
    return ['REF',split(" ",substr($hdr,1))];
}

sub readArgsBL {
    my ($af,$pMarker) = @_;
    report("reading $af");
    my $fh = openToRead($af);
    my (%score,$sum,$nt);
    while (<$fh>) {
	chomp;
	my ($lo,undef,$hi,undef,@rest) = split;
	my $sz = $pMarker->[$hi]-$pMarker->[$lo];
	if ($hi == $#$pMarker) {
	    $sz += $pMarker->[0]/2; # assume same as at start
	} else {
	    $sz += ($pMarker->[$hi+1]-$pMarker->[$hi])/2;
	}
	if ($lo == 0) {
	    $sz += $pMarker->[0]/2;
	} else {
	    $sz += ($pMarker->[$lo]-$pMarker->[$lo-1])/2;
	}
	my @mthr;
	my @descs;
	my $iMax = scalar(@rest)/2-1; # max daughter id
	$nt = ($iMax+1)/2+1;
	foreach my $i (0 .. $iMax) {
	    $mthr[$i] = my $m = $rest[$i*2+1];
	    if ($i < $nt) {
		push @{$descs[$m]},$i;
	    } else {
		push @{$descs[$m]},@{$descs[$i]};
	    }
	}
	my %seen;
	foreach my $i ($nt .. $iMax) {
	    my $pat = '0' x $nt;
	    foreach my $d (@{$descs[$i]}) {
		substr($pat,$d,1) = 1;
	    }
	    if (scalar(@{$descs[$i]}) > $nt/2) {
		$pat =~ tr/01/10/;
	    }
	    if ($pat =~ /1.*1/) {
		$seen{$pat} = 1;
	    }
	}
	foreach my $pat (keys %seen) {
	    $score{$pat} += $sz;
	}
	$sum += $sz;
    }
    report();
    return (\%score,$sum,$nt);
}
