#!/usr/local/bin/perl

# To create inputs:
# for c in $CHRS; do bsub -o cia_chrs/$c.bout -e cia_chrs/$c.berr compareImputedAcross.pl methc09 methc91 $c; done

use Utility;
use strict;

my ($HEADER,$MAF_HEADER);

# copied from compareImputedAcross ...
my @SUFFIXES = (undef,undef,'d','g','c','v','w','h','f','e','t','m','u',undef);

sub generalizeType {
    my ($type) = @_;
    if ($type eq '' || substr($type,0,1) eq '.') {
	return ($type);
    } else {
	my $x = substr($type,0,1);
	my $y = substr($type,1);
	my @ans;
	foreach my $tail (generalizeType($y)) {
	    push @ans,"$x$tail",".$tail";
	    if ($x > 0) {
		push @ans,"+$tail";
	    }
	}
	return @ans;
    }
}

sub decodeKMG {
    my ($n) = @_;
    if ($n =~ /^(\d+)([kmg])$/) {
	return ($1+0.5)*($2 eq 'k' ? 1000 : $2 eq 'm' ? 1000000 : 1000000000);
    } else {
	return $n;
    }
}

sub encodeKMG {
    my ($n) = @_;
    my $s = encodeKMG1($n);
    return ((4-length($s)) x ' ') . $s;
}

sub encodeKMG1 {
    my ($n) = @_;
    if ($n < 10000) { # allow 4 digits ...
	return $n || 0;
    } elsif ($n < 1000000) {
	return int(0.5+$n/1000) . 'k';
    } elsif ($n < 1000000000) {
	return int(0.5+$n/1000000) . 'm';
    } else {
	return int(0.5+$n/1000000000) . 'g';
    }
}

my (%N,%ERRS,$SUM,%MAF_COUNT,@MAF_KEYS);
foreach my $f (@ARGV) {
    my $fh = openToRead($f);
    while (<$fh>) {
	chomp;
	last if (/^Overall/);
	if (/^[NG][012][pm] /) {
	    #s/         / x/g; # fill in spaces left by missing data
	    my ($type,$n,undef,@rates) = split;
	    foreach my $ty (generalizeType($type)) {
		foreach my $k (0 .. $#rates) {
		    my $r = $rates[$k];
		    unless ($r eq '.') {
			my $c = int(0.5+$n*$r);
			$N{$ty}[$k] += $n;
			$ERRS{$ty}[$k] += $c;
		    }
		}
	    }
	} elsif (/^[NG][012][pm]\.\d\d /) {
	    my ($type,$n,undef,@counts) = split;
	    foreach my $ty (generalizeType($type)) {
		foreach my $k (0 .. $#counts) {
		    my $n = $counts[$k];
		    unless ($n eq '.') {
			$N{$ty}[$k] += decodeKMG($n);
		    }
		}
	    }
	} elsif (/^Type /) {
	    $HEADER = $_;
	} elsif (/^Method/) {
	    $MAF_HEADER= $_;
	} elsif (/^   [a-z]   /) {
	    my ($meth,@a) = split;
	    push @MAF_KEYS,$meth unless (defined $MAF_COUNT{$meth});
	    foreach my $i (0 .. $#a) {
		$MAF_COUNT{$meth}[$i] += $a[$i];
	    }
	}
    }
}

print $HEADER . "\n";
foreach my $type (sort keys %N) {
    #next if ($N{$type}[1] == 0);
    if (length($type) == 3) {
	printf("%3s    %9d %10.8f",$type,$N{$type}[1],$N{$type}[1]/$N{'...'}[1]);
	my $pE = $ERRS{$type};
	foreach my $k (0 .. $#SUFFIXES-1) {
	    my $ntk = $N{$type}[$k];
	    if ($ntk == 0 || ($k == 0 && $type !~ /p$/)) {
		printf(" %10s",'');
	    } else {
		printf(" %10.8f",$pE->[$k]/$ntk);
	    }
	}
    } else {
	printf("%6s %9s %10s",$type,'.','.');
	my $pN = $N{$type};
	foreach my $k (0 .. $#SUFFIXES-1) {
	    my $ntkE = $pN->[2*$k];
	    my $ntkT = $pN->[2*$k+1];
	    printf("  %4s %4s",encodeKMG($ntkE),encodeKMG($ntkT));
	}
    }
    print "\n";
}

print "\nSNP discovery\n\n";
print "$MAF_HEADER    #Poly   %Poly\n";
foreach my $type (@MAF_KEYS) {
    printf("%4s  ",$type);
    my $pC = $MAF_COUNT{$type};
    my $sum=0;
    foreach my $i (0 .. $#$pC) {
	printf(" %6d",$pC->[$i]);
	$sum += $pC->[$i];
    }
    my $np = $sum-$pC->[0];
    printf("  %7d %8.4f\n",$np,100*$np/$sum);
}

