#!/usr/bin/env perl

# Args: gff file and fa file, e.g. ALL.gff genome.fa 
# Output: e.g. 4waysyn.txt

# Prints all offsets that are inside a non-dubious coding region on at
# least one strand, and are four-way synonymous in whichever strands
# they are in such a region for.

use Utility;
use strict;

my ($gff,$fa) = @ARGV;

my %IS_SYN12 = map {$_ => 1} qw(TC CT CC CG AC GT GC GG);

my %SYN;
my $fh = openToRead($gff);
while (<$fh>) {
    chomp;
    my ($chr,undef,$key,$lo,$hi,undef,$ori,$phase) = split;
    next unless ($key eq 'CDS');
    next if (/Dubious/); # don't accept "dubious" genes
    my @list;
    if ($ori eq '+') {
	for (my $k=$lo+2-$phase; $k<=$hi; $k+=3) {
	    $SYN{$chr}{$ori}{$k} = 1;
	}
    } elsif ($ori eq '-') {
	for (my $k=$hi+$phase-2; $k>=$lo; $k-=3) {
	    $SYN{$chr}{$ori}{$k} = 1;
	}
    }
}
	
my $fh = openToRead($fa);
my ($chr,$seq);
while (<$fh>) {
    chomp;
    if (/^>(\S+)/) {
	processChr($chr,$seq) if ($chr);
	$chr = $1;
	$seq = "";
    } else {
	$seq .= $_;
    }
}
processChr($chr,$seq) if ($chr);

foreach my $chr (sort keys %SYN) {
    my @list = sort {$a <=> $b} (keys %{$SYN{$chr}{'+'}}, keys %{$SYN{$chr}{'-'}});
    $chr =~ tr/A-Z/a-z/;
    print "$chr @list\n";
}

sub processChr {
    my ($chr,$seq) = @_;
    my $pH = $SYN{$chr};
    return unless ($pH);
    foreach my $ori (qw(+ -)) {
	my @list = sort {$a <=> $b} keys %{$pH->{$ori}};
	foreach my $k (@list) {
	    my $codon12;
	    if ($ori eq '+') {
		$codon12 = substr($seq,$k-2,2);
	    } else {
		$codon12 = reverse(substr($seq,$k,2));
		$codon12 =~ tr/ACGT/TGCA/;
	    }
	    unless ($IS_SYN12{$codon12}) {
		# Delete in BOTH directions.
		delete $pH->{'+'}{$k};
		delete $pH->{'-'}{$k};
	    }
	}
    }
}
