#!/usr/bin/env perl

use Utility;
use strict;

# 0:500 for X, 0:750 for Y

my $D_CERE_PARA = 0.1428;
my $D_CERE_BAYA = 0.2813;

my @CERE = (100,650);
my @PARA = (400,650);
my @BAYA = (250);
# y (for baya) is such that
# sqrt(150^2+(600-y)^2) = 300*.2813/.1428
# 600 - y = sqrt(300*.2813/.1428 - 150^2)
my $d1 = ($PARA[0]-$CERE[0])*.2813/.1428;
my $d2 = $BAYA[0]-$CERE[0];
$BAYA[1] = $CERE[1] -  sqrt($d1*$d1 - $d2*$d2);

my $SCALE = 0.2813/distance(\@CERE,\@BAYA);

threeSpeciesPlot();

sub distance {
    my ($pA1,$pA2) = @_;
    my $sum = 0;
    foreach my $i (0 .. $#$pA1) {
	my $d = $pA2->[$i]-$pA1->[$i];
	$sum += $d*$d;
    }
    return sqrt($sum);
}

sub threeSpeciesPlot {
    my $gh = openToWrite("threeSpecies.ps");
    print $gh <<\END;
%!
1 setlinecap 1 setlinejoin 1 setlinewidth 0 setgray
/basefont /Times-Roman findfont 12 scalefont def
/titlefont /Times-Roman findfont 12 scalefont def
50 50 translate
-10 -10 moveto 510 -10 lineto 510 760 lineto -10 760 lineto -10 -10 lineto stroke
titlefont setfont
0 765 moveto (Three species plot
) show
basefont setfont
END

    addSpeciesToScale("cere.ps",@CERE,$SCALE,$gh);
    addSpeciesToScale("para.ps",@PARA,$SCALE,$gh);

    printf($gh "%d %d 5 -180 180 arc stroke\n",@BAYA);

    print $gh "showpage\n";

}

sub addSpeciesToScale {
    my ($f,$xOri,$yOri,$scale,$gh) = @_;
    my $fsc = fileScale($f);
    my $fac = $fsc/$scale;
    my $fh = openToRead($f);
    print $gh "0.7 0.3 0 setrgbcolor\n";
    my $dx = $xOri-250*$fac;
    my $dy = $yOri-375*$fac;
    while (<$fh>) {
	chomp;
	my ($x1,$y1,$mt,$x2,$y2,$lt,$st,@rest) = split;
	last if ($x2 > 480 && $y2 >= 720);
	if ($mt eq 'moveto' && $lt eq 'lineto' && $st eq 'stroke' && !@rest) {
	    $x1 *= $fac;
	    $y1 *= $fac;
	    $x2 *= $fac;
	    $y2 *= $fac;
	    $x1 += $dx;
	    $y1 += $dy;
	    $x2 += $dx;
	    $y2 += $dy;
	    printf($gh "%d %d moveto %d %d lineto stroke\n",
		   $x1,$y1,$x2,$y2);
	}
    }
    #printf($gh "%d %d moveto ($f to same scale) show\n",$dx,$dy+20);
}

sub fileScale {
    my ($f) = @_;
    my $fh = openFromPipe("tail -4 $f");
    chomp (my @lines = <$fh>);
    $lines[0] =~ s/.*\((.*)\).*/\1/;
    my @a = split(" ",$lines[1]);
    my @b = split(" ",$lines[2]);
    return $lines[0]/($b[0]-$a[0]);
}
