#!/usr/bin/env perl

use Utility;
use Getopt::Std;
use strict;

# Args:
#  -i chr
#  -f chr off
#  -b chr off

my %OPT;
getopts('ifb',\%OPT);
my ($chr,$off) = @ARGV;
$chr = "REF.$chr" unless ($chr =~ /^REF\./);

my $op;
if (defined $OPT{d} || !defined $off) {
    $op = 'd';
} elsif (defined $OPT{b}) {
    $op = 'b';
} else {
    $op = 'f';
}

my $mFile = "$ENV{SGR}/ref/cere/genome.map";

my @regs;
my $fh = openToRead($mFile);
while (<$fh>) {
    chomp;
    my ($c,$from,$to,$len) = split;
    if ($c eq $chr) {
	push @regs,[$from+1,$to+1,$len];
    }
}
die "Found nothing for $chr\n" unless (@regs);

if ($op eq 'd') {
    printInsertions(@regs);
} elsif ($op eq 'b') {
    printBackwardTranslation($off,@regs);
} else {
    printForwardTranslation($off,@regs);
}

sub printInsertions {
    my @regs = @_;
    foreach my $i (1 .. $#regs) {
	my ($from0,$to0,$len0) = @{$regs[$i-1]};
	my ($from1,$to1,$len1) = @{$regs[$i]};
	my $len = $to1-$len0-1;
	printf("Inserted before ref %7d : ext %7d to %7d with length %5d\n",
	       $from1,$to0+$len0,$to1-1,$to1-$to0-$len0-1);
    }
}

sub printBackwardTranslation {
    my ($off,@regs) = @_;
    foreach my $pA (@regs) {
	if ($pA->[1] <= $off) {
	    if ($pA->[1]+$pA->[2] >= $off) {
		printf("Ext %7d is ref %7d\n",$off,$off+$pA->[0]-$pA->[1]);
		return;
	    }
	} else {
	    printf("Ext %7d is inserted before ref %7d\n",$off,$pA->[0]);
	    return;
	}
    }
    printf("Ext %7d is inserted after end of ref\n",$off);
}

sub printForwardTranslation {
    my ($off,@regs) = @_;
    foreach my $pA (@regs) {
	if ($pA->[0] <= $off && $pA->[0]+$pA->[2] >= $off) {
	    printf("Ref %7d is ext %7d\n",$off,$off+$pA->[1]-$pA->[0]);
	    return;
	}
    }
    printf("Ext %7d is inserted after end of ref\n",$off);
}
