#!/usr/bin/env perl

# Args: genome.fa and genome.gff

# Probably best (e.g. for alicat) to split output like this:

# translateGFF.pl genome.fa genome.gff > translatedGFF.txt
# mkdir translatedGFF
# for c in `pf 1 translatedGFF.txt | sort -u`
#   do
#   grep ^$c translatedGFF.txt | sed 's/'$c'  *//' > translatedGFF/$c.txt
#   done

use Utility;
use strict;


my %TRANSLATION = ( TTT => 'Phe', TTC => 'Phe', TTA => 'Leu', TTG => 'Leu',
		    TCT => 'Ser', TCC => 'Ser', TCA => 'Ser', TCG => 'Ser', 
		    TAT => 'Tyr', TAC => 'Tyr', TAA => 'STP', TAG => 'STP', 
		    TGT => 'Cys', TGC => 'Cys', TGA => 'STP', TGG => 'Trp', 
		    CTT => 'Leu', CTC => 'Leu', CTA => 'Leu', CTG => 'Leu', 
		    CCT => 'Pro', CCC => 'Pro', CCA => 'Pro', CCG => 'Pro', 
		    CAT => 'His', CAC => 'His', CAA => 'Gln', CAG => 'Gln', 
		    CGT => 'Arg', CGC => 'Arg', CGA => 'Arg', CGG => 'Arg', 
		    ATT => 'Ile', ATC => 'Ile', ATA => 'Ile', ATG => 'Met',
		    ACT => 'Thr', ACC => 'Thr', ACA => 'Thr', ACG => 'Thr',
		    AAT => 'Asn', AAC => 'Asn', AAA => 'Lys', AAG => 'Lys', 
		    AGT => 'Ser', AGC => 'Ser', AGA => 'Arg', AGG => 'Arg',
		    GTT => 'Val', GTC => 'Val', GTA => 'Val', GTG => 'Val', 
		    GCT => 'Ala', GCC => 'Ala', GCA => 'Ala', GCG => 'Ala', 
		    GAT => 'Asp', GAC => 'Asp', GAA => 'Glu', GAG => 'Glu', 
		    GGT => 'Gly', GGC => 'Gly', GGA => 'Gly', GGG => 'Gly');

my ($faFile,$gffFile) = @ARGV;

my $fh = openToRead($faFile);
my %seq;
my ($buf,$chr);
while (<$fh>) {
    chomp;
    if (/^>(\S+)/) {
	if ($buf) {
	    $seq{$chr} = $buf;
	    $buf = ();
	}
	$chr = $1;
    } else {
	$buf .= $_;
    }
}
$seq{$chr} = $buf if ($buf);

my %genes;
$fh = openToRead($gffFile);
my $gh = openToPipe("sort");
while (<$fh>) {
    chomp;
    my @a = split;
    next unless ($a[2] eq 'CDS');
    my ($chr,$lo,$hi,$dir,$ph,$anno) = ($a[0],$a[3],$a[4],$a[6],$a[7],$a[8]);
    my $name;
    foreach my $pair (split(/;/,$anno)) {
	my ($key,$val) = split(/=/,$pair);
	if ($val) {
	    $name=$val;
	    last;
	}
    }
    if ($dir eq '+') {
	$lo += 3-$ph if ($ph);
	for (my $i=$lo; $i<=$hi-2; $i+=3) {
	    my $codon = substr($seq{$chr},$i-1,3); # -1 because 1->0 offset
	    my $aa = $TRANSLATION{$codon};
	    printf($gh "%s %7d %s %s %s %s\n",$chr,$i,$dir,$codon,$aa,$name);
	}
    } else {
	$hi -= 3-$ph if ($ph);
	for (my $i=$hi; $i>=$lo+2; $i-=3) {
	    my $codon = reverse(substr($seq{$chr},$i-3,3));
	    $codon =~ tr/ACGT/TGCA/;
	    my $aa = $TRANSLATION{$codon};
	    printf($gh "%s %7d %s %s %s %s\n",$chr,$i,$dir,$codon,$aa,$name);
	}
    }
}
