#!/usr/bin/env perl

# Inputs: strain name file, and file whose (first) line is in BL format
# Output: distance matrix, with strain name at end of each row

use Utility;
use strict;

my $pNames = readAndSortStrainNames($ARGV[0]);

sub byStrainName {
    $a eq 'REF' ? -1 : $b eq 'REF' ? 1 : $a cmp $b;
}

sub readAndSortStrainNames {
    my ($f) = @_;
    my $fh = openToRead($f);
    my @names = <$fh>;
    chomp @names;
    return [sort byStrainName @names];
}

my $fh = openToRead($ARGV[1]);
chomp (my $line = <$fh>);

my (undef,undef,undef,$tot,@rest) = split(" ",$line);
my $j=0;
# $descs[$m] is list of [$len,$desc] pairs.
my (@mthr,@dtrs,@descs,@dist);
# If there are $nr = scalar(@rest) items in @rest,
# there are $nn = ($nr/2)+1 nodes in the tree (the +1 is for
# the root). Just over half of these are terminal nodes,
# i.e. $nt = ($nn+1)/2 = $nr/4+1
my $nt = scalar(@rest)/4+1;
for (my $i=0; $i<$nt; $i++) {
    $descs[$i] = [[0,$i]];
    $dist[$i][$i] = 0.0;
}
for (my $i=0; $i<$#rest; $i+=2) {
    my $l = $rest[$i];
    my $m = $rest[$i+1];
    $mthr[$j] = $m;
    push @{$dtrs[$m]},$j;
    foreach my $pA (@{$descs[$j]}) {
	push @{$descs[$m]},[$pA->[0]+$l,$pA->[1]];
    }
    $j++;
}
foreach my $m ($nt .. $#dtrs) {
    my ($d1,$d2) = @{$dtrs[$m]};
    my @descs1 = @{$descs[$d1]};
    my @descs2 = @{$descs[$d2]};
    foreach my $pA1 (@descs1) {
	my ($len1,$d1) = @$pA1;
	foreach my $pA2 (@descs2) {
	    my ($len2,$d2) = @$pA2;
	    $dist[$d1][$d2] = $dist[$d2][$d1] = $len1+$len2;
	}
    }
}
foreach my $i (0 .. $#dist) {
    my $pRow = $dist[$i];
    foreach my $j (0 .. $#$pRow) {
	print " " if ($j > 0);
	printf("%10.8f",$tot*$dist[$i][$j]);
    }
    print " $pNames->[$i]\n";
}
