#!/usr/bin/env perl

=head1 NAME

which.pl

=head1 SYNOPSIS

which.pl cmd cmd cmd ...

=head1 DESCRIPTION

Report all places in which B<cmd> (either a shell command or a Perl module)
can be found in current PATH or PERL5LIB respectively.

=head1 AUTHOR

David Carter, C<dmc@sanger.ac.uk>

=head1 SEE ALSO

       

=cut

use Getopt::Std;

use strict;

my @xdirs = split(/:/,$ENV{PATH});
my @pdirs = @INC;

foreach my $cmd (@ARGV) {
    print $cmd . "\n";
    my $found1 = findIn($cmd,$cmd,@xdirs);
    (my $cmd2 = $cmd) =~ s@::@/@g;
    $cmd2 .= ".pm" unless ($cmd2 =~ /\.pm$/);
    my $found2 = findIn($cmd,$cmd2,@pdirs);
    print "   not found\n" unless ($found1 || $found2);
}

sub findIn {
    my ($cmd,$cmd2,@dirs) = @_;
    my $found = 0;
    my @files;
    foreach my $dir (@dirs) {
	my $f = "$dir/$cmd2";
	if (-f $f) {
	    push @files,$f;
	}
    }
    my $ok = @files > 0;
    print `ls -od @files | sed 's/^/   /'` if ($ok);
    return $ok;
}
