#!/usr/bin/env perl

use Utility;
use strict;

my $ROOT = '/lustre/scratch1/sanger/dmc/sgroot';

my $sFile = "$ROOT/ref/yong/ref4600_shred_lon.fasta";
my $qFile = "$ROOT/ref/yong/ref4600_shred_lon.fasta.qual";

my $rFile = "$ROOT/read/YONG/clip.fastq";
my $wFile = "$ROOT/read/YONG/clip.wm";
my $cFile = "$ROOT/read/YONG/name_corrections.txt";

my $fh = openToRead($sFile);
my $qh = openToRead($qFile);
my $gh = openToWrite($rFile);
my $wh = openToWrite($wFile);
my $ch = openToWrite($cFile);

my ($name,$seq);
while (<$fh>) {
    chomp;
    if (/^>(\S+)/) {
	if (defined $name) {
	    #report();
	    #report("reading qualities for $name");
	    my $quals = readQuals($qh,length($seq));
	    #report();
	    #report("writing fastq for $name");
	    printFastq($name,$seq,$quals,$gh,$wh,$ch);
	    #report();
	}
	$name = $1;
	#report("reading sequence for $name");
	$seq = "";
    } else {
	$seq .= $_;
    }
}
my $quals = readQuals($qh,length($seq));
printFastq($name,$seq,$quals,$gh,$wh,$ch);

sub readQuals {
    my ($fh,$n) = @_;
    <$fh>; # title
    my $quals;
    my $todo = $n;
    while ($todo > 0) {
	chomp (my $line = <$fh>);
	my @a = split(" ",$line);
	foreach my $k (@a) {
	    $quals .= chr(33+$k);
	}
	$todo -= scalar(@a);
    }
    return $quals;
}

sub printFastq {
    my ($name,$seq,$quals,$gh,$wh,$ch) = @_;
    my $munged = mungeContigName($name);
    print $ch "$name $munged\n";
    my $len = length($seq);
    print $gh "\@$name bases 1 to $len\n";
    print $gh $seq . "\n";
    print $gh "+$name bases 1 to $len\n";
    print $gh $quals . "\n";
    printf($wh "%s %6d %4d %6d\n",$name,$len,1,$len);
}

sub mungeContigName {
    my ($name) = @_;
    my $n = $name;
    $n =~ s/.*_0*//;
    my $n1 = int($n/24);
    my $end = sprintf("%2d",1+$n-$n1*24);
    $end =~ tr/ /0/;
    my $n2 = int($n1/16);
    my $mid = chr(ord('a')+$n1-$n2*16);
    return sprintf("YONG-%d%s%s.p1k",$n2+1,$mid,$end);
}
